//
// $Source: /afs/desy.de/group/hera-b/repository/ARTE/arte/include/PointerBase.hh,v $
// $Revision: 1.13 $
// $Author: natasha $
// $Date: 1999/07/02 10:25:09 $
//
// Created: Jan 1996,   C. Borgmeier
//
//
// This file is part of the C++ interface for Arte
// the HERA-B reconstruction and analysis tool
//
// This is the definition of the ArtePointerBase class.
//
// ArtePointerBase is the base class of the ArtePointer
// template class.
//
//
// .SS Usage // A Pointer to an GESL is declared as
//
// .ft B
//      ArtePointer<GESL> geslPoint;
//
//
// .SS See Also
//
// ArtePointer.hh ArteStructures.hh
//
// Author: Christoph Borgmeier, Hartwig Albrecht
//

#ifndef _ARTE_POINTER_BASE_HH_
#define _ARTE_POINTER_BASE_HH_


#ifndef __CINT__
#include <iosfwd>
#else
#include <iostream>
#endif


#ifdef USE_GATE
class TBuffer;
#endif


class ArtePointerBase
{
#ifdef USE_GATE
friend inline TBuffer& operator<<( TBuffer&, ArtePointerBase );
friend inline TBuffer& operator>>( TBuffer&, ArtePointerBase& );
#endif

public:
  inline ArtePointerBase( const ArtePointerBase& i ): idx( i.index() ) {}
#if !defined(ARTE_CXX_COMPAT_MODE) || (ARTE_CXX_COMPAT_MODE==1)
public:
#else
protected:
#endif
  inline ArtePointerBase( unsigned int i );
  inline unsigned int index() const;

  int operator==( const ArtePointerBase & arg ) const;
  int operator!=( const ArtePointerBase & arg ) const;

  //protected:
  inline unsigned int & indexReference();
  // Returns the private data.

  void increment() { ++idx; }
  void decrement() { --idx; }

  ArtePointerBase& operator+=( int offset ) { idx+=offset; return *this; }
  
public:
  void print( ostream& os ) const;

private:
  unsigned int idx;
};


inline ostream & operator<<( ostream & os, const ArtePointerBase & ap );
// Output to a stream.


#include "PointerBase.icc"

#endif
