*-- Author : A.Spiridonov
************************************************************
*                                                          *
      subroutine utfder(x0,bderiv)
*
*     Define derivatives of the field of HERA-B magnet 
*      for reconstruction program in ARTE environment
*
*            Version  3.00  Produced at 5/12/96
*
*
*      Input  :
*
*      x0(3) -  coordinates of the point in cm
*
*
*      Output :
*
*      bderiv(9) -  derivatives of field componentes with
*                   respect to coordinates (kG/cm) :
*                     dBx/dx,dBx/dy,dBx/dz,
*                     dBy/dx,dBy/dy,DBy/dz
*                     dBz/dx,dBz/dy,dBz/dz.
*                     
*
*
*     Author :  Alexander Spiridonov
*     E-mail :  spiridon@ifh.de
*                                                          *
*                                                          *
************************************************************
*
      implicit none
*
#include "arte/Arte.fi"
#include "gean/ZGESL.fi"
#include "gean/ZGMOTH.fi"
#include "auto/GESL.fi"
#include "auto/MF.fi"
*
*     integer S W I T C H  /0/
      integer S W I T C H 
*     integer Q I N T E R  /1/
      integer Q I N T E R       
      save switch
*
      integer j,J_P,N_P3,ix,iy,iz,isl,j1,j2
*     integer ID_reg,ID_reg_1,ID_reg_2/2/ 
      integer ID_reg,ID_reg_1,ID_reg_2
*
      real x(3),x1(3),x0(3),bderiv(9)
      real dBx_dx,dBx_dy,dBx_dz,dBy_dx,dBy_dy,dBy_dz
     +    ,dBz_dx,dBz_dy,dBz_dz
      real rotxy,rotxz,rotyz,div
      real xposmg,yposmg,zposmg
      real c_c(20)
*
      real e
      parameter (e=1.e-4)
*
      integer MAX_reg
      parameter ( MAX_reg = 21 )
      integer LOCF , LOC_MF (MAX_reg)
      integer JMFPA(1)
      equivalence ( JMF01 , JMFPA(1) )
*
      integer lk,nx,ny,nz,N_knot
      integer lk000,lk001,lk002,lk010,lk011,lk012,lk020,lk021,lk022
      integer lk100,lk101,lk102,lk110,lk111,lk112,lk120,lk121,lk122
      integer lk200,lk201,lk202,lk210,lk211,lk212,lk220,lk221,lk222
*
      real r00,r01,r02,r10,r11,r12,r20,r21,r22,rr0,rr1,rr2
      real xx0,xx1,xx2,yy0,yy1,yy2,zz0,zz1,zz2
      real xx_0,xx_1,xx_2,xx_3,y_1,y_2,y_3
      real l_inter,q_inter
*
      save ID_reg_2,LOC_MF
      save xposmg,yposmg,zposmg
      DATA  S W I T C H /0/, Q I N T E R  /1/, ID_reg_2/2/   
*     

#include "arte/Arte.ff"
#include "auto/GESL.ff" 
#include "auto/MF.ff"
*
*                                                location of knot
*
*     lk(ix,iy,iz)=(ix-1)*ny*nz+(iy-1)*nz+iz
*
*                                                linear interpolation
*
      l_inter(xx_1,xx_2,y_1,y_2,xx_0) =
     + ( (xx_0-xx_2)*y_1+(xx_1-xx_0)*y_2 )/(xx_1-xx_2)
*
*                                                quadratic interpolation
*
      q_inter ( xx_1,xx_2,xx_3,y_1,y_2,y_3,xx_0 ) =
     + (((xx_0-xx_3)  *( ((xx_0-xx_2)*y_1+(xx_1-xx_0)*y_2)
     + / (xx_1-xx_2) ) )
     + + ( (xx_1-xx_0)*( ((xx_0-xx_3)*y_2+(xx_2-xx_0)*y_3) 
     + / (xx_2-xx_3) ) ) )
     + / (xx_1-xx_3)
*
*==================================================================
*
*
*                                             Initialization
      if ( SWITCH .eq. 0 )  then
*
      LOC_MF( 1) = LOCF ( JMF01 ) - LOCF ( JMF01) + 1
      LOC_MF( 2) = LOCF ( JMF02 ) - LOCF ( JMF01) + 1
      LOC_MF( 3) = LOCF ( JMF03 ) - LOCF ( JMF01) + 1
      LOC_MF( 4) = LOCF ( JMF04 ) - LOCF ( JMF01) + 1
      LOC_MF( 5) = LOCF ( JMF05 ) - LOCF ( JMF01) + 1
      LOC_MF( 6) = LOCF ( JMF06 ) - LOCF ( JMF01) + 1
      LOC_MF( 7) = LOCF ( JMF07 ) - LOCF ( JMF01) + 1
      LOC_MF( 8) = LOCF ( JMF08 ) - LOCF ( JMF01) + 1
      LOC_MF( 9) = LOCF ( JMF09 ) - LOCF ( JMF01) + 1
      LOC_MF(10) = LOCF ( JMF10 ) - LOCF ( JMF01) + 1
      LOC_MF(11) = LOCF ( JMF11 ) - LOCF ( JMF01) + 1
      LOC_MF(12) = LOCF ( JMF12 ) - LOCF ( JMF01) + 1
      LOC_MF(13) = LOCF ( JMF13 ) - LOCF ( JMF01) + 1
      LOC_MF(14) = LOCF ( JMF14 ) - LOCF ( JMF01) + 1
      LOC_MF(15) = LOCF ( JMF15 ) - LOCF ( JMF01) + 1
      LOC_MF(16) = LOCF ( JMF16 ) - LOCF ( JMF01) + 1
      LOC_MF(17) = LOCF ( JMF17 ) - LOCF ( JMF01) + 1
      LOC_MF(18) = LOCF ( JMF18 ) - LOCF ( JMF01) + 1
      LOC_MF(19) = LOCF ( JMF19 ) - LOCF ( JMF01) + 1
      LOC_MF(20) = LOCF ( JMF20 ) - LOCF ( JMF01) + 1
      LOC_MF(21) = LOCF ( JMF21 ) - LOCF ( JMF01) + 1
*
*                                              Look for superlayer with
*                                              coordinates of magnet
        if(NGESL().lt.0) go to 996
        do isl=1,NGESL()
          if(IGESL_CMP(isl).eq.LGESL_FLD) then
            xposmg = FGESL_X1(isl)
            yposmg = FGESL_Y1(isl)
            zposmg = FGESL_Z1(isl) 
            if(IGESL_MFL(isl).eq.0) then
              switch = 1
            else
              switch = 2
            end if
           end if
        end do
        if(switch.eq.0) go to 996
      end if
*
*                                              Homogeneous magnetic field
*
      if ( SWITCH .eq. 1 ) then
*
        do j=1,9
          bderiv(j)=0.
        end do 
*
*                                              Inhomogeneous magnetic field
*
      else if ( SWITCH .eq. 2 ) then
*
      x1(1) = x0(1) - xposmg
      x1(2) = x0(2) - yposmg
      x1(3) = x0(3) - zposmg
*
      if(x1(1).lt.0.) then
        if(-x1(1).lt.1.e-10) x1(1)=0.
      else
        if( x1(1).lt.1.e-10) x1(1)=0.
      end if
      if(x1(2).lt.0.) then
        if(-x1(2).lt.1.e-10) x1(2)=0.
      else
        if( x1(2).lt.1.e-10) x1(2)=0.
      end if
      if(x1(3).lt.0.) then
        if(-x1(3).lt.1.e-10) x1(3)=0.
      else
        if( x1(3).lt.1.e-10) x1(3)=0.
      end if
*
*                                               Test if  point is inside
*                                             region treated during previous
*                                                    call of GUFLD
*
      if (
     + x1(1).ge.FMFLD_Xmin(ID_reg_2).and.
     + x1(1).le.FMFLD_Xmax(ID_reg_2).and.
     + x1(2).ge.FMFLD_Ymin(ID_reg_2).and.
     + x1(2).le.FMFLD_Ymax(ID_reg_2).and.
     + x1(3).ge.FMFLD_Zmin(ID_reg_2).and.
     + x1(3).le.FMFLD_Zmax(ID_reg_2)       ) then
*
        if( IMFLD_pipe(ID_reg_2).ne.0) then
*                                                                  e-pipe



          if (
     +      x1(3).ge.FMFLD_Zmin(1).and.
     +      x1(3).le.FMFLD_Zmax(1).and.
     +      x1(1).ge.FMFLD_Xmin(1).and.
     +      x1(1).le.FMFLD_Xmax(1).and.
     +      x1(2).ge.FMFLD_Ymin(1).and.
     +      x1(2).le.FMFLD_Ymax(1)       )   then
            ID_reg=1
            go to 7777
          else
            ID_reg=ID_reg_2
            go to 7777
          end if
*

        else
          ID_reg=ID_reg_2
          go to 7777
        end if
*
      end if
*
        do id_reg_1=2,NMFLD()                ! Search of region where point is
*
          if (    ID_reg_2.ne.ID_reg_1 .and.
     +    x1(1).ge.FMFLD_Xmin(ID_reg_1).and.
     +    x1(1).le.FMFLD_Xmax(ID_reg_1)      )    then
*
          if (
     +    x1(2).ge.FMFLD_Ymin(ID_reg_1).and.
     +    x1(2).le.FMFLD_Ymax(ID_reg_1).and.
     +    x1(3).ge.FMFLD_Zmin(ID_reg_1).and.
     +    x1(3).le.FMFLD_Zmax(ID_reg_1)       )   then
*
            ID_reg_2=ID_reg_1
*
            if( IMFLD_pipe(ID_reg_2).ne.0) then
*                                                                   e-pipe
              if (
     +        x1(3).ge.FMFLD_Zmin(1).and.
     +        x1(3).le.FMFLD_Zmax(1).and.
     +        x1(1).ge.FMFLD_Xmin(1).and.
     +        x1(1).le.FMFLD_Xmax(1).and.
     +        x1(2).ge.FMFLD_Ymin(1).and.
     +        x1(2).le.FMFLD_Ymax(1)       )   then
                ID_reg=1
                go to 7777
              else
                ID_reg=ID_reg_2
                go to 7777
              end if
*
            else
              ID_reg=ID_reg_2
              go to 7777
*
            end if
*
          end if                              ! end if on x,y test
          end if                              ! end if on z   test
*
        end do
*                                             Point is outside of any region
        do j=1,9
          bderiv(j)=0.
        end do 
*
        go to 999
*                                             Field evaluation
*
 7777 J_P  = JMFPA ( LOC_MF ( ID_reg ) )
*                                             Ordinary polinimial
      if(IMFLD_meth(ID_reg).eq.1) then
*
      N_P3 = IQ(J_P + 1)
*
      ix = int((x1(1)-FMFLD_Xmin(ID_reg)-e) / FMFLD_DR_x(ID_reg)) + 1
      iy = int((x1(2)-FMFLD_Ymin(ID_reg)-e) / FMFLD_DR_y(ID_reg)) + 1
      iz = int((x1(3)-FMFLD_Zmin(ID_reg)-e) / FMFLD_DR_z(ID_reg)) + 1
*
      x(1) = x1(1) * FMFLD_scal ( ID_reg )
      x(2) = x1(2) * FMFLD_scal ( ID_reg )
      x(3) = x1(3) * FMFLD_scal ( ID_reg )
*
      j = ( (ix-1) * IMFLD_nr_y(ID_reg) * IMFLD_nr_z(ID_reg)
     +    + (iy-1) * IMFLD_nr_z(ID_reg)
     +    + (iz-1) ) * N_P3 + 2 + J_P
*
      if ( N_P3 .eq. 60 ) then                 !    3rd power
*
*                                                    dBx/dx
      c_c( 1) = Q(j+ 2)        
      c_c( 2) = Q(j+ 5) * 2.
      c_c( 3) = Q(j+ 8)
      c_c( 4) = Q(j+ 9)
      c_c( 5) = Q(j+11) * 3.
      c_c( 6) = Q(j+16)
      c_c( 7) = Q(j+18)
      c_c( 8) = Q(j+14) * 2.
      c_c( 9) = Q(j+15) * 2.
      c_c(10) = Q(J+20)
*
        dBx_dx =  c_c(1)
     +  +x(1)*( c_c(2)+x(2)* c_c(8)+x(3)*c_c(9)+x(1)*c_c(5))
     +  +x(2)*( c_c(3)+x(3)*c_c(10)+x(2)*c_c(6) )
     +  +x(3)*( c_c(4)+x(3)*c_c(7) )
*                                                    dBx/dy
      c_c( 1) = Q(j+ 3)        
      c_c( 2) = Q(j+ 8)
      c_c( 3) = Q(j+ 6) * 2.
      c_c( 4) = Q(j+10)
      c_c( 5) = Q(j+14) 
      c_c( 6) = Q(j+12) * 3.
      c_c( 7) = Q(j+19)
      c_c( 8) = Q(j+16) * 2.
      c_c( 9) = Q(j+20) 
      c_c(10) = Q(J+17) * 2.
*
        dBx_dy =  c_c(1)
     +  +x(1)*( c_c(2)+x(2)* c_c(8)+x(3)*c_c(9)+x(1)*c_c(5))
     +  +x(2)*( c_c(3)+x(3)*c_c(10)+x(2)*c_c(6) )
     +  +x(3)*( c_c(4)+x(3)*c_c(7) )
*                                                    dBx/dz
      c_c( 1) = Q(j+ 4)        
      c_c( 2) = Q(j+ 9) 
      c_c( 3) = Q(j+10)
      c_c( 4) = Q(j+ 7) * 2.
      c_c( 5) = Q(j+15)  
      c_c( 6) = Q(j+17)
      c_c( 7) = Q(j+13) * 3.
      c_c( 8) = Q(j+20) 
      c_c( 9) = Q(j+18) * 2. 
      c_c(10) = Q(J+19) * 2.
*
        dBx_dz =  c_c(1)
     +  +x(1)*( c_c(2)+x(2)* c_c(8)+x(3)*c_c(9)+x(1)*c_c(5))
     +  +x(2)*( c_c(3)+x(3)*c_c(10)+x(2)*c_c(6) )
     +  +x(3)*( c_c(4)+x(3)*c_c(7) )
*                                                    dBy/dx
      c_c( 1) = Q(j+22)        
      c_c( 2) = Q(j+25) * 2.
      c_c( 3) = Q(j+28)
      c_c( 4) = Q(j+29)
      c_c( 5) = Q(j+31) * 3.
      c_c( 6) = Q(j+36)
      c_c( 7) = Q(j+38)
      c_c( 8) = Q(j+34) * 2.
      c_c( 9) = Q(j+35) * 2.
      c_c(10) = Q(J+40)
*
        dBy_dx =  c_c(1)
     +  +x(1)*( c_c(2)+x(2)* c_c(8)+x(3)*c_c(9)+x(1)*c_c(5))
     +  +x(2)*( c_c(3)+x(3)*c_c(10)+x(2)*c_c(6) )
     +  +x(3)*( c_c(4)+x(3)*c_c(7) )
*                                                    dBy/dy
      c_c( 1) = Q(j+23)        
      c_c( 2) = Q(j+28)
      c_c( 3) = Q(j+26) * 2.
      c_c( 4) = Q(j+30)
      c_c( 5) = Q(j+34) 
      c_c( 6) = Q(j+32) * 3.
      c_c( 7) = Q(j+39)
      c_c( 8) = Q(j+36) * 2.
      c_c( 9) = Q(j+40) 
      c_c(10) = Q(J+37) * 2.
*
        dBy_dy =  c_c(1)
     +  +x(1)*( c_c(2)+x(2)* c_c(8)+x(3)*c_c(9)+x(1)*c_c(5))
     +  +x(2)*( c_c(3)+x(3)*c_c(10)+x(2)*c_c(6) )
     +  +x(3)*( c_c(4)+x(3)*c_c(7) )
*                                                    dBy/dz
      c_c( 1) = Q(j+24)        
      c_c( 2) = Q(j+29) 
      c_c( 3) = Q(j+30)
      c_c( 4) = Q(j+27) * 2.
      c_c( 5) = Q(j+35)  
      c_c( 6) = Q(j+37)
      c_c( 7) = Q(j+33) * 3.
      c_c( 8) = Q(j+40) 
      c_c( 9) = Q(j+38) * 2. 
      c_c(10) = Q(J+39) * 2.
*
        dBy_dz =  c_c(1)
     +  +x(1)*( c_c(2)+x(2)* c_c(8)+x(3)*c_c(9)+x(1)*c_c(5))
     +  +x(2)*( c_c(3)+x(3)*c_c(10)+x(2)*c_c(6) )
     +  +x(3)*( c_c(4)+x(3)*c_c(7) )
*                                                    dBz/dx
      c_c( 1) = Q(j+42)        
      c_c( 2) = Q(j+45) * 2.
      c_c( 3) = Q(j+48)
      c_c( 4) = Q(j+49)
      c_c( 5) = Q(j+51) * 3.
      c_c( 6) = Q(j+56)
      c_c( 7) = Q(j+58)
      c_c( 8) = Q(j+54) * 2.
      c_c( 9) = Q(j+55) * 2.
      c_c(10) = Q(J+60)
*
        dBz_dx =  c_c(1)
     +  +x(1)*( c_c(2)+x(2)* c_c(8)+x(3)*c_c(9)+x(1)*c_c(5))
     +  +x(2)*( c_c(3)+x(3)*c_c(10)+x(2)*c_c(6) )
     +  +x(3)*( c_c(4)+x(3)*c_c(7) )
*                                                    dBz/dy
      c_c( 1) = Q(j+43)        
      c_c( 2) = Q(j+48)
      c_c( 3) = Q(j+46) * 2.
      c_c( 4) = Q(j+50)
      c_c( 5) = Q(j+54) 
      c_c( 6) = Q(j+52) * 3.
      c_c( 7) = Q(j+59)
      c_c( 8) = Q(j+56) * 2.
      c_c( 9) = Q(j+60) 
      c_c(10) = Q(J+57) * 2.
*
        dBz_dy =  c_c(1)
     +  +x(1)*( c_c(2)+x(2)* c_c(8)+x(3)*c_c(9)+x(1)*c_c(5))
     +  +x(2)*( c_c(3)+x(3)*c_c(10)+x(2)*c_c(6) )
     +  +x(3)*( c_c(4)+x(3)*c_c(7) )
*                                                    dBz/dz
      c_c( 1) = Q(j+44)        
      c_c( 2) = Q(j+49) 
      c_c( 3) = Q(j+50)
      c_c( 4) = Q(j+47) * 2.
      c_c( 5) = Q(j+55)  
      c_c( 6) = Q(j+57)
      c_c( 7) = Q(j+53) * 3.
      c_c( 8) = Q(j+60) 
      c_c( 9) = Q(j+58) * 2. 
      c_c(10) = Q(J+59) * 2.
*
        dBz_dz =  c_c(1)
     +  +x(1)*( c_c(2)+x(2)* c_c(8)+x(3)*c_c(9)+x(1)*c_c(5))
     +  +x(2)*( c_c(3)+x(3)*c_c(10)+x(2)*c_c(6) )
     +  +x(3)*( c_c(4)+x(3)*c_c(7) )
*
*
      else if ( N_P3 .eq. 30 ) then            !    2nd power
*
*                                                          dBx/dx,dy,dz      
      dBx_dx=Q(j+2)+x(1)*2.*Q(j+5)+x(2)*Q(j+8)+x(3)*Q(j+9) 
      dBx_dy=Q(j+9)+x(1)*Q(j+8)+x(2)*2.*Q(j+6)+x(3)*Q(j+10)
      dBx_dz=Q(j+4)+x(1)*Q(j+9)+x(2)*Q(j+10)+x(3)*2.*Q(j+7)
*                                                          dBy/dx,dy,dz      
      dBx_dx=Q(j+12)+x(1)*2.*Q(j+15)+x(2)*Q(j+18)+x(3)*Q(j+19) 
      dBx_dy=Q(j+19)+x(1)*Q(j+18)+x(2)*2.*Q(j+16)+x(3)*Q(j+20)
      dBx_dz=Q(j+14)+x(1)*Q(j+19)+x(2)*Q(j+20)+x(3)*2.*Q(j+17)
*                                                          dBz/dx,dy,dz      
      dBx_dx=Q(j+22)+x(1)*2.*Q(j+25)+x(2)*Q(j+28)+x(3)*Q(j+29) 
      dBx_dy=Q(j+29)+x(1)*Q(j+28)+x(2)*2.*Q(j+26)+x(3)*Q(j+30)
      dBx_dz=Q(j+24)+x(1)*Q(j+29)+x(2)*Q(j+30)+x(3)*2.*Q(j+27)
*
      else if ( N_P3 .eq. 12 ) then             !    1st  power
*        
*                                                          dBx/dx,dy,dz
      dBx_dx=Q(j+2)
      dBx_dy=Q(j+3)
      dBx_dz=Q(j+4)
*                                                          dBy/dx,dy,dz
      dBx_dx=Q(j+12)
      dBx_dy=Q(j+13)
      dBx_dz=Q(j+14)
*                                                          dBz/dx,dy,dz
      dBx_dx=Q(j+22)
      dBx_dy=Q(j+23)
      dBx_dz=Q(j+24)
*
      else if ( N_P3 .eq. 3 ) then             !    0 power
*
      if(ix.eq.IMFLD_nr_x(ID_reg)) then
        j1=j-IMFLD_nr_y(ID_reg) * IMFLD_nr_z(ID_reg)*N_p3
        j2=j
      else       
        j1=j
        j2=j+IMFLD_nr_y(ID_reg) * IMFLD_nr_z(ID_reg)*N_p3
      end if
*
      dBx_dx=(Q(j2+1)-Q(j1+1))/FMFLD_DR_x(ID_reg)*100.
      dBy_dx=(Q(j2+2)-Q(j1+2))/FMFLD_DR_x(ID_reg)*100.
      dBz_dx=(Q(j2+3)-Q(j1+3))/FMFLD_DR_x(ID_reg)*100.
*
      if(iy.eq.IMFLD_nr_y(ID_reg)) then
        j1=j-IMFLD_nr_z(ID_reg)*N_p3
        j2=j
      else       
        j1=j
        j2=j+IMFLD_nr_z(ID_reg)*N_p3
      end if
*
      dBx_dy=(Q(j2+1)-Q(j1+1))/FMFLD_DR_y(ID_reg)*100.
      dBy_dy=(Q(j2+2)-Q(j1+2))/FMFLD_DR_y(ID_reg)*100.
      dBz_dy=(Q(j2+3)-Q(j1+3))/FMFLD_DR_y(ID_reg)*100.
*
      if(iz.eq.IMFLD_nr_z(ID_reg)) then
        j1=j-N_p3
        j2=j
      else       
        j1=j
        j2=j+N_p3
      end if
*
      dBx_dz=(Q(j2+1)-Q(j1+1))/FMFLD_DR_z(ID_reg)*100.
      dBy_dz=(Q(j2+2)-Q(j1+2))/FMFLD_DR_z(ID_reg)*100.
      dBz_dz=(Q(j2+3)-Q(j1+3))/FMFLD_DR_z(ID_reg)*100.
*
*
      else                                    ! Error number of coefficients
        go to 997
      end if
*
*                                              Symmetrization of derivatives
*        
*  bderiv(6)=1)dBx/dx,2)dBx/dy, 3)dBx/dz, 4)dBy/dy, 5)dBy/dz, 6)dBz/dz.
*
      rotxy = dBx_dy - dBy_dx
      rotxz = dBx_dz - dBz_dx
      rotyz = dBy_dz - dBz_dy
      div   = dBx_dx + dBy_dy + dBz_dz
*
      Bderiv(1) = dBx_dx - div * 0.3333333333
      Bderiv(2) = dBx_dy - rotxy * 0.5
      Bderiv(3) = dBx_dz - rotxz * 0.5
      Bderiv(4) = dBy_dy - div * 0.3333333333
      Bderiv(5) = dBy_dz - rotyz * 0.5
      Bderiv(6) = dBz_dz - div * 0.3333333333
*
*                                              Store full matrix
      Bderiv(1) = dBx_dx * 0.01
      Bderiv(2) = dBx_dy * 0.01
      Bderiv(3) = dBx_dz * 0.01
      Bderiv(4) = dBy_dx * 0.01
      Bderiv(5) = dBy_dy * 0.01
      Bderiv(6) = dBy_dz * 0.01
      Bderiv(7) = dBz_dx * 0.01
      Bderiv(8) = dBz_dy * 0.01
      Bderiv(9) = dBz_dz * 0.01
*
*                                             Polinomial constrained by
*                                                Maxwell equation
*
      else if(IMFLD_meth(ID_reg).eq.4) then
*
      N_P3 = IQ(J_P + 1)
*
      ix = int((x1(1)-FMFLD_Xmin(ID_reg)-e) / FMFLD_DR_x(ID_reg)) + 1
      iy = int((x1(2)-FMFLD_Ymin(ID_reg)-e) / FMFLD_DR_y(ID_reg)) + 1
      iz = int((x1(3)-FMFLD_Zmin(ID_reg)-e) / FMFLD_DR_z(ID_reg)) + 1
*
      x(1) = x1(1) * FMFLD_scal ( ID_reg )
      x(2) = x1(2) * FMFLD_scal ( ID_reg )
      x(3) = x1(3) * FMFLD_scal ( ID_reg )
*
      j = ( (ix-1) * IMFLD_nr_y(ID_reg) * IMFLD_nr_z(ID_reg)
     +    + (iy-1) * IMFLD_nr_z(ID_reg)
     +    + (iz-1) ) * N_P3 + 2 + J_P
*                                                     1)  dBx/bx
      c_c( 1)=2.*Q(j+4)
      c_c( 2)=6.*Q(j+9)
      c_c( 3)=2.*Q(j+12)
      c_c( 4)=2.*Q(j+13)
      c_c( 5)=12.*Q(j+16)
      c_c( 6)=6.*(-Q(j+16)-Q(j+17)+Q(j+18))
      c_c( 7)=6.*(-Q(j+16)+Q(j+17)-Q(j+18))
      c_c( 8)=6.*Q(j+19)
      c_c( 9)=6.*Q(j+20)
      c_c(10)=-6.*(Q(j+22)+Q(j+24)) 
*
        dBx_dx =  c_c(1)
     +  +x(1)*( c_c(2)+x(2)* c_c(8)+x(3)*c_c(9)+x(1)*c_c(5))
     +  +x(2)*( c_c(3)+x(3)*c_c(10)+x(2)*c_c(6) )
     +  +x(3)*( c_c(4)+x(3)*c_c(7) )
*                                                     2)  dBx/dy
      c_c( 1)=Q(j+6)
      c_c( 2)=2.*Q(j+12)
      c_c( 3)=2.*Q(j+14)
      c_c( 4)=Q(j+15)
      c_c( 5)=3.*Q(j+19)
      c_c( 6)=3.*Q(j+21)
      c_c( 7)=-3.*(Q(j+19)+Q(j+21))
      c_c( 8)=12.*(-Q(j+16)-Q(j+17)+Q(j+18))
      c_c( 9)=-6.*(Q(j+22)+Q(j+24))
      c_c(10)=-6.*(Q(j+20)+Q(j+23))
*
        dBx_dy =  c_c(1)
     +  +x(1)*( c_c(2)+x(2)* c_c(8)+x(3)*c_c(9)+x(1)*c_c(5))
     +  +x(2)*( c_c(3)+x(3)*c_c(10)+x(2)*c_c(6) )
     +  +x(3)*( c_c(4)+x(3)*c_c(7) )
*                                                     3)  dBx/dz
      c_c( 1)=Q(j+7)
      c_c( 2)=2.*Q(j+13)
      c_c( 3)=Q(j+15)
      c_c( 4)=-6.*Q(j+9)-2.*Q(j+14)
      c_c( 5)=3.*Q(j+20)
      c_c( 6)=-3.*(Q(j+20)+Q(j+23))
      c_c( 7)=3.*Q(j+23)
      c_c( 8)=-6.*(Q(j+22)+Q(j+24))
      c_c( 9)=12.*(-Q(j+16)+Q(j+17)-Q(j+18))
      c_c(10)=-6.*(Q(j+19)+Q(j+21))
*
        dBx_dz =  c_c(1)
     +  +x(1)*( c_c(2)+x(2)* c_c(8)+x(3)*c_c(9)+x(1)*c_c(5))
     +  +x(2)*( c_c(3)+x(3)*c_c(10)+x(2)*c_c(6) )
     +  +x(3)*( c_c(4)+x(3)*c_c(7) )
*                                                     4)  dBy/dy
      c_c( 1)=2.*Q(j+5)
      c_c( 2)=2.*Q(j+14)
      c_c( 3)=6.*Q(j+10)
      c_c( 4)=-6.*Q(j+11)-2.*Q(j+13)
      c_c( 5)=6.*(-Q(j+16)-Q(j+17)+Q(j+18))
      c_c( 6)=12.*Q(j+17)
      c_c( 7)=6.*(Q(j+16)-Q(j+17)-Q(j+18))
      c_c( 8)=6.*Q(j+21)
      c_c( 9)=-6.*(Q(j+20)+Q(j+23))
      c_c(10)=6.*Q(j+22)
*
        dBy_dy =  c_c(1)
     +  +x(1)*( c_c(2)+x(2)* c_c(8)+x(3)*c_c(9)+x(1)*c_c(5))
     +  +x(2)*( c_c(3)+x(3)*c_c(10)+x(2)*c_c(6) )
     +  +x(3)*( c_c(4)+x(3)*c_c(7) )
*                                                     5)  dBy/dz
      c_c( 1)=Q(j+8)
      c_c( 2)=Q(j+15)
      c_c( 3)=-6.*Q(j+11)-2.*Q(j+13)
      c_c( 4)=-6.*Q(j+10)-2.*Q(j+12)
      c_c( 5)=-3.*(Q(j+22)+Q(j+24))
      c_c( 6)=3.*Q(j+22)
      c_c( 7)=3.*Q(j+24)
      c_c( 8)=-6.*(Q(j+20)+Q(j+23))
      c_c( 9)=-6.*(Q(j+19)+Q(j+21))
      c_c(10)=12.*(Q(j+16)-Q(j+17)-Q(j+18))
*
        dBy_dz =  c_c(1)
     +  +x(1)*( c_c(2)+x(2)* c_c(8)+x(3)*c_c(9)+x(1)*c_c(5))
     +  +x(2)*( c_c(3)+x(3)*c_c(10)+x(2)*c_c(6) )
     +  +x(3)*( c_c(4)+x(3)*c_c(7) )
*
      Bderiv(1) = dBx_dx * 0.01
      Bderiv(2) = dBx_dy * 0.01
      Bderiv(3) = dBx_dz * 0.01
      Bderiv(4) = dBx_dy * 0.01
      Bderiv(5) = dBy_dy * 0.01
      Bderiv(6) = dBy_dz * 0.01
      Bderiv(7) = dBx_dz * 0.01
      Bderiv(8) = dBy_dz * 0.01
      Bderiv(9) = -( dBx_dx + dBy_dy ) * 0.01
*
*
*                                             Interpolation
*
      else if(IMFLD_meth(ID_reg).eq.6) then
*
      N_knot = IQ(J_P + 2)/3
*
      ix = int((x1(1)-FMFLD_Xmin(ID_reg)-e) / FMFLD_DR_x(ID_reg)) + 1
      iy = int((x1(2)-FMFLD_Ymin(ID_reg)-e) / FMFLD_DR_y(ID_reg)) + 1
      iz = int((x1(3)-FMFLD_Zmin(ID_reg)-e) / FMFLD_DR_z(ID_reg)) + 1
*
      if(ix.gt.IMFLD_nr_x(ID_reg)-1) ix=IMFLD_nr_x(ID_reg)-1
      if(iy.gt.IMFLD_nr_y(ID_reg)-1) iy=IMFLD_nr_y(ID_reg)-1
      if(iz.gt.IMFLD_nr_z(ID_reg)-1) iz=IMFLD_nr_z(ID_reg)-1
*
      xx0 = FMFLD_Xmin(ID_reg) + float(ix-1) * FMFLD_DR_x(ID_reg)
      xx1 = xx0 + FMFLD_DR_x(ID_reg)
*
      yy0 = FMFLD_Ymin(ID_reg) + float(iy-1) * FMFLD_DR_y(ID_reg)
      yy1 = yy0 + FMFLD_DR_y(ID_reg)
*
      zz0 = FMFLD_Zmin(ID_reg) + float(iz-1) * FMFLD_DR_z(ID_reg)
      zz1 = zz0 + FMFLD_DR_z(ID_reg)
*
      nx=IMFLD_nr_x(ID_reg)
      ny=IMFLD_nr_y(ID_reg)
      nz=IMFLD_nr_z(ID_reg)
*      
*      lk(ix,iy,iz)=(ix-1)*ny*nz+(iy-1)*nz+iz
 
      lk000 = (ix-1) * ny*nz + (iy-1) * nz + iz + J_P + 2
      lk100 = lk000 + ny*nz
*      
      lk001 = lk000 + 1
      lk010 = lk000 + nz
      lk011 = lk010 + 1
* 
      lk101 = lk100 + 1
      lk110 = lk100 + nz
      lk111 = lk110 + 1
*
*                                                              dBx/dx
      dBx_dx = (Q(lk100)-Q(lk000))/(xx1-xx0)
*                                                              dBx/dy
      dBx_dy = (Q(lk010)-Q(lk000))/(yy1-yy0)
*                                                              dBx/dz
      dBx_dz = (Q(lk001)-Q(lk000))/(zz1-zz0)
*                                                              dBy/dx
      dBy_dx = (Q(lk100+n_knot)-Q(lk000+n_knot))/(xx1-xx0)
*                                                              dBy/dy
      dBy_dy = (Q(lk010+n_knot)-Q(lk000+n_knot))/(yy1-yy0)
*                                                              dBy/dz
      dBy_dz = (Q(lk001+n_knot)-Q(lk000+n_knot))/(zz1-zz0)      
*                                                              dBz/dx
      dBz_dx = (Q(lk100+2*n_knot)-Q(lk000+2*n_knot))/(xx1-xx0)
*                                                              dBz/dy
      dBz_dy = (Q(lk010+2*n_knot)-Q(lk000+2*n_knot))/(yy1-yy0)
*                                                              dBz/dz
      dBz_dz = (Q(lk001+2*n_knot)-Q(lk000+2*n_knot))/(zz1-zz0)
*
*
*                                              Symmetrization of derivatives
*        
*  bderiv(6)=1)dBx/dx,2)dBx/dy, 3)dBx/dz, 4)dBy/dy, 5)dBy/dz, 6)dBz/dz.
*
      rotxy = dBx_dy - dBy_dx
      rotxz = dBx_dz - dBz_dx
      rotyz = dBy_dz - dBz_dy
      div   = dBx_dx + dBy_dy + dBz_dz
*
      Bderiv(1) = dBx_dx - div * 0.3333333333
      Bderiv(2) = dBx_dy - rotxy * 0.5
      Bderiv(3) = dBx_dz - rotxz * 0.5
      Bderiv(4) = dBy_dy - div * 0.3333333333
      Bderiv(5) = dBy_dz - rotyz * 0.5
      Bderiv(6) = dBz_dz - div * 0.3333333333
*
*                                              Store full matrix
      Bderiv(1) = dBx_dx
      Bderiv(2) = dBx_dy
      Bderiv(3) = dBx_dz
      Bderiv(4) = dBy_dx
      Bderiv(5) = dBy_dy
      Bderiv(6) = dBy_dz
      Bderiv(7) = dBz_dx
      Bderiv(8) = dBz_dy
      Bderiv(9) = dBz_dz
*
      end if                                  ! END OF DERIVATIVES EVALUATION
*
      END IF                                  ! END of SWITCH=2
*
 999  return
*
 998  c78='UTFDER : TABLE  MFLD  IS  EMPTY'
      call ME78ER
      return
 997  c78='UTFDER : ERROR NUMBER OF COEFFICIENTS '
      call CH78BI(N_p3)
      call ME78ER
      return
 996  c78='UTFDER : NO MAGNET COORDINATES IN TABLE GESL  '
      call ME78ER
      return
*
      end
*
*.......................................................................
*     END OF UTFDER
*.......................................................................

