//                              -*- Mode: C++ -*-
//: CmdCareECDB.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#include "care/CmdCareECDB.hh"
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

#include "care/ECDBaseIO.hh"
#include "care/ECHF_status.hh"

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include "care/ECALDetector.hh"
#include "care/MMBank.hh"

/////////////// The function seen by kuip
#ifdef USEKUIP
extern "C" {

  void ecdbase_(){
    CmdCareECDB boardfile;
    boardfile.readAndExe();
//    boardfile.dump();
    ECALDetector::setDBflag(1);
  }
}

#endif

/////////////// Static data definition

char CmdCareECDB::filename[256] = "\0";

/////////////// CmdCareECDB class member function definition

CmdCareECDB::CmdCareECDB(){}
CmdCareECDB::~CmdCareECDB(){}

void CmdCareECDB::defValues(){
  strcpy(filename,"ECAL/EcalCalib.db");
  action = none;
  major = 1;
  minor = 10;
}


//--
//   Execute stored command // read or write from the DB
void CmdCareECDB::execute(){
  switch(action){
  case dbread:
    readDB();
    break;
  case dbwrite:
    writeDB();
    break;
  default:
    break;
  }
}

#ifdef USEKUIP
//--
//   Read acommand from external
void CmdCareECDB::read(){
  // 
  // read ASCII DB file name
  //
  char* name = readString();
  int i=0;
  for(; i<256 && name[i]!=' ' && name[i]!='\0'; i++)
    filename[i] = name[i];
  filename[i] = '\0';
  //
  // ACTION to do
  //
  int act = readInt();
  action = none;
  if( act==1) action = dbread;
  else if( act==2) action = dbwrite;
  //
  // DB version numbers
  //
  major = readInt();
  minor = readInt();
 
}
#endif

char* CmdCareECDB::calibfile(){
  if( filename[0]==0) 
    defValues();
  return filename;
}

void CmdCareECDB::dump(){
  arteMsg<<" Calibration file name :"<<filename;
  arteMsg.allp();
}

void CmdCareECDB::readDB(){

  MMBank<ECBO>::Drop();
  MMBank<ECHS>::Drop();
  MMBank<ECHF>::Drop();

  ECDBaseIO* db = ECDBaseIO::initialize(calibfile());
  arteMsg << "Open ECAL Database: " << calibfile();
  arteMsg.allp();
  // open the DB (the same as ECDBaseIO)
  db->open();
    arteMsg << "Fetching tables  ";
  arteMsg.allp();
  TimeTag_t myTime;
  myTime.minor = minor;
  myTime.major = major;
  db->fetchTables(myTime);
  int entriesEcbo = db->getEntriesForSTLBanksEcbo();
  int entriesEchs = db->getEntriesForSTLBanksEchs();
  int entriesEchf = db->getEntriesForSTLBanksEchf();

  ECBO board = {0,0,0,0,
                    {-1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
                     -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1}  };
  for(int i0= 0; i0<entriesEcbo; i0++)
        ArtePointer<ECBO>::append(board);
  ECHS channelbases = { 50., // info pedestal (peds)
                           {0., 1., 0., 0., 0., 0.}, // 6 cal const
                           {0., 1., 0., 0., 0., 0.}, // 6 cal errs
                           0.1, .10, // energy thresholds
                           10., 2., 4095., // MIP data
                           -1, -1, -1}; // FED channel data
  for(int i1= 0; i1<entriesEchs; i1++)
        ArtePointer<ECHS>::append(channelbases);
  ECHF channelbasef = { Echfs::usable | Echfs::readout | 
                            Echfs::pretrig | Echfs::usedcal | 
                            Echfs::avalcal,     // status of the channel
                            50.,2.,             // info pedestal (pedf,wid)
                            1.0}; 
  for(int i2= 0; i2<entriesEchf; i2++)
        ArtePointer<ECHF>::append(channelbasef);
 
  // define for ECDBaseIO the characters of the STL Banks
  db->setPointerToSTLBankEcbo(&(*(ArtePointer<ECBO>(1))));
  db->setPointerToSTLBankEchs(&(*(ArtePointer<ECHS>(1))));
  db->setPointerToSTLBankEchf(&(*(ArtePointer<ECHF>(1))));   
 
  db->read();
  arteMsg << "Closing "<<calibfile();
  arteMsg.allp();
  db->close();
}

void CmdCareECDB::writeDB(){
  ECDBaseIO* db = ECDBaseIO::initialize(calibfile()); 
  db->setDBname(calibfile());
  arteMsg << "Opening "<<calibfile();
  arteMsg.allp();
  // open the DB (the same as ECDBaseIO)
  db->open(ios::out);
  // define for ECDBaseIO the characters of the STL Banks
  db->setEntriesForSTLBanksEcbo(ArtePointer<ECBO>::nrow());
  db->setEntriesForSTLBanksEchs(ArtePointer<ECHS>::nrow());
  db->setEntriesForSTLBanksEchf(ArtePointer<ECHF>::nrow());
 // define for ECDBaseIO the characters of the STL Banks
  db->setPointerToSTLBankEcbo(&(*(ArtePointer<ECBO>(1))));
  db->setPointerToSTLBankEchs(&(*(ArtePointer<ECHS>(1))));
  db->setPointerToSTLBankEchf(&(*(ArtePointer<ECHF>(1))));   
  TimeTag_t myTime;
  myTime.minor = minor;
  myTime.major = major;
  arteMsg<<"Storing tables ";
  arteMsg.allp();
  db->store(myTime);
  arteMsg << "Closing "<<calibfile();
  arteMsg.allp();
}

void CmdCareECDB::setDB( char *name, int imaj, int imin){
  major = imaj;
  minor = imin;
  int i=0;
  for(; i<256 && name[i]!=' ' && name[i]!='\0'; i++)
      filename[i] = name[i];
  filename[i] = '\0';
  
}
