//                              -*- Mode: C++ -*-
//: CmdCareECDBMIZ.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#include "care/care.hh"
#include "care/CmdCareECDBMIZ.hh"
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "care/ECALDetector.hh"
#include "care/ECDBaseIO.hh"
#include "care/ECHF_status.hh"
#include "care/MMBank.hh"

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include "care/EcalDB.hh"

/////////////// The function seen by kuip
#ifdef USEKUIP
extern "C" {

  void ecdbmiz_(){
    CmdCareECDBMIZ boardfile;
    boardfile.readAndExe();
    boardfile.dump();
    ECALDetector::setDBflag(2);
  }
}

#endif

/////////////// Static data definition

char CmdCareECDBMIZ::filename[256] = "\0";

/////////////// CmdCareECDBMIZ class member function definition

CmdCareECDBMIZ::CmdCareECDBMIZ(){}
CmdCareECDBMIZ::~CmdCareECDBMIZ(){}

void CmdCareECDBMIZ::defValues(){
  strcpy(filename,"ECAL/EcalCalibMiz.db");
  action = none;
  match = 0;
  major = 1;
  minor = 10;
  tbset = TALL;
}


//--
//   Execute stored command // read or write from the DB
void CmdCareECDBMIZ::execute(){
  switch(action){
  case dbread:
    readDB(tbset, match);
    break;
  case dbwrite:
    writeDB(tbset, match);
    break;
  default:
    break;
  }
}

#ifdef USEKUIP
//--
//   Read acommand from external
void CmdCareECDBMIZ::read(){
  char* name = readString();
  int i=0;
  for(; i<256 && name[i]!=' ' && name[i]!='\0'; i++)
    filename[i] = name[i];
  filename[i] = '\0';

  // reading the TB set
  int itbset = readInt();
  tbset = TALL;
  if(itbset==1)  tbset = TSLOW;
  else if(itbset==2)  tbset = TFAST;
  
  int act = readInt();
  action = none;
  if( act==1) action = dbread;
  else if( act==2) action = dbwrite;
  //
  // DB version numbers
  //
  major = readInt();
  minor = readInt();

  // 
  // Match parameter 
  //
  char* smatch = readUppercaseString();
  const char tests[] = "EQGTLTGELE";
  for(i=0; i<5; i++){
    if( !strncmp(smatch,tests+i*2,2) ) break;
  }
  if( i<5 ) match = i;
}
#endif

char* CmdCareECDBMIZ::calibfile(){
  if( filename[0]==0) 
    defValues();
  return filename;
}

void CmdCareECDBMIZ::dump(){
  arteMsg<<"Calibration file name :"<<filename;
  arteMsg.allp();
  arteMsg<<" Tables:"<<tbset<<" action:"<<action
         <<" Tags: "<<major<<","<<minor<<"  match:"<<match;
  arteMsg.allp();
}

void CmdCareECDBMIZ::readDB(Tbset whichtable, int access){

  MMBank<ECBO>::Drop();
  MMBank<ECHS>::Drop();
  MMBank<ECHF>::Drop();

  arteMsg<< "Opening  " << calibfile();
  arteMsg.allp();
  EcalDB mydb;
  mydb.Open(calibfile(), 0);

  switch(whichtable){
  case TSLOW:
     mydb.ECBO.Book();
     mydb.ECHS.Book();
     if( mydb.ECBO.Match(major, minor,access)==0 ){
       arteMsg<<"This ECBO table version does not exists";
       arteMsg.warn();
       break;
     }
     if( mydb.ECHS.Match(major, minor,access)==0){
       arteMsg<<"This ECHS table version does not exists";
       arteMsg.warn();
       break;
     }
     arteMsg<<"Reading calibration data (SLOW): ";
     arteMsg.allp();
   
     MMBank<ECBO>::Book(mydb.ECBO.Nrow(major, minor,access));
     mydb.ECBO.Fetch(&(*(ArtePointer<ECBO>(1))));
     arteMsg<<"ECBO: Tags ("<< mydb.ECBO.Major()<<","<<mydb.ECBO.Minor()
            <<")   Entries: "<<ArtePointer<ECBO>::nrow();
     arteMsg.allp();
  
     MMBank<ECHS>::Book(mydb.ECHS.Nrow(major, minor,access));
     mydb.ECHS.Fetch(&(*(ArtePointer<ECHS>(1))));
     arteMsg<<"ECHS: Tags ("<< mydb.ECHS.Major()<<","<<mydb.ECHS.Minor()
            <<")   Entries: "<<ArtePointer<ECHS>::nrow();
     arteMsg.allp();

     mydb.ECBO.Unbook();
     mydb.ECHS.Unbook();
     break;
  case TFAST:
    mydb.ECHF.Book();
    if( mydb.ECHF.Match(major, minor,access)==0){
      arteMsg<<"This ECHF table version does not exists";
      arteMsg.warn();
      break;
    }
    arteMsg<<"Reading calibration data (FAST): ";
    arteMsg.allp();
   
    MMBank<ECHF>::Book(mydb.ECHF.Nrow(major, minor,access));
    mydb.ECHF.Fetch(&(*(ArtePointer<ECHF>(1))));
    arteMsg<<"ECHF: Tags ("<< mydb.ECHF.Major()<<","<<mydb.ECHF.Minor()
           <<")   Entries: "<<ArtePointer<ECHF>::nrow();
    arteMsg.allp();
      
    mydb.ECHF.Unbook();
    break;
  case TALL:
     mydb.ECBO.Book();
     mydb.ECHS.Book();
     mydb.ECHF.Book();
    if( mydb.ECBO.Match(major, minor,access)==0 ){
       arteMsg<<"This ECBO table version does not exists";
       arteMsg.warn();
       break;
     }
     if( mydb.ECHS.Match(major, minor,access)==0){
       arteMsg<<"This ECHS table version does not exists";
       arteMsg.warn();
       break;
     }
     if( mydb.ECHF.Match(major, minor,access)==0){
       arteMsg<<"This ECHF table version does not exists";
      arteMsg.warn();
      break;
     }
     arteMsg<<"Reading calibration data (ALL): ";
     arteMsg.allp(); 
     MMBank<ECBO>::Book(mydb.ECBO.Nrow(major, minor,access));
     mydb.ECBO.Fetch(&(*(ArtePointer<ECBO>(1))));
     arteMsg<<"ECBO: Tags ("<< mydb.ECBO.Major()<<","<<mydb.ECBO.Minor()
            <<")   Entries: "<<ArtePointer<ECBO>::nrow();
     arteMsg.allp();
  
     MMBank<ECHS>::Book(mydb.ECHS.Nrow(major, minor,access));
     mydb.ECHS.Fetch(&(*(ArtePointer<ECHS>(1))));
     arteMsg<<"ECHS: Tags ("<< mydb.ECHS.Major()<<","<<mydb.ECHS.Minor()
            <<")   Entries: "<<ArtePointer<ECHS>::nrow();
     arteMsg.allp();

     MMBank<ECHF>::Book(mydb.ECHF.Nrow(major, minor,access));
     mydb.ECHF.Fetch(&(*(ArtePointer<ECHF>(1))));
     arteMsg<<"ECHF: Tags ("<< mydb.ECHF.Major()<<","<<mydb.ECHF.Minor()
            <<")   Entries: "<<ArtePointer<ECHF>::nrow();
     arteMsg.allp();
     
     mydb.ECBO.Unbook();
     mydb.ECHS.Unbook();
     mydb.ECHF.Unbook();
     break;
  }
  arteMsg << "Closing " << calibfile();
  arteMsg.allp();
  mydb.Close();

}

void CmdCareECDBMIZ::writeDB(Tbset whichtable, int access){
  arteMsg<<"Opening "<<calibfile();
  arteMsg.allp();

  EcalDB mydb;
  mydb.Open(calibfile(), 0);
  switch(whichtable){
  case TSLOW:
     mydb.ECBO.Book();
     mydb.ECHS.Book();
     mydb.ECBO.Store(ArtePointer<ECBO>::nrow(),
		     &(*(ArtePointer<ECBO>(1))),
		       major, minor, access);
     mydb.ECHS.Store(ArtePointer<ECHS>::nrow(),
		     &(*(ArtePointer<ECHS>(1))),
		       major, minor, access);
     mydb.ECBO.Unbook();
     mydb.ECHS.Unbook();
     break;
  case TFAST:
    mydb.ECHF.Book();
    mydb.ECHF.Store(ArtePointer<ECHF>::nrow(),
		     &(*(ArtePointer<ECHF>(1))),
		    major, minor, access);
    mydb.ECHF.Unbook();
    break;
  case TALL:
     mydb.ECBO.Book();
     mydb.ECHS.Book();
     mydb.ECHF.Book();
     mydb.ECBO.Store(ArtePointer<ECBO>::nrow(),
		     &(*(ArtePointer<ECBO>(1))),
		     major, minor, access);
     mydb.ECHS.Store(ArtePointer<ECHS>::nrow(),
		     &(*(ArtePointer<ECHS>(1))),
		     major, minor, access);
     mydb.ECHF.Store(ArtePointer<ECHF>::nrow(),
		     &(*(ArtePointer<ECHF>(1))),
		     major, minor, access);
     mydb.ECBO.Unbook();
     mydb.ECHS.Unbook();
     mydb.ECHF.Unbook();
     break;
  }
  arteMsg << "Closing " << calibfile();
  arteMsg.allp();
  mydb.Close();
}

void CmdCareECDBMIZ::setDB( char *name, int imaj, int imin){
  major = imaj;
  minor = imin;
  int i=0;
  for(; i<256 && name[i]!=' ' && name[i]!='\0'; i++)
      filename[i] = name[i];
  filename[i] = '\0';
}

