//                              -*- Mode: C++ -*-
//: CmdCareLambda.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#include "care/CmdCareLambda.hh"
#include "care/ESCorCOGPos.hh"


/////////////// The function seen by kuip

extern "C" {
  void carelambda_(){
    CmdCareLambda lamCmd;
    lamCmd.readAndExe();
  }
}; // end of extern
//////////////////////////////////////////////////////////

/*
CLASS
  CmdCareLambda
  Base class to deal with the Lambda kuip command

class CmdCareLambda {
public:
  //-- Constructors
  CmdCareLambda();
  //-- Destructor
  // Destructor: currently void
  virtual ~CmdCareLambda();
  
  //-- Behavioural operations
  //   Reset stored data to default values
  virtual void defValues();
  //--
  //   Execute stored command
  virtual void execute();
  //--
  //   Read acommand from external
  virtual void read();
  //--
  //   Dump internal data
  virtual void dump();

private:
  ///////////////////// Data Part /////////////////////////////////
  //--
  Energy_t  inner,middle,outer;
};

*/

CmdCareLambda::CmdCareLambda() : subcal(Inner) , lambda(4.075) {}

CmdCareLambda::~CmdCareLambda(){}

void CmdCareLambda::defValues(){
  subcal = Inner;
  lambda = 4.075;
}

//--
//   Execute stored command
void CmdCareLambda::execute(){
  ESCorCOGPos::fill(subcal, lambda);
}

#ifdef USEKUIP
//--
//   Read a command from external
void CmdCareLambda::read(){
  subcal = (SubCal) readInt();
  lambda = readFloat();
  if(!(subcal>=Inner && subcal<=Outer)){
    defValues();
  }
}
#endif
                          
//--
//   Printout Information for the current lambda info stored
void CmdCareLambda::dump(){
   ESCorCOGPos::dump();
}


