//                              -*- Mode: C++ -*-
//:ELNearClusterFrac.C
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 27/5/97
// Last Update ........
//
//

#include"care/ELNearClusterFrac.hh"
#include"care/ELNearHitCellFrac.hh"
#include "care/EMClusterTree.hh"
#include "care/ELNearNeiCellLUT.hh"
#ifdef _OLD_NAMES_
#  include <algo.h>   
#else
#  include <algorithm>
#endif
#undef WRITE
/*
CLASS 
  ELNearClusterFrac

*/

///////////////// ELNearClusterFrac.C member function definitions ////////
//--
//  constructor
ELNearClusterFrac::ELNearClusterFrac() :  ELNearClusters() {}

//--
// constructor
ELNearClusterFrac::ELNearClusterFrac(ELNearHitCellFrac & nearT) {
  of(nearT);
}

//--
//  destructor
ELNearClusterFrac::~ELNearClusterFrac(){}


//--
//--
//  Get the neighbours of a given cell
void ELNearClusterFrac::of(const EMHitCell* hcell, const EMClusterTree & tree){
  //--
  // find the index of the cell pointed by the current cell
  of(hcell->cellIdx(), tree);
}

//--
//  Get neighbouring clusters of a given cell
void ELNearClusterFrac::of(CellIdx_t cell, const EMClusterTree & tree){

  ELNearHitCellFrac nearCells;
  nearCells.of(cell, tree); //  Get neibouring cells
  of(nearCells);
}

//--
//  Get neighbouring clusters of a given  set of cells
void ELNearClusterFrac::of(ELNearHitCellFrac & nearT){
  numNeib = 0;
  for(nearT.begin(); nearT.more(); nearT.next()){
    insert((*nearT)->getParent(), nearT.value());
  }
  curr = 0;
}


//--
//  Insert the current cluster if it is not already present
void ELNearClusterFrac::insert(EMCluster* clus, Energy_t fraction){
  unsigned int i=0;
  for(; i<numNeib; i++)
    if( neibClusters[i]==clus) break;
  if( i==numNeib){
    fractions[numNeib] = fraction;
    neibClusters[numNeib++] = clus;
  } else {
    fractions[i] += fraction;
  }
  //  cout<<" cell fraction: "<<fraction<<" on cluster "<<i
  //    <<" ( "<<clus<<", "<<neibClusters[i]<<")"<<endl;
  //clus->dump();
}

//--
//  Minimize the cluster list
//  This means take the cluster list and reduce it (following upstream
//  parent links) to a list of clusters with the same parent cluster.
//  Reduce the list eliminating the clusters equals and sort them in 
//  decreasing order in energy of the biggest signal.
void ELNearClusterFrac::minimize(){

  unsigned int i;
  for( i=0; i<numNeib; i++){
    if(neibClusters[i]->getHypotheses()<1){
      while(neibClusters[i]->getHypotheses()==0 && 
	    neibClusters[i]->depthLevel()<FIRSTLEVEL){
	neibClusters[i] = neibClusters[i]->getParent();
      }
      if(neibClusters[i]->depthLevel()==FIRSTLEVEL){
	 neibClusters[i] = 0;
      } else {
	while( neibClusters[i]->depthLevel()<FIRSTLEVEL
	       &&  neibClusters[i]->getParent()->getHypotheses()<2)
	  neibClusters[i] = neibClusters[i]->getParent();
      }
    } else if (neibClusters[i]->getHypotheses()>1){
      neibClusters[i] = 0;
    }
  }
  // for each cluster found test if there are more than one occurrence:
  // if it is so put to 0 all occurreces after the first
  oneOccurrence();
  //
  //  reduce the array
  reduce();
}
 
//--
//  Test if there are more than one occurrence of a given cluster
//  if it is so put to 0 all occurrencies after the first
void ELNearClusterFrac::oneOccurrence(){
  for(unsigned int i=0; i<numNeib-1; i++)
    if( neibClusters[i]!=0){
      for(unsigned int j=i+1; j<numNeib; j++)
	if( neibClusters[i] == neibClusters[j] ){
	  fractions[i] += fractions[j];
	  fractions[j] = 0.0;
	  neibClusters[j] = 0; // invalidate the pointer
	}
    }
}

//--
//  Get the number of occurrences of cluster
void ELNearClusterFrac::reduce(){

  unsigned int j=0;  // count number of valid pointers
  for(unsigned int i=0; i<numNeib; i++){
    neibClusters[j] = neibClusters[i];    
    fractions[j] = fractions[i];
    if( neibClusters[i] != 0) j++;
  }
  numNeib = j;
}

//--
//  Sort decreasing in energy
void  ELNearClusterFrac::sortClusters(){
  sort(&neibClusters[0], &neibClusters[numNeib], comp_cluster_pointers);
}








