//                              -*- Mode: C++ -*-
//: ESEMSPos.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 14/4/97
// Last Update ........
//

#include <math.h>
#include "arte/ArteMessage.hh"
#include "care/caredata.hh"
#include "care/EMClusterTree.hh"
#include "care/EMHitCell.hh"
#include "care/ECCellID.hh"
#include "care/ELLUTsManager.hh"
#include "care/ELNearNeiCellLUT.hh"
#include "care/ESEMSPos.hh"


//////////////////// ESEMSPOS Class Definitions ////////////////

ESEMSPos::ESEMSPos() : xmin(0.0), xmax(0.0), ymin(0.0), ymax(0.0) {}
ESEMSPos::~ESEMSPos(){}

//--
//  Start a cluster analysis
void ESEMSPos::evaluatePos(const EMCluster* clus){

  peakSt->evaluatePos(clus);    // get the peak
  posData = peakSt->getData(); // the peak position is now local
 
  float xpeak = posData.x();
  float ypeak = posData.y();
  xmin = xpeak;
  xmax = xpeak;
  ymin = ypeak;
  ymax = ypeak;

  ELLUTsManager LM;
  int ncells = LM.neiboursTable().howManyNeibouring(
    posData.cell()->whichBank()->gcce);
  for( int i=0; i<ncells; i++){
    ECCellID cellid(CellIdx_t(LM.neiboursTable()[i])->ceid);
    float x(0.0), y(0.0);
    cellid.getCoords(x,y);

    if( x<xmin ) xmin = x-0.1;
    if( x>xmax ) xmax = x+0.1;
    if( y<ymin ) ymin = y-0.1;
    if( y>ymax ) ymax = y+0.1;
  }

  // get first the center of gravity
  ESCOGPos::evaluatePos(clus);
  //
  // Now the correction can be evaluated
  ESCorCOGPos::correct();
}




//--
// HitCell Analyzer
void ESEMSPos::doAnalysis(const EMHitCell * hitT){
  // analyze only cells which are inside the confidence region
  if( xmin<= hitT->x() && hitT->x()<= xmax &&
      ymin<= hitT->y() && hitT->y()<= ymax ){
      ESCOGPos::doAnalysis(hitT);
  }
}




