//                              -*- Mode: C++ -*-
//:ECDBaseIO.hh
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 1/09/98
// Last Update ........
//
//
#ifndef ECDBASEIO_HH_
#define ECDBASEIO_HH_

#include "arte/arte.hh"
#include <fstream.h>

struct TimeTag_t
{
  int major;
  int minor;
};

const TimeTag_t newesttime = {0,0};

//class fstream;
typedef fstream* DBpointer_t;

/**
class ECDBASEIO
  Provide a collection of basic function for
  Input/Output from the ECAL calibration Data Base

  Patterns: Singleton, Bridge
  */

class ECDBaseIO {
protected:
  //--
  /// Constructor
  ECDBaseIO(char * database = 0);
  /// disallow copy constructor
  ECDBaseIO(const ECDBaseIO &){};

public:

  /// destructor
  virtual ~ECDBaseIO();
 
  /**@name Beheavioral methods */
  //@{ 
  //--
  /** Prepare internal data for analyzing the events. 
   */
  static ECDBaseIO* initialize(char * database = 0);
  //--
  /// Open the database
  virtual int open(int mode = ios::in);
  //--
  /// Load The DB Tables into LEDA Tables
  int fetchTables(TimeTag_t timetag = newesttime);
  //--
  /// Close the database
  virtual int close();
  //--
  /// Store the data in memory onto the database
  virtual int store(TimeTag_t timetag);
  //--
  /// Read the data from the database
  virtual int read(void);

  /// Set the pointers to the STL banks
  inline void setPointerToSTLBankEcbo(ECBO* pEcbo){ptrEcbo = pEcbo;};
  inline void setPointerToSTLBankEchs(ECHS* pEchs){ptrEchs = pEchs;};
  inline void setPointerToSTLBankEchf(ECHF* pEchf){ptrEchf = pEchf;};
  //--
  /// Set the entries for the STL banks
  inline void setEntriesForSTLBanksEcbo(int nEcbo) {entriesEcbo = nEcbo;};
  inline void setEntriesForSTLBanksEchs(int nEchs) {entriesEchs = nEchs;};
  inline void setEntriesForSTLBanksEchf(int nEchf) {entriesEchf = nEchf;};
  //--
  /// Get the entries for the STL banks
  inline int getEntriesForSTLBanksEcbo(void) {return entriesEcbo;};
  inline int getEntriesForSTLBanksEchs(void) {return entriesEchs;};
  inline int getEntriesForSTLBanksEchf(void) {return entriesEchf;};
  //--
  /// set the DB name
  char * setDBname(char *dbnam);  
  //--
  //@}

  /**@name access methods */
  //@{ 
  //--
  /// Return the instance
  inline static ECDBaseIO* instance();
  //--
  /// Get the name of the database
  inline const char* dbname() const {return databasename;}; 
  //--
  /// Get the last time tag used
  inline TimeTag_t lasttag() const {return lasttimetag;};
  /// The dbstatus value
  enum DBstatus_t {opened, closed};
  ///-- 
  /// Get the DBstatus value
  inline DBstatus_t status() const {return dbstatus;};
  //
  //@}  

private:
  /// The only instance
  static ECDBaseIO * ecdbaseio;
  /// name of the nameserver
  char* databasename;
  /// Last time tag used
  TimeTag_t  lasttimetag;
  /// The dbstatus
  DBstatus_t dbstatus;
  /// pointer to the ECBO STL vector based bank
  ECBO* ptrEcbo;
  /// pointer to the ECHS STL vector based bank
  ECHS* ptrEchs;
  /// pointer to the ECHF STL vector based bank
  ECHF* ptrEchf;
  /// entries for the ECBO bank
  int entriesEcbo;
  /// entries for the ECBO bank
  int entriesEchs;
  /// entries for the ECBO bank
  int entriesEchf;
  /// The database pointer
  DBpointer_t dbpointer;
};

// Return the instance
ECDBaseIO* ECDBaseIO::instance(){
  if(ecdbaseio==0){
    initialize();
  }
  return ecdbaseio;
}

#endif // ECDBASEIO_HH_


