//                              -*- Mode: C++ -*-
//:ELNearNeiCellLUT.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 8/4/97
// Last Update ........
//
#ifndef ELNearNeiCellLUT_HH_
#define ELNearNeiCellLUT_HH_


#include "arte/arte.hh"
#include "care/ELLUTsManager.hh"

/**
class  ELNearNeiCellLUT.
  This is a Look Up Table to store the information on neibouring cells of
  a given one. It can be used only with the help of the CellDesc LUT.
  There is only one instance of this class that is correctly initialized and
  is held by the LUTsManager.

WARNING
  Be sure to use this class only throu the LUTsManager interface.

*/

class  ELNearNeiCellLUT {
public:
  //-- 
  /**
    Constructor: it does nothing by default. Initialization performed
   by the LUTsManager class */
  inline ELNearNeiCellLUT();
  //-- 
  /// Destructor.
  inline ~ELNearNeiCellLUT();

  //--
  /// LookUp operation:return how many neibouring cells
  int howManyNeibouring(CellIdx_t idx);

  //--
  ///  Get cell index for neibour number i of the last accessed cell.
  inline CellIdx_t operator[](int i);
  //-- 
  ///  Get the size
  inline int size() const;
  friend class ELLUTsManager;
  //
private:
  /// The initializing function is this
  void initialize();
  //--
  ///   Find neibours using the Crux algorithm
  int crux(CellIdx_t * neibours, Coord_t xc, Coord_t yc, 
	   Coord_t xw, Coord_t yw);

  /// Store first neibour pointer of the last accessed cell
  ArtePointer<GNEI> firstneibour;

};

#include "care/ELNearNeiCellLUT.icc"
#endif // ELNearNeiCellLUT_HH_
