/************************************************************************
* KADGSEL
* former kagsel written by Siegmund Nowak 16/11/95 10.00
* updated 29/03/99
* the former dgsele.F will become obsolete
* This routines acts on kuip card DGSEL selecting region of interest
* in unpacking / digitising MIMPS / HITS

The syntax is dgsel name1 name2 .... namen, where

1. name1 ... namen are superlayer names (4 bytes each) 

2. alternatively if name1 = CMP then name 2 ... namem can be
   SVD        all Superlayers with gesl_cmp =1 will be taken (all SVD)  
   OTR                    "                  2      "        (all OTR)
   ITR                    "                  3      "        (all ITR) 
   PTL                    "                  4      "        (all PTL)
   RIC                    "                  5      "        (RICH)          
   TRD                    "                  6      "        (TRD)      
   ECA                    "                  7      "        (ECA) 
   MUD                    "                  8      "        (all MUDi)
   MXD                    "                  9      "        (all MXDi)

3. name1 ..  namen can be  
   MAGN      all SVD OTR,ITR,PTL chambers in the magnet will be
             taken, i.e.
             SI08 MS01 MS03 MS05 MS06 MC01 MC02 MC03 MC04 MC05 MC06
             MC08 PTL1 PTL2 PTL3

   PAT       all chambers from pattern recognition region will be taken
             MS10 MS11 MS12 MS13 PC01 PC02 PC03 PC04

   FLT       all detectors for first level trigger will be taken
             PC01 PC04 TC01 TC02 MS10 MS13 MS14 MS15 CALW MUD1 MUD2
             MUD3 MUD4 MXD1 MXD2 MXD3 MXD4

4. name can be * - all detectors will be taken

5. if a minus sign is preceeding the name (-SI01) the corresponding
detector is exluded from digitization

The default is that the digitization will be made for all devices.
However as soon as a dgsel command appears the default value will be
no digitization at all and  each wanted device (group of devices) must
be explicitely given - the whole list for instance can be implemented 
by DGSEL * 

examples:

DGSEL CMP ECAL SVD 
DGSEL  -SI06 MS10
DGSEL MAGN
DGSEL CMP -ITR
DGSEL PAT -FLT

The action of the cards is the following:
After the first card the following detectors will be included:
CALW SI01 SI02 SI03 SI04 SI05 SI06 SI07 SI08

the second card will drop from this list SI06 and add MS10 - the
result is
CALW SI01 SI02 SI03 SI04 SI05 SI07 SI08 MS10

the third card will add all chambers inside the magnet, the result is 
CALW SI01 SI02 SI03 SI04 SI05 SI07 SI08 MS10 MS01 MS03 MS05 MS06 
MC01 MC02 MC03 MC04 MC05 MC06 MC08 PTL1 PTL2 PTL3

the fourth card will drop all inner tracker chambers
CALW SI01 SI02 SI03 SI04 SI05 SI07 SI08 
MC01 MC02 MC03 MC04 MC05 MC06 MC08 PTL1 PTL2 PTL3

the fifth card will first add all pattern rec. superlayers and then
drop all superlayers (if they are in the list) which belong to FLT 
SI01 SI02 SI03 SI04 SI05 SI07 SI08 MC01 MC02 MC03 MC04 MC05 MC06 MC08
PTL1 PTL2 PTL3 MS11 MS12 PC02 PC03

************************************************************************/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "gean/hgdat.h"
#include "digi/zutsel.h"
#include "digi/subdet_names.h"

static int kadgsel_first=0, kadgsel_magn=15, kadgsel_patn=8, kadgsel_fltn=17;
static char kadgsel_mag[] = 
                "SI08MS01MS03MS05MS06MC01MC02MC03MC04MC05MC06MC08PTL1PTL2PTL3"; 
static char kadgsel_pat[] = "MS10MS11MS12MS13PC01PC02PC03PC04";
static char kadgsel_flt[] =
        "PC01PC04TC01TC02MS10MS13MS14MS15CALWMUD1MUD2MUD3MUD4MXD1MXD2MXD3MXD4"; 

extern "C" {
  char * ku_gete(); 
  void fill_hgsel(const char *);
  void drop_hgsel(const char *);

  void kadgsel()  {
// initialization
   char buffer[256], tag[10];
   if(!kadgsel_first) {
      memset(zutsel_.hgsel,' ',sizeof(zutsel_.hgsel));
      zutsel_.hgsel[sizeof(zutsel_.hgsel)-1] ='\0';
      kadgsel_first =1;
   }

// read in the whole command line
   memset(buffer,' ',255);
   buffer[255] = '\n';
   strcpy(buffer, ku_gete());
   int cmpflag = 0; 
   char *card = &buffer[0];   

// take each word separtely
   for (;;) {
     int nitem=sscanf(card,"%s",tag);
     if (nitem==-1) break;
//   check for minus sign or slash
     int dropflag = 0;           
     if(tag[0] =='-' ||tag[0] =='/' ) {
        card = strpbrk(card,"-/")+1; 
        nitem = sscanf(card,"%s",tag); 
        dropflag = 1; 
     }

     for (unsigned int i=0; i<strlen(tag); i++) tag[i] = toupper(tag[i]); 
     card+=itemlength(card,nitem);

//   command is *
     if(tag[0] == '*') { 
        for(ArteTableIterator<NESL> ns; ns.valid(); ns.next()){
	  if((*ns)->cmp < 10) {
             if(!dropflag) fill_hgsel((*ns)->name);
             else          drop_hgsel((*ns)->name);
          }   
	} 
     }

//   command is CMP  
     else if(!strncmp(tag,"CMP",3)) {
        cmpflag = 1;     continue;
     }

//   all commands within CMP range
     else if(cmpflag) {
         tag[3] ='\0';   
	 int i = iucomp(tag,subdet_names,9);
         if(i) {               
           for(ArteTableIterator<NESL> ns; ns.valid(); ns.next()){
             if((*ns)->cmp == i) {
                if(!dropflag) fill_hgsel((*ns)->name);
                else          drop_hgsel((*ns)->name);             
             } 
	   }
         }
         else {
           arteMsg << "kadgsel - CMP " << tag << " not found";      
           arteMsg.warn();
         }
     } 

//   command is MAGN
     else if(!strncmp(tag,"MAGN",4))  {
       for (int i = 0; i<kadgsel_magn; i++) {
            if(!dropflag) fill_hgsel(&kadgsel_mag[4*i]);
            else          drop_hgsel(&kadgsel_mag[4*i]);
	  }
     } 

//   command is PAT
     else if (!strncmp(tag,"PAT",3))   {
       for (int i = 0; i<kadgsel_patn; i++) {
            if(!dropflag) fill_hgsel(&kadgsel_pat[4*i]);
            else          drop_hgsel(&kadgsel_pat[4*i]);
          }
     }

//   comand is FLT       
     else if (!strncmp(tag,"FLT",3))   {    
	 for (int i = 0; i<kadgsel_fltn; i++) {
             if(!dropflag) fill_hgsel(&kadgsel_flt[4*i]);
             else          drop_hgsel(&kadgsel_flt[4*i]); 
          }
     }

//   single Superlayers       
     else {
	int found = 0;
        for(ArteTableIterator<NESL> ns; ns.valid(); ns.next()){
           if(!strncmp((*ns)->name,tag,4)) { 
	      if(!dropflag) fill_hgsel(tag);  
              else          drop_hgsel(tag);
              found =1;        break;
	   }
	}
	if(!found) {
	    arteMsg << "check dgsel card" << tag;    arteMsg.error();
        }   
     }       
   }
 }



void fill_hgsel(const char* a) {
  char cname[5];
  strncpy(cname,a,4);   cname[4] ='\0';
  if(!iucomp(cname,zutsel_.hgsel,zutsel_.ngsel)) {
    if(zutsel_.ngsel < 80) {  
       strncpy(&zutsel_.hgsel[4*zutsel_.ngsel],cname,4);
       zutsel_.ngsel++;
    }   
    else {  
       arteMsg << "fill_hgsel - maximal filling is 80 names"; 
       arteMsg.warn();
    }  
  }
}

void drop_hgsel(const char *a) {
  if(zutsel_.ngsel <=0 ) {
    arteMsg << "more selections dropped then existing, dgsel will not work!";
    zutsel_.ngsel = 0;
    arteMsg.error();
  }
  char cname[5];
  strncpy(cname,a,4);   cname[4] ='\0';
  int i =iucomp(cname,zutsel_.hgsel,zutsel_.ngsel);
  if(i != 0) {   
    zutsel_.hgsel[4*(i-1)] ='\0';
    strncat(zutsel_.hgsel,&zutsel_.hgsel[4*i],4*(zutsel_.ngsel-i));
    zutsel_.ngsel--; 
    strncpy(&zutsel_.hgsel[4*zutsel_.ngsel],"    ",4);
  }
}
}
