
// *****************************************************************************
//
// source:    faDaqLog.C
//
// type:      source code
//
// created:    1 Dec 98
// updated:    1 Jun 99 Andreas Gellrich
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// purpose:   send daq-formated event to logger (AG: 1 Dec 98)
//            includes (AG: 11 Jan 99)
//            return error code (AG: 15 Jan 99)
//            free memory (AG: 26 Jan 99)
//            includes (AG: 23 Feb 99)
//            polishing (AG: 11 Mar 99)
//            put includes (AG: 1 Jun 99)
//
// *****************************************************************************

// includes
#include <iostream>
#include <stdio.h>
#include <unistd.h>

#include "event.h"
#include "grr.h"
#include "esp.h"
#include "log.h"

#include "arte/ArteFlag.h"

#include "farm/fa.hh"

// externals
extern int *evtBuf;
extern int FADEB;
extern int DAQLOG;

//
// program code
//
int
faDaqLog()
{
  // reeturn code
  int rc = GRR_OK;

  // check flag
  if( DAQLOG ){
    
    // get event header
    EVENT_HDR evtHdr;
    ESP_get_event_hdr( (EVENT_HDR *) evtBuf, &evtHdr );
    
    // build log header
    LogHdr *hdr = new LogHdr;
    hdr->exp    = arteflag_.numexp;
    hdr->run    = arteflag_.numrun;
    hdr->evt    = arteflag_.numevt; // re-filled by logger
    hdr->datime = time( 0 );
    
    // print log header
#ifdef _DEB_
    if( FADEB ){
      cout << "LOG: "
	   << "exp="    << hdr->exp      << " "
	   << "run="    << hdr->run      << " "
	   << "evt="    << hdr->evt      << " "
	   << "datime=" << hdr->datime   << " "
	   << "bc="     << evtHdr.length << endl;
    }
#endif

    //
    // send event to logger
    //
    while( 1 ){
      if( ( rc = l4aLogEvt( (char *) evtBuf, evtHdr.length, hdr ) ) == GRR_OK ){
#ifdef _DEB_
	if( FADEB ){
	  cout << "LOG: l4aLogEvt()" << endl;
	}
#endif
	break;
      }
      // time out
      else if( rc == GRR_TIME ){
#ifdef _ERR_
	cerr << "LOG: l4aLogEvt() rc=GRR_TIME" << endl;
#endif
	sleep( 1 );
      }
      // reject
      else if( rc == GRR_REJECT ){
#ifdef _ERR_
	cerr << "LOG: l4aLogEvt() rc=GRR_REJECT" << endl;
#endif
	sleep( 1 );
      }
      // bad address
      else if( rc == GRR_BAD_ADDRESS ){
#ifdef _ERR_
	cerr << "LOG: l4aLogEvt() rc=GRR_BAD_ADDRESS" << endl;
#endif
	sleep( 1 );
      }
      // error
      else {
#ifdef _ERR_
	cerr << "LOG: ERROR: l4aLogEvt() rc=" << rc << endl;
#endif
	break;
      }
    }   

    // free memory
    delete hdr;
  }

  return rc;
}
