
// *****************************************************************************
//
// source:    faStop.C
//
// type:      source code
//
// created:    7 Jul 97
// updated:    1 Jun 99  Andreas Gellrich
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    fainit_()
//
// input:     void
// return:    int  ARTE_error_code
//
// called by: ATSTOP
//
// purporse:  termination of farm stuff called for FARM mode
//
// changes:   stop rpm (AG: 25-Nov-98)
//            multiple file input (AG: 27-Nov-98)
//            daq format logging (AG: 1-Dec-98)
//            includes (AG: 11 Jan 99)
//            removed emg status flag (AG: 12 Jan 99)
//            removed hbook filling (AG: 14 Jan 99)
//            more printout (AG: 21 Jan 99)
//            halt rpm (AG: 2 Feb 99)
//            removed faUtil.hh (AG: 23 Feb 99)
//            use grr error logger (AG: 25 Feb 99)
//            polishing (AG: 11 Mar 99)
//            put includes (AG: 1 Jun 99)
//
// *****************************************************************************

// includes
#include <iostream>

#include "rpm.h"
#include "grr.h"

#include "farm/fa.hh"

// externals
extern int  FARM;      // FARM mode
extern int  FAON;      // FARM online mode
extern char *DBNAME;   // database name
extern int  NOSPAR;    // sparsification
extern int  RPMINIT;   // rpm initialized
extern int  FAGRR;     // grr error logger     
extern char *PARNAME;  // emg partition
extern int  SEG_NUM;   // emg partitions
extern int  SEG_LEN;   // emg partition size

//
// program code
//
extern "C" int
fastop_()
{
  int rc = ARTE_GOOD;    
  //
  // farm mode
 //
  if( FARM ){

    //
    // online
    //
    if( FAON ){

      //
      // emg
      //
      if( ( rc = faStopEmg() ) != GRR_OK ){
	if( FAGRR ){
	  grr_error( GRR_SERROR, "STOP: Arte emg FAILED.", 0 );
	}
      }
    }
  }

  // grr error message
  if( FAGRR ){
    grr_error( GRR_SINFO, "STOP: Arte faStop() DONE.", 0 );
  }

  //
  // rpm
  //
  if( RPMINIT ){
    if( ( rc = rpm_halt() ) == GRR_OK ){
#ifdef _OUT_
      cout << "STOP: rpm_halt()" << endl;
#endif
    }
    else {
#ifdef _ERR_
      cerr << "STOP: ERROR: rpm_halt() rc=" << rc << endl;
#endif
    }
  }

  // return
  return rc;
}
