
          SUBROUTINE CAFMSU

C************************************************************
C*-- Author :    Olya Igonkina   12/12/95                   *
C* CAFMSU (vsn 1.00)                                        *
C*                                                          *
C* for   mini MonteCarlo for HERA-B ECAL                    *
C*                                                          *
C* IN:   VECT - initial particle parameters                 *
C* OUT:  HITPOS(2,IHIT) energy deposition in ECAL module    *
C*                                                          *
C* Use:  CAFMFO for shower parametrization  OUTER  ECAl     *
C*       CAFMFM for shower parametrization  MIDDLE ECAl     *
C*       CAFMFI for shower parametrization  INNER  ECAl     *
C************************************************************
       implicit none

      INTEGER INUM
      DIMENSION INUM(3)
      REAL RNO_MOD, HSIZ_MOD, RAD_MOD
      DIMENSION RNO_MOD(3), HSIZ_MOD(3), RAD_MOD(3)

      INTEGER   I, NUMED, IMNCL
      REAL      EDEP, RNORM
      REAL       SERR, XX
      DIMENSION XX(3)

      PARAMETER( SERR=0.5 )

      INTEGER    NVCL
      PARAMETER( NVCL = 10)

      INTEGER   NVOLCAL,HVOLCAL
      DIMENSION HVOLCAL(NVCL)
      LOGICAL   LFIRST
      SAVE      HVOLCAL,NVOLCAL,LFIRST

      INTEGER  IUCOMP
      EXTERNAL IUCOMP
      INTEGER  INCALO


#include "arte/Arte.fi"
#include "auto/GEO.fi"
#include "gean/ZGMOTH.fi"
#include "gean/ZGCALO.fi"
#include "gean/gcvolu.fi"
#include "gean/gctrak.fi"
#include "gean/gclink.fi"
#include "gean/cafmcp.fi"
#include "gean/cafmch.fi"

C..  O/M/I module division number
      DATA INUM/ 1, 2, 5/
C.. normalization factor (O/M/I) for Energy deposition
      DATA RNO_MOD/ 0.9, 0.9, 0.9/
C.. 0.5 * transverse size of scintillator plate
      DATA HSIZ_MOD/ 5.575, 2.7875, 1.115/
C.. radiation length
      DATA RAD_MOD/ 1.675, 1.675, 0.558/

      DATA  LFIRST /.TRUE./
      DATA  NVOLCAL /0/
      DATA  HVOLCAL /NVCL*0/

#include "arte/Arte.ff"
#include "auto/GEO.ff"

      INCALO(I) = IUCOMP(I,HVOLCAL(1),NVOLCAL)

      DO I=1,NHIT
        HITPOS(1,I) = 0.
        HITPOS(2,I) = 0.
      ENDDO

C.. initialization

      IF( LFIRST )THEN
       DO I=1, NGCAL()
         IF( IUCOMP( HGCAL_GVOL(I),HVOLCAL(1),NVOLCAL) .EQ. 0)THEN
           IF( NVOLCAL.EQ.NVCL )THEN
             PRINT *,'TOO MANY TYPE OF ECAL FOR CAFMSU SUBROUTINE'
             PRINT *,' YOU SHOULD INCREASE PARAMETER NVCL'
             NVOLCAL = NVCL - 1
           ENDIF
           NVOLCAL = NVOLCAL + 1
           HVOLCAL(NVOLCAL) = HGCAL_GVOL(I)
           LFIRST = .FALSE.
         ENDIF
       ENDDO
      ENDIF

C.. coordinates for initial particle, towers

      IHIT = 0
      EMOD = 0
      IF( VECT(6).LT.0.1E-6 )GOTO 999

      IF( (NLEVEL.LE.2.5) .OR.
     &    (INCALO(NAMES(NLEVEL-2)).EQ.0) )THEN
         PRINT *,'ERROR, particle not in the ECAL'
         PRINT *,'wrong use of CAFMSU, no HIT stored'
         GOTO 999
      ENDIF

      IMNCL = IGCAL_NCL(NUMBER(NLEVEL-2))
      IVN = 100*NUMBER(NLEVEL-2) + NUMBER(NLEVEL-1)*IMNCL
     +      + NUMBER(NLEVEL)

      PAR_MOD(2) = GTRAN(1,NLEVEL)
      PAR_MOD(3) = GTRAN(2,NLEVEL)

C.. search for main tower

      I = IUCOMP(IMNCL,INUM(1),3)

      IF( I.EQ.0 )THEN
         PRINT *,' ERROR IN CAFMSU :'
         PRINT *,' check division number of module ECAL'
         PRINT *,' it should be 1, 2 or 5'
         GOTO 999
      ENDIF

C        XX(1) = VECT(1) + 7.5 * RAD_MOD(I) * VECT(4)
C        XX(2) = VECT(2) + 7.5 * RAD_MOD(I) * VECT(5)
C        XX(3) = VECT(3) + 7.5 * RAD_MOD(I) * VECT(6)

         XX(3) = ZZERO + ZACAL(1) + 7.5 * RAD_MOD(I) * VECT(6)
         XX(1) = VECT(1) + (XX(3)-VECT(3))*VECT(4)/VECT(6)
         XX(2) = VECT(2) + (XX(3)-VECT(3))*VECT(5)/VECT(6)

        CALL GMEDIA(XX, NUMED)

        IF(NLEVEL.LE.2.5.OR.INCALO(NAMES(NLEVEL-2)).EQ.0)THEN
           XX(1) = VECT(1)
           XX(2) = VECT(2)    ! go back to the incident tower
           XX(3) = VECT(3)
           CALL GMEDIA(XX, NUMED)
        ENDIF

      IF( NLEVEL.GE.2.5) THEN
      IF( INCALO(NAMES(NLEVEL-2)) .NE. 0 )THEN

         IMNCL = IGCAL_NCL(NUMBER(NLEVEL-2))
         IVN = 100*NUMBER(NLEVEL-2) + NUMBER(NLEVEL-1)*IMNCL
     &       + NUMBER(NLEVEL)

         PAR_MOD(2) = GTRAN(1,NLEVEL)
         PAR_MOD(3) = GTRAN(2,NLEVEL)
         IF( IMNCL.EQ.INUM(3)) THEN
             MFL = 3
             CALL CAFMFI
         ELSE
           IF( IMNCL.EQ.INUM(2)) THEN
             MFL = 2
             CALL CAFMFM
           ELSE
             MFL = 1
             CALL CAFMFO
           ENDIF
         ENDIF
         IHIT = IHIT + 1
         HITPOS(1,IHIT) = IVN
         HITPOS(2,IHIT) = EMOD
       ENDIF
      ENDIF

      MPAR = PAR_MOD(1)
      MX = PAR_MOD(2)
      MY = PAR_MOD(3)

C.. upper neighbour

      XX(1) =  MX - MPAR - SERR
      XX(2) =  MY + MPAR + SERR

 10   CONTINUE

      CALL GMEDIA(XX, NUMED)

      IF( NLEVEL.GE.2.5) THEN
      IF( INCALO(NAMES(NLEVEL-2)) .NE. 0 )THEN

       IMNCL = IGCAL_NCL(NUMBER(NLEVEL-2))
       IVN = 100*NUMBER(NLEVEL-2) + NUMBER(NLEVEL-1)*IMNCL
     +       + NUMBER(NLEVEL)

        PAR_MOD(2) = GTRAN(1,NLEVEL)
        PAR_MOD(3) = GTRAN(2,NLEVEL)
        IF((PAR_MOD(3) - PAR_MOD(1))
     &             .LE.(MY +MPAR-SERR)) THEN
          XX(1) = MX - MPAR + SERR
          GOTO 20
        ENDIF
        IF( IMNCL.EQ.INUM(3)) THEN
            CALL CAFMFI
        ELSE
          IF( IMNCL.EQ.INUM(2)) THEN
            CALL CAFMFM
          ELSE
            CALL CAFMFO
          ENDIF
        ENDIF
        IHIT = IHIT + 1
        HITPOS(1,IHIT) = IVN
        HITPOS(2,IHIT) = EMOD

        IF( (PAR_MOD(2) + PAR_MOD(1))
     &             .LE.(MX - MPAR + SERR) )THEN
           XX(1) = MX - MPAR + SERR
           GOTO 20
        ENDIF
        IF( (PAR_MOD(2) + PAR_MOD(1))
     &             .GE.(MX +MPAR+SERR)) THEN
           XX(1) = MX + MPAR + SERR
           XX(2) = MY + MPAR - SERR
           GOTO 40
        ENDIF
        IF( (PAR_MOD(2) + PAR_MOD(1))
     &             .GE.(MX + MPAR - SERR)) THEN
           XX(1) = MX + MPAR + SERR
           GOTO 30
        ENDIF
        IF( (PAR_MOD(2) + PAR_MOD(1))
     &             .GE.(MX - MPAR + SERR) )THEN
           XX(1) = PAR_MOD(2) + PAR_MOD(1) + SERR
           GOTO 20
        ENDIF
      ENDIF
      ENDIF
      XX(1) = MX - MPAR + SERR

 20   CONTINUE

      CALL GMEDIA(XX, NUMED)

      IF( NLEVEL.GE.2.5) THEN
      IF( INCALO(NAMES(NLEVEL-2)) .NE. 0 )THEN

       IMNCL = IGCAL_NCL(NUMBER(NLEVEL-2))
       IVN = 100*NUMBER(NLEVEL-2) + NUMBER(NLEVEL-1)*IMNCL
     +       + NUMBER(NLEVEL)

        PAR_MOD(2) = GTRAN(1,NLEVEL)
        PAR_MOD(3) = GTRAN(2,NLEVEL)
        IF( IMNCL.EQ.INUM(3)) THEN
            CALL CAFMFI
        ELSE
          IF( IMNCL.EQ.INUM(2)) THEN
            CALL CAFMFM
          ELSE
            CALL CAFMFO
          ENDIF
        ENDIF
           IHIT = IHIT + 1
           HITPOS(1,IHIT) = IVN
           HITPOS(2,IHIT) = EMOD
        IF( (PAR_MOD(2) + PAR_MOD(1))
     &             .LE.(MX + MPAR - SERR)) THEN
           XX(1) = PAR_MOD(2) + PAR_MOD(1) + SERR
           GOTO 20
        ENDIF
        IF( (PAR_MOD(2) + PAR_MOD(1))
     &             .GE.(MX +MPAR+SERR)) THEN
           XX(1) = MX + MPAR + SERR
           XX(2) = MY + MPAR - SERR
           GOTO 40
        ENDIF
      ENDIF
      ENDIF
      XX(1) = MX + MPAR + SERR

 30   CONTINUE

      CALL GMEDIA(XX, NUMED)

      IF( NLEVEL.GE.2.5) THEN
      IF( INCALO(NAMES(NLEVEL-2)) .NE. 0 )THEN

       IMNCL = IGCAL_NCL(NUMBER(NLEVEL-2))
       IVN = 100*NUMBER(NLEVEL-2) + NUMBER(NLEVEL-1)*IMNCL
     +       + NUMBER(NLEVEL)

        PAR_MOD(2) = GTRAN(1,NLEVEL)
        PAR_MOD(3) = GTRAN(2,NLEVEL)
        IF( IMNCL.EQ.INUM(3)) THEN
            CALL CAFMFI
        ELSE
          IF( IMNCL.EQ.INUM(2)) THEN
            CALL CAFMFM
          ELSE
            CALL CAFMFO
          ENDIF
        ENDIF
           IHIT = IHIT + 1
           HITPOS(1,IHIT) = IVN
           HITPOS(2,IHIT) = EMOD

        IF( (PAR_MOD(3) - PAR_MOD(1))
     &             .GE.(MY + MPAR - SERR) )THEN
           XX(2) = MY + MPAR - SERR
           GOTO 40
        ENDIF
        IF( (PAR_MOD(3) - PAR_MOD(1))
     &             .LE.(MY - MPAR - SERR)) THEN
           XX(1) = MX + MPAR - SERR
           XX(2) = MY - MPAR - SERR
           GOTO 60
        ENDIF
        IF( (PAR_MOD(3) - PAR_MOD(1))
     &             .LE.(MY - MPAR + SERR)) THEN
           XX(2) = MY - MPAR - SERR
           GOTO 50
        ENDIF
        IF( (PAR_MOD(3) - PAR_MOD(1))
     &             .LE.(MY + MPAR - SERR) )THEN
           XX(2) = PAR_MOD(3) - PAR_MOD(1) - SERR
           GOTO 40
        ENDIF
      ENDIF
      ENDIF
      XX(2) = MY + MPAR - SERR

C.. middle level

 40   CONTINUE

      CALL GMEDIA(XX, NUMED)

      IF( NLEVEL.GE.2.5) THEN
      IF( INCALO(NAMES(NLEVEL-2)) .NE. 0 )THEN

       IMNCL = IGCAL_NCL(NUMBER(NLEVEL-2))
       IVN = 100*NUMBER(NLEVEL-2) + NUMBER(NLEVEL-1)*IMNCL
     +       + NUMBER(NLEVEL)

        PAR_MOD(2) = GTRAN(1,NLEVEL)
        PAR_MOD(3) = GTRAN(2,NLEVEL)
        IF( IMNCL.EQ.INUM(3)) THEN
            CALL CAFMFI
        ELSE
          IF( IMNCL.EQ.INUM(2)) THEN
            CALL CAFMFM
          ELSE
            CALL CAFMFO
          ENDIF
        ENDIF
           IHIT = IHIT + 1
           HITPOS(1,IHIT) = IVN
           HITPOS(2,IHIT) = EMOD
        IF( (PAR_MOD(3) - PAR_MOD(1))
     &             .GE.(MY-MPAR+SERR)) THEN
           XX(2) = PAR_MOD(3) - PAR_MOD(1) - SERR
           GOTO 40
        ENDIF
        IF( (PAR_MOD(3) - PAR_MOD(1))
     &             .LE.(MY-MPAR-SERR)) THEN
           XX(1) = MX + MPAR - SERR
           XX(2) = MY - MPAR - SERR
           GOTO 60
        ENDIF
      ENDIF
      ENDIF
      XX(2) = MY - MPAR - SERR

C.. bottom level

 50   CONTINUE

      CALL GMEDIA(XX, NUMED)

      IF( NLEVEL.GE.2.5) THEN
      IF( INCALO(NAMES(NLEVEL-2)) .NE. 0 )THEN

       IMNCL = IGCAL_NCL(NUMBER(NLEVEL-2))
       IVN = 100*NUMBER(NLEVEL-2) + NUMBER(NLEVEL-1)*IMNCL
     +       + NUMBER(NLEVEL)

        PAR_MOD(2) = GTRAN(1,NLEVEL)
        PAR_MOD(3) = GTRAN(2,NLEVEL)
        IF( IMNCL.EQ.INUM(3)) THEN
            CALL CAFMFI
        ELSE
          IF( IMNCL.EQ.INUM(2)) THEN
            CALL CAFMFM
          ELSE
            CALL CAFMFO
          ENDIF
        ENDIF
           IHIT = IHIT + 1
           HITPOS(1,IHIT) = IVN
           HITPOS(2,IHIT) = EMOD
        IF( (PAR_MOD(2) - PAR_MOD(1))
     &             .GE.(MX + MPAR - SERR) )THEN
           XX(1) = MX + MPAR - SERR
           GOTO 60
        ENDIF
        IF( (PAR_MOD(2) - PAR_MOD(1))
     &             .LE.(MX - MPAR-SERR)) THEN
           XX(1) = MX - MPAR - SERR
           XX(2) = MY - MPAR + SERR
           GOTO 80
        ENDIF
        IF( (PAR_MOD(2) - PAR_MOD(1))
     &             .LE.(MX - MPAR + SERR)) THEN
           XX(1) = MX - MPAR - SERR
           GOTO 70
        ENDIF
        IF( (PAR_MOD(2) - PAR_MOD(1))
     &             .LE.(MX + MPAR - SERR) )THEN
           XX(1) = PAR_MOD(2) - PAR_MOD(1) - SERR
           GOTO 60
        ENDIF
      ENDIF
      ENDIF
      XX(1) = MX + MPAR - SERR


 60   CONTINUE

      CALL GMEDIA(XX, NUMED)

      IF( NLEVEL.GE.2.5) THEN
      IF( INCALO(NAMES(NLEVEL-2)) .NE. 0 )THEN

       IMNCL = IGCAL_NCL(NUMBER(NLEVEL-2))
       IVN = 100*NUMBER(NLEVEL-2) + NUMBER(NLEVEL-1)*IMNCL
     +       + NUMBER(NLEVEL)

        PAR_MOD(2) = GTRAN(1,NLEVEL)
        PAR_MOD(3) = GTRAN(2,NLEVEL)
        IF( IMNCL.EQ.INUM(3)) THEN
            CALL CAFMFI
        ELSE
          IF( IMNCL.EQ.INUM(2)) THEN
            CALL CAFMFM
          ELSE
            CALL CAFMFO
          ENDIF
        ENDIF
           IHIT = IHIT + 1
           HITPOS(1,IHIT) = IVN
           HITPOS(2,IHIT) = EMOD
        IF( (PAR_MOD(2) - PAR_MOD(1))
     &             .GE.(MX-MPAR+SERR)) THEN
           XX(1) = PAR_MOD(2) - PAR_MOD(1) - SERR
           GOTO 60
        ENDIF
        IF( (PAR_MOD(2) - PAR_MOD(1))
     &             .LE.(MX-MPAR-SERR)) THEN
           XX(1) = MX - MPAR - SERR
           XX(2) = MY - MPAR + SERR
           GOTO 80
        ENDIF
      ENDIF
      ENDIF
      XX(1) = MX - MPAR - SERR

 70   CONTINUE

      CALL GMEDIA(XX, NUMED)

      IF( NLEVEL.GE.2.5) THEN
      IF( INCALO(NAMES(NLEVEL-2)) .NE. 0 )THEN

       IMNCL = IGCAL_NCL(NUMBER(NLEVEL-2))
       IVN = 100*NUMBER(NLEVEL-2) + NUMBER(NLEVEL-1)*IMNCL
     +       + NUMBER(NLEVEL)

        PAR_MOD(2) = GTRAN(1,NLEVEL)
        PAR_MOD(3) = GTRAN(2,NLEVEL)
        IF( IMNCL.EQ.INUM(3)) THEN
            CALL CAFMFI
        ELSE
          IF( IMNCL.EQ.INUM(2)) THEN
            CALL CAFMFM
          ELSE
            CALL CAFMFO
          ENDIF
        ENDIF
           IHIT = IHIT + 1
           HITPOS(1,IHIT) = IVN
           HITPOS(2,IHIT) = EMOD
       IF( (PAR_MOD(3) + PAR_MOD(1))
     &             .LE.(MY + MPAR - SERR) )THEN
           XX(2) = PAR_MOD(3) + PAR_MOD(1) + SERR
           GOTO 80
       ELSE
           GOTO 100
       ENDIF
      ENDIF
      ENDIF
      XX(2) = MY - MPAR + SERR

C.. middle level

 80   CONTINUE

      CALL GMEDIA(XX, NUMED)

      IF( NLEVEL.GE.2.5) THEN
      IF( INCALO(NAMES(NLEVEL-2)) .NE. 0 )THEN

       IMNCL = IGCAL_NCL(NUMBER(NLEVEL-2))
       IVN = 100*NUMBER(NLEVEL-2) + NUMBER(NLEVEL-1)*IMNCL
     +       + NUMBER(NLEVEL)

        PAR_MOD(2) = GTRAN(1,NLEVEL)
        PAR_MOD(3) = GTRAN(2,NLEVEL)
        IF( IMNCL.EQ.INUM(3)) THEN
            CALL CAFMFI
        ELSE
          IF( IMNCL.EQ.INUM(2)) THEN
            CALL CAFMFM
          ELSE
            CALL CAFMFO
          ENDIF
        ENDIF
           IHIT = IHIT + 1
           HITPOS(1,IHIT) = IVN
           HITPOS(2,IHIT) = EMOD
        IF( (PAR_MOD(3) + PAR_MOD(1))
     &             .LE.(MY+MPAR-SERR)) THEN
           XX(2) = PAR_MOD(3) + PAR_MOD(1) + SERR
           GOTO 80
        ENDIF
      ENDIF
      ENDIF

C..   back to the original volume

 100  CONTINUE

       XX(1) = VECT(1)
       XX(2) = VECT(2)
       XX(3) = VECT(3)
       CALL GMEDIA(XX, NUMED)

       IMNCL = IGCAL_NCL(NUMBER(NLEVEL-2))
       IVN = 100*NUMBER(NLEVEL-2) + NUMBER(NLEVEL-1)*IMNCL
     +       + NUMBER(NLEVEL)

C..  renormalization

      EDEP = 0

      IF( IHIT.EQ.0) GOTO 999

      DO I = 1, IHIT
        EDEP = EDEP + HITPOS(2, I)
      ENDDO

      I = IUCOMP(IMNCL,INUM(1),3)
      IF( I.NE.0.AND.EDEP.GT.0.1E-6) THEN
           RNORM = RNO_MOD(I) * VECT(7) / EDEP
      ENDIF

      DO I = 1, IHIT
       HITPOS(2,I) = HITPOS(2,I) * RNORM
C       print *,i,hitpos(1,i), hitpos(2,i)
      ENDDO

 999   RETURN
       END
C************************************************************



