/************************************************************************
*
* HGKINE
* written by Siegmund Nowak
*
* This routines translates ARTE tables (MTRA,MVER) from event
* generator into GEANT structures JVERT, JTRACK  
*
*       called from HGEVNT
***********************************************************************/
#include <vector.h>
#include <math.h>
#include <stdio.h>
#include "gean/gclink.h"
#include "gean/gcnum.h"
#include "gean/ZGMOTH.h"
#include "gean/zgkine.hh"
#include "gean/zgmimp.hh"
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "arte/ArteUtil.h"
#include "arte/ArteFlag.h"
extern
#include "gean/zginteract.hh"

vector<int>    *hg_vert;
vector<hgmtra> *hg_mtra;
vector<hgmimp> *normal_mimp;
vector<hgmimp> *modul_mimp;
vector<hgmimp> *TRD_mimp;
vector<hgmtcp> *ECA_ep;
vector<hgmcal> *ECA_mimp;
vector<hgmimp> *RICH_mimp;

extern "C" {
  void hgkine_();
  void gsvert_(float[],int*, int*, float*, int*, int* );
  void gskine_(float[],int*, int*, float*, int*, int* );
  void mmbook_(int &, const int&);
  void mmdrop_(int &);
}  
  void cr_ipoint(int &,float[],float &);

void hgkine_() {
 
// allocate memory for temporary storage - delete is done in guout
   hg_vert     = new vector<int>;
   if(hg_vert == NULL) {
      arteMsg << "hgkine: hg_vert = NULL";         arteMsg.error();}  
   hg_mtra     = new vector<hgmtra>;
   if(hg_mtra == NULL) {
      arteMsg << "hgkine: hg_mtra = NULL";         arteMsg.error();}  
   normal_mimp = new vector<hgmimp>;
   if(normal_mimp == NULL) {
      arteMsg << "hgkine: normal_mimp = NULL";     arteMsg.error();}  
   modul_mimp  = new vector<hgmimp>;
   if(modul_mimp == NULL) {
      arteMsg << "hgkine: modul_mimp = NULL";      arteMsg.error();}  
   TRD_mimp    = new vector<hgmimp>;
   if(TRD_mimp == NULL) {
      arteMsg << "hgkine: TRD_mimp = NULL";        arteMsg.error();}  
   ECA_ep      = new vector<hgmtcp>;
   if(ECA_ep == NULL) {
      arteMsg << "hgkine: ECA_ep = NULL";          arteMsg.error();}  
   ECA_mimp    = new vector<hgmcal>;
   if(ECA_mimp == NULL) {
      arteMsg << "hgkine: ECA_mimp = NULL";        arteMsg.error();}  
   RICH_mimp   = new vector<hgmimp>; 
   if(RICH_mimp == NULL) {
      arteMsg << "hgkine: RICH_mimp = NULL";       arteMsg.error();}  

   hg_vert->reserve(1000);
   hg_mtra->reserve(2000);
   normal_mimp->reserve(3000);
   modul_mimp->reserve(500);
   ECA_ep->reserve(500);
   ECA_mimp->reserve(1000);  

// create banks for tracks and mimps   
   if(ArtePointer<MVER>::getLink().getRef()) 
          mmbook_(ArtePointer<MVER>::getLink().getRef(),0);
   if(ArtePointer<MTRA>::getLink().getRef()) 
          mmbook_(ArtePointer<MTRA>::getLink().getRef(),0);
   if(ArtePointer<MIMP>::getLink().getRef())
          mmdrop_(ArtePointer<MIMP>::getLink().getRef());
   if(ArtePointer<HITC>::getLink().getRef()) 
          mmdrop_(ArtePointer<HITC>::getLink().getRef());
   mmbook_(ArtePointer<MIMP>::getLink().getRef(),0);
   mmbook_(ArtePointer<MCAL>::getLink().getRef(),0);

//      iq(jhead+6) = 0

// some useful quantities
   zgkine ZGKINE; 
   int nmcme = ArtePointer<MCME>::nrow();
   int nmver = ArtePointer<MVER>::nrow();
   int nmtra = ArtePointer<MTRA>::nrow();
   union { int ji; float jx;};
   int i2 = 0;  int i0 = 0;   int nvx;
   float vertex[3]; float plab[3]; float time_gen; float ubuf; 

// temporary storage for relation ARTE-GEAN - delete at the end of hgkine
   int *kv = new int[nmver];         int *kt = new int[nmtra];      
   for(int i =0; i<nmver; i++ ) kv[i] =0;
   for(int i =0; i<nmtra; i++ ) kt[i] =0;

// for every primary vertex: generate coordinates (cr_ipoint) and transform
// coordinates of all vertices belonging to this event  
   for(int j = 1; j <= nmcme; j++) {
      int i1 = i2 + 1;
      ArtePointer<MVER> mver(i1); 
      if(!mver->flag ) {
         cr_ipoint(j,vertex,time_gen);
         if(j < nmcme) { 
            ArtePointer<MCME> mcme(j+1); 
            ArtePointer<MVER> j2 = mcme->mver;
            i2 = j2.index() -1; 
            
         }
         else   i2 = nmver ;
         for(int i=i1; i<=i2; i++){
            ArtePointer<MVER> mve(i); 
            mve->x += vertex[0];
            mve->y += vertex[1];
            mve->z += vertex[2]; 
            if(i==i1) mve->flag = 1; 
         }       
      } 
   }
   ip_wire.erase(ip_wire.begin(),ip_wire.end());

// loop over all existing tracks 
   for( int ill = 1; ill<=nmtra; ill++) {        
      int nt = 0;
      ArtePointer<MTRA> mtra(ill); 
      if(mtra->flag == 2) continue;  // finish the loop if it is a resonance
      int nvorig = mtra->orig;       // no resonance - find vertex origin
 L10:  
      if(nvorig  >  0) {             // mother track is no resonance 
         ArtePointer<MVER> vorig(nvorig); 
         if(kv[nvorig-1] == 0) {     // vertex not yet translated to gean
            vertex[0] = vorig->x;
            vertex[1] = vorig->y;
            vertex[2] = vorig->z + zgmoth_.ZZERO;
            ArtePointer<MTRA> itox = vorig->btra; //find vertex origin track
            int ito = itox.index();
            ArtePointer<MTRA> itoa(ito); 
            if(ito > 0) {             // the vertex is not a primary vertex 
               ArtePointer<TOPP>  ipor = itoa->topp;
               int ipart = ipor->gean;                         
               if(ipart == 10 || ipart == 16) continue; //fin if mother is a K0
               ArtePointer<MVER> ivxbef = itox->bver;
               int kfvbef = ivxbef.index(); 
               int jvxbef =*(gcbank_.lq + gclink_.jvertx -1 - kv[kfvbef-1]);
               ji = *(gcbank_.iq + jvxbef + 3);      float len = jx;
               ito = kt[ito-1];
               gsvert_(vertex,&ito, &i0, &ubuf, &i0 , &nvx);
               int jtr = *(gcbank_.lq + gclink_.jkine -1 - ito);
               ji = *(gcbank_.iq + jtr    );  float px = jx;
               ji = *(gcbank_.iq + jtr + 1);  float py = jx;
               ji = *(gcbank_.iq + jtr + 2);  float pz = jx;
               ji = *(gcbank_.iq + jtr + 2);  float e  = jx;
               float beta = float(sqrt(px*px+py*py+pz*pz)/e);
               int jv = *(gcbank_.lq + gclink_.jvertx -1 - nvx);
               jx =  itox->leng/beta/AR_CLIGHT + len;                
               *(gcbank_.iq + jv + 3) =  ji; }
            else {                          // it is a primary vertex 
               gsvert_(vertex,&ito,&i0, &ubuf, &i0 , &nvx );
               int jv = *(gcbank_.lq + gclink_.jvertx -1 - nvx);
               jx = time_gen; 
               *(gcbank_.iq + jv + 3) = ji;}

            kv[nvorig-1] = nvx;             // fill vertex rel arte - gean
            ZGKINE.mvpush(nvx,nvorig); }    // fill vertex rel gean - arte

         else    nvx = kv[nvorig-1]; }      // vertex was already translated

      else {
         ArtePointer<MTRA> ml(-nvorig);     // mother track is a resonance, 
         nvorig = ml->orig;                 // go back to its vertex
         goto L10;   }

//    make an entry for geant
      ArtePointer<TOPP>  ip = mtra->topp;
      int ipart = ip->gean;                 // particle id to gean id
      if(ipart == 0) ipart = 48;
      if(ipart == 10 || ipart == 16)   mtra->flag = 0;
      plab[0]      = mtra->px;
      plab[1]      = mtra->py;
      plab[2]      = mtra->pz;
      gskine_(plab, &ipart, &nvx, &ubuf, &i0, &nt); // register track in gean
      if(!nt) {
         arteMsg << "hgkine: track booking error at ill = " << ill << 
               " for ipart = " << ipart;
         arteMsg.error();
         continue;  }

//    store existing data about the track in the temporary vector
      hgmtra mt;
      mt.mtra = ill;
      mt.nhit = 0;
      mt.len  = mtra->ml;
      mt.flag = mtra->flag;
      mt.ep = 0;
      kt[ill-1] = nt;                  // fill track rel arte - gean
      ZGKINE.mtpush(nt,mt);            // fill track rel gean - arte   
    }                                  // end of track loop

   delete [] kv;           delete [] kt;          
   }
     

















