        subroutine otrgeo
******************************************************************
*
* Reads definition of OTR superlayer from text files
* and declare related Arte tables and Geant volumes.
*
* Called from g97main, g98main
*
* External references:
*   OTR-specific:     otrmod,otrmed,otropen,otrvol,wtldat
*   Reading routines: otrcomc,otrcomr,iread
*   HBGeant:          cggesl,crgwal,crrotm,gradln
*   Geant:            gsposp
*   Arte:             mmbook,mmbrel,atstop,ioclos,me*,ch*
*   Cernlib:          uhtoc,uctoh,lnblnk,cltou,cutol
*
* Arguments  in/out       Description
*
* isl        in           superlayer number (index for NESL table)
*
* Author: Alexander Lanyov     Mar 1999
*
******************************************************************
*
* Some superlayer values already filled in kagesl.F are just overwritten.
*
************************************************************************
c Problem:
c Wire begin and end fnede_y1w(ilay) and fnede_y2w(ilay) vary also with
c the wire number!
c One should define some special inede_shp=42,34 with fnede_xmad=tan(+-60),
c fnede_ymad=tan(+-5), fnede_zmad=tan(+-5)
************************************************************************
        implicit none

#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "gean/gclink.fi"
#include "gean/gcflag.fi"
#include "gean/gcnum.fi"
#include "gean/ZGMOTH.fi"
#include "gean/ZGBPIP.fi"
#include "gean/ZGOTR.fi"
#include "gean/ZGDETV.fi"
#include "auto/GEO.fi"
#include "gean/ZGESL.fi"
#include "auto/CON.fi"

        integer maxwor,maxstereo
        parameter (maxwor=20,maxstereo=6)
        integer isl

#include "gean/otrgeo.fi"

        character*4 laynam*5,modnam*8,secnam*11
        real xmin_des,xmax_des,ymin_des,ymax_des,zmin_des,zmax_des,ymid
        integer nmodule



        integer maxsl
        parameter (maxsl=15)
        character*4 side*5,modtype*16,slname,name
        character*120 cnamfil,modfile,dir
        character*80 buf1(maxpar)

        integer nsl,i,l,ic,ipl,imod,nmod,nlast,isec,ierr
        integer il,is,ip,iz,iside,sec1,sec2,nsec,iy,iy1,iy2,ll
        logical lfirst,ebeam,lsldec,lhadec,shortmod
        save nsl,nmod,lfirst

        integer iNaN
        real NaN,angle,angle_old,sinang,cosang
        real rotz,rotzmax,sinrot,cosrot,dalign(3),xref,yref,zref
        real xrot,yrot,z,xlab,ylab,xp,yp,zp,xp_rot,yp_rot
        real xp_mo,yp_mo,zp_mo,xp_,yp_,xnew,ynew
        real xmin,xmax,ymin,ymax,zmin,zmax
        real xmin_lab,xmax_lab,ymin_lab,ymax_lab
        real xmin_de,xmax_de,ymin_de,ymax_de,zmin_de,zmax_de
        real xmin_mo,xmax_mo,ymin_mo,ymax_mo,zmin_mo,zmax_mo
        real xmin_ge,xmax_ge,ymin_ge,ymax_ge,zmin_ge,zmax_ge
        real sxmn_de,sxmx_de,symn_de,symx_de
        real ydivul(motlay,6),ydivdl(motlay,6)
        real d,pit,pit1,ydg10,dyg10,x1,x2,x1_,x2_,dx(5),dx1,dx2
        real otdisd,otdisds,covthick
        real zstereo(-1:1,maxstereo),cov_th,cov_d1,cov_d2,foipin,angpar
        real ping_up,ping_down
        real ping_st5,ping_st10,ping_mc5,ping_sp5,ping_sp10

        integer nstereo,nwords,lw(maxwor)
        character*80 words(maxwor)
        real xpin,ypin

        integer nwires,ncells,mcells,mcells_ge,dfi,dla,dfio,dlao
        logical nowires,ldiv,ldummy
        integer ilay,ilay1,ilay2,il0,il1,il2,jl
        integer ndiv,idiv,ndiv0,idiv0,lastdiv
        integer irot,jrot,ir,it,ir_g10
        save irot,jrot

        integer inede,inede1,nvol,ivo,jvo,numed,jtm,nmat,imat
        integer ityp,ityp0,ityp_g,ityp_ge,laytyp,imflag,npar
        integer ltyp_old,ltyp_new,ltyp_geo
        real par(10),xmsc,xms_tmp,tmp
        character*4 modshape,nam,matg10(2),matnam*20
c Function types
        integer lenocc,iucomp,matnum,otrsec,iread
        logical otrcomc,otrcomr

c required for icdeci (M432)
        integer nd,ne,nf,ng,num
        real dummy
        common /slate/ nd,ne,nf,ng,num(2),dummy(34)

        data ydg10/1./
        data lfirst/.true./

#include "arte/Arte.ff"
#include "auto/GEO.ff"
#include "auto/CON.ff"

c-------------------------------- initialization
        if (lfirst) then
           nmod=0       ! # of module definition loaded

           if (geomvers.eq.97.) then
              c78='OTRGEO: For 97 run geometry use ARTE version 1'
              call meerro(c78)
           else if (geomvers.ge.20. .and. geomvers.lt.97.) then
              c78='OTRGEO: Illegal geometry version'
              call ch78bi(int(geomvers))
              call meerro(c78)
           end if

           dyg10 = 1.5    ! was 1.0               ! width of G10 strip

c Possibly, this is not needed anymore (otrmed.F does everything itself)
c           call wtldat('OTR/otr.dat')

           call otrmed
           call otrvol

           matg10(1) = 'WTGT'
           matg10(2) = 'WTGM'

           lfirst=.false.
        end if
c-------------------------------- End of initialization

        isl = currsl

        call uhtoc(iq(hqnesl_name(isl)),4,slname,4)
        if (lenocc(slname).le.0)
     +         call meerro('OTRGEO: SLNAME not initialized!')

c------------------------- Install aging boxes at PC01 in 98-99 run
        if (slname.eq.'PC01' .and. abs(fnesl_y1(isl)).lt.100.) then
           c78 = 'Installed aging boxes at PC01 position'
           call me78wa
           call otrage(isl)
           return
        end if

        xmin_all = 1.e10 ! big positive number
        xmax_all =-xmin_all
        ymin_all = xmin_all
        ymax_all = xmax_all
        zmin_all = xmin_all
        zmax_all = xmax_all

        xmin_sl = fnesl_x1(isl)
        xmax_sl = fnesl_x2(isl)
        ymin_sl = fnesl_y1(isl)
        ymax_sl = fnesl_y2(isl)
        zmin_sl = fnesl_z1(isl)
        zmax_sl = fnesl_z2(isl)

        otdisd = 0.
        otdisds = 0.
        foipin = 7.  ! not used?

        covthick = 0.
        ping_st5  = 9.
        ping_st10 = 9.5
        ping_mc5  = 7.3
        ping_sp5  = 4.8
        ping_sp10 = 5.3

        nsl = INESL_NSL(ISL)
        if (nsl.le.0 .or. nsl .gt. maxsl) then
           c78 = 'Illegal SL number ='
           call ch78bi(nsl)
           call meerro(c78)
        endif

        cnamfil=slname(:lenocc(slname))//'.txt'
        call cutol(cnamfil)


        call zgetfil(dir,ll)
        call otropen(lread,cnamfil,dir)

        if (cnamfil(:1).eq.' ') then
           c78 = 'File'
           call ch78bc(cnamfil(:lenocc(cnamfil)))
           call ch78bc('was not opened')
           call meerro(c78)
           return
        end if

        lsldec = .false.
        lhadec = .false.

c To use e.g. NaN = 7FFFFFFF ?
        iNaN = -10000000        ! number which should give an error if used
        NaN = -1.e35            ! number which should give an error if used
        angle = NaN

        do i=1,6
           do iside=-1,1
              zstereo(iside,i) = 1500.
           end do
        end do
        iside = 0

        rotzwt(nsl) = 0.
        rotzmax = 0.
        dalign(1) = 0.
        dalign(2) = 0.
        dalign(3) = 0.
        xref = 0.
        yref = 0.
        zref = 0.

        ltyp_geo = inesl_ltyp(isl)      ! layer types/sequences
        iq(iqnesl_ltyp(isl)) = 0        ! layer types/sequences

c================================================ begin of reading loop
 100    continue
        read (lread,'(a)',end=991) buf

        do i=1,len(buf)
           ic=ichar(buf(i:i))
c           if (ic.ge.ichar('A') .and. ic.le.ichar('Z'))
c     @        buf(i:i)=char(ic+ichar('a')-ichar('A'))
            if (ic.eq. 9) buf(i:i)=' '           ! Tab
            if (ic.eq.13) buf(i:i)=' '           ! CR
            if (buf(i:i).eq.'"') buf(i:i)=' '
c           if (buf(i:i).ne.' ') nlast=i
        end do

c        buf = spaces(buf,1)  ! does not work for long lines
        call parse(buf,nwords,words,lw,nlast)
        buf = words(1)
        do i=2,nwords
           buf(lenocc(buf)+2:) = words(i)
        end do

        nlast = lenocc(buf)
c        do i=1,len(buf)
c           if (buf(i:i).ne.' ') nlast=i
c        end do

        call cutol(buf(:nlast))
        if (nlast.le.0) goto 100
        if (buf(:1).eq.'*') goto 100
        if (buf(:5).eq.'free ') goto 100
        if (buf(:2).eq.'" ') goto 100
        if (buf(:3).eq.'m.w') goto 100
        if (buf(:3).eq.'a.l') goto 100
        if (buf(1:1).eq.'l' .and. buf(3:3).eq.'r') goto 100


c------------------------------------------------ Ignored commands
             if (otrcomc('superlayer data - structure',cparam,-1)) then
        else if (otrcomc('stereo layer half'          ,cparam,-1)) then
        else if (otrcomc('stereo layer angle , mrad'  ,cparam,-1)) then
        else if (otrcomc('layer type'                 ,cparam,-1)) then
        else if (otrcomc('# of stereo layer halfs'    ,cparam,-1)) then
        else if (otrcomc('modules of stereo layer'    ,cparam,-1)) then

        else if (otrcomc('summary of coordinates and'  ,cparam,-8)) then
        else if (otrcomc('thickness of 5mm single'     ,cparam,-8)) then
        else if (otrcomc('thickness of 5mm double'     ,cparam,-8)) then
        else if (otrcomc('thickness of 10mm single'    ,cparam,-8)) then
        else if (otrcomc('thickness of 10mm double'    ,cparam,-8)) then
        else if (otrcomc('shift of pin-hole center of' ,cparam,-8)) then
        else if (otrcomc('shift of z-position of 10mm' ,cparam,-8)) then
        else if (otrcomc('thickness of distance piece' ,cparam,-8)) then
        else if (otrcomc('total thickness of'          ,cparam,-8)) then
        else if (otrcomc('length of modules around'    ,cparam,-8)) then
        else if (otrcomc('definition of pin-holder'    ,cparam,-8)) then
        else if (otrcomc('pin-position'                ,cparam,-8)) then
        else if (otrcomc('(m.walter,'                  ,cparam,-8)) then
        else if (otrcomc('thickness of cap for 1'      ,cparam,-8)) then
        else if (otrcomc('thickness of cap for stereo' ,cparam,-8)) then
        else if (otrcomc('structure:'                  ,cparam,-8)) then

c-------------------------------------------------- New commands Oct 1998
        else if (otrcomc('z0'  ,cparam,-9)) then
c       else if (otrcomc(''    ,cparam,-8)) then
c       else if (otrcomc(''    ,cparam,-8)) then

c-------------------------------- Read alignment corrections (N.B. unit is cm!)
        else if (otrcomr('dx',dalign(1),1)) then
        else if (otrcomr('dy',dalign(2),1)) then
        else if (otrcomr('dz',dalign(3),1)) then
        else if (otrcomr('dphiz',rotzwt(nsl),1)) then
        else if (otrcomr('xref',xref,1)) then
        else if (otrcomr('yref',yref,1)) then
        else if (otrcomr('zref',zref,1)) then

c-------------------------------- Reading of stereo layers position (zstereo)
        else if (otrcomc('z-position backside basisplate 1.module',
     @           cparam,-10)) then

c cparam should contain:
c old   '1.stereo layer at y=0, mm:' zstereo
c new   '1.stereo layer at y=0, -x/+x half of sl' zstereo_left  zstereo_right
c word:  1        2     3  4    5     6    7  8

           if (ichar(cparam(1)(1:1)).lt.ichar('0')
     @    .or. ichar(cparam(1)(1:1)).gt.ichar('9')) then
              call meerro('Illegal stereo layer '//cparam(1)(1:1))
           end if

           nstereo = ichar(cparam(1)(1:1)) - ichar('0')
           if (cparam(5)(:5).eq.'-x/+x') then ! new command - for MC
              if (lenocc(cparam(9)).gt.0) then
                 read (cparam(9),*,err=99) zstereo(-1,nstereo)
              else
                 zstereo(-1,nstereo) = 0.
              end if
              if (lenocc(cparam(10)).gt.0) then
                 read (cparam(10),*,err=99) zstereo(+1,nstereo)
              else
                 zstereo(+1,nstereo) = 0.
              end if
              zstereo(-1,nstereo) = 0.1 * zstereo(-1,nstereo)
              zstereo(+1,nstereo) = 0.1 * zstereo(+1,nstereo)
              if (zstereo(+1,nstereo) .eq. 0.)
     @           zstereo(+1,nstereo) = zstereo(-1,nstereo)

           else if (cparam(5)(:3).eq.'mm:') then ! old command - for PC and TC
              read (cparam( 6),*,err=99) zstereo(-1,nstereo)
              zstereo(-1,nstereo) = 0.1 * zstereo(-1,nstereo)
              zstereo(+1,nstereo) = zstereo(-1,nstereo)
           else
              call meerro('Illegal format for stereo layer '//
     @                    cparam(5)(:lenocc(cparam(5))))
           end if

           zstereo(-1,nstereo) = zstereo(-1,nstereo) + dalign(3)
           zstereo(+1,nstereo) = zstereo(+1,nstereo) + dalign(3)


c-------------------------------- Some distances for whole superlayer
        else if (otrcomr('distance between pin-hole centers'//
     @              ' for full length modules, mm',otdisd,1)) then
           otdisd = 0.1 * otdisd

        else if (otrcomr('distance between pin-hole centers'//
     @          ' for shorter modules (+x side), mm',otdisds,1)) then
           otdisds = 0.1 * otdisds

        else if (otrcomr('thickness of cap material, mm',cov_th,-1))then
           cov_th = 0.1 * cov_th
        else if (otrcomr('gap between cap and 1.module in z, mm',
     @                                          cov_d1,1))then
           cov_d1 = 0.1 * cov_d1
        else if (otrcomr('gap between last module and cap in z, mm',
     @                                          cov_d2,1))then
           cov_d2 = 0.1 * cov_d2

c-- dummy at the moment
      else if (otrcomr('gap between cap and 1.module in z, mm, double',
     @                                          dummy,1))then
      else if (otrcomr('gap between cap and 1.module in z, mm, single',
     @                                          dummy,1))then

        else if (otrcomr('distance between foil end'//
     @            ' and pin-hole center, mm',foipin,1))then
           foipin = 0.1 * foipin

c distance between pin-hole center and strip end (standard 5mm modules) , mm            90
c distance between pin-hole center and strip end (all mc5 ""sp5""  5mm modules) , mm    73
c distance between pin-hole center and strip end (""sp1"" mc3,4,6,8  5mm modules) , mm  48
c distance between pin-hole center and strip end (standard 10mm modules) , mm           95
c distance between pin-hole center and strip end (""sp1"" mc3,4,6   10mm modules) , mm  53

        else if (otrcomr('distance between pin-hole center and strip'//
     @   ' end (standard 5mm modules) , mm',ping_st5,1))then
           ping_st5 = 0.1 * ping_st5

        else if (otrcomr('distance between pin-hole center and strip'//
     @   ' end (standard 10mm modules) , mm',ping_st10,1))then
           ping_st10 = 0.1 * ping_st10

        else if (otrcomr('distance between pin-hole center and strip'//
     @   ' end (all mc5 sp5 5mm modules) , mm',ping_mc5,1))then
           ping_mc5 = 0.1 * ping_mc5

        else if (otrcomr('distance between pin-hole center and strip'//
     @   ' end ( sp1 mc3,4,6,8 5mm modules) , mm',ping_sp5,1))then
           ping_sp5 = 0.1 * ping_sp5

        else if (otrcomr('distance between pin-hole center and strip'//
     @   ' end ( sp1 mc3,4,6 10mm modules) , mm',ping_sp10,1))then
           ping_sp10 = 0.1 * ping_sp10

        else if (otrcomr('thickness of cover material in z, mm',
     @           covthick,1)) then
           covthick = 0.1 * covthick


c----------------------------------------------------------- command SL
        else if (otrcomc('name',cparam,1)
     @      .or. otrcomc('superlayer',cparam,-5)) then
c parameters 2-5 are ignored (nsl, zmin, zmax, n_channels_total)
           name = cparam(1)
           if (name(4:4).eq.' ') then
              name(4:4) = name(3:3)
              name(3:3) = '0'
           end if
           call cltou(name)
           if(slname.ne.name) call meerro(
     @            'Encountered '//name//' while expecting : '//slname)

           if (lsldec) then
              c78 = 'SL has been already declared in file "'
              call ch78ac(cnamfil(:lenocc(cnamfil)))
              call ch78ac('"')
              call meerro(c78)
              goto 991
           end if

           ipl = 0
           nmodule = 0

c These are the coordinates of the whole SL in Lab system.
        else if (otrcomr('xmin',xmin_sl,1)) then

c give warning - also for other coordinates
           if (fnesl_x1(isl).ne.0. .and. xmin_sl.lt.fnesl_x1(isl)) then
              c78 = 'New xmin is less than previous'
              call ch78bf(fnesl_x1(isl),-2)
              call me78wa
           end if

        else if (otrcomr('xmax',xmax_sl,1)) then
           continue
        else if (otrcomr('ymin',ymin_sl,1)) then
           continue
        else if (otrcomr('ymax',ymax_sl,1)) then
           continue

c
c S.Nowak: superlayer Z-sizes should be defined only in a special SL file.
c So ignore this information.
c

       else if (otrcomr('zmin',dummy,1)
     @     .or. otrcomr('minimal z-coord. of sl',dummy,1)) then
           continue

       else if (otrcomr('zmax',dummy,1)
     @     .or. otrcomr('maximum z-coord. of sl',dummy,1)) then
           continue


c----------------------------------------------------------- command Half
        else if (otrcomc('half',buf1,-7)) then

c parameters 4-7 are ignored (n_mod, n_mod_5mm, n_mod_10mm, n_channels_total)

           angle_old = angle

           read (buf1(1),*,err=99) angle
           angle = angle/1000. * 180. / 3.14159      ! mrad --> degrees
           
           angle = -angle
           
           iside = iread(buf1(3))
           if (iside.eq.-1) then
              side = 'left'
           else if (iside.eq.1) then
              side = 'right'
           else
              c78 = 'Wrong side in HALF command (98 geometry)'
              call meerro (c78)
           end if

           if (abs(angle-angle_old) .gt. 1.e-5) then
c In principle, it is better to check also the layer side (iside).
              ipl = ipl + 1

              if (abs(angle) .le. 1.1*rotzmax+0.001) then ! angle - in degrees
                 it=0
              else if (angle.gt.0.) then
                 it=1
              else if (angle.lt.0.) then
                 it=2
              end if

c layer types/sequences
c We will add 3 if later we find that this is double layer!
              iq(iqnesl_ltyp(isl)) = iq(iqnesl_ltyp(isl))*10 + it

              nmodule = 0
           end if

c laynam = '-t2-3' for left half of TC02, stereolayer 3
           if (iside.eq.-1) then
              laynam(1:1) = '-'
           else
              laynam(1:1) = '+'
           end if
           laynam(2:2) = slname(1:1)
           call cutol(laynam(2:2))
           laynam(3:3) = slname(4:4)
           laynam(4:4) = '-'
           write (laynam(5:),'(i1)') ipl


           sinrot = sin(rotzwt(nsl))
           cosrot = cos(rotzwt(nsl))
           rotz = rotzwt(nsl)*raddeg  ! correction of SL angles from alignment
           rotzmax = max(rotzmax, abs(rotz))
           angle = angle + rotz


c--------- find rotation matrix (create one if does not exist)
        call crrotm(irot,90.,-angle,90.,90.-angle,0.,0.)
        jrot = lq(jrotm-irot)
        if (jrot.eq.0) call meerro ('OTRGEO: illegal rotation matrix')
        ir=irot
        if (angle.eq.0.) ir=0
        currot = ir


        cosang = cos(angle * 0.01745329)
        sinang = sin(angle * 0.01745329)

c--------- Fill NESL table with new information
           if (.not. lsldec) then               ! Book new SL
c
c Alignment corrections (not used for superlayer borders)
c
c In principle, one should also take into account rotation!???
c             xmax_sl = xmax_sl + dalign(1)
c             ymax_sl = ymax_sl + dalign(2)
c             zmax_sl = zmax_sl + dalign(3)
c             xmin_sl = xmin_sl + dalign(1)
c             ymin_sl = ymin_sl + dalign(2)
c             zmin_sl = zmin_sl + dalign(3)

              q(qnesl_x1(isl)) = xmin_sl ! Already filled in KAGESL?
              q(qnesl_x2(isl)) = xmax_sl ! Already filled in KAGESL?
              q(qnesl_y1(isl)) = ymin_sl ! Already filled in KAGESL?
              q(qnesl_y2(isl)) = ymax_sl ! Already filled in KAGESL?
              q(qnesl_z1(isl)) = zmin_sl ! Already filled in KAGESL?
              q(qnesl_z2(isl)) = zmax_sl ! Already filled in KAGESL?

c           if (xmin_sl.lt.fnesl_x1(isl)) then
c              c78 = 'xmin changed'
c              call ch78bf(fnesl_x1(isl),-2)
c              call ch78bf(xmin_sl,-2)
c              call me78wa
c              q(qnesl_x1(isl)) = xmin_sl
c           end if
c           if (xmax_sl.gt.fnesl_x2(isl)) then
c              c78 = 'xmax changed'
c              call ch78bf(fnesl_x2(isl),-2)
c              call ch78bf(xmax_sl,-2)
c              call me78wa
c              q(qnesl_x2(isl)) = xmax_sl
c           end if
c           if (ymin_sl.lt.fnesl_y1(isl)) then
c              c78 = 'ymin changed'
c              call ch78bf(fnesl_y1(isl),-2)
c              call ch78bf(ymin_sl,-2)
c              call me78wa
c              q(qnesl_y1(isl)) = ymin_sl
c           end if
c           if (ymax_sl.gt.fnesl_y2(isl)) then
c              c78 = 'ymax changed'
c              call ch78bf(fnesl_y2(isl),-2)
c              call ch78bf(ymax_sl,-2)
c              call me78wa
c              q(qnesl_y2(isl)) = ymax_sl
c           end if
c           if (zmin_sl.lt.fnesl_z1(isl)) then
c              c78 = 'zmin changed'
c              call ch78bf(fnesl_z1(isl),-2)
c              call ch78bf(zmin_sl,-2)
c              call me78wa
c              q(qnesl_z1(isl)) = zmin_sl
c           end if
c           if (zmax_sl.lt.fnesl_z2(isl)) then
c              c78 = 'zmax changed'
c              call ch78bf(fnesl_z2(isl),-2)
c              call ch78bf(zmax_sl,-2)
c              call me78wa
c              q(qnesl_z2(isl)) = zmax_sl
c           end if

              iq(iqnesl_lay(isl)) = 0   ! number of layers

              imflag = inesl_mfl(isl)
              if (imflag.eq.0) then
                 call cggesl(slname,'MOTH',0)
              else
                 call cggesl(slname,'MAGN',0)
              end if

              xp_sl = 0.5*(xmax_sl + xmin_sl)
              yp_sl = 0.5*(ymax_sl + ymin_sl)
              zp_sl = 0.5*(zmax_sl + zmin_sl)

              lsldec = .true.
           end if                               ! End of definition of new SL

        lhadec = .true.

c----------------------------------------------------------- command Module
        else if (otrcomc('module',buf1,8)
     @    .or. buf(1:4).eq.laynam(2:5)) then

c last two parameters are ignored in MC superlayers (chan_top, chan_bot)


           nmodule = nmodule + 1
           if (buf(1:4).eq.laynam(2:5)) then
              modnam(1:1) = laynam(1:1)
              modnam(2:) = buf(:7)             ! modnam = '-t2-3-01'
              read (modnam(7:8),*,err=99) i
              if (i.ne.nmodule .and. slname(1:1).eq.'m') then
               c78 = 'Missmatch in module number: '//modnam//', N(mod)='
                 call ch78ai(nmodule)
                 call meerro (c78)
              end if

              buf(:7) = 'module '
              ldummy = otrcomc('module',buf1,8)
              if (.not.ldummy) call meerro('Eror in module command')
           else
              if (slname(1:1).eq.'m') then
                 write (modnam,'(a,i2.2)') laynam//'-',nmodule
              else
                 modnam = laynam//'-00'
              end if
           end if

           angpar = 0.
           do i=1,len(buf1(1))
              if (buf1(1)(i:i).eq.'/') buf1(1)(i:i)='_'
           end do
           modtype = buf1(1)

           if (buf1(2)(2:2).eq.'c'
     @         .and.(buf1(2)(3:3).eq.'-' .or. buf1(2)(4:4).eq.'-')) then
              buf1(2) = buf1(3)
              buf1(3) = buf1(4)
              buf1(4) = buf1(5)
              buf1(5) = buf1(6)
           end if
           read (buf1(2),*,err=99) xpin
           read (buf1(3),*,err=99) ypin
           read (buf1(4),*,err=99) z

c------------------------------- Extract module information
           l = min(len(cotnam(1)),len(modtype))
           do imod=1,nmod
              if (cotnam(imod)(:l).eq.modtype(:l)) goto 30
           end do

           write (modfile,'(a)') modtype(:lenocc(modtype))//'.txt'

           nmod=nmod+1
           imod=nmod

           call otrmod(imod,modfile,ierr)

           if (otdiam(imod).eq.0. .or. ierr.ne.0) then
              c78 = ' ? Module "'//modtype(:l)//'" not loaded'
              call me78wa

              nmod=nmod-1
              imod=nmod

              goto 100
           end if

           maxcell(imod) = 0
           do ilay=1,notlay(imod)
              do is=1,4                 ! is = sectors A,B,C,D
                 maxcell(imod) = max(maxcell(imod),notsew(ilay,is,imod))
              end do
           end do

30         continue
c----------------------------------------------- module found

c shortmod = short module above and below the beam
c            or some not complete module in the right half of PC,TC
           if (abs(otdist(imod)).lt.1.e-3) then
              shortmod = .true.
           else
              shortmod = .false.
           end if

c----------------------------------------------- Process coordinates
           xpin = 0.1 * xpin
           ypin = 0.1 * ypin
           z    = 0.1 * z

           xlab = xpin
           ylab = ypin

           if (otdisd .eq. 0.) then
              c78 = 'Illegal default pin distance:'
              call ch78bf(otdisd,-6)
              call ch78bc('cm.')
              call meerro(c78)
           end if
           
           if (otdist(imod) .gt. 0.
     @        .and. abs(otdist(imod)-otdisd) .gt. 1.e-3
     @        .and. abs(otdist(imod)-otdisds) .gt. 1.e-3) then
              c78 = 'Illegal pin distance:'
              call ch78bf(otdist(imod),-6)
              call ch78bc('cm, but default is')
              call ch78bf(otdisd,-6)
              if (otdisds.ne.0.) then
                 call ch78bc('or')
                 call ch78bf(otdisds,-6)
              end if
              call ch78bc('cm.')
              call meerro(c78)
           end if
           
c Calculate center of pins position from one pin position.
           d = 0.5 * otdisd
           if (ylab .lt. 0.) d = -d
           ylab = ylab - cosang * d
           xlab = xlab - sinang * d

c------------------------------------------------ Alignment corrections
           xlab = xlab + dalign(1)
           ylab = ylab + dalign(2)
           z    = z    + dalign(3)

c alignment rotation around Z axis
c          xnew =   xlab * cosrot + ylab * sinrot
c          ynew =  -xlab * sinrot + ylab * cosrot

c alignment rotation around a point with given X,Y-coordinates
           xnew = xref + (xlab-xref) * cosrot + (ylab-yref) * sinrot
           ynew = yref - (xlab-xref) * sinrot + (ylab-yref) * cosrot
           xlab = xnew
           ylab = ynew


           zstereo(iside,ipl) = min(z,zstereo(iside,ipl))
c-----------------------------------------------------------------------
c calculate distances Pin-G10 strip.
c (take into account that in database there are distances to G10 end,
c  not G10 middle line)
           if (slname.eq.'MC05') then
              ping_up = ping_mc5 - 0.5*dyg10
           else
              if (abs(otdiam(imod)-0.5).lt.1.e-3) then
                 ping_up = ping_st5 - 0.5*dyg10         ! 5-mm module
              else
                 ping_up = ping_st10 - 0.5*dyg10        ! 10-mm module
              end if
           end if

           ping_down = ping_up
           if (slname(1:1).eq.'M'
     @     .and. ichar(slname(4:4)).gt.ichar('2') .and. iside.eq.1) then
              if (abs(otdiam(imod)-0.5).lt.1.e-3) then
                 ping_down = ping_sp5 - 0.5*dyg10       ! 5-mm module
              else
                 ping_down = ping_sp10 - 0.5*dyg10      ! 10-mm module
              end if
           end if

           if (ping_up.eq.0) then
              ping_up = 100.
           end if
           if (ping_down.eq.0) then
              ping_down = 100.
           end if

           xrot =  xlab * cosang - ylab * sinang        ! lab --> rot
           yrot = +xlab * sinang + ylab * cosang

           if (abs(angle).gt.100.) then
              c78 = 'No HALF command was given (angle is undefined)'
              call meerro (c78)
           end if


        sec1 =  100
        sec2 = -100

c Patch e.g. for pc-5d32-12 or tc-5d24-12
c Modules with 4 sectors, sector A has no wires but sec1=1.
        if (otg10p(1,1,imod).ne.0.
     @.and. otg10p(1,2,imod).ne.0.
     @.and. otg10p(1,3,imod).ne.0.) then
           sec1=1
           sec2=4
        end if

        do ilay=1,notlay(imod)
           do is=1,4                 ! is = sectors A,B,C,D
              if (notsew(ilay,is,imod).ne.0
     @       .or. notnsw(ilay,is,imod).ne.0 ) then
                 sec1 = min(sec1,is)
                 sec2 = max(sec2,is)
              end if
           end do
        end do
        nsec = sec2-sec1+1

        if (sec1.ne.1 .and. .not.shortmod) then
           c78 = 'Illegal sector number for short modules: sec1 ='
           call ch78bi(sec1)
           call ch78ac(', sec2 =')
           call ch78bi(sec2)
           call meerro(c78)
        end if

c mc5-101, mc5-102 should be not rotated
        if (iside.eq.1 .or. modtype(:6).eq.'mc5-10') then  ! ======= right side or 97 run
           do ilay=1,notlay(imod)
              do iy=1,2                 ! iy = y>0, y<0
                 ttcefi(ilay,iy,imod) = otcefi(ilay,iy,imod)
                 ttcela(ilay,iy,imod) = otcela(ilay,iy,imod)
                 if (ttcefi(ilay,iy,imod).gt.otcela(ilay,iy,imod)) then
                    tmp = ttcela(ilay,iy,imod)
                    ttcela(ilay,iy,imod) = ttcefi(ilay,iy,imod)
                    ttcefi(ilay,iy,imod) = tmp
                 end if
              end do

              do iy=1,2                 ! iy = y>0, y<0
                 ttwifi(ilay,iy,imod) = min(ttcefi(ilay,1,imod),
     @                                      ttcefi(ilay,2,imod))
                 ttwila(ilay,iy,imod) = max(ttcela(ilay,1,imod),
     @                                      ttcela(ilay,2,imod))
              end do

              do is=1,nsec                 ! is = sectors A,B,C,D
                 if (notsew(ilay,is+sec1-1,imod).eq.0
     @         .and. fotsew(ilay,is+sec1-1,imod).eq.0) then
                    nttsew(ilay,is,imod) = 0
                    fttsew(ilay,is,imod) = 0
                 else
                    fttsew(ilay,is,imod) =
     @                   maxcell(imod) + 1 - notsew(ilay,is+sec1-1,imod)

                    nttsew(ilay,is,imod) =
     @                   maxcell(imod) + 1 - fotsew(ilay,is+sec1-1,imod)
                 end if
              end do

              do ip=1,3                 ! ip = up,mid,down
                 ttg10p(ilay,ip,imod) = otg10p(ilay,ip,imod)
              end do
              do ip=1,6                 ! ip = up:a,b,c, down:a,b,c
                 ttsupp(ilay,ip,imod) = otsupp(ilay,ip,imod)
              end do
           end do
           nttcht(imod) = notcht(imod)
           nttchb(imod) = notchb(imod)

        else if (iside.eq.-1) then      ! =============== left side

           do ilay=1,notlay(imod)
              do iy=1,2                 ! iy = y>0, y<0
                 ttcela(ilay,iy,imod) = otcefi(ilay,3-iy,imod) * iside
                 ttcefi(ilay,iy,imod) = otcela(ilay,3-iy,imod) * iside
                 if (ttcefi(ilay,iy,imod).gt.otcela(ilay,iy,imod)) then
                    tmp = ttcela(ilay,iy,imod)
                    ttcela(ilay,iy,imod) = ttcefi(ilay,iy,imod)
                    ttcefi(ilay,iy,imod) = tmp
                 end if
              end do

              do iy=1,2
                 ttwifi(ilay,iy,imod) = min(ttcefi(ilay,1,imod),
     @                                      ttcefi(ilay,2,imod))
                 ttwila(ilay,iy,imod) = max(ttcela(ilay,1,imod),
     @                                      ttcela(ilay,2,imod))
              end do

              do is=1,nsec                 ! is = sectors A,B,C,D
                 nttsew(ilay,is,imod)=notsew(ilay,sec2+1-is,imod)
                 fttsew(ilay,is,imod)=fotsew(ilay,sec2+1-is,imod)
              end do

              do ip=1,3                 ! ip = up,mid,down
                 ttg10p(ilay,ip,imod) = otg10p(ilay,4-ip,imod) * iside
              end do
              do ip=1,6                 ! ip = up:a,b,c, down:a,b,c
                 ttsupp(ilay,ip,imod) = otsupp(ilay,ip,imod) * iside
              end do
           end do

           if (cotsid(1,imod)(1:1).ne.' '
     @   .and. cotsid(2,imod)(1:1).ne.' ') then
              nttchb(imod) = notcht(imod)  ! up side down
              nttcht(imod) = notchb(imod)
           else
              nttcht(imod) = notcht(imod)  ! identical copy
              nttchb(imod) = notchb(imod)
           end if

        else
           c78 = 'Illegal iside ='
           call ch78bi(iside)
           call meerro(c78)
        end if

        if (shortmod) then  ! module with only one pin
           if (ypin.gt.0.) then         ! only top pin
              nttcht(imod) = notcht(imod) + notchb(imod)
              nttchb(imod) = 0
           else if (ypin.lt.0.) then    ! only bottom pin
              nttcht(imod) = 0
              nttchb(imod) = notcht(imod) + notchb(imod)
           end if
        end if



c###      c---------------------- In MC05 all modules have baseplate upstream.
c   Therefore one should rotate module around Y axis and modify Z coordinate
        if (slname.eq.'MC05') then


           z = z - 0.5 * (notlay(imod)+1) * otdiam(imod)
           do iy=1,2
              do ilay=1,notlay(imod)/2

                 tmp = -ttcela(notlay(imod)+1-ilay,iy,imod)
                 ttcela(notlay(imod)+1-ilay,iy,imod) =
     @                 -ttcefi(ilay,iy,imod)
                 ttcefi(ilay,iy,imod) = tmp

                 tmp = -ttcefi(notlay(imod)+1-ilay,iy,imod)
                 ttcefi(notlay(imod)+1-ilay,iy,imod) =
     @                 -ttcela(ilay,iy,imod)
                 ttcela(ilay,iy,imod) = tmp

                 tmp = -ttwila(notlay(imod)+1-ilay,iy,imod)
                 ttwila(notlay(imod)+1-ilay,iy,imod) =
     @                 -ttwifi(ilay,iy,imod)
                 ttwifi(ilay,iy,imod) = tmp

                 tmp = -ttwifi(notlay(imod)+1-ilay,iy,imod)
                 ttwifi(notlay(imod)+1-ilay,iy,imod) =
     @                 -ttwila(ilay,iy,imod)
                 ttwila(ilay,iy,imod) = tmp
              end do
           end do

           do is=1,nsec
              do ilay=1,notlay(imod)/2
                 if (nttsew(ilay,is,imod).ne.0
     @         .and. fttsew(ilay,is,imod).ne.0
     @         .and. nttsew(notlay(imod)+1-ilay,is,imod).ne.0
     @         .and. fttsew(notlay(imod)+1-ilay,is,imod).ne.0) then

                   tmp =
     @             maxcell(imod)-nttsew(notlay(imod)+1-ilay,is,imod)+1
                                 nttsew(notlay(imod)+1-ilay,is,imod) =
     @             maxcell(imod)-fttsew(ilay,is,imod)+1
                                 fttsew(ilay,is,imod) = tmp

                   tmp =
     @             maxcell(imod)-fttsew(notlay(imod)+1-ilay,is,imod)+1
                                 fttsew(notlay(imod)+1-ilay,is,imod) =
     @             maxcell(imod)-nttsew(ilay,is,imod)+1
                                 nttsew(ilay,is,imod) = tmp

                 else if (nttsew(ilay,is,imod).eq.0
     @              .and. fttsew(ilay,is,imod).eq.0
     @              .and. nttsew(notlay(imod)+1-ilay,is,imod).eq.0
     @              .and. fttsew(notlay(imod)+1-ilay,is,imod).eq.0) then

                    continue

                 else if (nttsew(ilay,is,imod).eq.0
     @              .and. fttsew(ilay,is,imod).eq.0) then

                    nttsew(ilay,is,imod) =
     @              maxcell(imod)-fttsew(notlay(imod)+1-ilay,is,imod)+1
                    fttsew(ilay,is,imod) =
     @              maxcell(imod)-nttsew(notlay(imod)+1-ilay,is,imod)+1

                    nttsew(notlay(imod)+1-ilay,is,imod) = 0
                    fttsew(notlay(imod)+1-ilay,is,imod) = 0

                 else if (nttsew(notlay(imod)+1-ilay,is,imod).eq.0
     @              .and. fttsew(notlay(imod)+1-ilay,is,imod).eq.0) then

                    nttsew(notlay(imod)+1-ilay,is,imod) =
     @              maxcell(imod)-fttsew(ilay,is,imod)+1
                    fttsew(notlay(imod)+1-ilay,is,imod) =
     @              maxcell(imod)-nttsew(ilay,is,imod)+1

                    nttsew(ilay,is,imod) = 0
                    fttsew(ilay,is,imod) = 0

                 else
                    call meerro('Illgeal nttsew/fttsew in MC05')
                 end if
              end do
           end do

           do ip=1,3
              do ilay=1,notlay(imod)/2
                 tmp = ttg10p(notlay(imod)+1-ilay,ip,imod)
                 ttg10p(notlay(imod)+1-ilay,ip,imod) =
     @              ttg10p(ilay,ip,imod)
                 ttg10p(ilay,ip,imod) = tmp
              end do
           end do

           do ip=1,6
              do ilay=1,notlay(imod)/2
                 tmp = ttsupp(notlay(imod)+1-ilay,ip,imod)
                 ttsupp(notlay(imod)+1-ilay,ip,imod) =
     @              ttsupp(ilay,ip,imod)
                 ttsupp(ilay,ip,imod) = tmp
              end do
           end do
        end if


c-----------------------------------------------
c Find out the volume division over y axis
c (maximum 4 volumes are possible).
c Division is made by dividing G10 strips.
c-----------------------------------------------

        if (nttcht(imod).eq.0 .and. nttchb(imod).eq.0)
     @               call meerro('Both nttcht and nttchb are zero')

        if (shortmod
     @     .and. nttcht(imod).ne.0 .and. nttchb(imod).ne.0 ) then
           c78 = 'Illegal: no pin distance,'//
     @           ' but both top and bottom channels'
           call meerro(c78)
        end if

        ndiv=1          ! ndiv will denote number of initialized divisions on y

        do ilay=1,notlay(imod)
           if (.not.shortmod) then
              ydivul(ilay,ndiv) = 0.5*otdisd - ping_up            ! normal module
           else if (ypin.gt.0.) then
              ydivul(ilay,ndiv) = 0.5*otdisd - ping_up            ! top pin
           else if (ypin.lt.0.) then
              ydivul(ilay,ndiv) = ttg10p(ilay,3,imod)             ! bottom pin
           end if
        end do

        lastdiv = 3
        if (shortmod) lastdiv = min(nsec-1,2)

        do idiv=1,lastdiv
           ldiv = .false.
           do ilay=1,notlay(imod)
              if (ttg10p(ilay,idiv,imod).ne.0.
     @            .and. (.not.ignore_g10(imod))) ldiv = .true.
           end do
           if (ldiv) then
              do ilay=1,notlay(imod)
                 ydivdl(ilay,ndiv  ) = ttg10p(ilay,idiv,imod)
                 ydivul(ilay,ndiv+1) = ttg10p(ilay,idiv,imod)
              end do
              ndiv = ndiv+1
           end if
        end do

        do ilay=1,notlay(imod)
           if (.not.shortmod) then                                ! normal module
              ydivdl(ilay,ndiv) = -0.5*otdist(imod) + ping_down
           else if (ypin.gt.0.) then                              ! top pin
              ydivdl(ilay,ndiv) = ttg10p(ilay,ndiv,imod)
           else if (ypin.lt.0.) then                              ! bottom pin
              ydivdl(ilay,ndiv) = - 0.5*otdisd + ping_down
           end if
        end do

c If ttg10p=0, then this position is not used for declaring wires.
c But it will be still used for ymin,ymax, so fill it with reasonable information
        do idiv=1,ndiv
           do ilay=1,notlay(imod)

              if (ydivdl(ilay,idiv).eq.0.) then
                 do i=1,notlay(imod)
                    if (ydivdl(i,idiv).ne.0.) then
                       ydivdl(ilay,idiv) = ydivdl(i,idiv)
                       goto 91
                    end if
                 end do
 91              continue
              end if

              if (ydivul(ilay,idiv).eq.0.) then
                 do i=1,notlay(imod)
                    if (ydivul(i,idiv).ne.0.) then
                       ydivul(ilay,idiv) = ydivul(i,idiv)
                       goto 92
                    end if
                 end do
 92              continue
              end if

           end do
        end do

c------------ in case of e-beam cutout we have additional vertical divisions.
c------------ But only for wide modules?

        ebeam = .false.
        do ip=1,2
           do il=1,notlay(imod)
              if (otbeam(il,ip,imod) .ne. 0) ebeam = .true.
           end do
        end do

c Narrow modules at e-beam pipe place (they are without wires here!)
c e.g. for PC[14] TC[12], module [pt]c-5d24-31*
c      also pc-5d32-51* ?

cccc il0 is not defined here!!!!!!!!!! ???????????? !!!!!!!!!!!!!
c              if (nttsew(il0,ndiv,imod).le.24
c     @      .or. modtype(:10).eq. 'pc-5d32-51') then
c                 ebeam = .false.
c              end if

        ndiv0 = ndiv
        if (ebeam) then


           ndiv = ndiv+2
           do ilay=1,notlay(imod)
              ydivul(ilay,ndiv  ) = otbeam(ilay,2,imod)
              ydivdl(ilay,ndiv  ) = ydivdl(ilay,ndiv-2)

              ydivul(ilay,ndiv-1) = otbeam(ilay,1,imod)
              ydivdl(ilay,ndiv-1) = otbeam(ilay,2,imod)

              ydivdl(ilay,ndiv-2) = otbeam(ilay,1,imod)
           end do
        end if

c---------- Extract trailing empty monolayers - they will be declared as walls.
        il1=100
        il2=-1
        do jl=1,notlay(imod)
           nowires=.true.
           do is=1,nsec
              if (nttsew(jl,is,imod).ne.0) nowires=.false.
           end do
           if (.not.nowires) il1=min(il1,jl)
           if (.not.nowires) il2=max(il2,jl)
        end do

        if ((il2-il1).gt.3 .or. (il2-il1).lt.0) then
           c78 = 'Module "'
           call ch78ac(modtype(:lenocc(modtype)))
           call ch78ac('" has illegal number of monolayers:')
           call ch78ai(il2-il1+1)
           call meerro(c78)
        end if



        if (lhadec) then
           if (il2-il1.gt.2 .and.mod(iq(iqnesl_ltyp(isl)),10).le.2)
     @     iq(iqnesl_ltyp(isl)) = iq(iqnesl_ltyp(isl)) + 3 ! layer types/sequences
           lhadec = .false.
        end if

c----- Form volume index ITYP
c--1     10 mm single layer    sensitive
c--2      5 mm single layer    sensitive
c--3      5 mm single layer nonsensitive
c--4     10 mm double layer    sensitive
c--5      5 mm double layer    sensitive
c--6      5 mm double layer nonsensitive
c
c--7     10 mm single layer    sensitive in magnetic field
c--8      5 mm single layer    sensitive in magnetic field
c--9      5 mm single layer nonsensitive in magnetic field
c
c--10    10 mm double layer    sensitive in magnetic field
c--11     5 mm double layer    sensitive in magnetic field
c--12     5 mm double layer nonsensitive in magnetic field

        pit1=otdiam(imod)*sqrt(3.)      ! ideal-geometry pitch

        if (otdiam(imod).eq.1.) then
           ityp0 = 1            ! sensitive wires (10 mm)
           pit = 1.74
        else if (otdiam(imod).eq.0.5) then

c          if (sense_wires) then
              ityp0 = 2         ! sensitive wires (5 mm)
c          else
c             ityp0 = 3         ! nonsensitive wires (5 mm)
c          end if

           pit = 0.87

        else
           c78 = 'Wrong OTR cell diameter:'
           call ch78bf(otdiam(imod),-2)
           call ch78ac(' cm')
           call meerro (c78)
        end if

        if (notlay(imod).le.2) then
           laytyp = 0      ! single layers
        else
           laytyp = 1      ! double layers
        end if

        ityp = 6*imflag + 3*laytyp + ityp0
        if (ityp.le.0 .or. ityp.gt.12) then
           c78 = 'Illegal ITYP = '
           call ch78bi(ityp)
           call atstop(c78)
        end if

        if (abs(pit/pit1-1.).gt.0.005) then
          c78 = 'For module "'//modtype(:lenocc(modtype))//'"'
             call ch78ac(', monolayer # ')
             call ch78ai(il0)
             call ch78ac(' Ideal pitch = ')
             call ch78af(pit1,-4)
             call ch78ac(' and real pitch = ')
             call ch78af(pit,-4)
          call meerro(c78)
        end if

c---------------------- loop on subvolumes (NEDE volumes)
        xmin_mo = 1.e10 ! big positive number
        xmax_mo =-xmin_mo
        ymin_mo = xmin_mo
        ymax_mo = xmax_mo
        zmin_mo = xmin_mo
        zmax_mo = xmax_mo
        mcells=0


        do idiv=1,ndiv          ! division on y

          call mmbook(jnede,1)

          inede = nnede()
          if (idiv.eq.1) inede1=inede

          if (iq(iqnede_fwl(inede1)).eq.0)
     @        iq(iqnede_fwl(inede1)) = nnwal() + 1
          if (iq(iqnesl_fwl(isl)).eq.0)
     @        iq(iqnesl_fwl(isl)) = nnwal() + 1

          call mmbrel(jnsnd,isl,inede)

          if (inesl_fpl(isl).eq.0) iq(iqnesl_fpl(isl)) = inede ! 1. plane
          iq(iqnesl_npl(isl)) = inesl_npl(isl) + 1             ! # of planes

          iq(iqnede_cmp_l())=lgesl_wch  ! detector component
          iq(iqnede_lay_l())=ipl        ! layer number within the superlayer

          q(qnede_pit(inede))=pit       ! pitch

          q(qnede_cos_l())= 1.  ! cos of the wire angle with respect to y-axis
          q(qnede_sin_l())= 0.

c copy 3x3 rotation matrix, transposing it
        jrot = lq(jrotm-irot)
        if (jrot.eq.0) call meerro ('OTRGEO: illegal rot.matrix!')
        q(qnede_rot(1,inede)) = q(jrot+1)
        q(qnede_rot(2,inede)) = q(jrot+4)
        q(qnede_rot(3,inede)) = q(jrot+7)
        q(qnede_rot(4,inede)) = q(jrot+2)
        q(qnede_rot(5,inede)) = q(jrot+5)
        q(qnede_rot(6,inede)) = q(jrot+8)
        q(qnede_rot(7,inede)) = q(jrot+3)
        q(qnede_rot(8,inede)) = q(jrot+6)
        q(qnede_rot(9,inede)) = q(jrot+9)


c-- Use some global file for this stuff? (or possibly local files also?)
       !q(qnede_acc_l()) = SIGWT(1)     ! measurement accuracy
        q(qnede_acc_l()) = 0.02         ! measurement accuracy

       !q(qnede_eff_l()) = EFFWT        ! hit efficiency
        q(qnede_eff_l()) = 0.98         ! hit efficiency

       !q(qnede_dod_l()) = 0.5*otdiam(imod) - 0.01 ! max drift distance
        q(qnede_dod_l()) = 0.3*pit                 ! max drift distance

        iq(iqnede_nsl_l())=il2-il1+1    ! number of sublayers in a segment

        xmin_de = 1.e10 ! big positive number
        xmax_de =-xmin_de
        ymin_de = xmin_de
        ymax_de = xmax_de
        zmin_de = xmin_de
        zmax_de = xmax_de

        xmin_des = xmin_de    ! point to sensitive volume only
        xmax_des = xmax_de
        ymin_des = ymin_de
        ymax_des = ymax_de
        zmin_des = zmin_de
        zmax_des = zmax_de

        do ilay=1,4
           iq(iqnede_nwi(ilay,inede))=0
        end do
        ilay = 0

        do il0=il1,il2    ! loop on monolayers
           il = il0-il1+1

c Here il0 = 2..5 - index for Module type arrays,
c      il  = 1..4 - index for NEDE banks.

c These are coordinates of NEDE volume in rotated system

           ymin = yrot + ydivdl(il0,idiv)
           ymax = yrot + ydivul(il0,idiv)


           ymid = 0.5*(ymin + ymax)
           if (ymid.gt.0.) then
              iy=1
           else
              iy=2
           end if

           x1 = ttcefi(il0,iy,imod)
           x2 = ttcela(il0,iy,imod)
c xmin, xmax set the bottom and the upper cell centers of cell row
           xmin = xrot + x1
           xmax = xrot + x2
           dx(il) = x2-x1

           if (xmin.ge.xmax) then
              c78 = 'For idiv='
              call ch78ai(idiv)
              call ch78ac(', il0=')
              call ch78ai(il0)
              call ch78ac(' xmin =')
              call ch78bf(xmin,-3)
              call ch78ac(', xmax =')
              call ch78bf(xmax,-3)
              call me78wa
           end if

c These are exact borders for this monolayer
           zmin = z + 0.5*(il0-1)*otdiam(imod)
           zmax = z + 0.5*(il0+1)*otdiam(imod)

c idiv  - index for ydivul, ydivdl
c idiv0 - index for module type arrays

           idiv0 = min(idiv,ndiv0)
           if (idiv.eq.3 .and. nsec.eq.4 .and. ndiv.eq.3
     @         .and. ttg10p(1,2,imod).eq.0.
     @         .and. ttg10p(3,2,imod).eq.0.) then
              idiv0 = 4
           end if

           if (modtype(:lenocc(modtype)).eq.'mc8-5d24-22'
     @      .and. idiv0.eq.2) then
              idiv0 = 3
           end if

           if (ttwifi(il0,iy,imod).ne.0.
     @         .and. fttsew(il0,idiv0,imod).ne.0) then

c Narrow modules at e-beam pipe place (they are without wires here!)
c e.g. for PC[14] TC[12], module [pt]c-5d24-31*
c      also pc-5d32-51* ?
              if (idiv.eq.ndiv0+1
     @      .and. (nttsew(il0,idiv0,imod).le.24
     @             .or. modtype(:10).eq. 'pc-5d32-51')) then
                 goto 20
              end if

              ilay = ilay + 1
              nwires = 1 + nttsew(il0,idiv0,imod)-fttsew(il0,idiv0,imod)
              iq(iqnede_nwi(ilay,inede)) = nwires

              if (nwires.le.1) then
                 c78 = 'Illegal nwires ='
                 call ch78bi(nwires)
                 call ch78bc('for inede =')
                 call ch78bi(inede)
                 call ch78ac(', ilay =')
                 call ch78bi(ilay)
                 call meerro (c78)
              end if

              q(qnede_xfw(ilay,inede)) = xrot + ttwifi(il0,iy,imod)
     @                               + (fttsew(il0,idiv0,imod)-1)*pit
              q(qnede_zfw(ilay,inede)) = z + 0.5*il0*otdiam(imod)

              q(qnede_y1w(ilay,inede)) = ymin
              q(qnede_y2w(ilay,inede)) = ymax

c########################################################
c              xmin_des = min(xmin, xmin_des)
c              xmax_des = max(xmax, xmax_des)

              if (ttwifi(il0,iy,imod) .gt. ttwila(il0,iy,imod)) then
                 call meerro('OTRGEO: ttwifi > ttwila')
              end if

              xmin_des = min(fnede_xfw(ilay,inede), xmin_des)
              xmax_des = max(fnede_xfw(ilay,inede)+(nwires-1)*pit,
     @                                              xmax_des)
c########################################################

              ymin_des = min(ymin, ymin_des)
              ymax_des = max(ymax, ymax_des)
              zmin_des = min(zmin, zmin_des)
              zmax_des = max(zmax, zmax_des)

           end if
20         continue
c##
           d = abs(ttcela(il0,iy,imod) - ttcefi(il0,iy,imod))
           d = x2 - x1
           ncells = int(d/pit1 + 1.5)
           mcells = max(mcells,ncells)          ! not needed now?
           if (ncells.gt.1 .and. d.gt.0.
     @      .and. abs(pit-d/(ncells-1)).gt.1.e-6) then
              call meerro('Something wrong with pitch')
           end if


c=======================================================================
c The following borders are calculated in rotated (with module) frame:
c
c xmin    - monolayer
c xmin_de - NEDE volume
c sxmn_de - sensitive NEDE volume
c xmin_mo - module (several NEDEs)
c xmin_mo - Geant volume (for common module xmin_mo=xmin_ge)
c
c The following sizes are in laboratory frame:
c
c xmin_all - SL - only modules volumes (used for finding positions of walls)
c xmin_sl  - SL Geant volume - should contain everything including walls
c                 (got from database)
c xmin_box - empty box inside walls
c                 (calculated and used inside wall routines)
c=======================================================================

c Now xmin, xmax set the bottom and the upper conners of cell row
           xmin = xmin - pit/3.
           xmax = xmax + pit/3.

           xmin_de = min(xmin,xmin_de)
           xmax_de = max(xmax,xmax_de)
           ymin_de = min(ymin,ymin_de)
           ymax_de = max(ymax,ymax_de)
           zmin_de = min(zmin,zmin_de)
           zmax_de = max(zmax,zmax_de)

           xmin_mo = min(xmin,xmin_mo)
           xmax_mo = max(xmax,xmax_mo)
           ymin_mo = min(ymin,ymin_mo)
           ymax_mo = max(ymax,ymax_mo)
           zmin_mo = min(zmin,zmin_mo)
           zmax_mo = max(zmax,zmax_mo)

c Now xmin, xmax set the bottom and top corners
c  of GEANT VOLUME (parallelepiped or trapezoid)
           xmin = xmin - 2./3.*pit
           xmax = xmax + 2./3.*pit

c rot --> lab
           xmin_lab =  xmin * cosang + min(ymin * sinang, ymax * sinang)
           xmax_lab =  xmax * cosang + max(ymin * sinang, ymax * sinang)
           ymin_lab = - max(xmin * sinang, xmax * sinang) + ymin*cosang
           ymax_lab = - min(xmin * sinang, xmax * sinang) + ymax*cosang

           xmin_all = min(xmin_lab,xmin_all)
           xmax_all = max(xmax_lab,xmax_all)
           ymin_all = min(ymin_lab,ymin_all)
           ymax_all = max(ymax_lab,ymax_all)
           zmin_all = min(zmin,zmin_all)
           zmax_all = max(zmax,zmax_all)

           if (il0.lt.il2
     @   .and. ttcefi(il0  ,iy,imod) .ne. 0.
     @   .and. ttcefi(il0+1,iy,imod) .ne. 0.) then

              dfio = dfi
              dlao = dla

              dfi =
     @  nint((ttcefi(il0+1,iy,imod) - ttcefi(il0,iy,imod))/(0.5*pit))
              dla =
     @  nint((ttcela(il0+1,iy,imod) - ttcela(il0,iy,imod))/(0.5*pit))


              if (il0 .gt. il1) then
                 if (dfi.ne.dfio .or. dla.ne.dlao) then
                    write (c78,*)'Non-standard geometry:',
     @                      ' module = ',modtype(:lenocc(modtype)),
     @                      ' idiv=',idiv,', il0=',il0,', iy=',iy
                     call meerro(c78)
                 end if
              end if

              if (dfi.eq.dla) then
                 modshape = 'PARA'
                 if (dfi.lt.0.) then
                    angpar = -60.
                 else
                    angpar = +60.
                 end if
              else if (dfi.eq.-dla) then
                 modshape = 'TRD1'
              else
                 modshape = 'Unknown'
                 c78 = 'Unknown module shape: dfi ='
                 call ch78bi(dfi)
                 call ch78ac(', dla =')
                 call ch78bi(dla)
                 call meerro(c78)
              end if

           end if

        end do                  ! loop on monolayers

c detector shape: 1-box, 2-TRD1, 10-parallelepiped
c TO INTRODUCE: 42 - another parallelepiped
c               34 - another TRD1
        if (modshape(:4).eq.'PARA') then
           iq(iqnede_shp_l()) = 42
        else if (modshape(:4).eq.'TRD1') then
           iq(iqnede_shp_l()) = 2               ! actually should be 34
        else
           c78 = 'Unknown module shape for inede ='
           call ch78bi(inede)
           call meerro(c78)
        end if



        iq(iqnede_nsl_l()) = ilay

c Rotate coordinates
        xp_ = 0.5*(xmin_de+xmax_de)
        yp_ = 0.5*(ymin_de+ymax_de)
        ynew = -sinang * xp_ + cosang * yp_  ! rot --> lab (only for y)
        if (ilay.ne.0) then
           isec = otrsec(imod,xp_,ynew,nsec)
        else
           isec = 0
        end if

        iq(iqnede_sec_l()) = isec       ! sector number

        if (isec.ne.0) then
             secnam(:9) = modnam // '-'             ! secnam  = '-t2-3-20-12'
             write (secnam(10:11),'(i2.2)') isec
             call uctoh(secnam//' ', iq(hqnede_hwc_l(1)), 12,12)
        else
             call uctoh('none        ', iq(hqnede_hwc_l(1)), 12,12)
        end if

        q(qnede_xmin_l()) = xmin_de - 2./3.*pit
        q(qnede_xmax_l()) = xmax_de + 2./3.*pit

        q(qnede_ymin_l()) = ymin_de
        q(qnede_ymax_l()) = ymax_de
        q(qnede_zmin_l()) = zmin_de
        q(qnede_zmax_l()) = zmax_de

c Fill sensitive volume borders only on the volume with sense wires
        sxmn_de = xmin_des - pit/3.
        sxmx_de = xmax_des + pit/3.
        symn_de = ymin_des + 0.5*(ydg10+dyg10)
        symx_de = ymax_des - 0.5*(ydg10+dyg10)

c Uncomment this if you want to fill borders on the volume of all cells
c (even without wires)
c        sxmn_de = xmin_de
c        sxmx_de = xmax_de
c        symn_de = ymin_de + 0.5*(ydg10+dyg10)
c        symx_de = ymax_de - 0.5*(ydg10+dyg10)

c If there are no sense wires in this GEDE, leave borders zero
        if (abs(sxmn_de).lt.100000.) then
           q(qnede_sxmn_l()) = sxmn_de ! x of the left edge of sensitive area
           q(qnede_sxmx_l()) = sxmx_de ! right
           q(qnede_symn_l()) = symn_de ! y left
           q(qnede_symx_l()) = symx_de ! y right
        end if

           if (symn_de.ge.symx_de
     @  .and. abs(symn_de).lt.100000.) then
              c78 = 'Wrong sensitive volume borders on Y:'
              call me78sp('T')
              c78 = 'symn_de ='
              call ch78bf(symn_de,-4)
              call ch78ac(', symx_de = ')
              call ch78af(symx_de,-4)
              call meerro(c78)
           end if

c------------- Put reference for material (copied from CRNEDE.F)
        CALL UHTOC(wtdt(ityp),4,nam,4)
        ivo = iucomp (nam, iq(jvolum+1), iq(jvolum-1))
        if (ivo.eq.0)  call meerro('Nonvalid volume'//nam)
        jvo = lq(jvolum-ivo)
        if (jvo.eq.0) then
           c78 = 'jvo = 0 for wtdt = '//nam//', ivo = '
           call ch78bi(ivo)
           call me78er
        endif

c Only first NEDE bank carries the GEANT name (used in digitization)
c idiv.gt.idiv0 - is it O.K. for dgo96.F ????????
        if (inede.eq.inede1 .or. idiv.gt.idiv0) then
           iq(hqnede_gvol_l()) = wtdt(ityp)
        else
           call uctoh('    ', iq(hqnede_gvol_l()), 4,4)
        end if

        numed = q(jvo+4)                ! numed  The same as tracking medium?
        jtm = iq(jtmed-numed-8)         ! jtm = lq(jtmed-numed)
        if (jtm.eq.0) then
           c78 = 'Illegal jvo ='
           call ch78bi(jtm)
           call me78er
        endif

        nmat = q(jtm+6)                 ! material # in Geant
        call uhtoc(q(iq(jmate-nmat-8)+1),4,matnam,20)

        imat = matnum(matnam(1:lenocc(matnam))) ! material # in Arte
        call mmbrel(jgmnd,imat,inede)

c Multiple scattering angle for 1 GeV particles
        xms_tmp = fgmat_radl(imat)
        if (xms_tmp.eq.0.) CALL MEERRO('OTRGEO: Wrong XMS')
        xmsc = (zmax_de-zmin_de)/xms_tmp
        if (xmsc.le.0.) CALL MEERRO('OTRGEO: Wrong XMSC')
        q(qnede_ms_l()) = xmsc*(0.0136*(1.+0.038*log(xmsc)))**2

       iq(iqnede_nwl_l())=0     ! number of walls connected to this detector
       iq(iqnede_fwl_l())=0     ! first wall connected to this detector

c Fill angle of parallelepiped (fnede_xmad) in degrees
       if (modshape .eq. 'PARA') then
          q(qnede_xmad_l())= tan(degrad*angpar)   ! angle of PARA: +-60 degrees
       else if (modshape .eq. 'TRD1') then
          q(qnede_xmad_l())= 0.
       end if

c Fill angle of NEDE bank borders: upper (fnede_ymad) and lower (fnede_zmad).
c Unit is degrees.
       q(qnede_ymad_l()) = 0.
       q(qnede_zmad_l()) = 0.
       if (fltmod(imod).eq.2) then
          if (idiv.gt.1 .and. idiv.le.ndiv0)
     @       q(qnede_ymad_l()) = tan(degrad*angle)
          if (idiv.lt.ndiv0)
     @       q(qnede_zmad_l()) = tan(degrad*angle)
       end if




        end do          ! division on y


c========================================= End of NEDE volume declaration


c========================================= Geant volume declaration
c---- Create Geant volume containing the whole module (several NEDE volumes)

        do idiv=ndiv0,ndiv      ! loop on vertical Geant volumes
                                ! above e-beam, near e-beam, under e-beam
           ymin_ge = ymin_mo
           ymax_ge = ymax_mo

           if (ndiv.ne.ndiv0) then
              if (idiv.eq.ndiv0) then
                 ymin_ge = +1000.
                 do ilay=1,notlay(imod)
                    if (otbeam(ilay,1,imod).ne.0.)
     @                 ymin_ge = min(yrot+otbeam(ilay,1,imod), ymin_ge)
                 end do
              else if (idiv.eq.ndiv0+1) then
                 ymin = -1000.
                 ymax = +1000.
                 do ilay=1,notlay(imod)
                    if (otbeam(ilay,1,imod).ne.0.)
     @                 ymax_ge = min(yrot+otbeam(ilay,1,imod), ymax_ge)
                    if (otbeam(ilay,2,imod).ne.0.)
     @                 ymin_ge = max(yrot+otbeam(ilay,2,imod), ymin_ge)
                 end do
              else if (idiv.eq.ndiv0+2) then
                 ymax_ge = -1000.
                 do ilay=1,notlay(imod)
                    if (otbeam(ilay,2,imod).ne.0.)
     @                 ymax_ge = max(yrot+otbeam(ilay,2,imod), ymax_ge)
                 end do
              else
                 call meerro('Something wrong with ndiv0')
              end if
           end if

           do iz=1,3    ! loop on horizontal (along Z) Geant volume:
                        ! wall, sensitive volume, wall

              if (iz.eq.1 .and. il1.eq.1) goto 200
              if (iz.eq.3 .and. il2.eq.notlay(imod)) goto 200

              mcells_ge = 0
              if (idiv.eq.ndiv0+1 .and. mcells .le. 24) goto 200

              x1_ = 1.e10
              x2_ = -1.e10
              if (iz.eq.1) then
                 ilay1=1
                 ilay2=il1-1
              else if (iz.eq.2) then
                 ilay1=il1
                 ilay2=il2
              else if (iz.eq.3) then
                 ilay1=il2+1
                 ilay2=notlay(imod)
              end if

              if (ilay1.gt.ilay2) then
                 c78 = 'Illegal ilay1='
                 call ch78ai(ilay1)
                 call ch78ac(', ilay2=')
                 call ch78ai(ilay2)
                 call meerro(c78)
              end if

              do il0=ilay1,ilay2
                 if (ymax_ge .ge. 0.) then
                    iy1 = 1
                 else
                    iy1 = 2
                 end if
                 if (ymin_ge .le. 0.) then
                    iy2 = 2
                 else
                    iy2 = 1
                 end if

                 if (iy1.gt.iy2) then
                    c78 = 'Illegal iy1='
                    call ch78ai(iy1)
                    call ch78ac(', iy2=')
                    call ch78ai(iy2)
                    call meerro(c78)
                 end if

                 x1 = +10000.
                 x2 = -10000.
                 do iy=iy1,iy2
                    if (ttcefi(il0,iy,imod).ne.0.) then
                       x1 = min(ttcefi(il0,iy,imod),x1)
                       if (idiv.eq.ndiv0+1)
     @                    x1 = max(ttcefi(il0,iy,imod)+15.,x1)
                    end if
                    if (ttcela(il0,iy,imod).ne.0.) then
                       x2 = max(ttcela(il0,iy,imod),x2)
                    end if

                    x1_ = min(x1,x1_)
                    x2_ = max(x2,x2_)
                 end do


                 dx(il0) = x2-x1
                 d = x2 - x1
                 ncells = int(d/pit1 + 1.5)
                 mcells_ge = max(mcells_ge,ncells)
              end do

              if (x1_.ge.x2_) then
                 c78 = 'Illegal x1_='
                 call ch78af(x1_,3)
                 call ch78ac(', x2_=')
                 call ch78af(x2_,3)
                 call meerro(c78)
              end if

              if (mcells_ge.le.0) then
                 c78 = 'Illegal mcells_ge='
                 call ch78ai(mcells_ge)
                 call meerro(c78)
              end if

              xmin_ge = xrot + x1_
              xmax_ge = xrot + x2_

              dx1 = dx(ilay1)
              dx2 = dx(ilay2)

              nvol = inede1 + (idiv-ndiv0)
              if (iz.ne.2) nvol = nnwal()+1

              zmin_ge = z + 0.5*(ilay1-1)*otdiam(imod)
              zmax_ge = z + 0.5*(ilay2+1)*otdiam(imod)
              if (iz.eq.1) then
                 zmax_ge = zmax_ge - 0.5*otdiam(imod)
                 if (modshape(:4).eq.'TRD1') then
                    dx2 = dx2 + pit
                 else
                    if (angpar.lt.0.) then
                       xmin_ge = xmin_ge + 0.5*pit
                    else
                       xmax_ge = xmax_ge - 0.5*pit
                    end if
                 end if


              else if (iz.eq.3) then
                 zmin_ge = zmin_ge + 0.5*otdiam(imod)
                 if (modshape(:4).eq.'TRD1') then
                    dx1 = dx1 - pit
                 else
                    if (angpar.gt.0.) then
                       xmin_ge = xmin_ge + 0.5*pit
                    else
                       xmax_ge = xmax_ge - 0.5*pit
                    end if
                 end if
              end if

              xp_rot = 0.5*(xmax_ge + xmin_ge)
              yp_rot = 0.5*(ymax_ge + ymin_ge)

              zp_mo  = 0.5*(zmax_ge + zmin_ge)
              zp = zp_mo - zp_sl

              if (iz.eq.2) then
                 ityp_g = 6*imflag + 3*laytyp + ityp0  ! sensitive: 10 & 5 mm
              else
                 ityp_g = 6*imflag + 3*laytyp + 3      ! nonsensitive: 5 mm
              end if

              if (modshape(:4).eq.'PARA') then !shape = PARA = parallelepiped
                 npar = 6
                 par(1) = 0.5*(dx1 + pit)
                 par(2) = 0.5*(ymax_ge - ymin_ge)
                 par(3) = 0.5*(zmax_ge - zmin_ge)
                 par(4) = 0.
                 par(5) = angpar
                 par(6) = 0.
                 do i=1,3
                    if (par(i).le.0.) then
                       c78 = 'PARA negative par('
                       call ch78ai(i)
                       call ch78ac(') =')
                       call ch78af(par(i),-3)
                       call meerro(c78)
                    end if
                 end do
                 ityp_ge = ityp_g

              else if (modshape(:4).eq.'TRD1') then ! shape = TRD1 = trapezoid 1
                 npar = 4
                 par(1) = 0.5*dx1
                 par(2) = 0.5*dx2
                 if ((par(2)-par(1)) .gt. 0.1*pit) then
                    par(2) = par(2) + pit
                 else
                    par(1) = par(1) + pit
                 end if
                 par(3) = 0.5*(ymax_ge - ymin_ge)
                 par(4) = 0.5*(zmax_ge - zmin_ge)
                 do i=1,4
                    if (par(i).le.0.) then
                       c78 = 'TRD1 negative par('
                       call ch78ai(i)
                       call ch78ac(') =')
                       call ch78af(par(i),-3)
                       call meerro(c78)
                    end if
                 end do

                 ityp_ge = ityp_g + 12
              end if

c rot --> lab
              xp_mo =  cosang * xp_rot + sinang * yp_rot
              yp_mo = -sinang * xp_rot + cosang * yp_rot
              xp = xp_mo - xp_sl
              yp = yp_mo - yp_sl


              call gsposp(wtdt(ityp_ge),nvol,slname,xp,yp,zp,
     @              ir,'ONLY',par,npar)

              if (iz.ne.2) then
                 call crgwal(wtdt(ityp_ge),nnwal()+1)
              end if


200           continue
           end do
        end do

c------------------------------------- Install the dividing G10-strips
        if (ipl.le.0 .or. ipl.gt.maxstereo) then
           c78 = 'Illegal number of stereo layer ='
           call ch78bi(ipl)
           call meerro(c78)
        end if
        if (zstereo(iside,ipl).eq.0.) then
           c78 = 'Illegal zstereo('
           call ch78ai(iside)
           call ch78ac(',')
           call ch78ai(ipl)
           call ch78ac(') =')
           call ch78bf(zstereo(iside,ipl),-3)
           call meerro(c78)
        end if

        do il = 1,notlay(imod)          ! monolayers
           do idiv = 1,3                ! vertical divisions
              par(2) = 0.5 * dyg10
              par(3) = 0.5 * 0.01

              if (ttg10p(il,idiv,imod) .ne. 0.) then

                 if ((yrot + ttg10p(il,idiv,imod)) .gt. 0.) then
                    iy = 1
                 else
                    iy = 2
                 end if

c##
                 if (ttcefi(il,iy,imod).eq.0.) then
                    c78 = 'For '//modtype(:lenocc(modtype))
                    call ch78ac(': ttcefi = 0. for il=')
                    call ch78ai(il)
                    call ch78ac(', iy=')
                    call ch78ai(iy)
                    call me78wa
                 end if

                 iy1 = iy
                 iy2 = iy
                 if (idiv.eq.2) then
                    iy1 = 1
                    iy2 = 2
                 end if

                 xmin = xrot + ttcefi(il,iy,imod) - pit/3.
                 xmax = xrot + ttcela(il,iy,imod) + pit/3.
                 do iy=iy1,iy2
                    xmin = min(xmin, xrot + ttcefi(il,iy,imod) - pit/3.)
                    xmax = max(xmax, xrot + ttcela(il,iy,imod) + pit/3.)
                 end do

                 par(1) = 0.5 * (xmax-xmin)

                 xp = 0.5 * (xmax + xmin)
                 yp = yrot + ttg10p(il,idiv,imod)

                 zmin = z + 0.5*il*otdiam(imod)
                 if (zstereo(iside,ipl).ne.0.) zmin = zstereo(iside,ipl)
     @                       - 0.01 / 0.25 * (zmin-zstereo(iside,ipl))

                 xp_ =  cosang * xp + sinang * yp - xp_sl
                 yp_ = -sinang * xp + cosang * yp - yp_sl
                 zp = zmin      - zp_sl

                 if (fltmod(imod).eq.1) then
                    ir_g10 = 0          ! div. G10 are along lab. x-axis
                 else if (fltmod(imod).eq.2) then
                    ir_g10 = ir         ! div. G10 are along rot. x-axis
                 else
                    call meerro('Illegal fltmod')
                 end if

                 call gsposp(matg10(imflag+1),nnwal()+1,slname(:4),
     @                 xp_,yp_,zp,ir_g10,'ONLY',par,3)
                 call crgwal(matg10(imflag+1),nnwal()+1)

c Put wall for last NEDE bank
                 iq(iqnede_nwl(inede1)) =
     @                              nnwal() - iq(iqnede_fwl(inede1)) + 1
                 iq(iqnesl_nwl(isl)) = nnwal() - iq(iqnesl_fwl(isl)) + 1


              end if
           end do                       ! vertical divisions

c------------------------------------- Install e-beam G10-strips
           do idiv = 1,2                ! vertical divisions
              par(2) = 0.5 * dyg10
              par(3) = 0.5 * 0.01

              if (otbeam(il,idiv,imod) .ne. 0.) then

                 if ((yrot + otbeam(il,idiv,imod)) .gt. 0.) then
                    iy = 1
                 else
                    iy = 2
                 end if

                 if (ttcefi(il,iy,imod).eq.0.) then
                    c78 = 'For '//modtype(:lenocc(modtype))
                    call ch78ac(': ttcefi = 0. for il=')
                    call ch78ai(il)
                    call ch78ac(', iy=')
                    call ch78ai(iy)
                    call me78wa
                 end if

                 xmin = xrot + ttcefi(il,iy,imod) - pit/3.
                 xmax = xrot + ttcela(il,iy,imod) + pit/3.

                 par(1) = 0.5 * (xmax-xmin)

                 xp = 0.5 * (xmax + xmin)
                 yp = yrot + otbeam(il,idiv,imod)

                 zmin = z + 0.5*il*otdiam(imod)
                 if (zstereo(iside,ipl).ne.0.) zmin = zstereo(iside,ipl)
     @                       - 0.01 / 0.25 * (zmin-zstereo(iside,ipl))
                 xp_ =  cosang * xp + sinang * yp - xp_sl
                 yp_ = -sinang * xp + cosang * yp - yp_sl
                 zp = zmin      - zp_sl

                 ir_g10 = ir

                 call gsposp(matg10(imflag+1),nnwal()+1,slname(:4),
     @                 xp_,yp_,zp,ir_g10,'ONLY',par,3)
                 call crgwal(matg10(imflag+1),nnwal()+1)
c--------

c Put wall for last NEDE bank
                 iq(iqnede_nwl(inede1)) =
     @                              nnwal() - iq(iqnede_fwl(inede1)) + 1
                 iq(iqnesl_nwl(isl)) = nnwal() - iq(iqnesl_fwl(isl)) + 1


              end if
           end do                       ! vertical divisions

c------------------------------------- Install the supporting G10-strips
           do idiv = 1,6                ! vertical divisions
              par(2) = 0.5 * 0.5   ! width of support strip = 5 mm
              par(3) = 0.5 * 0.01

              if (ttsupp(il,idiv,imod) .ne. 0.) then

                 if ((yrot + ttsupp(il,idiv,imod)) .gt. 0.) then
                    iy = 1
                 else
                    iy = 2
                 end if

c##
                 if (ttcefi(il,iy,imod).eq.0.) then
                    c78 = 'For '//modtype(:lenocc(modtype))
                    call ch78ac(': ttcefi = 0. for il=')
                    call ch78ai(il)
                    call ch78ac(', iy=')
                    call ch78ai(iy)
                    call me78wa
                 end if

                 xmin = xrot + ttcefi(il,iy,imod) - pit/3.
                 xmax = xrot + ttcela(il,iy,imod) + pit/3.

                 par(1) = 0.5 * (xmax-xmin)

                 xp = 0.5 * (xmax + xmin)
                 yp = yrot + ttsupp(il,idiv,imod)

                 zmin = z + 0.5*il*otdiam(imod)
                 if (zstereo(iside,ipl).ne.0.) zmin = zstereo(iside,ipl)
     @                       - 0.01 / 0.25 * (zmin-zstereo(iside,ipl))
                 xp_ =  cosang * xp + sinang * yp - xp_sl
                 yp_ = -sinang * xp + cosang * yp - yp_sl
                 zp = zmin      - zp_sl

                 call gsposp(matg10(imflag+1),nnwal()+1,slname(:4),
     @                 xp_,yp_,zp,ir,'ONLY',par,3)
                 call crgwal(matg10(imflag+1),nnwal()+1)

c Put wall for last NEDE bank
                 iq(iqnede_nwl(inede1)) =
     @                              nnwal() - iq(iqnede_fwl(inede1)) + 1
                 iq(iqnesl_nwl(isl)) = nnwal() - iq(iqnesl_fwl(isl)) + 1


              end if

           end do                       ! vertical divisions
        end do                          ! monolayers

c========================================= End of Geant volume declaration

c------------------------------------------------ Command was not found above.
        else
           c78 = 'Unknown command: '//buf(:nlast)
           call meerro(c78)
        end if
        goto 100

99      call meerro
     @     ('Error reading parameters in command: '//buf(:nlast))


c================================================ end of reading loop

c------------------- Here at end of file: Perform final checks, etc.

991     continue
        call ioclos(lread)
992     continue

c------------------------ Create walls (and assign them to the last NEDE bank)
        call otwall(covthick)

        iq(iqnede_nwl(inede1)) = nnwal() - iq(iqnede_fwl(inede1)) + 1
        iq(iqnesl_nwl(isl)) = nnwal() - iq(iqnesl_fwl(isl)) + 1

c Comparison of coordinates
        if (xmin_sl.gt.xmin_all) then
           c78 = 'Modules do not fit into declared SL volume:'
           call ch78ac(' xmin_sl = ')
           call ch78af(xmin_sl,-4)
           call ch78ac(' xmin_all = ')
           call ch78af(xmin_all,-4)
           call me78wa
        end if
        if (xmax_sl.lt.xmax_all) then
           c78 = 'Modules do not fit into declared SL volume:'
           call ch78ac(' xmax_sl = ')
           call ch78af(xmax_sl,-4)
           call ch78ac(' xmax_all = ')
           call ch78af(xmax_all,-4)
           call me78wa
        end if

        if (ymin_sl.gt.ymin_all) then
           c78 = 'Modules do not fit into declared SL volume:'
           call ch78ac(' ymin_sl = ')
           call ch78af(ymin_sl,-4)
           call ch78ac(' ymin_all = ')
           call ch78af(ymin_all,-4)
           call me78wa
        end if
        if (ymax_sl.lt.ymax_all) then
           c78 = 'Modules do not fit into declared SL volume:'
           call ch78ac(' ymax_sl = ')
           call ch78af(ymax_sl,-4)
           call ch78ac(' ymax_all = ')
           call ch78af(ymax_all,-4)
           call me78wa
        end if

        if (zmin_sl.gt.zmin_all) then
           c78 = 'Modules do not fit into declared SL volume:'
           call ch78ac(' zmin_sl = ')
           call ch78af(zmin_sl,-4)
           call ch78ac(' xmin_all = ')
           call ch78af(zmin_all,-4)
           call me78wa
        end if
        if (zmax_sl.lt.zmax_all) then
           c78 = 'Modules do not fit into declared SL volume:'
           call ch78ac(' zmax_sl = ')
           call ch78af(zmax_sl,-4)
           call ch78ac(' zmax_all = ')
           call ch78af(zmax_all,-4)
           call me78wa
        end if

        iq(iqnesl_lay(isl)) = ipl ! number of layers

c reverse digits in a decimal number
        ltyp_new = 0
        ltyp_old = inesl_ltyp(isl)
        do i=1,ipl
           l = mod(ltyp_old,10)
           ltyp_old = ltyp_old / 10
           ltyp_new = 10*ltyp_new + l
        end do
        iq(iqnesl_ltyp(isl)) = ltyp_new


        return

        end
