/*
************************************************************************
*
* PIPGEO
* written by Siegmund Nowak
*
* This routines defines geometry of the beam pipe outside the si-region
* and up to the muon system
*
*       called from G..MAIN
*
************************************************************************
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "gean/ZGMOTH.h"
#include "gean/ZGESL.h"
#include "gean/zgvers.hh"

  typedef struct {
  char nam[16];
  float ppos, pri, pro, plen, pdens;
    } Commonpipbuf;


  extern "C" {
    void pipgeo_();
    void gsvolu_(char*, char*, int*, float [], int*, int*);
    void gsatt_(char*, char*, int*);
    void crpos_(char*,char*,int*,int*,float*,float*,float*,int*,char*,float[]);
    void gsposp_(char*,int*,char*,float*,float*,float*,int*,char*,float[],int*);
    int mednum_(int &,char *,int);
    void crmixtc_(char*,int*,int*,int*,char*,int*,float[]);
    int  pipdat_(char*);
    extern Commonpipbuf pipbuf_;
  }

void pipgeo_() {

    Commonpipbuf& buf=pipbuf_;
    int ivol, imed, nwal, len; 
    int zero=0, one=1, two=2, three=3, twelve=12, twenty=20, fifty=50; 
    char shap[4], colo[4],  only[4], mot[4], vol[5],  mat[12];
    float par[3], vol_mix[2], zend, zp, dl, drest; 
    float fzero=0.;  float zstart=0.;
    zgvers ZGVERS;
    static char nam_mix[24];

//  select file name 
    ArtePointer<NESL> sl(zgmoth_.CURRSL);   
    int cmp = sl->cmp;
    char filnam[80];
    strcpy(filnam,ZGVERS.Get_vers_datafile(cmp));         

/*     standard materials */
    strncpy(shap,"TUBE",4); 
    strncpy(colo,"COLO",4);
    strncpy(only,"ONLY",4);

    len = 9;
    strncpy(mat,"ALUMINIUM",len);
    imed = mednum_(one, mat, len);
    strncpy(vol,"PPPN",4);
    gsvolu_(vol, shap, &imed, par, &zero, &ivol);
    gsatt_(vol, colo, &fifty);
    imed = mednum_(two, mat, len);
    strncpy(vol,"PPPM",4);
    gsvolu_(vol, shap, &imed, par, &zero, &ivol);
    gsatt_(vol, colo, &fifty);
    imed = mednum_(one, mat, len);
    strncpy(vol,"PPFN",4);
    gsvolu_(vol, shap, &imed, par, &zero, &ivol);
    gsatt_(vol, colo, &fifty);
    imed = mednum_(two, mat, len);
    strncpy(vol,"PPFM",4);
    gsvolu_(vol, shap, &imed, par, &zero, &ivol);
    gsatt_(vol, colo, &fifty);
    len=6;
    strncpy(mat,"VACUUM",len);
    imed = mednum_(two, mat, len);
    strncpy(vol,"PPVM",4);
    gsvolu_(vol, shap, &imed, par, &zero, &ivol);
    imed = mednum_(one, mat, len);
    strncpy(vol,"PPVN",4);
    gsvolu_(vol, shap, &imed, par, &zero, &ivol);

    len=4; 
    strncpy(mat,"IRON",4);
    imed = mednum_(one, mat, len);
    strncpy(vol,"PPPF",4);
    gsvolu_(vol, shap, &imed, par, &zero, &ivol);
    gsatt_(vol, colo, &twenty);
    imed = mednum_(one, mat, len);
    strncpy(vol,"PPFF",4);
    gsvolu_(vol, shap, &imed, par, &zero, &ivol);
    gsatt_(vol, colo, &twenty);

/*  according to Zimmer-Nixdorf the 5 mm Schweissnaht is equivalent to 
    3 mm pure aluminium */
    strncpy(nam_mix,"ALUMINIUM   AIR         ",24);
    vol_mix[0] = 0.6;
    vol_mix[1] = 0.4;
    len = 10;
    strncpy(mat,"ALUSCHWEIN",len);
    crmixtc_(mat,&len,&three, &two, nam_mix,&twelve, vol_mix);
    imed = mednum_(one, mat, len);
    strncpy(vol,"PPWN",4);
    gsvolu_(vol, shap, &imed, par, &zero, &ivol);
    gsatt_(vol, colo, &fifty);
    imed = mednum_(two, mat, len);
    strncpy(vol,"PPWM",4);
    gsvolu_(vol, shap, &imed, par, &zero, &ivol);
    gsatt_(vol, colo, &fifty);

//  define some often used quantities
    ArtePointer<NESL> magsl(zgmoth_.MAGNSL);
    float magz1 = magsl->z1;
    float magz2 = magsl->z2;

//  posit volumes within mother */
    static int i=0, ii=0, k=0; 

L100:
    int fi = pipdat_(filnam);
    if (fi == 1) {
	++i;
	if (buf.plen <= 0.01) {
            --i;
	    goto L100;	}     }
     else if (fi == 2) {
	++k;
	if (buf.plen <= .01) {
	    --k;
	    goto L100;  }     }
     else if (fi == 3) {      }
// do nothing                  
     else if (fi == 0) {
	goto L100;            } 
     else  goto L110;

    dl = buf.plen;
    drest = 0.;
    zstart = buf.ppos - buf.plen * 0.5;
    zend = buf.ppos + buf.plen * 0.5;

// divide a pipe which crosses boundary between mother and magnet
    if (zstart < magz2 && zend > magz2) {
    if(magz2 -zstart < dl) dl=magz2 -zstart; 
    drest = buf.plen - dl;
    zend = zstart + dl;              }
 
L102:
    if (dl < 0.01) goto L108;
 
//full in magnet
    if(zstart >= magz1 && zend <= magz2) {
      strncpy(mot,"MAGN",4);
       if(fi == 1) {
         if (buf.pdens >= 2.65) strncpy(vol, "PPPM", 4);
         else {
   	   vol_mix[0] = buf.pdens / 2.66;
	   vol_mix[1] = 1. - vol_mix[0];
	   ++ii;
           sprintf(mat, "ALUPIP%02d",ii);
           sprintf(vol, "CP%02d",ii);

         len=8;
	 crmixtc_(mat,&len, &two, &two, nam_mix, &twelve, vol_mix);
	 imed = mednum_(two, mat, len);
	 gsvolu_(vol,shap, &imed, par, &zero, &ivol);
           gsatt_(vol, colo, &fifty);                      }

// beam tube 
	 par[0] = buf.pri;
	 par[1] = buf.pro;
	 par[2] = dl * 0.5;
	 zp = zstart + par[2] - zgmoth_.ZMAGM;
	 crpos_(vol,mot,&i,&zero,&fzero,&fzero,&zp,&zero,only, par);
// vacuum in the beam tube 
	 par[0] = 0.;
	 par[1] = buf.pri;
	 nwal = ArtePointer<NWAL>::nrow();
         strncpy(vol,"PPVM",4);
	 gsposp_(vol,&nwal,mot,&fzero,&fzero,&zp,&zero,only,par,&three);
	} 

       else if (fi == 2) {
// flange 
	 par[0] = buf.pri;
	 par[1] = buf.pro;
	 par[2] = dl * 0.5;
	 zp = zstart + par[2] - zgmoth_.ZMAGM;
	 if (strncmp(buf.nam, "ST", 2) == 0){ 
            strncpy(vol,"PPFM",4);
            crpos_(vol,mot,&k,&i,&fzero,&fzero,&zp,&zero,only,par); }
	 else {
            strncpy(vol,"PPWM",4);
	    crpos_(vol,mot,&k,&i,&fzero,&fzero,&zp,&zero,only,par);
// vaccum under welting connection 
            par[0] = 0.;
	    par[1] = buf.pri;
	    nwal = ArtePointer<NWAL>::nrow();
            strncpy(vol,"PPVM",4);
	    gsposp_(vol,&nwal,mot,&fzero,&fzero,&zp,&zero,only,par,&three);
              }
        }

       else if (fi == 3) {
// support 
	 par[0] = buf.pri;
	 par[1] = buf.pro;
	 par[2] = dl * 0.5;
	 zp = zstart + par[2] - zgmoth_.ZMAGM;
	 if (buf.pdens >= 2.65 && buf.pdens <= 2.7) {
           strncpy(vol,"PPFM",4);
            crpos_(vol,mot,&zero,&zero,&fzero,&fzero,&zp,&zero,only,par);}
	 else {
        arteMsg << "other support material than AL in magnet not foreseen";
        arteMsg.warn();      }
     }}


/* ---  out of magnet */
    else if (zstart >= magz2) {
       strncpy(mot, "MOTH", 4);
       if (fi == 1) {
	  if (buf.pdens >= 2.65 && buf.pdens <= 2.7) strncpy(vol, "PPPN", 4);
	  else if(buf.pdens>=7.85 && buf.pdens<=7.9) strncpy(vol, "PPPF", 4);
	  else {
	    vol_mix[0] = buf.pdens / 2.66;      vol_mix[1] = 1. - vol_mix[0];
	    ++ii;
            sprintf(mat, "ALUPIP%02d",ii);
            sprintf(vol, "CP%02d",ii);
	    crmixtc_(mat,&len, &one, &two, nam_mix,&twelve, vol_mix);
	    imed = mednum_(one, mat, len);
	    gsvolu_(vol, shap, &imed, par, &zero, &ivol);
            gsatt_(vol, colo, &fifty);                       }
// beam tube 
	  par[0] = buf.pri;
	  par[1] = buf.pro;
	  par[2] = dl * 0.5;
	  zp = zstart + par[2] + zgmoth_.ZZERO;
	  crpos_(vol, mot,&i,&zero,&fzero,&fzero,&zp,&zero,only, par);
// vacuum in the beam tube
          par[0] = 0.;
	  par[1] = buf.pri;
	  nwal = ArtePointer<NWAL>::nrow();
          strncpy(vol,"PPVN",4);
	  gsposp_(vol,&nwal,mot,&fzero,&fzero,&zp,&zero,only,par,&three);
       }

        else if (fi == 2) {
//flange 
	  par[0] = buf.pri;
	  par[1] = buf.pro;
	  par[2] = dl * 0.5;
	  zp = zstart + par[2] + zgmoth_.ZZERO;
	  if (strncmp(buf.nam, "ST", 2) == 0) {
             strncpy(vol,"PPFN",4); 
	     crpos_(vol,mot,&k, &i,&fzero,&fzero,&zp,&zero,only,par); }
	  else  {
             strncpy(vol,"PPWN",4);
             crpos_(vol,mot,&k,&i,&fzero,&fzero,&zp,&zero,only,par); }
// vaccum under flange 
	     par[0] = 0.;
	     par[1] = buf.pri;
	     nwal = ArtePointer<NWAL>::nrow();
             strncpy(vol,"PPVN",4); 
	     gsposp_(vol,&nwal,mot,&fzero,&fzero,&zp,&zero,only,par,&three);
             }

        else if (fi == 3) {
// support 
	  par[0] = buf.pri;
	  par[1] = buf.pro;
	  par[2] = dl * 0.5;
	  zp = zstart + par[2] + zgmoth_.ZZERO;
	  if (buf.pdens >= 2.65&& buf.pdens <= 2.67) {
             strncpy(vol,"PPFN",4); 
             crpos_(vol,mot,&zero,&zero,&fzero,&fzero,&zp,&zero,only,par); }
	  else if (buf.pdens >= 7.85 && buf.pdens <= 7.9) {
             strncpy(vol,"PPFF",4); 
             crpos_(vol,mot,&zero,&zero,&fzero,&fzero,&zp,&zero,only,par); }
          }} 
         
L108:
    zstart += dl;
    if (drest >= .01) {
	dl = drest; 
	zend = zstart + dl;
	drest = 0.;
	goto L102;
      }
    goto L100;
  

/* --   Tube up to the muon system */
L110:
    ArtePointer<NESL> muonsl(zgmoth_.MUONSL);
    float muonz1 = muonsl->z1;
    dl = muonz1 - zstart;
    zend = zstart + dl;
    if (dl > (float).01)    {
        arteMsg << "beampipe is missing between: " << zstart << muonz1 ; 
        arteMsg.warn();      }
}  
 
 /* pipgeo_ */






