*CMZ :  1.01/01 02/01/96  16.55.08  by  Fedor Ratnikov
*CMZ :  1.00/01 05/04/95  00.17.17  by  Fedor Ratnikov
*-- Author :    Dominik Ressing   25/08/94
************************************************************************
*
* L1TFUMX
* written by Dominik Ressing, 25/08/94 14.46.56
*
* This routines action:
* simulates the TFU matrix.
*
* Arguments  kind         description
*          (in,update,out)
*-----------------------------------------------------------------------
* iHisZ       in        String of Hits for axial wires or upper row PADs
* iHisP       in        String of Hits for +alpha wires or lower row PADs
* iHisN       in        String of Hits for -alpha wires or middle row PADs
* iHeight     in        height of ROI in matrix coordinates (1..L1DEFtfuMxHei)
* kFlag       in        MatrixFlag: 1,2 wires 3: PADs
* Npoi        in/out    Number of found points
* iX,iY       out       Coordinates of found points (iX=0.., iY=0... )
* isX,isY     out       Width of cluster (found point)
* iHiPosZ     out       Hit pointer for the found hits 0 layer
* iHiPosP     out       Hit pointer for the found hits + alpha layer
* iHiPosN     out       Hit pointer for the found hits - alpha layer
*
************************************************************************
c
      SUBROUTINE L1TFUMX(iHisZ,iHisP,iHisN,iHeight,kFlag,
     &                   Npoi,iX,iY,isX,isY,iHiPosZ,iHiPosP,iHiPosN)
c
      IMPLICIT NONE
c
#include "l1si/l1param.fi"
#include "l1si/l1geodf.fi"
#include "l1si/l1detdf.fi"
c
      INTEGER iHisZ(L1HITlen,*),iHisP(L1HITlen,*),iHisN(L1HITlen,*),
     &        iHeight,kFlag,Npoi,iX(*),iY(*),isX(*),isY(*),
     &        iHiPosZ(L1HITlen,*),iHiPosP(L1HITlen,*),
     &        iHiPosN(L1HITlen,*)
      INTEGER iP,iZ,iN,iXh,iYh,iNrange,iHeiPlu,iWiMiHe,iWiLaCo,
     &        iPfi,iPla,iNla,iNfi,iZla,iZfi,dZ,iAdd,iAddHa,MaxPo,
     &        iMaxZer,i,ii,k,N,irow,iJump,
     &        iHeiMin,iXf,iYf,minf,maxf
      LOGICAL found,WireLike
c
      N = 0
      iJump = 0
c
      IF ( kFlag.eq.2 ) THEN  ! first axial wire does not cover reference cell
         iAdd = 2
         iAddHa = 1
         WireLike = .TRUE.
      ELSE IF ( kFlag.eq.1 ) THEN  ! first axial wire covers reference cell
         iAdd = 1
         iAddHa = 0
         WireLike = .TRUE.
      ELSE IF ( kFlag.eq.3 ) THEN  ! PAD matrix
         WireLike = .FALSE.
         GO TO 555
      ELSEIF (kFlag.LT.0)    THEN
         WireLike = .FALSE.
         iJump = -kFlag
      ELSE
         WRITE (L1LUNprn,*) 'L1TFUMX==> Invalid Flag ',kFlag
         RETURN
      END IF
c
c               WIRE matrix part
c
      iNrange = 2*iHeight
      iHeiPlu = iHeight+1
      iHeiMin = iHeight-1
      iWiMiHe = L1DEFtfuMxWid-iHeight+1
      iWiLaCo = L1DEFtfuMxWid+iWiMiHe
      iMaxZer = iWiMiHe
c     looking for hit in 3 orientations
c
      DO iP = 1, L1DEFtfuMxWid
         IF ( iHisP(L1HITnpo,iP).ne.0 ) THEN    ! +alpha hit set ??
            iNla = MIN(iP + iHeight,L1DEFtfuMxWid)     ! calculate valid -alpha range
            iNfi = MAX(1,iP - iHeight)
c
            DO iN = iNfi, iNla
               IF ( iHisN(L1HITnpo,iN).ne.0 ) THEN   ! -alpha hit set ??
c                                                    ! calc. coordinates of cross
                  iXh =  iN+iP-2                     ! x-count of half cells in matrix
                  iYh =  iN-iP                       ! y-count of half cells in matrix
                  iZla = 1+(iXh+iAdd-iHeiMin)/2      ! last zero cell for cross
                  iZfi = MAX(1,iZla-1)               ! first zero cell for cross
                  iZla = MIN(iZla,iMaxZer)           ! limit to valid
                                                     ! range
                  IF(iJump.NE.0)   THEN
c                    handle both kFlags:
                     iZla = 2+(iXh-iHeiMin)/2
c                    consider different x-steps for 0 and +- wires:
                     iZla = iZla+(iZla+iHeight/2)/iJump
                     iZfi = MAX(1,iZla-2) ! first zero cell for cross
                     iZla = MIN(iZla,iMaxZer) ! limit to valid
                  ENDIF
c
                  DO iZ = iZfi,iZla
                     IF ( iHisZ(L1HITnpo,iZ).ne.0 ) THEN    ! zero hit set ??
                        iXf  = iZ ! x from zero projection
                        iYf  = iYh
c==================================================================================
c
c    ------------ Do clusterisation ---------------
c
      DO I=1,N                  ! do clustering
         IF ( iXf.ge.(iX(i)-1) .and.
     &        iXf.le.(iX(i)+isX(i)).and.
     &        (iYf+2).ge.iY(i).and.
     &        iYf.le.(iY(i)+isY(i))) THEN
            minf = MIN(iXf,iX(i))
            maxf = MAX(iXf+1,iX(i)+isX(i))
            iX(i)  = minf
            isX(i) = maxf-minf
            minf = MAX(-iHeight+1,MIN(iYf,iY(i)))
            maxf = MIN(iHeight+1,
     &           MAX(iYf+2,iY(i)+isY(i)))
            isY(i) = maxf-minf
            iY(i)  = minf
            CALL l1hitsu(iHisZ(1,iZ),iHiPosZ(1,i)) !copy zero pointers
            CALL l1hitsu(iHisP(1,iP),iHiPosP(1,i)) !copy + pointers
            CALL l1hitsu(iHisN(1,iN),iHiPosN(1,i)) !copy - pointers
            DO ii = i+1,N       ! does it glue another ! cluster?
               IF(iX(i).LE.iX(ii)+isX(ii).AND.
     &              iX(i)+isX(i).GE.iX(ii).AND.
     &              iY(i).LE.iY(ii)+isY(ii).AND.
     &              iY(i)+isY(i).GE.iY(ii))   THEN
                  minf = MIN(iX(i),iX(ii))
                  maxf = MAX(iX(i)+isX(i),
     &                 iX(ii)+isX(ii))
                  iX(i) = minf
                  isX(i) = maxf-minf
                  minf = MIN(iY(i),iY(ii))
                  maxf = MAX(iY(i)+isY(i),
     &                 iY(ii)+isY(ii))
                  iY(i) = minf
                  isY(i) = maxf-minf
                  CALL l1hitsu(iHiPosZ(1,ii),iHiPosZ(1,i)) !copy zero
                                                           !pointers
                  CALL l1hitsu(iHiPosP(1,ii),iHiPosP(1,i)) !copy + pointers
                  CALL l1hitsu(iHiPosN(1,ii),iHiPosN(1,i)) !copy -
                                                              !pointers
                  IF(ii.LT.N)   THEN ! remove glued cluster
                     iX(ii) = iX(N) ! move last on place of removed
                     isX(ii) = isX(N)
                     iY(ii) = iY(N)
                     isY(ii) = isY(N)
                     iHiPosZ(1,ii) = iHiPosZ(1,N)
                     iHiPosP(1,ii) = iHiPosP(1,N)
                     iHiPosN(1,ii) = iHiPosN(1,N)
                  ENDIF
                  N = N-1
               ENDIF
            ENDDO
            GOTO 100    ! new point already included
         ENDIF
      ENDDO                     ! form new cluster
      IF(N.GE.Npoi)   THEN      ! Exceeds point memory
         PRINT
     &        '(''L1TFUMX==> Error: points memory limit'',2I3)',
     &        N,Npoi
         GOTO 777
      ENDIF
      N = N + 1                 ! store new cluster
      iX(N)  = iXf
      isX(N) = 1                ! cluster width = 1
      iY(N)  = MAX(-iHeight+1,iYf) ! strip on ROI
      isY(N) = MIN(iHeight+1,iYf+2)-iY(N)
      iHiPosZ(L1HITpos,N) = iHisZ(L1HITpos,iZ) ! copy references
      iHiPosP(L1HITpos,N) = iHisP(L1HITpos,iP)
      iHiPosN(L1HITpos,N) = iHisN(L1HITpos,iN)
 100  CONTINUE
c==================================================================================
                     ENDIF
                  END DO
               END IF
            END DO
         END IF
      END DO
c
      GOTO 777   ! RETURN
c
 555  CONTINUE   ! here is the PAD part
c
      DO irow=1,iHeight
         DO iXf = 1, L1DEFtfuMxWid
            IF ((irow.eq.1.AND.iHisZ(L1HITnpo,iXf).NE.0).OR.
     &           (irow.eq.2.AND.iHisN(L1HITnpo,iXf).NE.0).OR.
     &           (irow.eq.3.AND.iHisP(L1HITnpo,iXf).NE.0))  THEN
               iYf = 2*(irow-1)
c==================================================================================
c
c    ------------ Do clusterisation ---------------
c
      DO I=1,N                  ! do clustering
         IF ( iXf.ge.(iX(i)-1) .and.
     &        iXf.le.(iX(i)+isX(i)).and.
     &        (iYf+2).ge.iY(i).and.
     &        iYf.le.(iY(i)+isY(i))) THEN
            minf = MIN(iXf,iX(i))
            maxf = MAX(iXf+1,iX(i)+isX(i))
            iX(i)  = minf
            isX(i) = maxf-minf
            minf = MAX(-iHeight+1,MIN(iYf,iY(i)))
            maxf = MIN(iHeight+1,
     &           MAX(iYf+2,iY(i)+isY(i)))
            isY(i) = maxf-minf
            iY(i)  = minf
            IF(irow.EQ.1)   THEN
               CALL l1hitsu(iHisZ(1,iXf),iHiPosZ(1,i)) !copy pointers
            ELSEIF(irow.EQ.2)   THEN
               CALL l1hitsu(iHisN(1,iXf),iHiPosZ(1,i)) !copy pointers
            ELSEIF(irow.EQ.3)   THEN
               CALL l1hitsu(iHisP(1,iXf),iHiPosZ(1,i)) !copy pointers
            ENDIF
            DO ii = i+1,N       ! does it glue another ! cluster?
               IF(iX(i).LE.iX(ii)+isX(ii).AND.
     &              iX(i)+isX(i).GE.iX(ii).AND.
     &              iY(i).LE.iY(ii)+isY(ii).AND.
     &              iY(i)+isY(i).GE.iY(ii))   THEN
                  minf = MIN(iX(i),iX(ii))
                  maxf = MAX(iX(i)+isX(i),
     &                 iX(ii)+isX(ii))
                  iX(i) = minf
                  isX(i) = maxf-minf
                  minf = MIN(iY(i),iY(ii))
                  maxf = MAX(iY(i)+isY(i),
     &                 iY(ii)+isY(ii))
                  iY(i) = minf
                  isY(i) = maxf-minf
                  CALL l1hitsu(iHiPosZ(1,ii),iHiPosZ(1,i)) !copy zero
                                                           !pointers
                  IF(ii.LT.N)   THEN ! remove glued cluster
                     iX(ii) = iX(N) ! move last on place of removed
                     isX(ii) = isX(N)
                     iY(ii) = iY(N)
                     isY(ii) = isY(N)
                     iHiPosZ(1,ii) = iHiPosZ(1,N)
                  ENDIF
                  N = N-1
               ENDIF
            ENDDO
            GOTO 200    ! new point already included
         ENDIF
      ENDDO                     ! form new cluster
      IF(N.GE.Npoi)   THEN      ! Exceeds point memory
         PRINT
     &        '(''L1TFUMX==> Error: points memory limit'',2I3)',
     &        N,Npoi
         GOTO 777
      ENDIF
      N = N + 1                 ! store new cluster
      iX(N)  = iXf
      isX(N) = 1                ! cluster width = 1
      iY(N)  = iYf
      isY(N) = 2                ! height in half-cell units
      IF(irow.EQ.1)   THEN
         iHiPosZ(L1HITpos,N) = iHisZ(L1HITpos,iXf) ! copy references
      ELSEIF(irow.EQ.2)   THEN
         iHiPosZ(L1HITpos,N) = iHisP(L1HITpos,iXf) ! copy references
      ELSEIF(irow.EQ.3)   THEN
         iHiPosZ(L1HITpos,N) = iHisN(L1HITpos,iXf) ! copy references
      ENDIF
 200  CONTINUE
c==================================================================================
            ENDIF
         ENDDO
      ENDDO
c
 777  CONTINUE
      Npoi = N
      RETURN
      END





