// F.Ratnikov: rearrange MU GEDE chambers in increasing X order

#include "arte/Arte.hh"
#include "auto/NEDE.hh"
#include "auto/NESL.hh"
#include <set.h>
#include <stdio.h>

struct myGEDE {
  NEDE gede;
  NEDE gede2;
  int oldHb;
  int oldHb2;
  int ivo1;
  int ivo2;
  int in1;
  int in2;
};

struct GEDEless {
  bool operator() (const myGEDE &gede1, const myGEDE &gede2) const {
    double dx = gede1.gede.xmin - gede2.gede.xmin;
    if (dx > 0.1 || dx < -0.1)   
      return (bool) (dx < 0.);  // plain case
    else   // short chamber
      return (bool) (gede1.gede.ymin < gede2.gede.ymin); // arrange in any way
  } 
};

extern "C" {
  void muarrange_ ();
  void masetgeant_ (int* ivo, int* in, int* newId);
  void magetvolume_ (char* name, int* oldId, int* ivo, int* in);
} 

void muarrange_ ()
{
  const char* slName [4] = {"MUD1", "MUD2", "MUD3", "MUD4"};
  
  int isl = 4;
  while (--isl >= 0)  // loop over SL
    {
      ArteTableIterator<NESL> sl;
      for(; sl.valid (); sl.next ())
	{
	  if (!strncmp(sl->name,slName [isl], 4))
	    {
	      // found. Exit this loop
	      break;
	    }
	}
      if (!sl.valid ())
	{
	  cerr << "muarrange: can not find SL " << slName [isl] << endl;
	  return;
	}
      int hbFirst = sl->fpl;
      int hbLast = sl->fpl + sl->npl;
      myGEDE* GedeTmp = new myGEDE [sl->npl];
      int iGedeTmp = 0;
      int iStereo = -1;
      for (; iStereo < 2; iStereo++)
	{
	  int up = 0;
	  for (; up < 2; up++)
	    {
	      set <myGEDE, GEDEless> gedeSet;
	      int hb = hbFirst;
	      for (; hb < hbLast; hb++)
		{
		  ArteTableIterator<NEDE> gede (hb);
		  double x = 0.5 * (gede->xmax + gede->xmin);
		  double y = 0.5 * (gede->ymax + gede->ymin);
		  y = y * gede->rot [0] - x * gede->rot [1];

		  if ((up && y > 0. || !up && y < 0) &&
		      (iStereo < 0 && gede->rot [1] < 0. ||
		       iStereo == 0 && gede->rot [1] == 0. ||
		       iStereo > 0 && gede->rot [1] > 0.))
		    {
		      myGEDE newGede;
		      newGede.gede = *gede;
		      newGede.oldHb = hb;
		      magetvolume_ (gede->gvol, &hb, &newGede.ivo1, &newGede.in1);
		      gede.next ();
		      newGede.gede2 = *gede;
		      newGede.oldHb2 = ++hb;
		      magetvolume_ (gede->gvol, &hb, &newGede.ivo2, &newGede.in2);
		      gedeSet.insert (newGede);
		    }
		}
	      // fill new data
	      const myGEDE* pGedeShort = 0;
	      double oldX = -999.;
	      set <myGEDE, GEDEless> ::iterator pGede = gedeSet.begin ();
	      for (; pGede != gedeSet.end (); pGede++)
		{
		  double dx = (*pGede).gede.xmin - oldX;
		  if (dx < -1. || dx > 1.)
		    {
		      GedeTmp [iGedeTmp] = *pGede;
		      iGedeTmp++;
		      oldX = (*pGede).gede.xmin;
		    }
		  else   // short chamber: put into the end of the list
		    {
		     if (pGedeShort)
		       cerr << "muarrange: another short chamber found" << endl;
		     else
		       {
			 pGedeShort = &(*pGede);
		       }
		    }
		}
	      if (pGedeShort)
		{
		 GedeTmp [iGedeTmp] = *pGedeShort;
		 iGedeTmp++;
		}
	    }
	}
      if (2 * iGedeTmp != sl->npl)
	{
	  cerr << "muarrange: different number of modules " << slName [isl] << endl;
	  return;
	}
      int hb = 0;
      for (; hb < iGedeTmp; hb++)
	{
	  int idNew = hbFirst + 2 * hb;
	  myGEDE* newGede = GedeTmp + hb;
	  *(ArteTableIterator<NEDE> (idNew)) = newGede->gede;
	  masetgeant_ (&(newGede->ivo1), &(newGede->in1), &idNew);
	  idNew++;
	  *(ArteTableIterator<NEDE> (idNew)) = newGede->gede2;
	  masetgeant_ (&(newGede->ivo2), &(newGede->in2), &idNew);
	}

      delete [] GedeTmp;
    }

}






