#ifndef _MCFUNCTION_HH
#define _MCFUNCTION_HH_
//---------------------------------------------------------------------
// Created: 21.04.98 by   Olya Igonkina

// functions which check MTRA information used in RcSegClusBuilder class

//---------------------------------------------------------------------

#include <vector>

#include "arte/arte.hh"
#include "marple/RcPoint.hh"

///return 1 if at least one element in mtraList1 == any element in mtraList2
extern int RcMtraOk( const vector<ArtePointer<MTRA> > & mtraList1, 
		     const vector<ArtePointer<MTRA> > & mtraList2  );
///return 1 if mtraEntry == one of the elements in mtraList
extern int RcMtraOk( ArtePointer<MTRA>  & mtraEntry, 
		     const vector<ArtePointer<MTRA> > & mtraList );

//return 1 if RcPoint leaved by lepton from J/psi
extern int RcFromJpsi( const RcPoint &);

/// return 1 if MC particle can be used as a reference one for match
extern int RcMtraReference( const ArtePointer<MTRA> & mtra );

/// return 1 if at least one element in mtraList1 == any element in mtraLis and can be used as a reference one for match
extern int RcMtraReferenceOk( const vector<ArtePointer<MTRA> > & mtraList1, 
			      const vector<ArtePointer<MTRA> > & mtraList2 );

extern int RcMtraReferenceOk( ArtePointer<MTRA>  & mtraEntry, 
			      const vector<ArtePointer<MTRA> > & mtraList );
//------------ gamma----------------------------
extern int RcGamma(const ArtePointer<MTRA> mtra );
extern vector<ArtePointer<MTRA> > RcGamma( const vector<ArtePointer<MTRA> > 
					 mtraList );

//-----------bremstralung------------------------
extern ArtePointer<MTRA> RcOrigOfBrem( const ArtePointer<MTRA> mtra, 
				     float beforeZ = 200. );
// add all bremstralung origin to mtraList
extern vector<ArtePointer<MTRA> > RcMtraListWithBrem( const 
						    vector<ArtePointer<MTRA> > 
						    mtraList , 
						    float beforeZ = 200.);
// add only bremstralung origin to mtraList
extern vector<ArtePointer<MTRA> > RcMtraListOnlyBrem( const 
						    vector<ArtePointer<MTRA> > 
						    mtraList , 
						    float beforeZ = 200.);
//---------------elepton-------------------------
extern int RcElepton(const ArtePointer<MTRA> mtra );
extern vector<ArtePointer<MTRA> > RcElepton( const vector<ArtePointer<MTRA> > 
					    mtraList );

//---------------elepton-from-Jpsi-----------------------

extern int RcEleptonFromJpsi(const ArtePointer<MTRA> mtra );
extern vector<ArtePointer<MTRA> > RcEleptonFromJpsi( const 
						    vector<ArtePointer<MTRA> > 
						    mtraList );
//---------------electron-------------------------
extern int RcElectron(const ArtePointer<MTRA> mtra );
extern vector<ArtePointer<MTRA> > RcElectron( const vector<ArtePointer<MTRA> > 
					    mtraList );

//---------------electron-from-Jpsi-----------------------

extern int RcElectronFromJpsi(const ArtePointer<MTRA> mtra );
extern vector<ArtePointer<MTRA> > RcElectronFromJpsi( const 
						    vector<ArtePointer<MTRA> > 
						    mtraList );
//---------------positron-------------------------
extern int RcPositron(const ArtePointer<MTRA> mtra );
extern vector<ArtePointer<MTRA> > RcPositron( const vector<ArtePointer<MTRA> > 
					    mtraList );

//---------------positron-from-Jpsi-----------------------

extern int RcPositronFromJpsi(const ArtePointer<MTRA> mtra );
extern vector<ArtePointer<MTRA> > RcPositronFromJpsi( const 
						    vector<ArtePointer<MTRA> > 
						    mtraList );
//----------------J/psi----------------------------
extern int RcFromJpsi(const ArtePointer<MTRA> mtra );
extern int RcFromJpsi(const vector<ArtePointer<MTRA> > mtraList );

#endif
