#ifndef _RCMSOFGESL_HH_
#define _RCMSOFGESL_HH_
//**********************************************************************/
// Created: 23.08.97,   Olya Igonkina
//
// MARPLE class contains the average multiple scattering 
//              for each superlayer GESL
//
//**********************************************************************/

#include <iostream.h>
//#include <CLHEP/config/CLHEP.h>
#include <CLHEP/Vector/ThreeVector.h>


#include "arte/arte.hh"

class RcMsOfGesl
{
public:
  inline RcMsOfGesl(); //constructors
  
  // if default, the value is taken from GESL table
  RcMsOfGesl( const ArtePointer <GESL> gesl, 
	      HepDouble ms = 0.,
	      HepDouble zcenter = 0.,
	      int insideMagnet = -1); 

  inline RcMsOfGesl( const RcMsOfGesl &rms);

  inline ~RcMsOfGesl();  //destructor

  inline void       setgesl( const   ArtePointer <GESL> gesl);
  inline void       setms(   const   HepDouble  ms) ;
  inline void       setzcenter(const HepDouble zcenter);
  inline void       setMagnet(int iFlag = 0);

  inline ArtePointer<GESL>  gesl() const;  // returns the poiter to GESL table
  inline HepDouble  ms() const;     // returns the average multiple scattering
  inline HepDouble  zcenter() const;// returns the middle of the superlayer
  inline int        insideMagnet() const;
                                   // returns 0 if superlayer outside of magnet

  int check( Hep3Vector &rp ) const; // returns 0 if rp outside GESL

  inline RcMsOfGesl & operator = (const RcMsOfGesl &rms); //copy operator

private:

  ArtePointer<GESL>  fgesl;
  HepDouble          fms;
  HepDouble          fz_center;
  int                finsideMagnet;
};

ostream & operator << (ostream & os, const RcMsOfGesl &rms);

#include "marple/RcMsOfGesl.icc"

#endif
