//--------------------------------------------------------------------
// Created: 02.09.97 by   Olya Igonkina

//   MARPLE matching of SVD segments and segments reconstructed 
//   in the Tracker area behind magnet(OT)
//   with estimation of momentum information 
//   by requirement of segment point to target

//--------------------------------------------------------------------
  

//--------------------------------------------------------------------

#include "marple/marple.hh"
#include "marple/RcMatchBuilder.hh"

#include "arte/ArteMessage.hh"
#include "arte/RSEG_cmp.hh"

//definition of constants for SVD-OT matching ( see \Ref{RcMatchConst})
// flag, z_plane, z_min, z_max, use_pair_cut, n_pair_left, n_pair_right, 
// use_delta_cut, delta_x, delta_y, delta_tx, delta_ty, delta_pe, chi2, 
RcMatchConst rcSvdOtConst = { 
11110,     160.,   30.,  800.,      1,          1,            1,
    1,      3.,      5.,      0.01,      .01,   1.e+10,   100.,  
};


extern "C"  
void FORTRAN_ROUTINE(matchsvdot)() 
{

  RcMatchBuilder< RcSegPoint, RcSegPoint> SvdOtMatch;

  //--------------------------------------------------------------------
  // put constants to the MatchBuilder
  //--------------------------------------------------------------------
  SvdOtMatch.setConst(rcSvdOtConst);

  //--------------------------------------------------------------------
  // create vector of SVD segments
  //--------------------------------------------------------------------
  
  int iErr;
  for( ArteTable<RSEG>::iterator irseg = ArteTable<RSEG>::begin();
        irseg != ArteTable<RSEG>::end(); ++irseg )
    {
      if( (*irseg)->fit >= 0 && (*irseg)->cmp & Rsegc::vxd && 
	  SvdOtMatch.ptToMatchConst()->geomAccept(HepDouble((*irseg)->ze)) )
	{ 
	  RcSegPoint * ptToPoint = new RcSegPoint( (*irseg) , 1 );
	  ptToPoint->prolongate( SvdOtMatch.ptToMatchConst()->z_plane,
				 iErr);
	  if( iErr )
	    {
	      delete ptToPoint;
	    }else{
	      SvdOtMatch.addPointLeft( *ptToPoint);
	    }
	  ptToPoint = 0;
	}
    }
  //--------------------------------------------------------------------
  // create vector of Outer Tracker segments 
  //--------------------------------------------------------------------
  
  for( ArteTable<RSEG>::iterator irseg = ArteTable<RSEG>::begin();
        irseg != ArteTable<RSEG>::end(); ++irseg )
    {
      if( (*irseg)->fit >= 0 && (*irseg)->cmp & Rsegc::patt && 
	  SvdOtMatch.ptToMatchConst()->geomAccept(HepDouble((*irseg)->zf)) )
	{
	  RcSegPoint * ptToPoint = new RcSegPoint( (*irseg), 0 );

	  ptToPoint->estimatePEfromTAR( 5, iErr);
	  if( iErr ){ delete ptToPoint; ptToPoint = 0; continue; }
	  ptToPoint->prolongate( SvdOtMatch.ptToMatchConst()->z_plane, iErr);
	  if( iErr ){ delete ptToPoint; ptToPoint = 0; continue; }

	  SvdOtMatch.addPointRight( *ptToPoint);
	  ptToPoint = 0;	      
	}
    }  
  
  //--------------------------------------------------------------------
  // create a list of matching points with chi2 
  //--------------------------------------------------------------------

  SvdOtMatch.doMatch();
  
  //--------------------------------------------------------------------
  // print result of matching
  //--------------------------------------------------------------------

  if( MARPLE_DEBUG >= 0)
    {
      static int firstSVDOT = 1;
      if( firstSVDOT )
	{
	  cout << *SvdOtMatch.ptToMatchConst();
	  firstSVDOT = 0;
	}
      SvdOtMatch.stat(MARPLE_DEBUG);
    }
      
  if( MARPLE_DEBUG >= 20)
    {  
      SvdOtMatch.fillDeltas(8213,8313);
      SvdOtMatch.fillMatch( 8113);
    }else if( MARPLE_DEBUG >= 10){
      SvdOtMatch.fillDeltas(8213);
    }
  //--------------------------------------------------------------------
  // save found relation to the table
  //--------------------------------------------------------------------
  if( MARPLE_MAKERSEG ) SvdOtMatch.saveRSEG(iErr);

  SvdOtMatch.saveRelation(iErr);
  if( iErr )
    { arteMsg << "MARPLE: some relations were not saved "; arteMsg.allp();}

  //    refit information
  //SvdOtMatch.refitLeftFound(       iErr);
  //SvdOtMatch.refitRightFound(      iErr);
  //    and update RSEG tables
  //SvdOtMatch.updateARTELeftFound(  iErr);
  //SvdOtMatch.updateARTERightFound( iErr);

  //--------------------------------------------------------------------
  // clean everything
  //--------------------------------------------------------------------
  SvdOtMatch.deleteAll();  
}

