//##
//      /**Author:        Boris Fominykh
//
//      Date:             March, 10,  1999
//
//      Structure:        MuDigiHitb
//
//      Description:
//      Structure MuDigiHitb and the set of related routines are
//      for the digi->hitb transformation of muon detector hits.*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
// #include <math.h>
#include <fstream.h>
#include <iostream.h>
#include <iomanip.h>
#include <stdlib.h>
#include "arte/ArteMessage.hh"
#include "arte/Arte.hh"
#include "auto/GESL.hh"
#include "auto/GEDE.hh"
#include "auto/DMUT.hh"
#include "auto/DMUP.hh"
#include "auto/DMUX.hh"

typedef unsigned int   LWord;
typedef unsigned short Word;

extern "C" {
void mursht_();
void muaddt_();
void muaddp_();
int MuDHini();
void MuDiHi();
int ArteIni();
int ConnectIni();
void TubeLoops();
void PadLoops();
void LoopsPW(Word,Word,Word);
int CheckBit(int,Word,Word);
float pad_occup(int,int);
float wire_occup(int,int,int);
}

struct MuDigiHitb {

    Word Ivalid;

// tube chambers
    const Word TubeGeoNx     = 30;
    const Word TubeGeoUpDown = 2;
    const Word TubeGeoLay    = 3;
    const Word TubeGeoSl     = 2;
    const Word TubeGeoSd     = 2;

  //## number of tube chambers
    Word NchambT[TubeGeoSl][TubeGeoLay][TubeGeoUpDown];
  //## number of wires in the tube chamber
    Word TubeGeoWire;
  //## gede
    LWord TubeGede[TubeGeoSl][TubeGeoLay][TubeGeoUpDown][TubeGeoNx][TubeGeoSd];
  //## chamber names
    char ChamListT[TubeGeoSl][TubeGeoLay][TubeGeoUpDown][TubeGeoNx][9];
  //## number of half-connectors per chamber
    Word NconnT[TubeGeoSl][TubeGeoLay][TubeGeoUpDown][TubeGeoNx];
    const Word Nconn      = 2;
  //## number of connected wires by half-connector
    const Word  NumbT = 8;
  //## mask
    Word  MaskT[TubeGeoSl][TubeGeoLay][TubeGeoUpDown][TubeGeoNx][Nconn];
  //## first connected wire number
    Word  Wire1T[TubeGeoSl][TubeGeoLay][TubeGeoUpDown][TubeGeoNx][Nconn][TubeGeoSd];
  //## first bit inside crate
    Word  Nbit1T[TubeGeoSl][TubeGeoLay][TubeGeoUpDown][TubeGeoNx][Nconn][TubeGeoSd];
  //##  crate
    Word  CrateT[TubeGeoSl][TubeGeoLay][TubeGeoUpDown][TubeGeoNx];
  //## step (+2 or -2)
    int  StepT[TubeGeoSl][TubeGeoLay][TubeGeoUpDown][TubeGeoNx];

// pad chambers
    const Word PadGeoSl     = 2;
    const Word PadGeoUpDown = 2;
    const Word PadGeoCh     = 34;
    const Word PadGeoSd     = 2;
    const Word PadGeoRow    = 2;

  //## number of pad chambers
    Word NchambP[PadGeoSl][PadGeoUpDown];
  //## number of wires in the pad chamber
    Word PadGeoWire;
  //## number of pads per row
    Word Npads[PadGeoSl][PadGeoUpDown][PadGeoCh];
  //## gede
    LWord PadGede[PadGeoSl][PadGeoUpDown][PadGeoCh][PadGeoSd];
  //## chamber names
    char ChamListP[PadGeoSl][PadGeoUpDown][PadGeoCh][9];

  //## number of tube readout half-connectors per chamber
    Word NconnP[PadGeoSl][PadGeoUpDown][PadGeoCh];
  //## tube readout mask
    Word  MaskP[PadGeoSl][PadGeoUpDown][PadGeoCh][Nconn];
  //## first connected wire
    Word  Wire1P[PadGeoSl][PadGeoUpDown][PadGeoCh][Nconn][PadGeoSd];
  //## first bit inside crate
    Word  Nbit1P[PadGeoSl][PadGeoUpDown][PadGeoCh][Nconn][PadGeoSd];
  //## crate for wire readout
    Word  CrateP[PadGeoSl][PadGeoUpDown][PadGeoCh];
  //## step for wire readout(+2 or -2)
    int  StepP[PadGeoSl][PadGeoUpDown][PadGeoCh];

  //## number of pad readout connectors per chamber
    Word NconPP[PadGeoSl][PadGeoUpDown][PadGeoCh];
    const Word NconP = 4;
  //## pad readout mask
    Word  MaskPP[PadGeoSl][PadGeoUpDown][PadGeoCh][NconP];
  //## first connected pad
    Word  Pad1[PadGeoSl][PadGeoUpDown][PadGeoCh][NconP];
  //## number of connected pads per connector
    Word  NumbP[PadGeoSl][PadGeoUpDown][PadGeoCh][NconP];
  //## first bit inside crate
    Word  Nbit1PP[PadGeoSl][PadGeoUpDown][PadGeoCh][NconP];
  //## crate number
    Word  CratePP[PadGeoSl][PadGeoUpDown][PadGeoCh];
};

static MuDigiHitb  MDH;

extern "C" 
{

int MuDHini()
{
  int st1 = ArteIni();
  int st2 = ConnectIni();
  mursht_();
  if(st1 == 1 && st2 == 1) return 1;
  else return 0;
}

void MuDiHi()
{
  TubeLoops();
  PadLoops();
}

void clear()
{

// tube chambers
  for(Word sl=0; sl<MDH.TubeGeoSl; sl++) {
    for(Word lay=0; lay<MDH.TubeGeoLay; lay++) {
      for(Word ud=0; ud<MDH.TubeGeoUpDown; ud++) {
        MDH.NchambT[sl][lay][ud] = 0;
        for(Word nx=0; nx<MDH.TubeGeoNx; nx++) {
          char* name = "        ";
          strncpy(&MDH.ChamListT[sl][lay][ud][nx][0],name,9);
          MDH.TubeGede[sl][lay][ud][nx][0] = 0;
          MDH.TubeGede[sl][lay][ud][nx][1] = 0;
          MDH.NconnT[sl][lay][ud][nx]=0;
          MDH.CrateT[sl][lay][ud][nx]=0;
          MDH.StepT[sl][lay][ud][nx] =0;
          for(Word nc=0; nc<MDH.Nconn; nc++) {
            for(Word sd=0; sd<MDH.TubeGeoSd; sd++) {
              MDH.Wire1T[sl][lay][ud][nx][nc][sd]=0;
              MDH.Nbit1T[sl][lay][ud][nx][nc][sd]=0;
            }
          }
        }
      }
    }
  }

// pad chambers
  for(Word sl=0; sl<MDH.PadGeoSl; sl++) {
    for(Word ud=0; ud<MDH.PadGeoUpDown; ud++) {
      MDH.NchambP[sl][ud] = 0;
      for(Word nx=0; nx<MDH.PadGeoCh; nx++) {
        char* name = "        ";
        strncpy(&MDH.ChamListP[sl][ud][nx][0],name,9);
        MDH.Npads[sl][ud][nx]    = 0;
        MDH.NconPP[sl][ud][nx]    = 0;
        MDH.PadGede[sl][ud][nx][0] = 0;
        MDH.PadGede[sl][ud][nx][1] = 0;
        MDH.NconnP[sl][ud][nx]=0;
        MDH.NconPP[sl][ud][nx]=0;
        MDH.CrateP[sl][ud][nx]=0;
        MDH.CratePP[sl][ud][nx]=0;
        MDH.StepP[sl][ud][nx] =0;
        for(Word nc=0; nc<MDH.Nconn; nc++) {
          for(Word sd=0; sd<MDH.PadGeoSd; sd++) {
            MDH.MaskP[sl][ud][nx][nc]=0;
            MDH.Wire1P[sl][ud][nx][nc][sd]=0;
            MDH.Nbit1P[sl][ud][nx][nc][sd]=0;
          }
        }
        for(Word nc=0; nc<MDH.NconP; nc++) {
          MDH.MaskPP[sl][ud][nx][nc]=0;
          MDH.Pad1[sl][ud][nx][nc]=0;
          MDH.Nbit1PP[sl][ud][nx][nc]=0;
          MDH.NumbP[sl][ud][nx][nc]=0;
        }
      }
    }
  }

  MDH.Ivalid = 0;
}

void MuDHprint()
{
// create the log-file and pipe the programm output
  ofstream out("MuDigiHitb.print");
  if (!out) 
  {
    arteMsg << "MuDigiHitb - cannot open file for print" << endl;
    arteMsg.warn();
    return;
  }
  cout = out;
  cerr = out;

  int print=3;
  
  if(print == 1 || print == 3 || print == 5 || print == 7) {
  cout << "MuDigiHitb - Tube Chambers, Ivalid= " << MDH.Ivalid << endl;
  for(Word sl=0; sl<MDH.TubeGeoSl; sl++) {
    for(Word lay=0; lay<MDH.TubeGeoLay; lay++) {
      for(Word ud=0; ud<MDH.TubeGeoUpDown; ud++) {
	cout << "tube chamber sl " << dec << sl << " lay " << lay;
        cout << " UpDown " << ud << " Nchamb " << MDH.NchambT[sl][lay][ud] << endl; 
        if(MDH.NchambT[sl][lay][ud] > 0) {
          for(Word nx=0; nx<MDH.NchambT[sl][lay][ud]; nx++) {
            char name[9];
            strncpy(name,&MDH.ChamListT[sl][lay][ud][nx][0],9);
	    cout << name << " nx " << nx;
            cout << " Ncable " << MDH.NconnT[sl][lay][ud][nx] << endl;
            if(MDH.NconnT[sl][lay][ud][nx] > 0) {
              for(Word nc=0; nc<MDH.NconnT[sl][lay][ud][nx]; nc++) {
                for(Word sd=0; sd<MDH.TubeGeoSd; sd++) {
        	  cout << "   #cable " << nc;
                  cout << " mask " << hex << MDH.MaskT[sl][lay][ud][nx][nc];
                  cout << " side " << dec << sd;
		  cout << " wire1 " << dec << MDH.Wire1T[sl][lay][ud][nx][nc][sd];
		  cout << " crate " << hex << MDH.CrateT[sl][lay][ud][nx];
		  cout << " Nbit1 " << dec << MDH.Nbit1T[sl][lay][ud][nx][nc][sd];
                  Word word = MDH.Nbit1T[sl][lay][ud][nx][nc][sd] / 32;
                  Word bit  = MDH.Nbit1T[sl][lay][ud][nx][nc][sd] - word*32;
                  cout << " (" << word << "-" << bit << ") ";
		  cout << " step " << dec << MDH.StepT[sl][lay][ud][nx];
		  cout << " gede " << dec << MDH.TubeGede[sl][lay][ud][nx][sd];
                  cout << endl;
                }
              }
            }
          }
        }
      }
    }
  }
  }

  if(print == 2 || print == 3 || print == 6 || print == 7) {
  cout << "MuDigiHitb - Pad Chambers, Ivalid= " << MDH.Ivalid << endl;
  for(Word sl=0; sl<MDH.PadGeoSl; sl++) {
    for(Word ud=0; ud<MDH.PadGeoUpDown; ud++) {
      cout << "pad chamber sl " << dec << sl+2;
      cout << " UpDown " << ud << " Nchamb " << MDH.NchambP[sl][ud] << endl; 
      if(MDH.NchambP[sl][ud] > 0) {
        for(Word nx=0; nx<MDH.NchambP[sl][ud]; nx++) {
          char name[9];
          strncpy(name,&MDH.ChamListP[sl][ud][nx][0],9);
 cout << name << " nx " << nx;
 cout << " wires " << MDH.PadGeoWire << " Ncab " << MDH.NconnP[sl][ud][nx];
 cout << " pads " << 2*MDH.Npads[sl][ud][nx] << " Ncab " << MDH.NconPP[sl][ud][nx] << endl;
          if(MDH.NconnP[sl][ud][nx] > 0) {
            for(Word nc=0; nc<MDH.NconnP[sl][ud][nx]; nc++) {
              for(Word sd=0; sd<MDH.PadGeoSd; sd++) {
                cout << "   #cableW " << nc;
                cout << " mask " << hex << MDH.MaskP[sl][ud][nx][nc];
                cout << " side " << dec << sd;
                cout << " wire1 " << dec << MDH.Wire1P[sl][ud][nx][nc][sd] ;
		cout << " crate " << hex << MDH.CrateP[sl][ud][nx];
		cout << " Nbit1 " << dec << MDH.Nbit1P[sl][ud][nx][nc][sd] ;
                Word word = MDH.Nbit1P[sl][ud][nx][nc][sd] / 32 ;
                Word bit  = MDH.Nbit1P[sl][ud][nx][nc][sd] - word*32 ;
                cout << " (" << word << "-" << bit << ") ";
		cout << " step " << dec << MDH.StepP[sl][ud][nx];
		cout << " gede " << dec << MDH.PadGede[sl][ud][nx][sd] ;
                cout << endl;
              }
            }
          }
          if(MDH.NconPP[sl][ud][nx] > 0) {
            for(Word nc=0; nc<MDH.NconPP[sl][ud][nx]; nc++) {
              cout << "   #cableP " << nc;
              cout << " mask " << hex << MDH.MaskPP[sl][ud][nx][nc];
              cout << " pad1 " << dec << MDH.Pad1[sl][ud][nx][nc];
              cout << " Npads " << dec << MDH.NumbP[sl][ud][nx][nc];
              cout << " crate " << hex << MDH.CratePP[sl][ud][nx];
              cout << " Nbit1 " << dec << MDH.Nbit1PP[sl][ud][nx][nc];
              Word word = MDH.Nbit1PP[sl][ud][nx][nc] / 32 ;
              Word bit  = MDH.Nbit1PP[sl][ud][nx][nc] - word*32 ;
              cout << " (" << word << "-" << bit << ") ";
              cout << " gede1 " << dec << MDH.PadGede[sl][ud][nx][0];
              cout << " gede2 " << dec << MDH.PadGede[sl][ud][nx][1];
              cout << endl;
            }
          }
        }
      }
    }
  }
  }
}

// initiate the structure MDH from the ARTE tables
int ArteIni() 
{
  clear();
// number of tube chambers per layer
//       ud  l  sl
// sl=0, UpDown=0
  MDH.NchambT[0][0][0] = 27;
  MDH.NchambT[0][1][0] = 28;
  MDH.NchambT[0][2][0] = 28;
// sl=0, UpDown=1
  MDH.NchambT[1][0][0] = 29;
  MDH.NchambT[1][1][0] = 28;
  MDH.NchambT[1][2][0] = 28;
// sl=1, UpDown=0
  MDH.NchambT[0][0][1] = 28;
  MDH.NchambT[0][1][1] = 29;
  MDH.NchambT[0][2][1] = 29;
// sl=1, UpDown=1
  MDH.NchambT[1][0][1] = 30;
  MDH.NchambT[1][1][1] = 29;
  MDH.NchambT[1][2][1] = 29;

// number of pad chambers per superlayer
  MDH.NchambP[0][0] = 33;
  MDH.NchambP[0][1] = 34;
  MDH.NchambP[1][0] = 33;
  MDH.NchambP[1][1] = 34;

// loops over GEDE
  LWord Ngede = 0;

  char memo[9] = "        ";
  for ( ArteTableIterator<GEDE> iv; 
        iv.valid();
        iv.next() ) {
    Ngede++;
    int   cmp  = ArtePointer<GEDE>(iv)->cmp;
// tube and pad chambers
    if(cmp == 8) {
      int  lay  = ArtePointer<GEDE>(iv)->lay - 1;   // # layer
      ArtePointer<GESL> gesl = (ArtePointer<GESL>) ArtePointer<GEDE>(iv)->gesl;
      int msl  = gesl->nsl;            // # muon superlayer
      //      float xd1  = ArtePointer<GEDE>(iv)->xmin;  // Xmin of chamber
      //      float xd2  = ArtePointer<GEDE>(iv)->xmax;  // Xmax of chamber
      float yd1  = ArtePointer<GEDE>(iv)->ymin;  // Ymin of chamber
      float yd2  = ArtePointer<GEDE>(iv)->ymax;  // Ymax of chamber
      //      float zd1  = ArtePointer<GEDE>(iv)->zmin;  // Zmin of chamber
      //      float zd2  = ArtePointer<GEDE>(iv)->zmax;  // Zmax of chamber

      int iup=0;                                // UpDown
      float absYd1 = (yd1>=0. ? yd1 : -yd1);
      float absYd2 = (yd2>=0. ? yd2 : -yd2);
      if(absYd1 > absYd2) iup=1;

// number of rows
//      int Nrow = ArteTableIterator<GEDE>::nrow();
//  cout << " " << endl;
//  cout << "Ngede " << Ngede << " Nrow " << Nrow;
//  cout << " iv,sl,lay: " << iv << " " << msl << " " << lay << " iup " << iup;
//  cout << " xd1,xd2 " << xd1 << " " << xd2 << endl;
//  cout << "yd1,yd2 " << yd1 << " " << yd2;
//  cout << " zd1,zd2 " << zd1 << " " << zd2 << endl;

// chamber name
      char name[9] = "        ";
      strncpy(name,ArtePointer<GEDE>(iv)->hwc,9);

// decode chamber number
      char numb[11] = "0123456789";
      int n1 = 0;
      char ch2 = name[6];
      for(int i=0; i<11; i++) {
        char ch1 = numb[i]; 
        if(ch1 == ch2) {
          n1 = i;
          break;
        }
      }
      int n2 = 0;
      ch2 = name[7];
      for(int i=0; i<11; i++) {
        char ch1 = numb[i]; 
        if(ch1 == ch2) {
          n2 = i;
          break;
        }
      }
      int nx = 10*n1+n2-1;

// side
      int iside=0;
      if(!strcmp(name,memo)) iside = 1;
      strncpy(memo,name,9);

//    cout << name << " msl,lay " << msl << " " << lay << " nx " << nx;
//    cout << " side " << iside << " n1,n2 " << n1 << " " << n2;
//    cout << " Ngede " << Ngede << endl;

// the tube chambers
      if(msl < 3) {
        int sl=msl-1;
// the special case of chamber 99
        if(iup == 1 && nx == 98) {
          nx=MDH.NchambT[sl][lay][iup]-1;         
        }

        if(nx > MDH.NchambT[sl][lay][iup]-1 || nx < 0) {
          arteMsg  << "MuDigiHitb: wrong tube ch. nx = " << nx << endl;
          arteMsg.warn();
	  return 0;
        }

        MDH.TubeGeoWire = ArtePointer<GEDE>(iv)->nwi[0];  // number of wires
        strncpy(&MDH.ChamListT[sl][lay][iup][nx][0],name,9);
        MDH.TubeGede[sl][lay][iup][nx][iside]    = Ngede;
      }

// the pad  chambers
      if(msl > 2) {
        int sl=msl-3;
// the special case of chamber 99
        if(iup == 1 && nx == 98) {
          nx=MDH.NchambP[sl][iup]-1;         
        }

        if(nx > MDH.NchambP[sl][iup]-1 || nx < 0) {
          arteMsg << "MuDigiHitb: wrong pad ch. nx = " << nx << endl;
          arteMsg.warn();
	  return 0;
        }

        MDH.PadGeoWire = ArtePointer<GEDE>(iv)->nwi[0];         // number of wires
        MDH.Npads[sl][iup][nx] = ArtePointer<GEDE>(iv)->nwi[2]; // number of pads
        strncpy(&MDH.ChamListP[sl][iup][nx][0],name,9);
        MDH.PadGede[sl][iup][nx][iside]    = Ngede;
      }
    }
  }
  MDH.Ivalid = 1;
  return 1;
}

// initiate the cable connection from the Mark's files
int ConnectIni() 
{

  char line[150];
  FILE *conf;
  conf = fopen("$HBROOT/MUON/db/info.muonconfig", "r");
  if(!conf) {
    arteMsg << "MuDigiHitb: NO configuration file info.muonconfig" << endl;
    arteMsg.error();
    return 0;
  }
  int cratr=0;
  int cablr =0;
  int st[16];
  Word crate=0;
  Word cable =0;
  char cablname[10];
  char cablchname[11];
  int icb=0;

  while (fgets(line, sizeof(line), conf) != NULL) {
    if (sscanf(line, "FED %x   CABLE %d   NAME %s   CHNAME %s   " 
                     "STATUS %d%d%d%d%d%d%d%d%d%d%d%d%d%d%d%d\n",
               &cratr,&cablr,cablname,cablchname,
               &st[0],&st[1],&st[2],&st[3],
               &st[4],&st[5],&st[6],&st[7],
               &st[8],&st[9],&st[10],&st[11],
               &st[12],&st[13],&st[14],&st[15]) == 20) {

      crate = Word(cratr);
      cable = Word(cablr);

// code the status into 16-bit word
      Word status;
      status = 0;
      Word one    = 0x1;
      for(int i=0; i<16; i++) {
        if(st[i] >= 0 && st[i] < 9) status = status + (one << i);
      }

// cable bit position inside crate
      Word nbit = 16*cable; 

//      cout << dec << icb << " crate " << hex << crate << " cable " << dec << cable;
//      cout << " " << cablname << " nbit " << dec << nbit;
//      cout << " stat " << hex << status << "  st " << dec;
//      cout << st[0] << st[1] << st[2] << st[3] << "  " << st[4];
//      cout << st[5] << st[6] << st[7] << "  " << st[8] << st[9];
//      cout << st[10] << st[11] << "  " << st[12] << st[13] << st[14];
//      cout << st[15] << endl;

// decode cablname
      char numb[11] = "0123456789";
      char ch;

      char chi = cablname[1];
      char* cht = "t";
      char* chp = "p";
      //      char* chx = "x";

      int sl=-1;
      ch = cablname[2];
      for(int i=0; i<11; i++) {
        char chn = numb[i]; 
        if(chn == ch) {
          sl = i;
          break;
        }
      }
      sl = sl-1;

      int type = 0;
      if(chi == *cht && sl >= 0 && sl < 2) type=1; // tube wires
      if(chi == *cht && sl > 1) type=2;            // pad wires
      if(chi == *chp && sl > 1) type=3;            // pads

      int ind=0;
      ch = cablname[4];
      for(int i=0; i<11; i++) {
        char chn = numb[i]; 
        if(chn == ch) {
          ind = i;
          break;
        }
      }
      int lay = (ind-1)/2;
      int iup = ind-lay*2-1;

      int ncab=-1;
      ch = cablname[8];
      char* chA = "A";
      char* chB = "B";
      char* chC = "C";
      char* chD = "D";
      if(ch == *chA) ncab=0;
      if(ch == *chB) ncab=1;
      if(ch == *chC) ncab=2;
      if(ch == *chD) ncab=3;

      int nx1 = 0;
      ch = cablname[6];
      for(int i=0; i<11; i++) {
        char chn = numb[i]; 
        if(chn == ch) {
          nx1 = i;
          break;
        }
      }
      int nx2 = 0;
      ch = cablname[7];
      for(int i=0; i<11; i++) {
        char chn = numb[i]; 
        if(chn == ch) {
          nx2 = i;
          break;
        }
      }
      int nx = 10*nx1+nx2-1;
// the special case of chamber 99
      if(iup == 1 && nx == 98) {
        if(type == 1) nx=MDH.NchambT[sl][lay][iup]-1;         
        if(type > 1 && type < 4)  nx=MDH.NchambP[sl-2][iup]-1;         
      }

//      if(type == 2) {
//        cout << "  sl " << sl << " ind " << ind << " lay " << lay;
//        cout << " iup " << iup << " nx " << nx << " type " << type;
//        cout << " ncab " << ncab << endl; 
//      }

// tube wires
      if(type == 1) {
        if(nx > MDH.NchambT[sl][lay][iup]-1 || nx < 0) {
          arteMsg << "MuDigiHitb - tube wires: wrong nx = " << nx << endl;
          arteMsg.warn();
//          cout << "  sl " << sl << " ind " << ind << " lay " << lay;
//          cout << " iup " << iup << " nx " << nx << " type " << type;
//          cout << " ncab " << ncab << endl; 
//          cout << line << endl;
          return 0;
        }
        int nc = MDH.NconnT[sl][lay][iup][nx];
        MDH.NconnT[sl][lay][iup][nx] += 1;
        MDH.MaskT[sl][lay][iup][nx][nc] = status;
        if(iup == 0) {
          MDH.StepT[sl][lay][iup][nx] =-2;
          MDH.StepT[sl][lay][iup][nx] =-2;

          MDH.CrateT[sl][lay][iup][nx]       =crate;
          MDH.Nbit1T[sl][lay][iup][nx][nc][0]=nbit+1;
          MDH.Nbit1T[sl][lay][iup][nx][nc][1]=nbit;

          if(ncab == 0) {
            MDH.Wire1T[sl][lay][iup][nx][nc][0]=15;
            MDH.Wire1T[sl][lay][iup][nx][nc][1]=15;
          }
          if(ncab == 1) {
            MDH.Wire1T[sl][lay][iup][nx][nc][0]=7;
            MDH.Wire1T[sl][lay][iup][nx][nc][1]=7;
          }
        }
        if(iup == 1) {
          MDH.StepT[sl][lay][iup][nx] =2;
          MDH.StepT[sl][lay][iup][nx] =2;

          MDH.CrateT[sl][lay][iup][nx]       =crate;
          MDH.Nbit1T[sl][lay][iup][nx][nc][0]=nbit;
          MDH.Nbit1T[sl][lay][iup][nx][nc][1]=nbit+1;

          if(ncab == 0) {
            MDH.Wire1T[sl][lay][iup][nx][nc][0]=0;
            MDH.Wire1T[sl][lay][iup][nx][nc][1]=0;
          }
          if(ncab == 1) {
            MDH.Wire1T[sl][lay][iup][nx][nc][0]=8;
            MDH.Wire1T[sl][lay][iup][nx][nc][1]=8;
          }
        }
      }

// pad chamber wires
      if(type == 2) {
        Word slp = sl-2;
        if(nx > MDH.NchambP[slp][iup]-1 || nx < 0) {
          arteMsg << "MuDigiHitb-pad chamber wires: wrong nx = " << nx << endl;
          arteMsg.warn();
//          cout << "  sl " << sl << " ind " << ind << " lay " << lay;
//          cout << " iup " << iup << " nx " << nx << " type " << type;
//          cout << " ncab " << ncab << endl; 
//          cout << line << endl;
          return 0;
        }
        int nc = MDH.NconnP[slp][iup][nx];
        MDH.NconnP[slp][iup][nx] += 1;
        MDH.MaskP[slp][iup][nx][nc] = status;
        if(iup == 0) {
          MDH.StepP[slp][iup][nx]=-2;
          MDH.StepP[slp][iup][nx]=-2;

          MDH.CrateP[slp][iup][nx]       =crate;
          MDH.Nbit1P[slp][iup][nx][nc][0]=nbit+1;
          MDH.Nbit1P[slp][iup][nx][nc][1]=nbit;

          if(ncab == 0) {
            MDH.Wire1P[slp][iup][nx][nc][0]=15;
            MDH.Wire1P[slp][iup][nx][nc][1]=15;
          }
          if(ncab == 1) {
            MDH.Wire1P[slp][iup][nx][nc][0]=7;
            MDH.Wire1P[slp][iup][nx][nc][1]=7;
          }
        }
        if(iup == 1) {
          MDH.StepP[slp][iup][nx]=2;
          MDH.StepP[slp][iup][nx]=2;

          MDH.CrateP[slp][iup][nx]       =crate;
          MDH.Nbit1P[slp][iup][nx][nc][0]=nbit;
          MDH.Nbit1P[slp][iup][nx][nc][1]=nbit+1;

          if(ncab == 0) {
            MDH.Wire1P[slp][iup][nx][nc][0]=0;
            MDH.Wire1P[slp][iup][nx][nc][1]=0;
          }
          if(ncab == 1) {
            MDH.Wire1P[slp][iup][nx][nc][0]=8;
            MDH.Wire1P[slp][iup][nx][nc][1]=8;
          }
        }
      }

// pads
      if(type == 3) {
        Word slp = sl-2;
        if(nx > MDH.NchambP[slp][iup]-1 || nx < 0) {
          arteMsg << "MuDigiHitb-pad chamber pads: wrong nx = " << nx << endl;
          arteMsg.warn();
//          cout << "  sl " << sl << " ind " << ind << " lay " << lay;
//          cout << " iup " << iup << " nx " << nx << " type " << type;
//          cout << " ncab " << ncab << endl; 
//          cout << line << endl;
          return 0;
        }
        int nc = MDH.NconPP[slp][iup][nx];
        MDH.NconPP[slp][iup][nx] += 1;
        MDH.MaskPP[slp][iup][nx][nc] = status;
        MDH.CratePP[slp][iup][nx]    = crate;
        MDH.Nbit1PP[slp][iup][nx][nc]=nbit;
        MDH.NumbP[slp][iup][nx][nc]=15;
        if(ncab == 0 || ncab == 2) {
          if(slp == 1) {
            MDH.NumbP[slp][iup][nx][nc]=14;
            MDH.Nbit1PP[slp][iup][nx][nc]=nbit+1;
          }
        }
        Word padsh = 15;

// chambers Mp 15, 16, 17
        if(nx > 14 && nx < 18) {
          if(ncab == 1 || ncab == 3) {
            MDH.NumbP[slp][iup][nx][nc]=12;
            if(slp == 1) MDH.Nbit1PP[slp][iup][nx][nc]=nbit;
          }
          padsh=12;
        }
// chamber 18 bottom 
        if(iup == 1 && nx == 18) {
          if(ncab == 1 || ncab == 3) {
            MDH.Nbit1PP[slp][iup][nx][nc]=nbit;
            MDH.NumbP[slp][iup][nx][nc]=4;
            if(slp == 1) MDH.NumbP[slp][iup][nx][nc]=5;
          }
          if(ncab == 0 || ncab == 2) {
            MDH.Nbit1PP[slp][iup][nx][nc]=nbit+1;
          }
          padsh=4;
          if(slp == 1) padsh=5;
        }

        if(iup == 0) {
          if(ncab == 1) MDH.Pad1[slp][iup][nx][nc]=MDH.Npads[slp][iup][nx];
          if(ncab == 0) MDH.Pad1[slp][iup][nx][nc]=MDH.Npads[slp][iup][nx]+padsh;
          if(ncab == 3) MDH.Pad1[slp][iup][nx][nc]=0;
          if(ncab == 2) MDH.Pad1[slp][iup][nx][nc]=padsh;
        }
        if(iup == 1) {
          if(ncab == 1) MDH.Pad1[slp][iup][nx][nc]=0;
          if(ncab == 0) MDH.Pad1[slp][iup][nx][nc]=padsh;
          if(ncab == 3) MDH.Pad1[slp][iup][nx][nc]=MDH.Npads[slp][iup][nx];
          if(ncab == 2) MDH.Pad1[slp][iup][nx][nc]=MDH.Npads[slp][iup][nx]+padsh;
// add the link to chamber 99
          if(nx == 18) {
            int nx99=int(MDH.NchambP[slp][iup])-1;         
            if(ncab == 1 || ncab == 3) {
              int nc99 = int(MDH.NconPP[slp][iup][nx99]);
              MDH.NconPP[slp][iup][nx99] += 1;
              MDH.MaskPP[slp][iup][nx99][nc99] = status;
              MDH.CratePP[slp][iup][nx99]    =crate;
              MDH.Nbit1PP[slp][iup][nx99][nc99]=nbit+9;
              MDH.NumbP[slp][iup][nx99][nc99]=MDH.Npads[slp][iup][nx99];
              if(ncab == 1) MDH.Pad1[slp][iup][nx99][nc99]=0;
              if(ncab == 3) MDH.Pad1[slp][iup][nx99][nc99]=MDH.Npads[slp][iup][nx99];
            }
          }
        }
      }
      icb++;
    }
  }
  fclose(conf);
  return 1;
}

//*** loops over tube chambers
void TubeLoops() 
{
  if(MDH.Ivalid != 1) {
    arteMsg << "MuDigiHitb.TubeLoops: Ivalid = " << MDH.Ivalid << endl;
    arteMsg.error();
    return;
  }

  for(Word sl=0; sl<MDH.TubeGeoSl; sl++) {
    for(Word lay=0; lay<MDH.TubeGeoLay; lay++) {
      for(Word ud=0; ud<MDH.TubeGeoUpDown; ud++) {
        if(MDH.NchambT[sl][lay][ud] > 0) {
          for(Word sd=0; sd<MDH.TubeGeoSd; sd++) {
            for(Word nx=0; nx<MDH.NchambT[sl][lay][ud]; nx++) {
              Word ncon = MDH.NconnT[sl][lay][ud][nx];
              if(ncon > 0) {
                for(Word nc=0; nc<ncon; nc++) {
                  Word crate = MDH.CrateT[sl][lay][ud][nx];
                  Word nbit1 = MDH.Nbit1T[sl][lay][ud][nx][nc][sd];
                  Word wir1   = MDH.Wire1T[sl][lay][ud][nx][nc][sd];
                  int  step   = MDH.StepT[sl][lay][ud][nx];
                  int astp   = (step<0)? -step : step;
                  int gede = int(MDH.TubeGede[sl][lay][ud][nx][sd]);
                  for(Word wr=0; wr<MDH.NumbT; wr++) {
                    int wir;
                    if(step > 0) wir = int(wir1+wr);
                    if(step < 0) wir = int(wir1-wr);
                    wir=15-wir;
                    Word nbit = nbit1 + astp*wr;
// check mask
                    Word mskb = astp*wr;
                    if(MDH.Nbit1T[sl][lay][ud][nx][nc][sd] > 
                       MDH.Nbit1T[sl][lay][ud][nx][nc][1-sd]) mskb += 1;
                    Word msk = 0;
                    Word one = 0x8000;
                    msk = msk + (one >> mskb);
                    Word mask = MDH.MaskT[sl][lay][ud][nx][nc] & msk;
                    if(mask != 0) {
// check DMUT
                      int check = CheckBit(1,crate,nbit);
                      if(check == 1) {

// add the tube wire into HITB
			//                        float rate=wire_occup(gede,sd,wir);
			//                        if(rate < 0.1) muaddt_(&gede,&wir);
                        muaddt_(&gede,&wir);

//     char nam[9];
//     strncpy(nam,&MDH.ChamListT[sl][lay][ud][nx][0],9);
//     cout << "wire of " << nam;
//     cout << " added in HITB, sl " << dec << sl << " lay " << lay;
//     cout << " ud " << ud << " nx " << nx << " nc " << nc;
//     cout << " sd " << sd << " wir " << wir << " gede " << dec << gede;
//     cout << " crate " << hex << crate << dec << " nbit " << nbit;
//     Word ww = nbit / 32 ;
//     Word bb  = nbit - ww*32 ;
//     cout << " (" << ww << "-" << bb << ") " << endl;

                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }
}

// loops over pad chambers
void PadLoops()
{
// create the log-file and pipe the programm output
  //  ofstream out("MuDigiHitb.loops");
  //  if (!out) 
  //  {
  //    cout << "MuDigiHitb - cannot open file for print" << endl;
  //    return;
  //  }
  //  cout = out;
  //  cerr = out;

  if(MDH.Ivalid != 1) {
    arteMsg << "PadWireLoops: Ivalid = " << MDH.Ivalid << endl;
    arteMsg.error();
    return;
  }

  for(Word sl=0; sl<MDH.PadGeoSl; sl++) {
    for(Word udt=0; udt<MDH.PadGeoUpDown; udt++) {
      Word ud = MDH.PadGeoUpDown-1-udt;

// loops over wire connectors of side 0
      LoopsPW(sl,ud,0);

// loops over pad connectors
      if(MDH.NchambP[sl][ud] > 0) {
        for(Word nx=0; nx<MDH.NchambP[sl][ud]; nx++) {
          Word ncon = MDH.NconPP[sl][ud][nx];
          if(ncon > 0) {
            Word crate= MDH.CratePP[sl][ud][nx];
            int gede1 = int(MDH.PadGede[sl][ud][nx][0]);
            int gede2 = int(MDH.PadGede[sl][ud][nx][1]);
            for(Word nc=0; nc<ncon; nc++) {
              Word nbit1 = MDH.Nbit1PP[sl][ud][nx][nc];
              Word pad1   = MDH.Pad1[sl][ud][nx][nc];
              int  npd    = MDH.NumbP[sl][ud][nx][nc];
              if(npd > 0) {
                for(Word pd=0; pd<npd; pd++) {
                  int pad = int(pad1+npd-pd-1);
                  Word nbit = nbit1+pd;
// check mask
                  Word one = 0x8000;
                  Word msk = 0;
                  if(npd >= 14 || npd < 12) msk = one >> pd+15-npd;
                  if(npd == 12) msk = one >> pd;
                  Word mask = MDH.MaskPP[sl][ud][nx][nc] & msk;

//  if(nx > 14 && nx < 18) {
//   cout << " nx " << dec << nx << " ud " << ud << " pad " << pad;
//   cout << " pd,npd " << pd << " " << npd << endl;
//   cout << " sour " << hex << MDH.MaskPP[sl][ud][nx][nc];
//   cout << " msk " << hex << msk;
//   cout << " mask " << hex << mask << endl;
//  }

                  if(mask != 0) {
// check DMUT
                    int check = CheckBit(2,crate,nbit);

//  if(nx > 14 && nx < 18) {
//    cout << " nbit " << dec << nbit << " check= " << dec << check << endl;
//  }

                    if(check == 1) {

		      //   cout << " nx " << dec << nx << " ud " << ud << " pad " << pad;
		      //   cout << " pd,npd " << pd << " " << npd << endl;
		      //   cout << " sour " << hex << MDH.MaskPP[sl][ud][nx][nc];
		      //   cout << " msk " << hex << msk;
		      //   cout << " mask " << hex << mask << endl;

// add the pad into HITB
		      //	              float rate=pad_occup(gede1,pad);
		      //	              if(rate < 0.05) {
		      //  cout << " gede1,2 " << gede1 << " " << gede2 << " pad " << pad << endl;
                        muaddp_(&gede1,&pad);
                        muaddp_(&gede2,&pad);
		      //                      }

// if(nx > 14 && nx < 18 && pad < 3) {
//   char nam[9];
//   strncpy(nam,&MDH.ChamListP[sl][ud][nx][0],9);
//   cout << "pd " << dec << pd << "  " << nam;
//   cout << " added in HITB, sl " << dec << sl;
//   cout << " ud " << ud << " nx " << nx << " nc " << nc;
//   cout << " pad " << pad;
//   cout << " gede1 " << gede1;
//   cout << " gede2 " << dec << gede2;
//   cout << " crate " << hex << crate << " nbit " << dec << nbit;
//   Word ww = nbit / 32 ;
//   Word bb  = nbit - ww*32 ;
//   cout << " (" << ww << "-" << bb << ") " << endl;
// }

                    }
                  }
                }
              }
            }
          }
        }
      }

// loops over wire connectors of side 1
      LoopsPW(sl,ud,1);

    }
  }
}


// loops over pad chamber wire connectors of side sd
void LoopsPW(Word sl,Word ud,Word sd) 
{

  if(MDH.NchambP[sl][ud] > 0) {
    for(Word nx=0; nx<MDH.NchambP[sl][ud]; nx++) {
      Word ncon = MDH.NconnP[sl][ud][nx];
      if(ncon > 0) {
        Word crate = MDH.CrateP[sl][ud][nx];
        int gede = int(MDH.PadGede[sl][ud][nx][sd]);
        int  step   = MDH.StepP[sl][ud][nx];
        Word astp   = (step<0)? -step : step;
        for(Word nc=0; nc<ncon; nc++) {
          Word nbit1 = MDH.Nbit1P[sl][ud][nx][nc][sd];
          Word wir1   = MDH.Wire1P[sl][ud][nx][nc][sd];
          for(Word wr=0; wr<MDH.NumbT; wr++) {
            int wir;
            if(step > 0) wir = int(wir1+wr);
            if(step < 0) wir = int(wir1-wr);
            wir=15-wir;
            Word nbit = nbit1 + astp*wr;
// check mask
            Word mskb = astp*wr;
            if(MDH.Nbit1P[sl][ud][nx][nc][sd] > 
              MDH.Nbit1P[sl][ud][nx][nc][1-sd]) mskb += 1;
            Word msk = 0;
            Word one = 0x8000;
            msk = msk + (one >> mskb);
            Word mask = MDH.MaskP[sl][ud][nx][nc] & msk;
            if(mask != 0) {
// check DMUT
              int check = CheckBit(1,crate,nbit);
              if(check == 1) {

// add the pad wire into HITB
		//           float rate=wire_occup(gede,sd,wir);
		//           if(rate < 0.1) muaddt_(&gede,&wir);
                muaddt_(&gede,&wir);

//         char nam[9];
//         strncpy(nam,&MDH.ChamListP[sl][ud][nx][0],9);
//         Word word = nbit / 32 ;
//         Word bit  = nbit - word*32 ;
//         cout << "wire of " << nam;
//         cout << " added in HITB, sl " << dec << sl;
//         cout << " ud " << ud << " nx " << nx << " nc " << nc;
//         cout << " sd " << sd << " wir " << wir;
//         cout << " gede " << dec << gede;
//         cout << " crate " << hex << crate << " nbit " << dec << nbit;
//         Word ww = nbit / 32 ;
//         Word bb  = nbit - ww*32 ;
//         cout << " (" << ww << "-" << bb << ") " << endl;

              }
            }
          }
        }
      }
    }
  }
}

// check the bit nbit in DMUT(table=1), DMUP(table=2), DMUX(table=3)
int CheckBit(int table,Word crate,Word nbit) 
{
  int* nmax   = 0;
  if(table == 1) nmax=(int*)(ArtePointer<DMUT>(0)->bits);
  if(table == 2) nmax=(int*)(ArtePointer<DMUP>(0)->bits);
  if(table == 3) nmax=(int*)(ArtePointer<DMUX>(0)->bits);

// crate offset
  int nword = 0;
  int ncrate = *nmax/34;
  LWord* word = 0;
  int first=0;
  for (int ncr=0; ncr<ncrate; ncr++) {
    first=2+34*ncr;
    if(table == 1) word=(LWord*)(ArtePointer<DMUT>(first)->bits);
    if(table == 2) word=(LWord*)(ArtePointer<DMUP>(first)->bits);
    if(table == 3) word=(LWord*)(ArtePointer<DMUX>(first)->bits);

    int tmp1 = *word << 16;
    Word  tmp2=Word(tmp1 >> 16);
    if(tmp2 == crate) {
      nword=first+1+int(nbit)/32;
      break;
    }
  }
  if(nword == 0) {
    //  arteMsg << "CheckBit: no crate " << hex << crate << " in table " << table << endl;
    //    arteMsg.error();
    //  arteMsg.warn();
    return 0;
  }
  if(nword < 0 || nword > *nmax) {
    //    arteMsg << "CheckBit: nword= " << nword << " nmax= " << *nmax << " table " << table  << endl;
    //    arteMsg.error();
    //    arteMsg.warn();
    return 0;
  } 

  if(table == 1) word=(LWord*)(ArtePointer<DMUT>(nword+1)->bits);
  if(table == 2) word=(LWord*)(ArtePointer<DMUP>(nword+1)->bits);
  if(table == 3) word=(LWord*)(ArtePointer<DMUX>(nword+1)->bits);

  if(*word == 0) return 0;
  Word nw = nbit/32;
  Word nbtt = nbit-32*nw;
  Word nbyte = nbtt/8;
  Word nbb = nbtt-8*nbyte;
  Word nb = 7-nbb;  // change the order of bits inside byte
  Word nbt = 8*nbyte+nb;
  const LWord one = 0x1;
  LWord mask = one << nbt;
  LWord res = *word & mask;
  if(res == 0) return 0;
  else return 1;
}

}




