// Author:  Vitaly Eiges,  22.03.99
// Main routine for muon digi-->hitb conversion

#include <iostream.h>
#include <fstream.h>
#include "arte/Arte.hh"
#include "auto/GEDE.hh"
#include "auto/DIG.hh"

extern int GetOffset( int );
extern int GetPosition( int );
extern int GetPlane( int );
extern int GetGede( int );
extern unsigned int GetHot( int, int, int, int );

extern "C" {
void mdfillt_(const int &, unsigned int);
void mdfillp_(const int &, unsigned int, unsigned int, const int &);
void mmbook_( int & , const int & );
void mdreset_();
void mdsort_(int &, int &);
}

void GetBits(unsigned int* t, unsigned int* b)
{
  int j; 
  unsigned int bit_mask=1;
  
  for (j=0; j<8; j++)
    {
      *(b+j+ 0) = (((*(unsigned char* )t)     & (bit_mask << j)) >> j);
      *(b+j+ 8) = (((*((unsigned char* )t+1)) & (bit_mask << j)) >> j);
      *(b+j+16) = (((*((unsigned char* )t+2)) & (bit_mask << j)) >> j);
      *(b+j+24) = (((*((unsigned char* )t+3)) & (bit_mask << j)) >> j);
    }
}

void md2hitb()
{
  int iT = 0;
  int iP = 0;
  int i,j,l,plane,npmx;
  unsigned int* t[700]; 
  unsigned int* p[700];
  int FED_id;
  unsigned a1[32], a2[32], b1[32], b2[32];
  int t99, p99;
  int iCH = 0;
  int n1, n2;

  // Reset GEDE hit first - hit last info
  mdreset_();

  // First position for muon hits in HITB
  n1 = ArtePointer<HITB>::nrow()+1;

  for ( ArteTableIterator<DMUT> iDT; iDT.valid(); iDT.next() ) 
    {
      t[++iT] = (unsigned int* )ArtePointer<DMUT>(iDT)->bits;
    }
  for ( ArteTableIterator<DMUP> iDP; iDP.valid(); iDP.next() ) 
    {
      p[++iP] = (unsigned int* )ArtePointer<DMUP>(iDP)->bits;
    }

  // Start handling DMUT table
  // Table DMUT runs from i = 1 to iT (32 bit strings)
  
  // cout << "NDMUT= " << iT << "  NDMUP= " << iP << endl;

  int i_offset = 2;
  while(i_offset + 33 <= iT)
    {
      FED_id = (int )*((unsigned char*)t[i_offset] + 3) + 
	256 * (int )*((unsigned char*)t[i_offset] + 2);
      iCH = 0;  // bit counter in FED data
      // cout << "FED_id " << hex << FED_id << dec << endl;

      for (i=i_offset+2; i<=i_offset+33; i++)   // i_offset + 2 : offset in FED data
	{
	  t99 = 0;  // 99 flag init
	  plane = GetGede(GetPlane(GetOffset(FED_id)+i-i_offset-2));
	  // cout << "Plane " << plane << "  Offset " << GetOffset(FED_id) << endl;
	  if ((GetOffset(FED_id) != 1) && (plane != 0)) // FED & plane OK
	    {
	      GetBits(t[i], b1);
	      //   for (j=0; j<8; j++)
	      //	{
	      //	  cout << b1[j]; 
	      //	}
	      // cout << " ";
	      // for (j=8; j<16; j++)
	      //	{
	      //	  cout << b1[j]; 
	      //	}
	      //  cout << " ";
	      // for (j=16; j<24; j++)
	      //	{
	      //	  cout << b1[j]; 
	      //	}
	      //  cout << " ";
	      // for (j=24; j<32; j++)
	      //	{
	      //  cout << b1[j]; 
	      //	}
	      // cout << endl;
	      // Check hot channels (GetHot 0-hot, 1-normal !!)
	      for (j=0; j<32; j++) {b1[j] = (b1[j] & GetHot(iCH++, FED_id, plane, 0));}

	      if (FED_id == 0x111 && (i-i_offset-2) == 24) // 1.2-16, 1.2-99
		{
		  GetBits(t[i+1], b2);
		  for (j=0; j<32; j++) {b2[j] = (b2[j] & GetHot(iCH++, FED_id, GetGede(1024), 0));}
		  for (l=0; l<16; l++) 
		    { 
		      a1[l]=b1[l]; a1[l+16]=b2[l];
		      a2[l]=b1[l+16]; a2[l+16]=b2[l+16];
		    }
		  mdfillt_(GetGede(1024), (unsigned int)a1); // 1.2-16
		  mdfillt_(GetGede(1048), (unsigned int)a2); // 1.2-99
		  t99=1;
		}
	      if (FED_id == 0x111 && (i-i_offset-2) == 25) // 1.2-16, 1.2-99
		{
		  t99=1; iCH += -32; // ok, skip this time
		}
	      if (FED_id == 0x132 && (i-i_offset-2) == 0) // 3.2-19, 3.2-99
		{
		  GetBits(t[i+1], b2);
		  for (j=0; j<32; j++) {b2[j] = (b2[j] & GetHot(iCH++, FED_id, GetGede(3030), 0));}
		  for (l=0; l<16; l++)
		    {
		      a1[l]=b2[l+16]; a1[l+16]=b1[l+16];
		      a2[l]=b1[l];    a2[l+16]=b2[l];
		    }
		  for (l=0; l<16; l++)
		    {
		      b1[2*l]=a1[2*l+1];  b1[2*l+1]=a1[2*l];
		      b2[2*l]=a2[2*l+1];  b2[2*l+1]=a2[2*l];
		    }
		  mdfillt_(GetGede(3030), (unsigned int)b1); // 3.2-19
		  mdfillt_(GetGede(3060), (unsigned int)b2); // 3.2-99
		  t99=1;
		}
	      if (FED_id == 0x132 && (i-i_offset-2) == 1) // 3.2-19, 3.2-99
		{
		  t99=1; iCH += -32; // ok, skip this time
		}
	      if (FED_id == 0x132 && (i-i_offset-2) == 16) // 4.2-19, 4.2-99
		{
		  GetBits(t[i+1], b2);
		  for (j=0; j<32; j++) {b2[j] = (b2[j] & GetHot(iCH++, FED_id, GetGede(4030), 0));}
		  for (l=0; l<16; l++)
		    {
		      a1[l]=b2[l+16]; a1[l+16]=b1[l+16];
		      a2[l]=b1[l];    a2[l+16]=b2[l];
		    }
		  for (l=0; l<16; l++)
		    {
		      b1[2*l]=a1[2*l+1];  b1[2*l+1]=a1[2*l];
		      b2[2*l]=a2[2*l+1];  b2[2*l+1]=a2[2*l];
		    }
		  mdfillt_(GetGede(4030), (unsigned int)b1); // 4.2-19
		  mdfillt_(GetGede(4060), (unsigned int)b2); // 4.2-99
		  t99=1;
		}
	      if (FED_id == 0x132 && (i-i_offset-2) == 17) // 4.2-19, 4.2-99
		{
		  t99=1; iCH += -32; // ok, skip this time
		}

	      // Fill bottom tube hits
	      if (GetPosition(FED_id)==0 && t99 == 0) 
		{ 
		  if (FED_id != 0x132 && 
                      FED_id != 0x452 &&
		      FED_id != 0x521) { mdfillt_(plane, (unsigned int)b1); } // normal bottom
		  if (FED_id == 0x132)  // +X, Mt3.2, Mt4.2
		    { 
		      for (l=0; l<16; l++)
			{
			  a1[l]=b1[l+16];
			  a1[l+16]=b1[l];
			}
		      for (l=0; l<16; l++)
			{
 			  b1[2*l]=a1[2*l+1];
			  b1[2*l+1]=a1[2*l];
			}
		      mdfillt_(plane, (unsigned int)b1); 
		    }
		  if (FED_id == 0x452 || FED_id == 0x521) // -X, Mt3.2, Mt4.2 
		    { 
		      for (l=0; l<16; l++)
			{
 			  a1[2*l]=b1[2*l+1];
			  a1[2*l+1]=b1[2*l];
			}
		      mdfillt_(plane, (unsigned int)a1); 
		    }
		}
	      if (FED_id == 0x131 && (i-i_offset-2) == 0) // 3.1-19 (revert to AB) !!!
		{
                  for (l=0; l<16; l++) {a1[l]=b1[15-l]; a1[l+16]=b1[31-l];} 
                  for (l=0; l<16; l++) {b1[l]=a1[l+16]; b1[l+16]=a1[l];} //  A <--> B
		  mdfillt_(GetGede(3000), (unsigned int)b1); // 3.1-19
		  t99=1; // just to flag
		}
	      if (FED_id == 0x131 && (i-i_offset-2) == 16) // 4.1-19 (revert to AB) !!!
		{
                  for (l=0; l<16; l++) {a1[l]=b1[15-l]; a1[l+16]=b1[31-l];} 
                  for (l=0; l<16; l++) {b1[l]=a1[l+16]; b1[l+16]=a1[l];} //  A <--> B
		  mdfillt_(GetGede(4000), (unsigned int)b1); // 4.1-19
		  t99=1; // just to flag
		}
              // Fill top tube hits
	      if (GetPosition(FED_id)==1 && t99 == 0) 
		{
		  if (FED_id != 0x451 && FED_id != 0x512)  // normal top
		    { 
		      for (l=0; l<16; l++) {a1[l]=b1[15-l]; a1[l+16]=b1[31-l];} 
		      mdfillt_(plane, (unsigned int)a1); 
		    }
		  if (FED_id == 0x451 || FED_id == 0x512)  // -X, Mt3.1, Mt4.1 (AB !!)
		    { 
		      for (l=0; l<32; l++) {a1[l]=b1[31-l];} 
		      mdfillt_(plane, (unsigned int)a1); 
		    }
		}
	    }
	  if ((GetOffset(FED_id) == 1) || (plane == 0)) { iCH += 32; } // Skip
	}
      i_offset += 34; 
    }
  
  // Start handling DMUP table
  // Table DMUP runs from i = 1 to iP (32 bit strings)

  i_offset = 2;
  while(i_offset + 33 <= iP)
    {
      FED_id = (int )*((unsigned char*)p[i_offset] + 3) + 
	256 * (int )*((unsigned char*)p[i_offset] + 2);
      iCH = 0;  // bit counter in FED data
      // cout << "FED_id pad " << hex << FED_id << dec << endl;

      for (i=i_offset+2; i<=i_offset+33; i += 2)
	{
	  p99 = 0;  // 99 flag init
	  plane = GetGede(GetPlane(GetOffset(FED_id)+i-i_offset-2));
	  if ((GetOffset(FED_id) != 1) && (plane != 0)) // FED & plane OK
	    {
	      GetBits(p[i],   b1);
	      GetBits(p[i+1], b2);
	      //  for (j=0; j<8; j++)
	      //	{
	      //	  cout << b1[j]; 
	      //	}
	      // cout << " ";
	      // for (j=8; j<16; j++)
	      //	{
	      //	  cout << b1[j]; 
	      //	}
	      // cout << " ";
	      // for (j=16; j<24; j++)
	      //	{
	      //	  cout << b1[j]; 
	      //	}
	      // cout << " ";
	      // for (j=24; j<32; j++)
	      //	{
	      //  cout << b1[j]; 
	      //	}
	      // cout << endl;
	      // for (j=0; j<8; j++)
	      //	{
	      //	  cout << b2[j]; 
	      //	}
	      // cout << " ";
	      // for (j=8; j<16; j++)
	      //	{
	      //	  cout << b2[j]; 
	      //	}
	      // cout << " ";
	      // for (j=16; j<24; j++)
	      //	{
	      //	  cout << b2[j]; 
	      //	}
	      // cout << " ";
	      // for (j=24; j<32; j++)
	      //	{
	      //  cout << b2[j]; 
	      //	}
	      // cout << endl;
	      // Check hot channels (GetHot 0-hot, 1-normal !!)
	      for (j=0; j<32; j++) {b1[j] = (b1[j] & GetHot(iCH++, FED_id, plane, 1));}
	      for (j=0; j<32; j++) {b2[j] = (b2[j] & GetHot(iCH++, FED_id, plane, 1));}
              // Fill top pad hits (ABCD)
	      if (GetPosition(FED_id)==1) 
		{ 
		  npmx = ArtePointer<GEDE>(plane)->nwi[2];
		  mdfillp_(plane, (unsigned int)b1, (unsigned int)b2, npmx); 
		}
	      if (FED_id == 0x122 && (i-i_offset-2) == 0) // 3.2-19, 3.2-99
		{
		  p99 = 1;
		  npmx = 6; // 6 pads
		  a1[0] = 0;
		  for (j=1; j<=6; j++) {a1[j] = b1[j];}
		  for (j=7; j<=31; j++) {a1[j] = 0;}
		  a2[0] = 0;
		  for (j=1; j<=6; j++) {a2[j] = b2[j];}
		  for (j=7; j<=31; j++) {a2[j] = 0;}
		  mdfillp_(GetGede(3060), (unsigned int)a1, (unsigned int)a2, npmx); // 3.2-99
		  npmx = 19; // 19 pads
                  a1[0] = 0; a1[16] = 0;
		  for (j=1; j<=4; j++) {a1[j] = b1[j+11];}
		  for (j=5; j<=15; j++) {a1[j] = b1[j+12];}
		  for (j=17; j<=20; j++) {a1[j] = b1[j+11];}
		  for (j=21; j<=31; j++) {a1[j] = 0;}
		  mdfillp_(GetGede(3030), (unsigned int)a1, (unsigned int)a2, npmx); // 3.2-19
		}
	      if (FED_id == 0x232 && (i-i_offset-2) == 0) // 4.2-19, 4.2-99
		{
		  p99 = 1;
		  npmx = 6; // 6 pads
		  a1[0] = 0;
		  for (j=1; j<=6; j++) {a1[j] = b1[j];}
		  for (j=7; j<=31; j++) {a1[j] = 0;}
		  a2[0] = 0;
		  for (j=1; j<=6; j++) {a2[j] = b2[j];}
		  for (j=7; j<=31; j++) {a2[j] = 0;}
		  mdfillp_(GetGede(4060), (unsigned int)a1, (unsigned int)a2, npmx); // 4.2-99
		  npmx = 19; // 19 pads
                  a1[0] = 0; a1[16] = 0;
		  for (j=1; j<=5; j++) {a1[j] = b1[j+10];}
		  for (j=6; j<=15; j++) {a1[j] = b1[j+11];}
		  for (j=17; j<=20; j++) {a1[j] = b1[j+10];}
		  for (j=21; j<=31; j++) {a1[j] = 0;}
		  mdfillp_(GetGede(4030), (unsigned int)a1, (unsigned int)a2, npmx); // 4.2-19
		}
	      // Fill bottom pad hits (CDAB)
	      if (GetPosition(FED_id)==0 && p99 != 1) 
		{ 
 		  npmx = ArtePointer<GEDE>(plane)->nwi[2];
		  mdfillp_(plane, (unsigned int)b1, (unsigned int)b2, npmx); 
		}
	      // Fill -X pad hits (top CDAB, bottom ABCD)
	      if (GetPosition(FED_id)==2) 
		{ 
 		  npmx = ArtePointer<GEDE>(plane)->nwi[2];
		  mdfillp_(plane, (unsigned int)b2, (unsigned int)b1, npmx);  // b1 <--> b2 ! 
		}
	      // Fill -X Mp3 pad hits (mix) 27 pads
	      if (GetPosition(FED_id)==3) 
		{ 
		  for (j=4; j<16; j++) { a1[j-3] = b1[j]; a2[j-3] = b2[j]; }
		  for (j=17; j<20; j++) { a1[j-4] = b1[j]; a2[j-4] = b2[j]; }
		  for (j=20; j<32; j++) { a1[j-3] = b1[j]; a2[j-3] = b2[j]; }
 		  npmx = ArtePointer<GEDE>(plane)->nwi[2];
		  mdfillp_(plane, (unsigned int)a2, (unsigned int)a1, npmx);  // a1 <--> a2 ! 
		}
	      // Fill -X Mp4 pad hits (mix) 26 pads
	      if (GetPosition(FED_id)==4) 
		{ 
		  for (j=4; j<16; j++) { a1[j-3] = b1[j]; a2[j-3] = b2[j]; }
		  for (j=17; j<20; j++) { a1[j-4] = b1[j]; a2[j-4] = b2[j]; }
		  for (j=20; j<31; j++) { a1[j-3] = b1[j]; a2[j-3] = b2[j]; }
 		  npmx = ArtePointer<GEDE>(plane)->nwi[2];
		  mdfillp_(plane, (unsigned int)a2, (unsigned int)a1, npmx);  // a1 <--> a2 ! 
		}
	    }
	  if ((GetOffset(FED_id) == 1) || (plane == 0)) { iCH += 64; } // Skip
	}
      i_offset += 34; 
    }

  // Last position for muon hits in HITB
  n2 = ArtePointer<HITB>::nrow();

  mdsort_(n1, n2);

}
