// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// class rangerRMap
// stores history information for R table objects

#ifndef rangerRMap_HH
#define rangerRMap_HH

#include <list.h>
#include "rang/HitMap.hh"
#include "arte/arte.hh"


class rangerRMap {

public:
  rangerRMap(){};
  rangerRMap(ArtePointer<RSEG> rseg);
  rangerRMap(ArtePointer<RTRA> rtra);
  rangerRMap(const rangerRMap& rm);  // copy constructor
  rangerRMap& operator=(const rangerRMap& rm); // assignment
  ~rangerRMap(){};

  void addHitMap(const rangerHitMap& hm);
  void prepareIdPnt();  // create or relate RPNTs for particle id
  void addPnt(rangerNode* np);  // ensure RPNT assignment in hit map
  const ArtePointer<RHIT> getRhit(rangerNode* np) const;
  const ArtePointer<RPNT> getRpnt(rangerNode* np) const;
  const ArtePointer<RTRA> getRtra() const { return rt; };
  const ArtePointer<RSEG> getRseg() const { return rs; };
  const list<rangerHitMap> & getHMList() { return HMList; };
  rangerNode* getNodeZLow() const;  // get node of lowest   z in RMap
  rangerNode* getNodeZHigh() const;  // get node of highest z in RMap
  double chiSq() const;  // calculate total chiSquare for nodes in map
  void dump() const;

private:
  // rt and rs are mutually exclusive
  ArtePointer<RTRA> rt;
  ArtePointer<RSEG> rs;
  list<rangerHitMap> HMList;
  list<ArtePointer<RPNT> > RPList;

  friend ostream& operator<<(ostream &os,const rangerRMap &rmap);

};

#endif


