// 
//  Author:  Rainer Mankel                     Created:  07-Mar-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 


#include "rang/DomObj.hh"
#include "rang/Domain.hh"
#include "rang/DomWin.hh"
#include "rang/DomLay.hh"
#include "rang/DomSeg.hh"
#include "rang/SNM.hh"
#include "rang/SNMList.hh"
#include "rang/SeedTableau.hh"
#include "rang/SeedSet.hh"
#include "rang/rginit.hh"
#include "rang/Geom.hh"
#include "rang/NavItem.hh"
#include "rang/XPro.hh"
#include "rang/XWin.hh"
#include "rang/VLTSet.hh"
#include "rang/VLT.hh"
#include "rang/CompLay.hh"
#include "rang/CompLaySet.hh"
#include "rang/rgzone.hh"

#undef PREVPC1

extern rangerDomObj* rangerDO;   // global DO pointer

void rangerMakeSeed(string dName,const int direction) {
  vector<rangerDomain*>::iterator di=rangerDO->findDom(dName);
  if (di != rangerDO->dom().end()) {
    rangerSeedTableau* st = new  rangerSeedTableau(*di,direction);
    if (st) {
      for (vector <rangerDomWin*>::iterator dw=(*di)->win().begin();
	   dw != (*di)->win().end(); ++dw) {

	if ((*dw)->cmp() == 2) {  // outer tracker seeds
	  rangerSeedSet* ss = new rangerSeedSet(*dw,1,direction,1,2,3);
	  if (ss && ss->OK()) {
	    st->appSeedSet(ss);
	  }
	  else {
	    delete ss;
	  }
	  ss = new rangerSeedSet(*dw,1,direction,1,2,4);
	  if (ss && ss->OK()) {
	    st->appSeedSet(ss);
	  }
	  else {
	    delete ss;
	  }
	  ss = new rangerSeedSet(*dw,2,direction,1,2,4);
	  if (ss && ss->OK()) {
	    st->appSeedSet(ss);
	  }
	  else {
	    delete ss;
	  }
	  ss = new rangerSeedSet(*dw,2,direction,1,3,4);
	  if (ss && ss->OK()) {
	    st->appSeedSet(ss);
	  }
	  else {
	    delete ss;
	  }
	}
	else {      // inner tracker seeds
	  rangerSeedSet* ss = new rangerSeedSet(*dw,1,direction,1,3,4);
	  if (ss && ss->OK()) {
	    st->appSeedSet(ss);
	  }
	  else {
	    delete ss;
	  }
	  ss = new rangerSeedSet(*dw,1,direction,2,3,4);
	  if (ss && ss->OK()) {
	    st->appSeedSet(ss);
	  }
	  else {
	    delete ss;
	  }
	  ss = new rangerSeedSet(*dw,2,direction,1,2,4);
	  if (ss && ss->OK()) {
	    st->appSeedSet(ss);
	  }
	  else {
	    delete ss;
	  }
	  ss = new rangerSeedSet(*dw,2,direction,1,3,4);
	  if (ss && ss->OK()) {
	    st->appSeedSet(ss);
	  }
	  else {
	    delete ss;
	  }
	}
      }
      rangerDO->appSeedTab(st);
    }
  }
}




extern "C" {
  void rgdom_() {
    if (!rangerDO) {
      cout << "rgdom: rangerDO=" << rangerDO << endl;
      return;
    }

    // test initialization
    rginit_();
    rangerGeom::condUpdate();

    // loop over default domains
    for (vector<rangerDomain*>::iterator di=rangerDO->def().begin();
	 di != rangerDO->def().end(); ++di) {
      rangerDomain* dn = (*di)->cloneSl();
      dn->updSls();
      dn->addWin(1,1,8,-0.060,0.060,1);
      dn->addWin(2,1,12,-0.002,0.002,20);
      dn->addWin(3,1,8,-0.002,0.002,1);
      rangerDO->appDom(dn);
      if (dn->sco() == 3) {
	rangerDomSeg* ds = new rangerDomSeg(dn);
	rangerDO->appDomSeg(ds);
	dn->setDomSeg(ds);
      }
      //  cout << "rgdom: appended domain " << *dn << endl;
#ifdef DOMPRINT
      dn->dump();
#endif
      //...
    }

    // define sector neighbourhood mask
    rangerSNMList SNMs;

    // outer tracker sectors
    SNMs.add(new rangerSNM(2,1,2,9,2,7));
    SNMs.add(new rangerSNM(2,2,2,8,2,10));
    SNMs.add(new rangerSNM(2,3,2,7,3,2));
    SNMs.add(new rangerSNM(2,4,2,8,3,4));
    SNMs.add(new rangerSNM(2,5,2,9,3,7));
    SNMs.add(new rangerSNM(2,6,2,10,3,5));
    SNMs.add(new rangerSNM(2,7,2,1,2,3,2,11));
    SNMs.add(new rangerSNM(2,8,2,2,2,4,2,11));
    SNMs.add(new rangerSNM(2,9,2,1,2,5,2,12));
    SNMs.add(new rangerSNM(2,10,2,2,2,6,2,12));
    SNMs.add(new rangerSNM(2,11,2,7,2,8));
    SNMs.add(new rangerSNM(2,12,2,9,2,10));
    // inner tracker sectors
    SNMs.add(new rangerSNM(3,1,3,2,3,8));
    SNMs.add(new rangerSNM(3,2,2,3,3,1));
    SNMs.add(new rangerSNM(3,3,3,4,3,6));
    SNMs.add(new rangerSNM(3,4,2,4,3,3));
    SNMs.add(new rangerSNM(3,5,2,6,3,6));
    SNMs.add(new rangerSNM(3,6,3,3,3,5));
    SNMs.add(new rangerSNM(3,7,2,5,3,8));
    SNMs.add(new rangerSNM(3,8,3,1,3,7));
    // vds sectors
    SNMs.add(new rangerSNM(1,1));
    SNMs.add(new rangerSNM(1,2));
    SNMs.add(new rangerSNM(1,3));
    SNMs.add(new rangerSNM(1,4));
    
//     cout << " SNM test..." << endl;
//     SNMs.test(3);
//     cout << " .............." << endl;

    // create VLTs
//     rangerVLTSet vs1;
//     vs1.append(new rangerVLT(2,1,2,3,4,5,6));
//     vs1.append(new rangerVLT(3,1,6,2,3,5,6));

//     rangerVLTSet vs2;
//     vs2.append(new rangerVLT(2,1,3,4,6,5,2));
//     vs2.append(new rangerVLT(3,1,2,3,4,5,6));

    // define some composite layers
    rangerCompLaySet clset_MC1;
    clset_MC1.append
      ( new rangerCompLay( make_pair(1,rangerSubLayerSet(3,1)),
			   make_pair(1,rangerSubLayerSet(1,1)) ) );
    clset_MC1.append
      ( new rangerCompLay( make_pair(1,rangerSubLayerSet(2,1)),
                           make_pair(3,rangerSubLayerSet(3,3)),
			   make_pair(3,rangerSubLayerSet(1,3)) ) );
    clset_MC1.append
      ( new rangerCompLay( make_pair(2,rangerSubLayerSet(2,2)),
                           make_pair(4,rangerSubLayerSet(3,2)),
			   make_pair(4,rangerSubLayerSet(1,2)) ) );
    clset_MC1.append
      ( new rangerCompLay( make_pair(3,rangerSubLayerSet(2,3)),
                           make_pair(2,rangerSubLayerSet(3,2)),
			   make_pair(2,rangerSubLayerSet(1,2)) ) );



    rangerCompLaySet clset_PC1;
    clset_PC1.append
      ( new rangerCompLay( make_pair(1,rangerSubLayerSet(2,1)),
			   make_pair(1,rangerSubLayerSet(3,1)) ) );
    clset_PC1.append
      ( new rangerCompLay( make_pair(2,rangerSubLayerSet(2,2)),
			   make_pair(3,rangerSubLayerSet(3,3)) ) );
    clset_PC1.append
      ( new rangerCompLay( make_pair(3,rangerSubLayerSet(2,3)),
			   make_pair(2,rangerSubLayerSet(3,2)) ) );
    clset_PC1.append
      ( new rangerCompLay( make_pair(4,rangerSubLayerSet(2,4,1,2)),
			   make_pair(6,rangerSubLayerSet(3,6)) ) );
    clset_PC1.append
      ( new rangerCompLay( make_pair(5,rangerSubLayerSet(2,4,3,4)),
			   make_pair(7,rangerSubLayerSet(3,7)) ) );
    clset_PC1.append
      ( new rangerCompLay( make_pair(6,rangerSubLayerSet(2,5,1,2)),
			   make_pair(5,rangerSubLayerSet(3,5)) ) );
    clset_PC1.append
      ( new rangerCompLay( make_pair(7,rangerSubLayerSet(2,5,3,4)),
			   make_pair(8,rangerSubLayerSet(3,8)) ) );
    clset_PC1.append
      ( new rangerCompLay( make_pair(8,rangerSubLayerSet(2,6,1,4)),
			   make_pair(4,rangerSubLayerSet(3,4)) ) );

    //clset_PC1.dump(9);


    rangerCompLaySet clset_PC2;
    clset_PC2.append
      ( new rangerCompLay( make_pair(1,rangerSubLayerSet(2,1)),
			   make_pair(1,rangerSubLayerSet(3,1)) ) );
    clset_PC2.append
      ( new rangerCompLay( make_pair(2,rangerSubLayerSet(2,2)),
			   make_pair(3,rangerSubLayerSet(3,3)) ) );
    clset_PC2.append
      ( new rangerCompLay( make_pair(3,rangerSubLayerSet(2,3)),
			   make_pair(4,rangerSubLayerSet(3,4)) ) );
    clset_PC2.append
      ( new rangerCompLay( make_pair(4,rangerSubLayerSet(2,4)),
			   make_pair(2,rangerSubLayerSet(3,2)) ) );
    clset_PC2.append
      ( new rangerCompLay( make_pair(5,rangerSubLayerSet(2,5)) ) );
    clset_PC2.append
      ( new rangerCompLay( make_pair(6,rangerSubLayerSet(2,6)) ) );

    rangerCompLaySet clset_PC4;
    clset_PC4.append
      ( new rangerCompLay( make_pair(1,rangerSubLayerSet(2,1)),
			   make_pair(1,rangerSubLayerSet(3,1)) ) );
    clset_PC4.append
      ( new rangerCompLay( make_pair(2,rangerSubLayerSet(2,2)),
			   make_pair(3,rangerSubLayerSet(3,3)) ) );
    clset_PC4.append
      ( new rangerCompLay( make_pair(3,rangerSubLayerSet(2,3)),
			   make_pair(2,rangerSubLayerSet(3,2)) ) );
    clset_PC4.append
      ( new rangerCompLay( make_pair(4,rangerSubLayerSet(2,4,1,2)),
			   make_pair(6,rangerSubLayerSet(3,6)) ) );
    clset_PC4.append
      ( new rangerCompLay( make_pair(5,rangerSubLayerSet(2,4,3,4)),
			   make_pair(7,rangerSubLayerSet(3,7)) ) );
    clset_PC4.append
      ( new rangerCompLay( make_pair(6,rangerSubLayerSet(2,5,1,2)),
			   make_pair(5,rangerSubLayerSet(3,5)) ) );
    clset_PC4.append
      ( new rangerCompLay( make_pair(7,rangerSubLayerSet(2,5,3,4)),
			   make_pair(8,rangerSubLayerSet(3,8)) ) );
    clset_PC4.append
      ( new rangerCompLay( make_pair(8,rangerSubLayerSet(2,6,1,4)),
			   make_pair(4,rangerSubLayerSet(3,4)) ) );
    //clset_PC4.dump(9);
	
    rangerCompLaySet clset_TC1;
    clset_TC1.append
      ( new rangerCompLay( make_pair(1,rangerSubLayerSet(2,1,1,2)),
			   make_pair(3,rangerSubLayerSet(3,3)) ) );
    clset_TC1.append
      ( new rangerCompLay( make_pair(2,rangerSubLayerSet(2,1,3,4)),
			   make_pair(4,rangerSubLayerSet(3,4)) ) );
    clset_TC1.append
      ( new rangerCompLay( make_pair(3,rangerSubLayerSet(2,2,1,2)),
			   make_pair(1,rangerSubLayerSet(3,1)) ) );
    clset_TC1.append
      ( new rangerCompLay( make_pair(4,rangerSubLayerSet(2,2,3,4)),
			   make_pair(2,rangerSubLayerSet(3,2)) ) );
    clset_TC1.append
      ( new rangerCompLay( make_pair(5,rangerSubLayerSet(2,3,1,2)),
			   make_pair(5,rangerSubLayerSet(3,5)) ) );
    clset_TC1.append
      ( new rangerCompLay( make_pair(6,rangerSubLayerSet(2,3,3,4)),
			   make_pair(6,rangerSubLayerSet(3,6)) ) );
    //clset_TC1.dump(9);

    // create the domain layers
    for (vector<rangerDomain*>::iterator di=rangerDO->dom().begin();
	 di != rangerDO->dom().end(); ++di) {
#ifdef DOMPRINT
      cout << "Make domain layers for " << *(*di) << endl;
#endif
      for (vector <rangerDomWin*>::iterator wi=(*di)->win().begin();
	   wi != (*di)->win().end(); ++wi) {
	rangerSNM* s = SNMs.getSNM((*wi)->cmp(),(*wi)->sec());
	if (s) {
	  for (int gLayer=1; gLayer<=10; ++gLayer) {
// 	    cout << "domain " << *(*di) << " window cmp=" << (*wi)->cmp()
// 		 << " sector " << (*wi)->sec() << " layer " << gLayer
// 		 << " : " << endl;
	    // tentative creation of new domain layer
	    rangerDomLay* dl = new rangerDomLay(*wi);
	    if (! dl) break;
	    // collect planes for a layer
	    int nAcc = 0;
	    
	    if ((*di)->nam() == "MC1") {
	      nAcc = dl->accumComp(*wi,gLayer,s,clset_MC1,-1);
	    }
	    else if ((*di)->nam() == "PC1") {
	      nAcc = dl->accumComp(*wi,gLayer,s,clset_PC1,-1);
#ifdef VERBOSE
	      cout << "rgdom: layer planes for PC1 win " << *(*wi)
		   << " gLayer " << gLayer << " nAcc=" << nAcc << endl;
#endif
	    }
	    else if ((*di)->nam() == "PC4") {
	      nAcc = dl->accumComp(*wi,gLayer,s,clset_PC4,-1);
#ifdef VERBOSE
	      cout << "rgdom: layer planes for PC4 win " << *(*wi)
		   << " gLayer " << gLayer << " nAcc=" << nAcc << endl;
#endif
	    }
	    else if ((*di)->nam() == "PC2" || (*di)->nam() == "PC3") {
	      nAcc = dl->accumComp(*wi,gLayer,s,clset_PC2,-1);
	    }
	    else if ((*di)->nam() == "TC1" || (*di)->nam() == "TC2") {
	      nAcc = dl->accumComp(*wi,gLayer,s,clset_TC1,-1);
	    }
	    else {
	      nAcc = dl->accum(*wi,gLayer,s,-1);
	    }

	    // check native layer planes
	    int nNative = dl->cntLP((*wi)->cmp(),(*wi)->sec());
	    if (nAcc && nNative) {
	      (*wi)->addDomLay(dl);  // keep
	      dl->setHL(rangerDO->makeHL(dl));  // prepare hitlist entity
// 	      cout << "  added " << nAcc << " domain layer planes" << endl;
	    }
	    else {
	      delete dl;          // or destroy
	      break;
	    }
	  }
	}
// 	cout << "call updLsn for this window" << endl;
	(*wi)->updLsn();

	// add entrance/exit layers for navigation
	(*wi)->addEntrance(+1,new rangerDomLay(*wi));
	(*wi)->addEntrance(-1,new rangerDomLay(*wi));

// 	(*wi)->dump(5);
      }

#ifdef PROXY
      // add proxy windows and layers
      if ((*di)->sco() >=2 && (*di)->sco() <=4) {
	int nProxy = 0;
	nProxy += (*di)->addProxy(2,9,2,5.,2,10); // proxies for 10mm sectors
	nProxy += (*di)->addProxy(2,10,1,5.,2,9);
	nProxy += (*di)->addProxy(2,7,2,5.,2,8);
	nProxy += (*di)->addProxy(2,8,1,5.,2,7);
	nProxy += (*di)->addProxy(2,5,2,5.,2,6);  // proxies for 5mm middle
	nProxy += (*di)->addProxy(2,6,1,5.,2,5);
	nProxy += (*di)->addProxy(2,3,2,5.,2,4);
	nProxy += (*di)->addProxy(2,4,1,5.,2,3);

	nProxy += (*di)->addProxy(2,1,2,5.,2,5);  // proxies for 4fold division
	nProxy += (*di)->addProxy(2,5,1,5.,2,1);
	nProxy += (*di)->addProxy(2,1,2,5.,2,3);
	nProxy += (*di)->addProxy(2,3,1,5.,2,1);

	nProxy += (*di)->addProxy(2,6,2,5.,2,2);
	nProxy += (*di)->addProxy(2,2,1,5.,2,6);
	nProxy += (*di)->addProxy(2,4,2,5.,2,2);
	nProxy += (*di)->addProxy(2,2,1,5.,2,4);

	nProxy += (*di)->addProxy(2,1,2,5.,3,1);  // proxies outer/inner tr
	nProxy += (*di)->addProxy(2,1,2,5.,3,2);  // above
	nProxy += (*di)->addProxy(2,1,2,5.,3,7);
	nProxy += (*di)->addProxy(2,1,2,5.,3,8);

	nProxy += (*di)->addProxy(3,1,1,5.,2,1);  // reverse
	nProxy += (*di)->addProxy(3,2,1,5.,2,1);
	nProxy += (*di)->addProxy(3,7,1,5.,2,1);
	nProxy += (*di)->addProxy(3,8,1,5.,2,1);

	nProxy += (*di)->addProxy(2,2,1,5.,3,3);  // proxies outer/inner tr
	nProxy += (*di)->addProxy(2,2,1,5.,3,4);  // below
	nProxy += (*di)->addProxy(2,2,1,5.,3,5);
	nProxy += (*di)->addProxy(2,2,1,5.,3,6);

	nProxy += (*di)->addProxy(3,3,2,5.,2,2);  // reverse
	nProxy += (*di)->addProxy(3,4,2,5.,2,2);
	nProxy += (*di)->addProxy(3,5,2,5.,2,2);
	nProxy += (*di)->addProxy(3,6,2,5.,2,2);

	nProxy += (*di)->addProxy(3,2,2,5.,3,4);  // proxies for inner tracker
	nProxy += (*di)->addProxy(3,4,1,5.,3,2);
	nProxy += (*di)->addProxy(3,7,2,5.,3,5);
	nProxy += (*di)->addProxy(3,5,1,5.,3,7);


	int nProxyLay = (*di)->setProxyLayers();
	cout << " Domain " << (*di)->nam() << " proxy windows created "
	     << nProxy << " layers " << nProxyLay << endl;
      }
#endif
    }

    // create domain window navigation table

    // define navigation mask
    rangerSNMList NNMs;

    // outer tracker sectors
    NNMs.add(new rangerSNM(2,1,2,9,2,7,2,3,2,5,3,7,3,8,3,1,3,2));
    NNMs.add(new rangerSNM(2,2,2,8,2,10,2,4,2,6,3,5,3,6,3,3,3,4));
    NNMs.add(new rangerSNM(2,3,2,7,3,2,2,4,2,1));
    NNMs.add(new rangerSNM(2,4,2,8,3,4,2,3,2,2));
    NNMs.add(new rangerSNM(2,5,2,9,3,7,2,6,2,1));
    NNMs.add(new rangerSNM(2,6,2,10,3,5,2,5,2,2));
    NNMs.add(new rangerSNM(2,7,2,1,2,3,2,11,2,8,2,4));
    NNMs.add(new rangerSNM(2,8,2,2,2,4,2,11,2,7,2,3));
    NNMs.add(new rangerSNM(2,9,2,1,2,5,2,12,2,10,2,6));
    NNMs.add(new rangerSNM(2,10,2,2,2,6,2,12,2,9,2,5));
    NNMs.add(new rangerSNM(2,11,2,7,2,8));
    NNMs.add(new rangerSNM(2,12,2,9,2,10));
    // inner tracker sectors
    NNMs.add(new rangerSNM(3,1,3,2,3,8,2,1,1,1,1,2));
    NNMs.add(new rangerSNM(3,2,2,3,3,1,2,1,3,8,3,4,2,4,1,1,1,2));
    NNMs.add(new rangerSNM(3,3,3,4,3,6,2,2,1,3,1,2));
    NNMs.add(new rangerSNM(3,4,2,4,3,3,2,2,3,6,3,2,2,3,1,2,1,3));
    NNMs.add(new rangerSNM(3,5,2,6,3,6,2,2,3,3,3,7,2,5,1,3,1,4));
    NNMs.add(new rangerSNM(3,6,3,3,3,5,2,2,1,3,1,4));
    NNMs.add(new rangerSNM(3,7,2,5,3,8,2,1,3,1,3,5,2,6,1,4,1,1));
    NNMs.add(new rangerSNM(3,8,3,1,3,7,2,1,1,1,1,4));
    // vds sectors
    NNMs.add(new rangerSNM(1,1,1,2,1,4));
    NNMs.add(new rangerSNM(1,2,1,1,1,3));
    NNMs.add(new rangerSNM(1,3,1,2,1,4));
    NNMs.add(new rangerSNM(1,4,1,3,1,1));

//     NNMs.test(3);
//     NNMs.dump(3);

    // loop over domain windows
    for (vector<rangerDomain*>::iterator di=rangerDO->dom().begin();
	 di != rangerDO->dom().end(); ++di) {
      for (vector <rangerDomWin*>::iterator wIt=(*di)->win().begin();
	   wIt != (*di)->win().end(); ++wIt) {
	rangerNavItem* ni = new rangerNavItem(*wIt);
	if (! ni) break;
	rangerDO->appNav(ni);
	(*wIt)->setNI(ni);
      }
    }

#ifdef DOMPRINT
    cout << " rgdom: number of NavItems " << rangerDO->nav().size()
	 << endl;
#endif
    for (vector<rangerNavItem*>::iterator nii=rangerDO->nav().begin();
	 nii != rangerDO->nav().end(); ++nii) {
      if ((*nii)->win()->dom()->sco() == 1) {
	(*nii)->setNavTable(rangerDO->dom().begin(),rangerDO->dom().end(),
			    NNMs,1,1,5);
      }
      else {
	(*nii)->setNavTable(rangerDO->dom().begin(),rangerDO->dom().end(),
			    NNMs,2,4,5);
      }
//       (*nii)->dump(2);
    }
    

    // Prepare seeds
    rangerMakeSeed("PC4",-1);
    rangerMakeSeed("PC1",+1);

#ifdef NAVCHECK
    // check the navigation table

    // locate domain PC4
    for (vector<rangerDomain*>::iterator di=rangerDO->dom().begin();
	 di != rangerDO->dom().end(); ++di) {
      if ((*di)->nam() == "PC4") {
	int method = 0;
	int dir = +1;
	int zone = 6;
	rangerDomLay* dl = (*(*di)->win().begin())->firstDL(method,1);
	cout << " look at window " << endl;
	(*(((*di)->win()).begin()))->dump();
	if (dl) {
	  dl->dump();
	  rangerDomLay* targetVec[100];
	  int nTarget = 0;
	  rgzone_(dl,method,zone,dir,3,3,1,100,targetVec,nTarget);
	  cout << " number of targets=" << nTarget << endl;
	  for (int i=0; i<nTarget && i<100; ++i) {
	    targetVec[i]->win()->dump();
	    targetVec[i]->dump();
	  }
	}
      }
    }
#endif

    // build the x projection structure
    for (vector<rangerDomain*>::iterator di=rangerDO->dom().begin();
	 di != rangerDO->dom().end(); ++di) {
      if ((*di)->sco() == 3) {
	rangerXWin* xwi = new rangerXWin((*di));
	rangerDO->xpro()->appXWin(xwi);
	(*di)->setXWin(xwi);
      }
    }

#ifdef DOMPRINT
    cout << "rgdom: number of domains: " << rangerDO->dom().size() << endl;
    int nWin = 0;
    for (vector<rangerDomain*>::iterator di=rangerDO->dom().begin();
	 di != rangerDO->dom().end(); ++di) {
      nWin += (*di)->win().size();
    }
    cout << "rgdom: number of windows: " << nWin  << endl;
    cout << "rgdom: number of hitlist items:" 
	 << (rangerDO->hls()).size() << endl;
    cout << "rgdom: number of seed tableaus : " 
	 << (rangerDO->stab()).size() 
	 << " seed sets : " << rangerDO->nSeedSets() << endl;
    cout << "rgdom: number of XWin items : "
	 << (rangerDO->xpro()->xws()).size() << endl;
#endif
    //exit(1);
    //rangerDO->dump(5);
  }
}

    
