// 
//  Author:  Rainer Mankel                     Created:  23-Apr-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

#include <math.h>
#include "rang/Geom.hh"
#include "rang/DomObj.hh"
#include "rang/DomLay.hh"
#include "rang/HL.hh"
#include "rang/zfpar.hh"
#include "rang/zcut.hh"
#include "rang/rgxseg.hh"
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

extern rangerDomObj* rangerDO;   // global DO pointer

extern "C" {
  void rghl_(rangerDomLay*& dl,
	       const int& maxHits,
	       ArtePointer<HITB>* hitVec,
	       int& nHits) {

    if (!rangerDO) {
      cout << "rghl: rangerDO=" << rangerDO << endl;
      return;
    }

    nHits = 0;

    // loop over hits in domain layer
    for (vector<ArtePointer<HITB> >::const_iterator hi=dl->hl()->h().begin();
	 hi != dl->hl()->h().end(); ++hi) {

      // remove hit if already used
      if (zfpar_.segstor) {
	if (((*hi)->rseg).begin() != ((*hi)->rseg).end()) continue;
      }
      else {
	if (((*hi)->rtra).begin() != ((*hi)->rtra).end()) continue;
      }

      // put into array
      if (nHits < maxHits) {
	hitVec[nHits] = (*hi);
	++nHits;
      }
      else {
	arteMsg << "rghl: overflowing nhl, some hits skipped" << endl;
	arteMsg.warn();
	break;
      }
    }
  }
}
    



