// 
//  Author:  Rainer Mankel                     Created:  20-Apr-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

#include <math.h>
#include "rang/rginit.hh"
#include "rang/Geom.hh"
#include "rang/DomObj.hh"
#include "rang/DomWin.hh"
#include "rang/DomLay.hh"
#include "rang/LayPl.hh"
#include "rang/HL.hh"

extern rangerDomObj* rangerDO;   // global DO pointer

extern "C" {
  void rghlevnt_() {

    if (!rangerDO) {
      cout << "rgdom: rangerDO=" << rangerDO << endl;
      return;
    }

    // test initialization
    rginit_();
    rangerGeom::condUpdate();

    // loop over hit lists
    int nAdded = 0;
    int nhls = 0;
    for (vector<rangerHL*>::iterator hli=rangerDO->hls().begin();
	 hli != rangerDO->hls().end(); ++hli) {
      ++nhls;
      (*hli)->erase();

      // associated layer planes
      if (! (*hli)->dl()) continue;
      for (vector<rangerLayPl*>::const_iterator 
	     lpi=(*hli)->dl()->LPs().begin();
	   lpi != (*hli)->dl()->LPs().end(); ++lpi) {

	// hits in layer plane
#ifdef LPHITS
	if ((*hli)->dl()->win()->dom()->nam() == "PC1") {
	  cout << "rghlevnt: for " <<  " c/s=" << (*hli)->dl()->win()->cmp() 
	       << "/"
	       << (*hli)->dl()->win()->sec() 
	       << " lay " << (*hli)->dl()->getLsn(0)
	       << " ge=" << (*lpi)->ge() << " hitb= ";
	}
#endif
	for (ArteTable<HITB>::iterator hi=(*lpi)->ge()->hit1;
	     hi != (*lpi)->ge()->hitl+1; ++hi) {

	  if ((*lpi)->ge()->cmp == 2 && (*lpi)->ge()->nsl <= 4) {  
	    // check sublayer for OT
	    int subl=0;
	    float minDist = 9999;
	    for (int isubl=0; isubl<(*lpi)->ge()->nsl; ++isubl) {
	      float dz = fabs((*lpi)->ge()->zfw[isubl] - (*hi)->z);
	      if (dz < minDist) {
		subl = isubl + 1;
		minDist = dz;
	      }
	    }
	    if (subl < (*lpi)->subgl1() || subl > (*lpi)->subgl2()) continue;
	  }
	  
	  if ((*hi)->x > (*lpi)->umin() && (*hi)->x < (*lpi)->umax()) {
#ifdef LPHITS
	    if ((*hli)->dl()->win()->dom()->nam() == "PC1") { 
	      cout << *hi << " ";
	    }
#endif
	    (*hli)->addHit(*hi);
	    ++nAdded;
	  }
	}
#ifdef LPHITS
 	if ((*hli)->dl()->win()->dom()->nam() == "PC1") cout << endl;
#endif
      }
    }
    cout << " rghlevnt: # domain hits stored=" << nAdded
	 << " in " << nhls << " domain layers" << endl;

    for (vector<rangerDomain*>::iterator di=rangerDO->dom().begin();
	 di != rangerDO->dom().end(); ++di) {
      int nhd = 0;
      for (vector <rangerDomWin*>::iterator dw=(*di)->win().begin();
	   dw != (*di)->win().end(); ++dw) {
	for (vector <rangerDomLay*>::iterator dl=(*dw)->dls().begin();
	     dl != (*dw)->dls().end(); ++dl) {
	  if ((*dl)->hl()) nhd += (*dl)->hl()->h().size();
#ifdef PCOUT
	  if (((*di)->nam()) == "PC1") {
	    cout << ((*di)->nam()) << " c/s=" << (*dw)->cmp() << "/"
		 << (*dw)->sec() << " lay " 
		 << dl-(*dw)->dls().begin()+1 << " nloc=" 
		 <<  (*dl)->hl()->h().size() << endl;
	  }
#endif
	}
      }
//       cout << " Domain " << (*di)->nam() << " domhits " << nhd << endl;
    }
  }
}

	
      

    
