// ********************************************************************
//
//  Routine:      rc99AStandAlone
//  Author:       T.Oest/D.Samtleben
//
//  Description:  stand alone reconstruction
//
//                reco level 0: no reconstruction
//                           1: hitb only
//                           2: reconstruction only
//                           3: hitb + reconstruction
//
//  Modification:  - drop RICH hits
//                 - recoTARGET,... variables no longer static
//                                                        27/05/99 Oest
//                 - VDS==SVD for kumac-steering
//                 - error-message if given subdetector is not known
//                                                        10/06/99 Samtleben
//                 - if at least one reconstruction package of a subdetector
//                   is called the Rxxx-tables are cleared
//                                                        01/07/99 Samtleben
//
// ********************************************************************

#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include "arte/arte.hh"

#include "reco/rc99Areco.hh"

int recoTARGET  = 0;
int recoVDS     = 0;
int recoOTR     = 0;
int recoECAL    = 0;
int recoMUON    = 0;
int recoRICH    = 0;
int recoTRD     = 0;
int recoHPT     = 0;

//extern "C" void mmdrop_(const int &);

//********************************************************************
// rc99AStandAlone
//********************************************************************
  int rc99AStandAlone() {


    if (recoTARGET) int TARGETflag  = rc99ATARGET(recoTARGET);


//===================================================================
// remove the GTAR (for real data) and Rxxx-tables
//===================================================================
//------  remove GTAR in case of reconstruction in real data

    if (recoTARGET && ArteTable<MTRA>::begin()==ArteTable<MTRA>::end()){
        ArteTable<GTAR>::clear();
}
//------  remove Rxxx-tables
    if(recoVDS||recoECAL||recoRICH||recoHPT||recoTRD||recoMUON){
        ArteTable<RREL>::clear();
        ArteTable<RPNT>::clear();
        ArteTable<RVER>::clear();
        ArteTable<RHIT>::clear();
        ArteTable<RSEG>::clear();
        ArteTable<RTRA>::clear();
        ArteTable<RCCL>::clear();
}
//===================================================================
//  start with the reconstruction where the HITBs can be dropped
//===================================================================
    int RICHflag = 0;

    if (recoRICH)      RICHflag    = rc99ARICH(recoRICH);
        if ( RICHflag == -1 )
//       mmdrop_(ArtePointer<HITB>::getLink().getInt());
      ArteTable<HITB>::clear();

//===================================================================
//  stand alone tracking
//===================================================================

    if (recoVDS) int SVDflag     = rc99AVDS(recoVDS);
    int ITRflag     = 1;
    if (recoOTR)    int OTRflag     = rc99AOTR(recoOTR);

    int HPTflag     = 0;
    if (recoHPT)      HPTflag    = rc99AHPT(recoHPT);

    if (recoMUON)   int MUONflag    = rc99AMUON(recoMUON);

//===================================================================
//  ECAL needs tracks for the 2. reconstruction step
//===================================================================

    if (recoECAL)   int ECALflag    = rc99AECAL(recoECAL);

    int TRDflag     = 0;
    if (recoTRD)   TRDflag    = rc99ATRD(recoTRD);


//===================================================================
//  flag for stand alone reconstruction
//===================================================================

    return 0;

  }


/************************************************************************
* KASUBD
*
* called from  KUIP
***********************************************************************/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "arte/Arte.hh"

extern "C" void kugetc_( char*, int *, int );


extern "C" void kasubd_() {   

//---------------------------------------------------------------
//   read in subdetector
//---------------------------------------------------------------

  int len;        
  int len1 = 6;
  char input[7];
  kugetc_(input, &len, len1);

  char *subdetector;
  subdetector  = (char *) malloc(len+1);
  sprintf(subdetector  ,"%s", input);

  for(int i=0; i<len; i++) subdetector[i] = toupper(subdetector[i]);

//---------------------------------------------------------------
//   read in subdetetcor status
//---------------------------------------------------------------

  len1 = 3;
  kugetc_(input, &len, len1);

  char *status;
  status  = (char *) malloc(len+1);
  sprintf(status  ,"%s", input);

  if(len>0) for(int i=0; i<len; i++) status[i] = toupper(input[i]);

//---------------------------------------------------------------
//   read in HITB flag
//---------------------------------------------------------------

  len1 = 4;
  kugetc_(input, &len, len1);

  char *hitb;
  hitb   = (char *) malloc(len+1);
  sprintf(hitb  ,"%s", input);

  if(len>0) for(int i=0; i<len; i++) hitb[i] = toupper(hitb[i]);

//---------------------------------------------------------------
//   set reconstruction status
//---------------------------------------------------------------
  int istatus;


  if (!strncmp(status,"ON",2)){
    if (!strncmp(hitb,"HITB",4)){
       istatus = 1;
    } else {
       istatus = 3;
    };
  } else {
    if (!strncmp(hitb,"HITB",4)){
       istatus = 2;
    } else {
       istatus = 0;
    };
  };

  if (!strncmp(status,"ON",2)){

   if (!strncmp(subdetector,"VDS"    ,3)) recoVDS    = recoVDS    | istatus;
   if (!strncmp(subdetector,"SVD"    ,3)) recoVDS    = recoVDS    | istatus;
   if (!strncmp(subdetector,"OTR"    ,3)) recoOTR    = recoOTR    | istatus;
   if (!strncmp(subdetector,"ECAL"   ,4)) recoECAL   = recoECAL   | istatus;
   if (!strncmp(subdetector,"MUON"   ,4)) recoMUON   = recoMUON   | istatus;
   if (!strncmp(subdetector,"RICH"   ,4)) recoRICH   = recoRICH   | istatus;
   if (!strncmp(subdetector,"TARG"   ,4)) recoTARGET = recoTARGET | istatus;
   if (!strncmp(subdetector,"TRD"   ,3)) recoTRD = recoTRD | istatus;
   if (!strncmp(subdetector,"HPT"   ,3)) recoHPT = recoHPT | istatus;

   if (!strncmp(subdetector,"ALL" ,3)) {
      recoTARGET  = recoTARGET | istatus;
      recoVDS     = recoVDS    | istatus;
      recoOTR     = recoOTR    | istatus;
      recoECAL    = recoECAL   | istatus;
      recoMUON    = recoMUON   | istatus;
      recoRICH    = recoRICH   | istatus;
      recoTRD     = recoTRD   | istatus;
      recoHPT     = recoHPT   | istatus;
   };
   if(strncmp(subdetector,"VDS",3)&&strncmp(subdetector,"SVD",3)&&
      strncmp(subdetector,"OTR",3)&&strncmp(subdetector,"ECAL",4)&&
      strncmp(subdetector,"MUON",4)&&strncmp(subdetector,"RICH",4)&&
      strncmp(subdetector,"TARG",4)&&strncmp(subdetector,"TRD",3)&&
      strncmp(subdetector,"HPT",3)&&strncmp(subdetector,"ALL",3)){
     cerr << "****** RECO/SUBD:   unknown subdetector!!! ************** "
<< endl;
   }

  } else {

   if (!strncmp(subdetector,"VDS"    ,3)) recoVDS    = recoVDS    & istatus;
   if (!strncmp(subdetector,"OTR"    ,3)) recoOTR    = recoOTR    & istatus;
   if (!strncmp(subdetector,"ECAL"   ,4)) recoECAL   = recoECAL   & istatus;
   if (!strncmp(subdetector,"MUON"   ,4)) recoMUON   = recoMUON   & istatus;
   if (!strncmp(subdetector,"RICH"   ,4)) recoRICH   = recoRICH   & istatus;
   if (!strncmp(subdetector,"TARG"   ,4)) recoTARGET = recoTARGET & istatus;
   if (!strncmp(subdetector,"TRD"   ,3))  recoTRD = recoTRD & istatus;
   if (!strncmp(subdetector,"HPT"   ,3))  recoHPT = recoHPT & istatus;

   if (!strncmp(subdetector,"ALL" ,3)) {
      recoTARGET  = recoTARGET & istatus;
      recoVDS     = recoVDS    & istatus;
      recoOTR     = recoOTR    & istatus;
      recoECAL    = recoECAL   & istatus;
      recoMUON    = recoMUON   & istatus;
      recoRICH    = recoRICH   & istatus;
      recoTRD     = recoTRD    & istatus;
      recoHPT     = recoHPT    & istatus;
   };

  };

};
