// ********************************************************************
//
//  Routine:      rc99ATARGET
//  Author:       T.Oest/D.Samtleben
//
//  Description:  target rates, position
//
//  1998 data: 8 wires
//             1. GTAR table row  Ia    Al
//             2.            row  Ib    Ti
//             3.            row  Ii    Al
//             4.            row  Io    Al
//             5.            row IIa    Fe  (45 degree)
//             6.            row IIb    Al  (45 degree)
//             7.            row IIi    C
//             8.            row IIo    Ti
//
//
//  Modification:
//                        dropping of GTAR is now included in
//                        rc99AStandAlone
//                                             01/07/99   Samtleben
//
// ********************************************************************

#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "arte/arte.hh"

#include "online/esp.h"


static int Nrfile  = 0;
static char * DBfile[100];


extern "C" 
{
#include "mizzi/table.h"
}

//extern "C" void mmdrop_(const int &);
int rc99AfillGTARfromDTAR();

//*****************************************************************
// function, no action for MC
//*****************************************************************

int rc99ATARGET(int level) {

 if (ArteTable<MTRA>::begin() != ArteTable<MTRA>::end()) return 0;

//==================================================================
// 1. drop target banks
//==================================================================

//   mmdrop_(ArtePointer<GTAR>::getLink().getInt());
 
//    ArteTable<GTAR>::clear();

//==================================================================
// 2. get time from DDAQ bank, fill event header bank
//==================================================================
  ArtePointer<DDAQ> ddaq( 2 );
  
  int datime = 0;

  EVT_DATA *p = (EVT_DATA *) &ddaq->bits;
  ESP_get_int( &p->event_time, &datime, 1 );

  struct tm *tmTime = localtime( (time_t * ) &datime );

  ArtePointer<EVHD>(1)->date = 
     10000*tmTime->tm_year + 100*(1+tmTime->tm_mon) + tmTime->tm_mday;
  ArtePointer<EVHD>(1)->time = 
     10000*tmTime->tm_hour + 100*tmTime->tm_min + tmTime->tm_sec;

  if (ArtePointer<EVHD>(1)->exp == 1998)  ArtePointer<EVHD>(1)->exp =1;

//==================================================================
// 3. open target data base
//==================================================================

  if (ArtePointer<EVHD>(1)->date > 990200) {
   return rc99AfillGTARfromDTAR();
    return 0;
  };


  int    flagData        = -1;
  int    EntriesPosition =  0;
  int    EntriesSelect   =  0;

  Tables    x;
  int       tx, major;
  float     WirePosition[8];
  float     Rate;
  char      ByteArray[8];
  int       WireSelect[8];

  for (int i=0; i < Nrfile && flagData != 0 ; i++){

    x = OpenTables(DBfile[i] ,TABLEreadonly);

    if (x) {
//----------------------------------------------------------------   
// get wire position from target/Data
//----------------------------------------------------------------   

     if (flagData != 0) {

     tx = BookTable(x,"target/Data",0,0);

     if (tx) {

      TableStamp(x,datime,0);

      if (FetchTable(x,tx,TABLEge)) {

       major=TableMajor(x);

       if (major == datime){

        SkipTableField(x,1);
        EntriesPosition = GetTableField(x,TABLEfloat,8,WirePosition);
        SkipTableField(x,3);
                          GetTableField(x,TABLEfloat,1,&Rate);

        flagData = 0;

       };};};};

//----------------------------------------------------------------   
// get wire status from target/Auto (same db file)
//----------------------------------------------------------------   

     if (flagData == 0) {

     tx = BookTable(x,"target/Auto",0,0);

     if (tx) {

      TableStamp(x,datime,0);

      if (FetchTable(x,tx,TABLEle))   { major  = TableMajor(x);}
      else { 
        if (FetchTable(x,tx,TABLEge)) { major  = TableMajor(x);}
        else                          { major  = 0; };
      };

      if ( major != 0){

        SkipTableField(x,5);
        EntriesSelect = GetTableField(x,TABLEbyte,8,ByteArray);
        for (int i=0; i<8 ; i++) WireSelect[i] = ByteArray[i];

       };};};


     CloseTables(x);

  };
  };

//==================================================================
// 2. fill target bank
//                       the rate is copied to GTAR_MS 
//==================================================================

  GTAR target;

       float rot[9];
       float rote[9] = {1.0,0.0,0.0, 0.0,1.0,0.0, 0.0,0.0,1.0};

       rot[0] =  sqrt(0.5);
       rot[1] =  sqrt(0.5);
       rot[2] =  0.0;
       rot[3] = -sqrt(0.5);
       rot[4] =  sqrt(0.5);
       rot[5] =  0.0;
       rot[6] =  0.0;
       rot[7] =  0.0;
       rot[8] =  1.0;

  if (!flagData) {

                 target.cmp   = 0;
                 target.x0bm  = 0.0;
                 target.y0bm  = 0.0;
                 target.sxbm  = 0.0;
                 target.sybm  = 0.0;
                 target.shap  = 0;

    for (int i=0; i < EntriesPosition; i++){

      if ( i==1 || i==5 || i==3 || i==7 ){
            WirePosition[i] =  0.1 * WirePosition[i];
       } else {
            WirePosition[i] = -0.1 * WirePosition[i];
       };

                 target.wnr = i+1;
                 target.dz  = 0.05;

       if ( i==0 || i== 1 ) {
                 target.z   = -1.0;
                 target.dx  = 3.0;
                 target.dy  = 0.005;
                 target.x   = 0.;
                 target.y   = WirePosition[i];
                 target.rot = rote;
       };
       if ( i==2 || i== 3 ) {
                 target.z   = -0.7;
                 target.dx  = 0.005;
                 target.dy  = 3.0;
                 target.x   = WirePosition[i];
                 target.y   = 0.;
                 target.rot = rote;
       };
       if ( i==4 || i== 5 ) {
                 target.z   = -5.0;
                 target.dx  = 3.0;
                 target.dy  = 0.005;
                 target.x   = 0.;
                 target.y   = WirePosition[i];
                 target.rot = rot;
       };
       if ( i==6 || i== 7 ) {
                 target.z   = -4.7;
                 target.dx  = 0.005;
                 target.dy  = 3.0;
                 target.x   = WirePosition[i];
                 target.y   = 0.;
                 target.rot = rote;
       };

       if (EntriesSelect>i && WireSelect[i]==1) {target.ms   = Rate;}
       else                                     {target.ms   = 0.0;};


       ArteTable<GTAR>::push_back( target );

    };

  };

     return flagData;
}
/************************************************************************
* KATRGT  target database files
*
* called from  KUIP
***********************************************************************/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "arte/Arte.hh"

extern "C" void kugetc_( char*, int *, int );


extern "C" void katrgt_() {

//---------------------------------------------------------------
//   read in subdetector
//---------------------------------------------------------------
  if (Nrfile < 100){

   int len;
   int len1 = 100;
   char input[120];
   kugetc_(input, &len, len1);
   for (int i =0; i<len; i++) {input[i] = tolower(input[i]);};
   input[len] = 0;

   DBfile[Nrfile]  = (char *) malloc(strlen(input)+1);
   sprintf(DBfile[Nrfile],"%s", input);

   Nrfile++;

  } else {
    cout << " too many target database files " << endl;
  };

};
