// ********************************************************************
//
//  Routine:      rc99SelVdsRichEcal
//  Author:       T. Oest
//
//  Description:  VDS/ECAL/RICH match
//
//  Modification:
//
// ********************************************************************

// includes
#include <iostream.h>
#include <stdlib.h>
#include <math.h>

#include "arte/arte.hh"

#include "reco/rc99fit98.hh"

extern float MatchChi2Cut;
extern float tyRICHcorr;
extern float MatchdtyCut;
extern float MatchdtyMaxCut;
extern float MatchdyMaxCut;
extern float MatchpCut;

int rc99SelVDSerrors(ArtePointer<RSEG>, const double *);

//********************************************************************
// function
//********************************************************************
void rc99SelVdsRichEcal(ArtePointer<RSEG> rVDS, ArtePointer<RSEG> rRICH, 
                    ArtePointer<RCCL> rccl)
{
    float xECAL, yECAL, zECAL, txRICH, tyRICH;
    float xVDS,  yVDS , zVDS,  txVDS,  tyVDS;
    const float zMAG = 450.;

const int index[5][5] = { {0, 1,  3, 6,10},
                          {1, 2,  4, 7,11},
                          {3, 4,  5, 8,12},
                          {6, 7,  8, 9,13},
                          {10,11,12,13,14}};
const int index2[5][5] = { {0,1, 2, 3, 4},
                           {1,5, 6, 7, 8},
                           {2,6, 9,10,11},
                           {3,7,10,12,13},
                           {4,8,11,13,14}};
//-----------------------------------------------------------------
//  select candidates
//-----------------------------------------------------------------
    float alpha, pinv, yimpact, dty, ycut, tycut;

      xECAL    = rccl->x;
      yECAL    = rccl->y;
      zECAL    = rccl->z;
      xVDS     = rVDS->xe;
      yVDS     = rVDS->ye;
      zVDS     = rVDS->ze;
      txVDS    = rVDS->txe;
      tyVDS    = rVDS->tye;
      txRICH   = rRICH->txf;
      tyRICH   = rRICH->tyf + tyRICHcorr;

      yimpact  = yVDS + tyVDS * (zECAL - zVDS) - yECAL;
      dty      = tyRICH - tyVDS;

      if (MatchChi2Cut > 1000.){
       ycut  = MatchdyMaxCut; 
       tycut = MatchdtyMaxCut; 
      } else {
       if (rccl->esum > 100.) {
         ycut = 1.;
       } else {
         ycut  = 1. + 3. * (1. - rccl->esum / 100.);
       };
       tycut = MatchdtyCut;
      };

      if (fabs(yimpact) < ycut && fabs(dty) < tycut ){

//-----------------------------------------------------------------
//  fit 
//-----------------------------------------------------------------
        double pstart[6];
        double z, pMeas[5], wMeas[5][5];

        alpha    = (xVDS+ txVDS*(zECAL-zVDS)-xECAL)/(zECAL-zMAG);
        pinv     = -1.61 * alpha;

        pstart[0] = zVDS;
        pstart[1] = xVDS;
        pstart[2] = yVDS;
        pstart[3] = txVDS;
        pstart[4] = tyVDS;
        pstart[5] = pinv;

        for (int i=0;i<5;i++){for (int j=0;j<5;j++){wMeas[i][j]=0.0;};}

        track_98 track(pstart);

 	// ECAL, RICH

        z        = zECAL;
        pMeas[0] = xECAL;
        pMeas[1] = yECAL;
        pMeas[2] = txRICH;
        pMeas[3] = tyRICH;
        pMeas[4] = pinv;

        float a;
        a = rccl->cve[0] * rccl->cve[2] - 
            rccl->cve[1] * rccl->cve[1];
        if (a != 0.0){
         wMeas[0][0] =   rccl->cve[2] / a;
         wMeas[1][1] =   rccl->cve[0] / a;
         wMeas[1][0] = - rccl->cve[1] / a;
         wMeas[0][1] = wMeas[1][0];
        } else {
         wMeas[0][0] = 1./rccl->cve[0];
         wMeas[1][1] = 1./rccl->cve[2];
         wMeas[1][0] = 0.0;
         wMeas[0][1] = 0.0;
        };

        a = rRICH->cvf[5]  * rRICH->cvf[9] - 
            rRICH->cvf[8]  * rRICH->cvf[8];
        if (a != 0.0){
         wMeas[2][2] =   rRICH->cvf[9] / a;
         wMeas[3][3] =   rRICH->cvf[5]  / a;
         wMeas[3][2] = - rRICH->cvf[8]  / a;
         wMeas[2][3] = wMeas[3][2];
        } else {
         wMeas[2][2] = 1./rRICH->cvf[5];
         wMeas[3][3] = 1./rRICH->cvf[9];
         wMeas[3][2] = 0.0;
         wMeas[2][3] = 0.0;
        };

        track.add_measurement(z, &pMeas[0], &wMeas[0][0]);

	// VDS segment

        z        = zVDS;
        pMeas[0] = xVDS;
        pMeas[1] = yVDS;
        pMeas[2] = txVDS;
        pMeas[3] = tyVDS;
        pMeas[4] = pinv;
        for (int i=0; i<=3 ;i++){ for (int j=0; j<=3 ;j++){
         wMeas[i][j] = rVDS->cve[index[i][j]];
        };};


        if (rc99SelVDSerrors(rVDS, &wMeas[0][0]) == 0){

         track.add_measurement(z, &pMeas[0], &wMeas[0][0]);


//-----------------------------------------------------------------
//  fit 
//-----------------------------------------------------------------

        double chi2 = track.fit();

//-----------------------------------------------------------------
//  fill RTRA row
//-----------------------------------------------------------------
        double * parameter;
        parameter   = track.track_parameters();
        float p     = 1./fabs(float(parameter[5]));

        if (chi2 < MatchChi2Cut && chi2 >0.0 &&
            p > MatchpCut){

        double * covariance;
        covariance  = track.track_errors();

        RTRA rtra;

        rtra.hsi = 0;
        rtra.hin = 0;
        rtra.hou = 0;
        rtra.hri = 0;
        rtra.hca = 0;
        rtra.htr = 0;
        rtra.hmu = 0;

        rtra.lre = 0.0;
        rtra.lrmu= 0.0;
        rtra.lrpi= 0.0;
        rtra.lrk = 0.0;
        rtra.lrp = 0.0;
        rtra.flr = 0;

        rtra.lte = 0.0;
        rtra.lth = 0.0;
        rtra.flt = 0;

        rtra.lee = 0.0;
        rtra.lemi= 0.0;
        rtra.leh = 0.0;
        rtra.fle = 0;

        rtra.lmmu= 0.0;
        rtra.lmh = 0.0;
        rtra.flm = 0;


        rtra.fit = 10;

        rtra.xf  = rVDS->xf;
        rtra.yf  = rVDS->yf;
        rtra.zf  = rVDS->zf;
        rtra.txf = rVDS->txf;
        rtra.tyf = rVDS->tyf;
        rtra.pf  = float(parameter[5]);
        rtra.chi = float(chi2);

        rtra.lre = rRICH->time;
        rtra.lee = fabs(rccl->esum * float(parameter[5])) ;


        for (int i=0; i<=3; i++){for (int j=0; j<=3; j++)
          rtra.cvf[index[i][j]] = rVDS->cvf[index[i][j]];};
        for (int i=0; i<=4; i++){
          rtra.cvf[index[i][4]] = covariance[i*5+4];
          rtra.cvf[index[4][i]] = covariance[4*5+i];
        };

        rtra.lre = pinv;

        ArteTable<RTRA>::push_back( rtra );

        ArtePointer<RTRA> Prtra(ArteTable<RTRA>::size());

        createRelation( Prtra, rRICH, &RTRA::rseg );
        createRelation( Prtra, rVDS,  &RTRA::rseg );
        createRelation( Prtra, rccl,  &RTRA::rccl );

        if (ArteTable<MTRA>::size() > 0 && rRICH->fit != 0){
         ArtePointer<MTRA> mtra (rRICH->fit);
         for (ArteRelationIterator<MTRA> mccl = rccl->mtra.begin();
              mccl != rccl->mtra.end(); mccl++){
	 if (mtra == (*mccl)){

         ArteRelationIterator<MTRA> mvds = rVDS->mtra.begin();

          if (mvds != rVDS->mtra.end() && mtra == (*mvds))
            createRelation( Prtra, mtra, &RTRA::mtra );

         };
	 };};
        
        };

      };
     };


      
};



