// ********************************************************************
//
//  Routine:      rc99fit98
//  Author:       T. Oest
//
//  Description:  general fitting routine
//
//  Modification:
//
// ********************************************************************
#include <math.h>
#include <complex.h>
#include <iostream.h>
#include "reco/rc99fit98.hh"


//#define show
//#define showall

extern "C"
{
 void rkmain_(const int    & icovar, 
              const double & zout, 
              const double & fstep, 
              const double * parameters, 
              const double * prec, 
              const double * p_track, 
              const double * derivatives, 
              const int    & ierror);
 void deqinv_(const int    & n,
              const double * G,
              const int    & idim,
              const float  * r,
              const int    & ifail,
              const int    & k,
              const double * g);
}

//---------------------------------------------------------------------------
// constructor for track_98
//--------------------------------------------------------------------------

track_98::track_98(){
 n_measurements = 0;
 chi2           = -1.0;

 for(int i=0; i<5; i++) {
  parameters[i]     = 0.0;
  for(int j=0; j<5; j++) {
   covariance[i][j] = -1.0;
  };
 };

 weight         = covariance;
};


track_98::track_98(const double * p){
 n_measurements = 0;
 chi2           = -1.0;

 for(int i=0; i<5; i++) {
   parameters[i]    = p[i];
  for(int j=0; j<5; j++) {
   covariance[i][j] = -1.0;
  };
 };
 parameters[5]    = p[5];

 weight         = covariance;
};

//-------------------------------------------------------------------
//  add a measurement
//--------------------------------------------------------------------

int track_98::add_measurement(const double & z, const double * m , 
                              const double * C ){
  if (n_measurements < max_meas){

     z_position[n_measurements] = z;
     for(int i=0; i<5; i++) {
      measurements[n_measurements][i] = m[i];
      for(int j=0; j<5; j++) {
       errors[n_measurements][i][j] = C[i*5 + j];
      };
     };

     n_measurements++;
     return 0;
     }
  else return 1;
};

//-------------------------------------------------------------------
//  return track parameters, errors
//-------------------------------------------------------------------

double * track_98::track_parameters(){
  return parameters;
};

double * track_98::track_errors(){
  return & covariance[0][0];
};

//-------------------------------------------------------------------
//  fit
//-------------------------------------------------------------------

double track_98::fit(){

  int i=1;

  double chi2_old;
  double dchi = -1.0;

  double test=1.;
  while ((dchi < 0.0 || i <= 2 ) &&
         (fabs(test) > 0.01 && i<20 && test!=-1000.)){
     chi2_old = chi2;
     test=fit_iteration();
     dchi = chi2 - chi2_old;
     //
     //        cout << " iteration " 
     //	  << i 
     //	  << " CHI2 : " 
     //	  << chi2 
     //	  << " " 
     //	  << test << endl ;
     i++;
   };

   if (i == 20 || test==-1000. || chi2 == 0.0) return -1000.;

 return chi2;
};

//-------------------------------------------------------------------
//  fit iteration
//-------------------------------------------------------------------

double track_98::fit_iteration(){

 double    delta_p[5]       = {};
 double    g[5]             = {};
 double    G[5][5]          = {};
 double    fstep=50.;

 double delta_chi2;

 if (chi2 == -1.0) chi2 = 0.0;

 for(int j=0; j<5; j++)
 {
    g[j] = 0.0;
    for(int k=0; k<5; k++)
    {
      G[j][k] = 0.0;
    };
 };

 chi2=0.;

 for(int i=0; i<n_measurements; i++) {

   const int    icovar=2;    
   double       prec[4];
   double       derivatives[5][5]={};
   double       p_track[6];

   //
   // tracing returns derivatives, p_track
   //
   if (z_position[i] != parameters[0]) {

     double zout=z_position[i];
     int    ierror;

     prec[0] = 0.001/abs(zout-parameters[0]);
     prec[1] = prec[0];
     prec[2] = 0.0001/abs(zout-parameters[0]);
     prec[3] = prec[2];
 
     //     cout << "going to move " << parameters[0] << " " << parameters[1] << " " << parameters[2] << " " << parameters[3] << " " << parameters[4] << endl;

     rkmain_(icovar, zout, fstep, parameters, prec, p_track, 
	     & derivatives[0][0], ierror);
 
     //     cout << "finished " << p_track[0] << " " << p_track[1] << " " << p_track[2] << " " << p_track[3] << " " << p_track[4] << endl;

     if (ierror != 0) return -1000.;

   }
   else {
     for(int j=0; j<5; j++){
       derivatives[j][j] = 1.0;
       p_track[j+1]        = parameters[j+1];
     };
   };


   //
   // derivatives is a fortran matrix 
   //
   //         calculate g and G
   //          g = 0.5 * d chi2 / d p
   //          G = 0.5 * d**2 chi2 / d p**2
   //
   for(int j=0; j<5; j++) {
     for(int k=0; k<5; k++) {
       for(int l=0; l<5; l++) {
	 g[j] = g[j] - derivatives[j][k] * errors[i][k][l] * 
	   (p_track[l+1]-measurements[i][l]);

       };
     };
   };

   for (int j=0; j<5; j++) {
     for (int k=0; k<5; k++) {
       for (int l=0; l<5; l++) {
	 for (int m=0; m<5; m++) {
	   G[j][k] = G[j][k] + 
	     derivatives[j][l]*errors[i][l][m]*derivatives[k][m];
	 };
       };
       weight[j][k] = G[j][k];

     };
   };

   //
   // chi2
   //   if (chi2_old == -1.0){
   for(int j=0; j<5; j++){
     for(int k=0; k<5; k++){
       chi2 = chi2 + (measurements[i][j] - p_track[j+1]) * 
	 errors[i][j][k] * (measurements[i][k] - p_track[k+1]);

#ifdef showall      
       cout << "whatshappening ?"<< i << " " << j << " " << k << " " 
	    << measurements[i][j]<< " " 
	    << p_track[j+1]      << " " 
	    << errors[i][j][k]   << " " 
	    << measurements[i][k]<< " " 
	    << p_track[k+1]      << " " 
	    << chi2 << endl;
#endif
     };
   };
 };


 for (int j=0;j<5;j++) {delta_p[j] = g[j];}
 //
 // now call CERNLIB routine to solve for delta_p (-->g)
 //
 const int n=5;
 const int idim=5;
 float     r[5];
 int       ifail;
 const int kdim=1;

 // G is symmetric


 deqinv_(n, &G[0][0] , idim, r, ifail, kdim, g);
#ifdef showall
  cout << "calculation failed???? " << ifail << endl;
#endif

 //
 // new parameters
 //
 parameters[1] = parameters[1] + g[0];
 parameters[2] = parameters[2] + g[1];
 parameters[3] = parameters[3] + g[2];
 parameters[4] = parameters[4] + g[3];
 parameters[5] = parameters[5] + g[4];

 for(int j=0; j<5; j++){
#ifdef show
   cout << "new parameter ...." 
	<< parameters[j+1]
	<< endl;
#endif

   for(int k=0; k<5; k++){ 
     covariance[j][k] = G[j][k];
   };
 };

 delta_chi2=0.0;
 for (int j=0;j<5;j++) delta_chi2=delta_chi2 - g[j]*delta_p[j];
 // chi2 = chi2 + delta_chi2;

 return delta_chi2;
};
