#include "rise/RICHEventMC.h"
#include "rich/vessel.h"
#include "arte/RSEG_cmp.hh"
#include "rich/PhiLambda.h"
#ifdef USE_ROOT
#include "TROOT.h"
#include "TMinuit.h"
#endif

// -----------------------  RICH photons ----------------------------
ArtePointer<GEDE> RichGedePointer=0;

RICHphotons::RICHphotons() {
  if (RichGedePointer==0)
    for (ArteTable<GEDE>::iterator igede=ArteTable<GEDE>::begin();
	 igede!=ArteTable<GEDE>::end(); ++igede) {
      if ((*igede)->cmp==5) {
        RichGedePointer=(*igede);
        break;
      }
    }
  
  for (ArteTable<MIMP>::iterator imimp=RichGedePointer->mim1;
       imimp!=RichGedePointer->miml; ++imimp) {
    unsigned imtra=(*imimp)->mtra.index();
    photonList.insert( pair<const unsigned, unsigned>(imtra,(*imimp).index()));
  }
  it=photonList.begin();
  itl=photonList.end();
}

void RICHphotons::print() {
  for (begin(); valid(); next()) cout<<(*it).first<<" "<<(*it).second<<" "<<id()<<endl;
}

void RICHphotons::print(unsigned imtra) {
  for (begin(imtra); valid(); next()) cout<<(*it).first<<" "<<(*it).second<<" "<<id()<<endl;
}


// ----------------------- MC RICH rings ----------------------------

RICHrings::RICHrings() {
  cout<<"RICHrings::RICHrings()"<<endl;
  extern RICHVessel *RICH_Vessel;
  RICHVessel &rv=*RICH_Vessel;
  photons=new RICHphotons;
  unsigned oldMtra=0;
  data.mimps=0;
  float sumxz=0,sumyz=0,sumz=0,sumz2=0,sumx=0,sumy=0;
  for (photons->begin(); photons->valid(); photons->next()) {
    if (photons->mtra()!=oldMtra) {
      if (data.mimps>3) {
	// -- direction
	float delta=data.mimps*sumz2-sumz*sumz;
	data.tx=(data.mimps*sumxz-sumx*sumz)/delta;
	data.ty=(data.mimps*sumyz-sumy*sumz)/delta;
	//cout<<data.tx<<" "<<data.ty<<endl;
	// -- index of refraction
	data.nrefr/=float(data.mimps);
	// -- radius
	data.radius=0;
	for (photons->begin(oldMtra); photons->valid(); photons->next()) {
	  float dtx=data.tx-photons->mimp()->tx/photons->mimp()->p;
	  float dty=data.ty-photons->mimp()->ty/photons->mimp()->p;
	  float rad=sqrt(dtx*dtx+dty*dty);
	  data.radius+=rad;
	}
	data.radius/=float(data.mimps);

	ArtePointer<MTRA> imtra(oldMtra); // obsolete!!!!
	data.vertex=imtra->bver;
	data.topp  =imtra->topp;
	data.mother=imtra->btra->topp;

	ringList.insert(pair<const unsigned, RichRingData>(oldMtra,data));
      }
      
      // -- new ring
      if (!photons->end()) {
	oldMtra=photons->mtra();
	float x=photons->mimp()->x;
	float y=photons->mimp()->y;
	float z=photons->mimp()->z;
	sumxz=x*z;
	sumyz=y*z;
	sumz=z;
	sumz2=z*z;
	sumx=x;
	sumy=y;
	data.mimps=1;
	if (photons->detected()) data.hitbs=1;
	else data.hitbs=0;
	float e=norm(Vec3(photons->mimp()->tx,photons->mimp()->ty,photons->mimp()->p));
	float l=energy2lambda(e);
	data.nrefr=rv.Radiator().RefractiveIndex(l);
      }
    }
    else {
      // -- direction
      float x=photons->mimp()->x;
      float y=photons->mimp()->y;
      float z=photons->mimp()->z;
      //cout<<x<<" "<<y<<" "<<z<<endl;
      sumxz+=x*z;
      sumyz+=y*z;
      sumz+=z;
      sumz2+=z*z;
      sumx+=x;
      sumy+=y;
      // -- index of refraction
      float e=norm(Vec3(photons->mimp()->tx,photons->mimp()->ty,photons->mimp()->p));
      float l=energy2lambda(e);
      data.nrefr+=rv.Radiator().RefractiveIndex(l);
      // -- number of photons
      data.mimps++;
      if (photons->detected()) data.hitbs++;
      // -- coordinate
      data.x=Vec3(x,y,z);
    }
  }
  begin();
}


void RICHrings::print() {
  for (begin(); valid(); next()) {
    cout<<endl
	<<"mtra="<<mtra()<<endl
	<<"type="<<topp()->lund<<endl
	<<"mother="<<mother()->lund<<endl
	<<"hitbs="<<DetectedPhotons()<<endl
	<<"mimps="<<RadiatedPhotons()<<endl
	<<"RICH position="<<x()<<endl
	<<"vertex z="<<vertex()->z<<endl
	<<"direction="<<phi()<<","<<lambda()<<endl
	<<"radius="<<radius()<<endl
	<<"nrefr="<<RefractionIndex()<<endl
	<<"p="<<momentum()<<endl;
  }
}

void RICHrings::fill(HBookTuple *tuple) {
  for (begin(); valid(); next()) {
    tuple->column("tx",tx());
    tuple->column("ty",ty());
    tuple->column("rad",radius());
    tuple->column("p",momentum());
    tuple->column("x",x().x());
    tuple->column("y",x().y());
    tuple->column("z",x().z());
    tuple->column("hitb",DetectedPhotons());
    tuple->column("mimp",RadiatedPhotons());
    tuple->column("z0",vertex()->z);
    tuple->dumpData();
  }
}



float RICHrings::momentum() {
  float px=mtra()->px;
  float py=mtra()->py;
  float pz=mtra()->pz;
  return sqrt(px*px+py*py+pz*pz);
}



// ----------------------- RC RICH rings ----------------------------

RICHreco::RICHreco() {

  if (int((*ArteTable<RSEG>::end()).index())-
      int((*ArteTable<RSEG>::begin()).index())<0) return;
  mcRings= new RICHrings;
  for (ArteTable<RSEG>::iterator irseg=ArteTable<RSEG>::begin();
       irseg!=ArteTable<RSEG>::end(); ++irseg) {
    if ((*irseg)->cmp!=Rsegc::rich) continue;
    unsigned imtra=unsigned((*irseg)->fit);
    // -- find correspoding MC particle
    int fFlag=0;
    if (mcRings->find(imtra)) fFlag=1;
    // -- ring parameters and errors
    data.tx      = (*irseg)->txf;
    if (fFlag) key.dtx=data.tx-mcRings->tx();
    else key.dtx=0;
    data.ty      = (*irseg)->tyf;
    if (fFlag) key.dty=data.ty-mcRings->ty();
    else key.dty=0;
    data.radius  = (*irseg)->time;
    if (fFlag) key.dradius=data.radius-mcRings->radius();
    else key.dradius=0;
    data.photons = (*irseg)->ze;
    if (fFlag) key.dphotons=data.photons-mcRings->DetectedPhotons();
    else key.dphotons=0;
    key.mtra=imtra;
    recoList.insert( pair<const RecoKey,RichRecoData>(key,data) );
    }
  begin();
}


void RICHreco::print() {
  for (begin(); valid(); next()) {
    cout<<endl
	<<"mtra="<<(*it).first.mtra<<endl
	<<"phi="<<phi()<<", lambda="<<lambda()<<endl
	<<"dphi="<<dtx()<<", dlambda="<<dty()<<endl
	<<"radius="<<radius()<<endl
	<<"dradius="<<dradius()<<endl
	<<"photons="<<photons()<<endl
	<<"dphotons="<<dphotons()<<endl;
  }
}

void RICHreco::fill(HBookTuple *tuple) {
  unsigned oldMtra=0;
  int      multi=1;
  for (begin(); valid(); next()) {
    unsigned imtra=mtra().index();
    if (imtra!=oldMtra) {
      oldMtra=imtra;
      multi=1;
    }
    else multi++;
    mcRings->find(imtra);
    tuple->column("phi",mcRings->phi());
    tuple->column("phif",phi());
    tuple->column("lam",mcRings->lambda());
    tuple->column("lamf",lambda());
    tuple->column("rad",mcRings->radius());
    tuple->column("radf",radius());
    tuple->column("p",mcRings->momentum());
    tuple->column("x",mcRings->x().x());
    tuple->column("y",mcRings->x().y());
    tuple->column("z",mcRings->x().z());
    tuple->column("z0",mcRings->vertex()->z);
    tuple->column("multi",multi);
    tuple->dumpData();
  }
}



// ----------------------- RICH angles --------------------------------

#ifdef USE_ROOT

#define MAX 10000

extern "C" {
  void ringfit(int &npar,double *gin,double &f,double *x,int iflag);
  double x[MAX],y[MAX],z[MAX];
  double D_horiz_u,D_vert_u,D_horiz_d,D_vert_d,A_d,A_u;
  double P_horiz_u,P_vert_u,P_horiz_d,P_vert_d;
  int    ni; 
  }

RICHangles::RICHangles() {
  TMinuit *gMinuit=new TMinuit(3);
  gMinuit->mninit(5,6,7);
  gMinuit->SetFCN(ringfit);
}


void RICHangles::fitRings(HBookTuple *tuple) {

  static int magnetFlag=-1;
  if (magnetFlag<0) {
    for (unsigned i=1;i<=ArteTable<GESL>::nrow();i++) {
      ArtePointer<GESL> gesl(i);
      if (gesl->cmp==12) {
	magnetFlag=gesl->mfl;
	break;
      }
    } 
  if (magnetFlag) printf("Magnet ON\n");
  else printf("Magnet OFF\n");
  }

  mcRings=new RICHrings;

  // -- loop over MC rings
  for (mcRings->begin(); mcRings->valid(); mcRings->next()) {

    // -- initial quality cuts on rings
    if (mcRings->momentum()<2) continue;
    if (mcRings->DetectedPhotons()<10) continue;
    
    // -- ring fitting
    unsigned imtra=mcRings->mtra().index();
    ni=0;
    // -- get data
    for (mcRings->firstPhoton(imtra); mcRings->validPhoton(); mcRings->nextPhoton()) {
      if (!mcRings->GetPhotons()->detected()) continue;
      x[ni]=mcRings->GetPhotons()->DetectedPhi();
      y[ni]=mcRings->GetPhotons()->DetectedLambda();
      if (mcRings->GetPhotons()->id() > 7000000) z[ni]=1;
      else z[ni]=0;
      ni++;
    }

    // -- get corrections
    RICH_LambdaPhi->Correct()->newRing(mcRings->phi(),mcRings->lambda(),mcRings->radius(),
				       mcRings->T().x()*.01,mcRings->T().y()*.01);

    // --- minuit
    int    nprm[3]  = {0,1,2};
    double par[3],err[3],low[3],upp[3];
    double vstrt[3] = {mcRings->phi(),mcRings->lambda(),mcRings->radius()};

    double stp[3]   = {        .0005,           .0005,            .001};
    int ierflg;
    double p0=0,pm1=-1;

    gMinuit->mnexcm("SET PRINT", &pm1 ,1,ierflg);
    gMinuit->mnparm(nprm[0],   "phi", vstrt[0], stp[0], 0,0,ierflg);
    gMinuit->mnparm(nprm[1],"lambda", vstrt[1], stp[1], 0,0,ierflg);
    gMinuit->mnparm(nprm[2],"radius", vstrt[2], stp[2], 0,0,ierflg);
    if (ierflg) printf(" Unable to define parameter \n");
    
    gMinuit->mnexcm("MIGRAD", &p0 ,0,ierflg);
    gMinuit->mnexcm("MINOS", &p0 ,0,ierflg);

    // -- get parameters
    char *name;
    gMinuit->mnpout(0,name,par[0],err[0],low[0],upp[0],ierflg);
    gMinuit->mnpout(1,name,par[1],err[1],low[1],upp[1],ierflg);
    gMinuit->mnpout(2,name,par[2],err[2],low[2],upp[2],ierflg);
    tuple->column("phi",mcRings->phi());
    tuple->column("phif",par[0]);
    tuple->column("lam",mcRings->lambda());
    tuple->column("lamf",par[1]);
    tuple->column("rad",mcRings->radius());
    tuple->column("radf",par[2]);
    tuple->column("p",mcRings->momentum());
    tuple->column("z",mcRings->vertex()->z);
    tuple->dumpData();
  }
}



void ringfit(int &npar,double *gin,double &f,double *par,int iflag) {

  double chi2=0;
  for (int i=0;i<ni;i++) {
    double D_horiz=RICH_LambdaPhi->Correct()->horizontalDistortion(z[i]);
    double D_vert=RICH_LambdaPhi->Correct()->verticalDistortion(z[i]);
    double P_horiz=RICH_LambdaPhi->Correct()->horizontalShift(z[i]);
    double P_vert=RICH_LambdaPhi->Correct()->verticalShift(z[i]);
    double A=RICH_LambdaPhi->Correct()->skewing(z[i]);  

    double idev=1/(D_horiz*D_vert-A*A);
    double y1=( (y[i]-par[1]+P_vert)*D_horiz-(x[i]-par[0]+P_horiz)*A ) * idev;
    double x1=( (x[i]-par[0]+P_horiz)*D_vert-(y[i]-par[1]+P_vert )*A ) * idev * cos(y[i]);
    double f=sqrt(x1*x1+y1*y1)-par[2];
    chi2=chi2+f*f;          
  }
  chi2=chi2/ni;
  f=chi2;
}



#endif









