#include <stdio.h>
#include "rise/RingSearchDB.h"
#include "rich/ricutil.h"

extern "C" {
#include "table.h"
}


void RingSearchDB::getSearchParameters(int run_no, int verb) {
  static int oldRun_no=0;
  if (oldRun_no==run_no) return;

  char dbsite[256];
  dbsite[0]=0;
  char *getdbsite=getenv("RICH_RECODB");
  if (!getdbsite) {
    if (verb>0) {
      fprintf(stderr," $$$ RISE Warning: RICH_RECODB not defined, using default values for RingSearch\n");
      fprintf(stderr," $$$ RISE Warning: %f %f %f %d %d\n",MinSearchRadius,MaxSearchRadius,SearchStep,Threshold,OccupancyCut);
    }
    return;
  }
  strcpy(dbsite,getdbsite);

  Tables x=OpenTables(dbsite,0);
  if (!x) { fprintf(stderr,"Cannot fatch a table for run %d/table not found\n",run_no); Exit(1); }  

  int tx=BookTable(x,"RICH_RADIUS",0,0);
  if (!tx) { fprintf(stderr,"Cannot fatch a table for run %d/table not found\n",run_no); Exit(2); }

  TableStamp(x,run_no,1);
  if (!FetchTable(x,tx,TABLEge)) { fprintf(stderr,"Cannot fatch a table for run %d/table not found\n",run_no); Exit(3); }

  SkipTableField(x,1);
  int   major=TableMajor(x);
  int   minor=TableMinor(x);
  float radius,temperature,pressure,time;
  GetTableField(x,TABLEfloat,10000,&pressure);
  GetTableField(x,TABLEfloat,10000,&temperature);
  GetTableField(x,TABLEfloat,10000,&radius);
  GetTableField(x,TABLEfloat,10000,&time);
  GetTableField(x,TABLEfloat,10000,&MinSearchRadius);
  GetTableField(x,TABLEfloat,10000,&MaxSearchRadius);
  GetTableField(x,TABLEfloat,10000,&SearchStep);
  GetTableField(x,  TABLEint,10000,&OccupancyCut);
  GetTableField(x,  TABLEint,10000,&Threshold);

  //printf(" major minor x TABLEint %d %d %d %d \n",major,minor,(int)x,TABLEint);
  //printf("RUN %d:   P=%f  T=%f  r=%f  t=%f : %f %f %f : %d %d\n",
  //run_no,pressure,temperature,radius,time,MinSearchRadius,MaxSearchRadius,SearchStep,OccupancyCut,Threshold);

  if (verb>0) {
    fprintf(stderr," $$$ RISE Warning: RingSearch parameters changed (new reference run %d):\n",run_no); 
    fprintf(stderr," $$$ RISE lower radius=%f  upper radius=%f search step=%f threshold=%d occupancy cut=%d\n",
	    MinSearchRadius,MaxSearchRadius,SearchStep,Threshold,OccupancyCut);
  }

  if (verb>1) PrintTables(x,1);

  CloseTables(x);

  oldRun_no=run_no;
}


