#ifndef _riter_h_
#define _riter_h_
#include "rich/rich.h"

//#define RITER_DEBUG 1
#ifdef RITER_DEBUG
#include "CLHEP/Hist/HBookFile.h"
#endif
#include "CLHEP/Vector/ThreeVector.h"

#ifdef  RITER_MC
extern "C" {
  extern void ricini_(); 
}
#endif

#define pi M_PI
#define RITER_ERROR 1
#define RITER_OK 0

#define NCHAN 86016
#define NWCHA (NCHAN-1)/32+1


#define NDEL 512       //track_buffer size
#define NFOTB 10000    // photon buffer size
#define NTB 100000     // rich buffer size
#define NHIP  7        // number of hipothesis

#define ifix(x) (int)x 
#define max(x,y) (x>y) ? x : y
#define min(x,y) (x<y) ? x : y
#define sqr(x) x*x

inline Hep3Vector toHep3Vector(const Vec3 &x) {
return Hep3Vector(x.x(), x.y(), x.z());
}


inline double power(double x,int n) {
  double prod=1;
  for (int i=0;i<n;i++){
   prod*=x;
  }
return prod;
}


inline void vzero(float  *x,int y) { for (int vz=0;vz<(y);vz++) x[vz]=0; return;} 


extern "C" {
extern float alogam_(float&);
}

inline float alogam(float &x) {return alogam_(x); }  // CERNLIB ln Gamma(x) 

inline float rnd(void) {return (float) rand()/32767.0;
// rand() uses a multiplicative, congruential, random-number generator
//      with period 2^32 that returns successive pseudo-random numbers in the
//      range from 0 to 2^15-1=32767.

}
inline int btest(int a,int b) {return (1<<b)&a; } 
inline int ibset(int a,int b) {return (1<<b)|a; }

float pois_log(float mean,float n);


inline Hep3Vector virtual_position(const Hep3Vector &x,const Hep3Vector &x0,const Hep3Vector &norm) {
float sp=(x-x0)*norm;
return x-2*sp*norm;
}

inline Hep3Vector virtual_direction(const Hep3Vector &x,const Hep3Vector &norm) {
  float sp=x*norm;
  return x-2*sp*norm;
}



// Marko Staric iterative search identification routines
//-------------------------------------------------------------

typedef struct {
  //  float x,y,z,sig,wt,eff;
  Hep3Vector r;
  float sig,wt,eff;
  int nh;
} channel;


class RICHGeometry {

 public:
  float sigthc;   
  float sig_sfa;  
  float r_mi;
  
  // sigma^2 =sig0^2+sigthc^2+sig_sfa^2  
  // sig0 =nn*sig_pad; nn={4-m1,2-m4, 1-m1}
  float sig_pad;
  float f_pad;   // f_pad=f/d_pad

#define NUMROW 24
#define NUMCOL 12
#define NUMSM  32
#define NUMCH  16
  
  int idet[128][48][14];
  Hep3Vector ixyz[NUMSM][NUMROW][NUMCOL][NUMCH];
  int  bit_map[NWCHA];
  channel  ch[NCHAN];

  float an_zero;  
  float ref_ind;  // refractive index -average
  
  float  th_0,fi_0; 
  Hep3Vector rc_mu,rc_md;    // spherical mirrors CoC
  Hep3Vector rp_mu,rp_md;    // planar mirrors -position
  Hep3Vector ezp_mu,ezp_md;  // planar mirrors -normal
  float rmax2;    // square of the maximum Ch.radius 
  
  Hep3Vector  r0[14],vx[14],vy[14],vn[14];  // geometry of supermodules
  float       dvx[14],dvy[14],dxsm[14],acc_lens;
  float z_rich_min;                         // start and and of the rich
  float z_rich_max;
  
  float x_cut, y_cut, z_cut, z_magnet,p_min;
  int print();
  int    set_pmt(int type,int ism,int ir,int ic,int &ichan);
  void   read_pmt(FILE *fp1,int &ism,int &irow,int &icol,int &itp,long &line,int &ok);
  void   to_xyz(int ism,int ir,int ic,int nn,Hep3Vector &r,float &sig);  

  Hep3Vector src2xyz(int ism,int ir,int ic,int ch);
  RICHGeometry();
  ~RICHGeometry(){};

};

/* --------------------------------------------------------------------------*/
typedef struct { 
  Hep3Vector r,dir;      //! entrance point, direction
  float p ;              //! momentum
  HepDouble sig;
  int  itype;            //! particle type (unknown=0,e=1,mu=2,pi=3,k=4,p=5)
  Hep3Vector  r0;        //! mean photon emission point
  Hep3Vector r1u, r1d;   //! particle hit on (up, down) sph. mirror
  Hep3Vector diru, dird; //! direction after reflection (up, down) 
  Hep3Vector rcu, rcd;   //! ring centers (up and down detector)
  short  wm ;            //! =1 if particle hits upper sph. mirror
  float thc[NHIP];          //! expected cerenkov angles for {0,e,mu,pi,k,p,e+e-} 
  float  stf[NHIP];         //! expected number of signal photons for {..}
  float bgr[NHIP];          //! expected number of bgr photons for {..}
} track_buffer;


typedef struct { 
  Hep3Vector rf;  // ! photon hits
  float  sig;     // ! angular error (sigma)
  float  utez;    // ! photon weight (channel efficiency, occupancy)
} photon_buffer;

typedef struct { 
  int ir;
  int ihit;
  int ifot;
  float utez;
  float thc;
  float w;                 //! result of iteration procedure
} richtab;
     
/* --------------------------------------------------------------------------*/
class RICHFaktor{
 public:
float bgrfak[10];
float faktor[100];
 RICHFaktor();
~ RICHFaktor(){};
};
/* --------------------------------------------------------------------------*/
class Riter {

private:
  RICHGeometry *RICH_geo;
  RICHFaktor   *Faktor;  
  int track_selected[NDEL];  
  track_buffer  track[NDEL];
  photon_buffer foton[NFOTB];
  richtab       rich[NTB];
 
  int  itip_mc[NDEL];        //! particle type from montecarlo //
  int  iu[NDEL];             //! nonzero if from b
  float   plkh[NDEL][NHIP];     //! likelihood
  float   sfot[NDEL][NHIP];     //! measured number of photons
  int irm;                   //! max radius in bins
  int niter;                   // number od iterations
  int ntracks;                 // number of tracks in the event
  int nfot;                    // number of photons in the event
  float anzero_mul;            // NzeroNormalize
  float bgr_mul;               // BgrNormalize
  float pf[256][NDEL];         //! histogram (radius,track)

  //char  file_tracks[80];
  //char  file_photons[80];
  //char  nt_file[80];
  //char  file_pmt[80];

float p_mass[NHIP];
//={0, .511e-3,105.66e-3,139.57e-3,493.68e-3,938.27e-3,.511e-3};
float bin;                     //! bin size

#define  nnn 100 
  float cfi[nnn], sfi[nnn];   // for exp_no_phot

 float gean_mass(int ipart);
 int   itip_gean(int ipart);
 int   itip_lund(int ipart);

 int   read_ini( const char *);
 Hep3Vector   toRICH(int x,int y,int z,Hep3Vector s);
 int   print_tracks_par(track_buffer *t);
 int   calc_tracks_par( track_buffer *t );
 void  zero_iter(  int ihit, float pf[256][NDEL], int &irm );
 int   calc_thcer( photon_buffer &f,track_buffer  &t,float &th_cer,Hep3Vector &dirf);
 int   calc_full_thcer( photon_buffer &f,track_buffer  &t,float &th_cer,Hep3Vector &dirf);
 int   ring_tab(   int ihit,int ifot,float thc,float bin,float pf[256][NDEL],float utez,richtab *rich,int &n);
 void  ring_iter(richtab *rich,int n,float pf[256][NDEL],int nh,int nf);
 int   fill_lkh(float x,float wi,float x0,float sig,float sph,float bph,float &plkh,float &sfot);

 
 float  fun_bgr(float x);
 int    presek_detector( Hep3Vector &r, Hep3Vector &dir, int kje, Hep3Vector &r1, float  &alfa,float  &beta,int &ism);
 float  cher_angle(float p,float mass,float ref_ind);
 int    exp_no_phot(float thc,float path,float theta, float a, float b, int ism,  float &stfs, float &sig_mean, float &acg);
 float  exp_no_bgr(int i,float *thc,float *sig,float *acg);
 void   oznaci_bine( float *xxx, float *yyy, int nch,float bin, short *hip, float * sig, float *th, float *acg);

 float  acc_the(float theta);
 void   measured_dir(track_buffer &t);
 int    part_type(float p_mass);
 int    test_hyp(float *thc,int itype,float th_cer,float dthc);
 int    cerenkov_kot(Hep3Vector &r0,Hep3Vector &dir,Hep3Vector &rh,Hep3Vector &rc,float r_mi,float &th_cer,Hep3Vector &dirf);
 int    cerenkov_kot_0( Hep3Vector &r0, Hep3Vector &dir, Hep3Vector &rh, Hep3Vector &rc,HepDouble r_mi,float  &th_cer,Hep3Vector &dirf);
 int    resi_sin(float a,float b,float c,float &x, float &y,int & iter,int max_iter);
 int    resi_sin_0(float a,float b,float c,float &x,float &y);
 // ********************** geometrical routines
 
 float  com(Hep3Vector &dir,int izb);
 void   smer(HepDouble theta,HepDouble fi,Hep3Vector &dir);
 
 void   premica(Hep3Vector &rinit,Hep3Vector &dir,Hep3Vector &rfinl,float &path,HepInt izb);
 
 void   presek_sfera( Hep3Vector  &r,Hep3Vector  &dir,HepDouble radij, Hep3Vector  &rc, Hep3Vector  &r1, Hep3Vector  &r2, int &seka);
 
 void   odboj_na_sferi(Hep3Vector &r,Hep3Vector &dir,HepDouble radij,Hep3Vector &rc);
 
 int    select_track(track_buffer *t);
 
 int    Filltracks_MIMP1();
 int    Filltracks_MIMP2();
 int    Filltracks_MIMP();
 int    Filltracks_RCCL();
 int    Filltracks_RSEG();
 int    Filltracks_RPNT();
 int    Fillfotons_HITB();
 void   FillRSEG();
 void   FillRTRA();
 int    main(int idog,int ntra,int neve);
// *********************** public members 
 public:
int  Riter_evnt();
 Riter();
 ~Riter(){
   delete RICH_geo;
   delete Faktor;
 };
};
// ---------------------------------------------------------------------

inline float Riter::com(Hep3Vector &dir,int izb)
{
  if (izb==1) return dir.x();  
  if (izb==2) return dir.y();  
  if (izb==3) return dir.z();
  return 0;
}

void fill_ntuple_4(int iev,track_buffer *t,int iu,int itip,float *plkh,float *sfot);
void fill_ntuple_5(int ism,int icol,int irow,channel *ch);


#endif 












