#include <stdio.h>
#include <iostream.h>
#include <iomanip.h>
#include <strings.h>
#include <stdlib.h>

// some Arte stuff
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "silarte/silarte.hh"
#include "silarte/EVinfo.hh"

// prototype declarations
#include "aframelib.h"
#include "vdsleda.h"
#include "silraw.h"
#include "marvin.h"
#include "vdsrhp.h"

// return codes
#include "grr.h"

// sparse event definition
#include "event.h"
#include "etb.h"

//-----------------------------------------------------------------------------

int SvdEvnt()
{
  int  rc;

  // size and location of sparsified event
#define SEVT_INIT_SIZE    0x7FFF     // size of event buffers 
  static char sparsevt[SEVT_INIT_SIZE];
  Marvin_Sevt_t *sevt;

  // flag to indicate first call to SvdEvnt 
  static int first = 1;

  char *routine = "SvdEvnt";

  //
  // No SVD Digitization in case of MC  
  //
  if ( CheckMC() )
    return GRR_OK;

  //
  // REAL DATA
  //

  // get event pointer
  EVENT_HDR *event = ARTE_get_event();
  if ( !event ) {
    aframe_mess(routine,GRR_SERROR,"Could not find event!");
    return GRR_FISHY;
    }      

  // I'd really like to do this in SvdInit, but we don't have a run# there yet.
  // The same holds for the exp # in EVHD: Thats why we fill the geometry
  // here rather than in SvdInit. 
  if (first) {

    //    Get constants from database
    //
    // Time stamp is evaluated in this order:
    //  1. date/time   from vdsleda IOL
    //  2. major/minor from vdsleda IOL
    //  3. event time from DDAQ (if all IOL parameters are =0)
    int major = 0;
    int minor = 0;
    int ndaq;
    EVT_DATA_RECORD daq;
    ETB_get_daqs( event, 1, &daq, &ndaq );
    if ( ndaq > 0 ) {
      major = daq.data.event_time;
      minor = 0;
    }
    rc = vdsleda_startprogram( major, minor );
    if ( rc != GRR_OK )
      return rc;

    // start silraw
    rc = silraw_startprogram();
    if ( rc != GRR_OK )
      return rc;

    // start marvin
    marvin_init();

    // vdsrhp init
    rc = vdsrhp_init();
    if ( rc != GRR_OK )
      return rc;
    
    // fill GEDE table with geometry from file
    silarte_updateGEDE();

    // we are through now
    first = 0;
  }

  // ---------------------
  // Normal event handling
  // ---------------------

  // Call silraw - check for correct format handling first
  if ( silraw_iol.rawdata_type.value != 0 ) {
    aframe_mess(routine,GRR_SERROR,
		"Silraw raw data type not EVENT format - check IOL parameter!");
    return GRR_FISHY;
  }
  rc = silraw_event( (char*) event, 0 );
  if ( rc != GRR_OK )
    return GRR_FISHY;
  
  // Do cluster finding
  sevt = marvin_fetch_sevt();
  if ( !sevt ) {
    aframe_mess(routine,GRR_SERROR,
		"no Marvin_Sevt_t available (marvin not initialized?)");
    return GRR_FISHY;
  }
  sevt->data = sparsevt; 
  sevt->size = SEVT_INIT_SIZE;
  sevt->next = sevt->data; 
  marvin_event();

  // Fill HITB table
  fillHITBfromSPARSEVT();

  // Call RHP monitoring
  rc = vdsrhp_event( event );
  if ( rc != GRR_OK )
    return GRR_FISHY;

  return GRR_OK;

}
