#ifdef USE_ROOT
 #include <TH2.h>
 #include <TH1.h>
 #include <TNtuple.h>
 #include "wax/OTrEfficiencyH1F.hh"
#endif

#include "wax/OTrAlignment.hh"
#include "wax/OTrAlignmentProjection.hh"
#include "wax/OTrEvent.hh"
#include "wax/OTrSuperCluster.hh"
#include "wax/OTrNodeProjection.hh"
#include "wax/OTrSection.hh"
#include "wax/OTrTrack.hh"
#include "wax/OTrTrackCandidate.hh"
#include "wax/OTrDetector.hh"

#ifdef USE_ROOT
#endif

//__________________________________________________________________
// OTrAlignment
//
// Container for:
// * All histograms that have to do with alignment
// Functionality:
// * Histogramming (etc)
//
// Author: Wouter Hulsbergen
//__________________________________________________________________

#ifdef __OTR_CLASSDEF__
ClassImp(OTrAlignment) ;
#endif

OTrAlignment::OTrAlignment() {} ;

//__________________________________________________________________
// OTrMCHit
//
// Utility class for determining efficiencies.
//__________________________________________________________________


class OTrMCHit
{
private:
  const OTrWire* fWire ;
  OTrNodeProjection* fNodeProjection ;
  const OTrHit* fMeasuredHit ;
public:
  OTrMCHit(const OTrWire*) ;
  ~OTrMCHit() ;
  OTrNodeProjection* GetNodeProjection() ;
  void SetMeasuredHit(const OTrHit* hit) { fMeasuredHit=hit; } 
  const OTrHit* GetMeasuredHit() const { return fMeasuredHit ; } 
  const OTrWire* GetWire() const { return fWire ; } 
} ;

inline
OTrMCHit::~OTrMCHit() { delete fNodeProjection ; }

inline
OTrMCHit::OTrMCHit(const OTrWire* w) 
  : fWire(w), fNodeProjection(0), fMeasuredHit(0) {}

inline OTrNodeProjection* 
OTrMCHit::GetNodeProjection()
{
  if(!fNodeProjection) fNodeProjection = new OTrNodeProjection(fWire) ;
  return fNodeProjection ;
}

void
OTrAlignment::Add(OTrEvent* event)
{
#ifdef USE_ROOT
  static TH2F* txtyResMC2VsMSL1H2 = 
    new TH2F("txtyResMC2VsMSL1H2","",100,-0.002,0.002,100,-0.03,0.03) ;
  static TH2F* txtyResMC2VsMSL2H2 =
    new TH2F("txtyResMC2VsMSL2H2","",100,-0.003,0.003,100,-0.05,0.05) ;
  static TH2F* txtyResMC2VsMTC1H2 = 
    new TH2F("txtyResMC2VsMTC1H2","",100,-0.150,0.150,100,-0.15,0.15) ;
  static TH2F* txtyResMSL1VsMSL2H2 = 
    new TH2F("txtyResMSL1VsMSL2H2","",100,-0.010,0.010,100,-0.05,0.05) ;
  static TH2F* txtyResMSL1VsMTC1H2 = 
    new TH2F("txtyResMSL1VsMTC1H2","",100,-0.010,0.010,100,-0.15,0.15) ;
  static TH2F* txtyResMSL2VsMTC1H2 = 
    new TH2F("txtyResMSL2VsMTC1H2","",100,-0.010,0.010,100,-0.15,0.15) ;
  static TH2F* txtyResOthersH2 = 
    new TH2F("txtyResOthersH2","",100,-0.010,0.010,100,-0.15,0.15) ;
  static TH1F* xResMC2VsMSL1H1 = 
    new TH1F("xResMC2VsMSL1H1","",1000,-5.0,5.0) ;
  static TH1F* xResMC2VsMSL2H1 =
    new TH1F("xResMC2VsMSL2H1","",1000,-5.0,5.0) ;
  static TH1F* xResMC2VsMTC1H1 = 
    new TH1F("xResMC2VsMTC1H1","",1000,-5.0,5.0) ;
  static TH1F* xResMSL1VsMSL2H1 = 
    new TH1F("xResMSL1VsMSL2H1","",1000,-5.0,5.0) ;
  static TH1F* xResMSL1VsMTC1H1 = 
    new TH1F("xResMSL1VsMTC1H1","",1000,-5.0,5.0) ;
  static TH1F* xResMSL2VsMTC1H1 = 
    new TH1F("xResMSL2VsMTC1H1","",1000,-5.0,5.0) ;
  static TH1F* xResOthersH1 = 
    new TH1F("xResOthersH1","",1000,-5.0,5.0) ;
  static TH1F* yResMC2VsMSL1H1 = 
    new TH1F("yResMC2VsMSL1H1","",100,-5,5) ;
  static TH1F* yResMC2VsMSL2H1 =
    new TH1F("yResMC2VsMSL2H1","",100,-5,5) ;
  static TH1F* yResMC2VsMTC1H1 = 
    new TH1F("yResMC2VsMTC1H1","",100,-5,5) ;
  static TH1F* yResMSL1VsMSL2H1 = 
    new TH1F("yResMSL1VsMSL2H1","",100,-5,5) ;
  static TH1F* yResMSL1VsMTC1H1 = 
    new TH1F("yResMSL1VsMTC1H1","",100,-5,5) ;
  static TH1F* yResMSL2VsMTC1H1 = 
    new TH1F("yResMSL2VsMTC1H1","",100,-5,5) ;
  static TH1F* yResOthersH1 = 
    new TH1F("yResOthersH1","",100,-5,5) ;
  static TH1F* alphaResMC2VsMSL1H1 = 
    new TH1F("alphaResMC2VsMSL1H1","",100,-0.005,0.005) ;
  static TH1F* alphaResMC2VsMSL2H1 =
    new TH1F("alphaResMC2VsMSL2H1","",100,-0.005,0.005) ;
  static TH1F* alphaResMC2VsMTC1H1 = 
    new TH1F("alphaResMC2VsMTC1H1","",100,-0.005,0.005) ;
  static TH1F* alphaResMSL1VsMSL2H1 = 
    new TH1F("alphaResMSL1VsMSL2H1","",100,-0.005,0.005) ;
  static TH1F* alphaResMSL1VsMTC1H1 = 
    new TH1F("alphaResMSL1VsMTC1H1","",100,-0.005,0.005) ;
  static TH1F* alphaResMSL2VsMTC1H1 = 
    new TH1F("alphaResMSL2VsMTC1H1","",100,-0.005,0.005) ;
  static TH1F* alphaResOthersH1 = 
    new TH1F("alphaResOthersH1","",100,-0.005,0.005) ;

//   static TH2F* TxVsTyMC2H2  = new TH2F("TxVsTyMC2H2","",1000,-0.3,0.3,1000,-0.3,0.3) ;
//   static TH2F* TxVsTyMSL1H2 = new TH2F("TxVsTyMSL1H2","",1000,-0.11,-0.03,1000,-0.07,0.07) ;
//   static TH2F* TxVsTyMSL2H2 = new TH2F("TxVsTyMSL2H2","",1000,-0.09,-0.03,1000,-0.05,0.05) ;
//   static TH2F* TxVsTyMT1CH2 = new TH2F("TxVsTyMTC1H2","",1000,-0.09,-0.03,1000,-0.2,0.2) ;
  static TH2F* TxVsTyMC2H2  = new TH2F("TxVsTyMC2H2","",1000,-0.3,0.3,1000,-0.3,0.3) ;
  static TH2F* TxVsTyMSL1H2 = new TH2F("TxVsTyMSL1H2","",1000,-0.3,0.3,1000,-0.3,0.3) ;
  static TH2F* TxVsTyMSL2H2 = new TH2F("TxVsTyMSL2H2","",1000,-0.3,0.3,1000,-0.3,0.3) ;
  static TH2F* TxVsTyMT1CH2 = new TH2F("TxVsTyMTC1H2","",1000,-0.3,0.3,1000,-0.3,0.3) ;
  
  TH2F *txtyResH2, *txtyH2 ;
  TH1F *xResH1, *yResH1, *alphaResH1 ;
  for(OTrEvent::SuperLayerEventContainer::const_iterator 
  	sleIt=event->GetSuperLayerEvents().begin();
      sleIt!=event->GetSuperLayerEvents().end(); sleIt++) {
    switch((*sleIt).first)
      {
      case 2:  txtyH2=TxVsTyMC2H2; break ;
      case 31: txtyH2=TxVsTyMSL1H2; break ;
      case 32: txtyH2=TxVsTyMSL2H2; break ;
      default: txtyH2=TxVsTyMT1CH2;
      }
    
    for(OTrSuperLayerEvent::ClusterContainer::const_iterator 
	  clIt=(*sleIt).second.GetClusters().begin() ;
	clIt!=(*sleIt).second.GetClusters().end() ; clIt++) {
      txtyH2->Fill((float)(*clIt)->GetTx(),(float)(*clIt)->GetTy()) ;
    }  
  
   // for(OTrEvent::SuperLayerEventContainer::const_iterator 
   //	  sleIt2=fEvent->GetSuperLayerEvents().begin();
   //	sleIt2!=sleIt; sleIt2++)
    for(OTrEvent::SuperLayerEventContainer::const_iterator 
	  sleIt2=sleIt;
	sleIt2!=event->GetSuperLayerEvents().end(); sleIt2++)
      if(sleIt!=sleIt2) { // To get rid of diagonal terms
	int slId1 = (*sleIt).first ;
	int slId2 = (*sleIt2).first ;
	switch (slId1*100 + slId2) 
	  {
	  case 231: 
	    txtyResH2=txtyResMC2VsMSL1H2;
	    xResH1=xResMC2VsMSL1H1;
	    alphaResH1=alphaResMC2VsMSL1H1;
	    yResH1=yResMC2VsMSL1H1;
	    break ;
	  case 232:
	    txtyResH2=txtyResMC2VsMSL2H2;
	    xResH1=xResMC2VsMSL2H1;
	    alphaResH1=alphaResMC2VsMSL2H1;
	    yResH1=yResMC2VsMSL2H1;
	    break ;
	  case 233:
	    txtyResH2=txtyResMC2VsMTC1H2;
	    xResH1=xResMC2VsMTC1H1;
	    alphaResH1=alphaResMC2VsMTC1H1;
	    yResH1=yResMC2VsMTC1H1;
	    break ;
	  case 3132:
	    txtyResH2=txtyResMSL1VsMSL2H2;
	    xResH1=xResMSL1VsMSL2H1;
	    alphaResH1=alphaResMSL1VsMSL2H1;
	    yResH1=yResMSL1VsMSL2H1;
	    break ;
	  case 3133:
	    txtyResH2=txtyResMSL1VsMTC1H2;
	    xResH1=xResMSL1VsMTC1H1;
	    alphaResH1=alphaResMSL1VsMTC1H1;
	    yResH1=yResMSL1VsMTC1H1;
	    break ;
	  case 3233:
	    txtyResH2=txtyResMSL2VsMTC1H2;
	    xResH1=xResMSL2VsMTC1H1;
	    alphaResH1=alphaResMSL2VsMTC1H1;
	    yResH1=yResMSL2VsMTC1H1;
	    break ;
	  default:
	    txtyResH2=txtyResOthersH2 ;
	    alphaResH1=alphaResOthersH1 ;
	    xResH1=xResOthersH1 ;
	    yResH1=yResOthersH1 ;
	  }
	
	for(OTrSuperLayerEvent::ClusterContainer::const_iterator clIt=
	      (*sleIt).second.GetClusters().begin() ;
	    clIt!=(*sleIt).second.GetClusters().end() ; clIt++) {
	  // Residuals of clusters
	  for(OTrSuperLayerEvent::ClusterContainer::const_iterator 
		clIt2=(*sleIt2).second.GetClusters().begin() ;
	      clIt2!=(*sleIt2).second.GetClusters().end() ; clIt2++) 
	    txtyResH2->Fill(float((*clIt)->GetTx()-(*clIt2)->GetTx()),
			    float((*clIt)->GetTy()-(*clIt2)->GetTy())) ;
	  // Residuals of hits
	  for(OTrSuperLayerEvent::HitContainer::const_iterator 
		hitIt=(*sleIt2).second.GetHits().begin() ;
	      hitIt!=(*sleIt2).second.GetHits().end() ; hitIt++) {
	    OTrNodeProjection* nodeProjection = 
	      new OTrNodeProjection(*hitIt) ;
	    nodeProjection->Project(*clIt) ;
	    float cosAlpha = (*hitIt)->GetWire()->GetGede()->GetCosStereoAngle() ;
	    float sinAlpha = -(*hitIt)->GetWire()->GetGede()->GetSinStereoAngle() ;
	    float rawZ = (*hitIt)->GetWire()->GetRawZ() ;
	    float noemer = sqrt(1+sqr((*clIt)->GetTx()*cosAlpha+
				      (*clIt)->GetTy()*sinAlpha)) ;
	    
	    if((*hitIt)->GetWire()->GetGede()->
	       WithinYAcceptance((*clIt)->GetTx(),(*clIt)->GetTy())) {
	      float deltaX = cosAlpha * nodeProjection->GetResidualD()*noemer ;
	      xResH1->Fill(deltaX) ;
	      if(fabs(deltaX)<0.5) 
		alphaResH1->Fill(nodeProjection->GetResidualD()*noemer/
				 ((*clIt)->GetTx()*rawZ*sinAlpha-
				  (*clIt)->GetTx()*rawZ*cosAlpha)) ;
	    }
	    if((*hitIt)->GetWire()->GetGede()->GetEStereoAngle()!=
	       OTrGede::kZeroDegrees) {
	      float deltaY = sinAlpha * nodeProjection->GetResidualD()*noemer ;
	      if(fabs(deltaY)>0.005) yResH1->Fill(deltaY) ;
	    }
	    delete nodeProjection ;
	  }
	}
      }
  }
  
  // The projection. Take only tracks defined by MC2 and target.
  //   if(OTrSection::GetAlignmentOn()) {
  //     OTrEvent::SuperLayerEventContainer::const_iterator 
  //       sleIt = event->GetSuperLayerEvents().find(2) ;
  //     if(sleIt!=event->GetSuperLayerEvents().end()) {
  //       for(OTrSuperLayerEvent::ClusterContainer::const_iterator clIt=
  // 	    (*sleIt).second.GetClusters().begin() ;
  // 	  clIt!=(*sleIt).second.GetClusters().end() ; clIt++) 
  // 	for(OTrEvent::SuperLayerEventContainer::const_iterator 
  // 	      sleIt2=event->GetSuperLayerEvents().begin();
  // 	    sleIt2!=event->GetSuperLayerEvents().end(); sleIt2++)
  // 	  for(OTrSuperLayerEvent::HitContainer::const_iterator 
  // 		hitIt=(*sleIt2).second.GetHits().begin() ;
  // 	      hitIt!=(*sleIt2).second.GetHits().end() ; hitIt++) {
  // 	    (*hitIt)->GetWire()->GetSection()->GetAlignmentProjection()->
  // 	      Add(*clIt,*hitIt) ;
  // 	  }
  //     }
  //   }
  
  OTrEvent::SuperLayerEventContainer::const_iterator 
    sleMSL2It = event->GetSuperLayerEvents().find(32) ;
  OTrEvent::SuperLayerEventContainer::const_iterator 
    sleMC2It = event->GetSuperLayerEvents().find(2) ;
  OTrEvent::SuperLayerEventContainer::const_iterator 
    sleMSL1It = event->GetSuperLayerEvents().find(31) ;
  OTrSuperLayer* slMSL1 = OTrDetectorComponent::GetDetector()->GetSuperLayer(31) ;

  static TH1F* xResMSL1H1 = new TH1F("xResMSL1H1","",500,-5,5) ;
  static TH1F* xResMSL1BH1 = new TH1F("xResMSL1BH1","",100,-3,3) ;
  static TH1F* dMSL105mmH1 = new TH1F("dMSL105mmH1","",500,-5,5) ;
  static TH1F* dMSL110mmH1 = new TH1F("dMSL110mmH1","",500,-5,5) ;
  float MAXNWIREPITCHES = 1;
  float MAXNSIGMAS = 10 ;
  static TH1F* gedeMultH1 = new TH1F("gedeMultH1","",10,-0.5,9.5) ;
  static TNtuple* alignmenTuple = new TNtuple("alignmenTuple","","x_0:y_0:xresMSL1") ;
  static TNtuple* efficienTuple = new TNtuple("efficienTuple","","d1:d2:d3") ;
  static TH1F* efficiencyHisto = new TH1F("efficiencyHisto","",12,0.5,12.5) ;
  
  if(sleMSL2It!=event->GetSuperLayerEvents().end() &&
     sleMC2It!=event->GetSuperLayerEvents().end() &&
     sleMSL1It!=event->GetSuperLayerEvents().end()) {
    for(OTrSuperLayerEvent::ClusterContainer::const_iterator 
	  clIt=(*sleMC2It).second.GetClusters().begin() ;
	clIt!=(*sleMC2It).second.GetClusters().end() ; clIt++) 
      for(OTrSuperLayerEvent::ClusterContainer::const_iterator 
	    clIt2=(*sleMSL2It).second.GetClusters().begin() ;
	  clIt2!=(*sleMSL2It).second.GetClusters().end() ; clIt2++) { 
	OTrTrackCandidate* track = new OTrTrackCandidate(*clIt) ;
	track->Add(*clIt2) ; //->AddIfValid(*clIt2) ;
	if(track->Fit() && fabs(track->GetTrack()->GetX()) < 2 &&
	   fabs(track->GetTrack()->GetY()) < 4 ) {
	  int mult=0;  
	  for(OTrSuperLayerEvent::HitContainer::const_iterator 
		hitIt=(*sleMSL1It).second.GetHits().begin() ;
	      hitIt!=(*sleMSL1It).second.GetHits().end() ; hitIt++) {
	    OTrNodeProjection* nodeProjection = 
	      new OTrNodeProjection(*hitIt) ;
	    nodeProjection->Project(track->GetTrack()) ;
	    float cosAlpha = (*hitIt)->GetWire()->GetGede()->GetCosStereoAngle() ;
	    float sinAlpha = -(*hitIt)->GetWire()->GetGede()->GetSinStereoAngle() ;
	    //float rawZ = (*hitIt)->GetWire()->GetRawZ() ;
	    float noemer = sqrt(1+sqr(track->GetTrack()->GetTx()*cosAlpha+
				      track->GetTrack()->GetTy()*sinAlpha)) ;
	    if((*hitIt)->GetWire()->GetGede()->
	       WithinYAcceptance(track->GetTrack()->GetTx(),track->GetTrack()->GetTy())) {
	      float deltaX = cosAlpha * nodeProjection->GetResidualD()*noemer ;
	      xResMSL1H1->Fill(deltaX) ;
	      if(fabs(deltaX)<5)
		alignmenTuple->Fill(track->GetTrack()->GetX(),track->GetTrack()->GetY(),deltaX) ;
	      //float R = fabs(nodeProjection->GetResidualD()) ;
	      //int layId = (*hitIt)->GetWire()->GetLayerId()-1 ;
	    }
	  } 
	}
	delete track ;
      }
  }
#endif
}

