#include <CLHEP/Matrix/Vector.h>
#include <CLHEP/Matrix/SymMatrix.h>
#include "wax/OTrAlignmentProjection.hh"
#include "wax/OTrTrack.hh"
#include "wax/OTrSection.hh"
#include "wax/OTrWire.hh"
#include "wax/OTrHit.hh"
#include "wax/OTrNodeProjection.hh"

#ifdef USE_ROOT
#include <TH1.h>
#endif

//__________________________________________________________________
// OTrAlignmentProjection
//
// Contains the alignment derivatives for one section. 
//
// Author: Wouter Hulsbergen 
//__________________________________________________________________

//#ifdef __OTR_CLASSDEF__
//ClassImp(OTrAlignmentProjection) ;
//#endif

OTrAlignmentProjection::OTrAlignmentProjection(OTrSection* section) :
  fSection(section)
{
#ifdef USE_ROOT
  TString nameX(fSection->GetName()) ;
  nameX.Append("deltaXH1") ;
  fDeltaXH1 = new TH1F(nameX,nameX,1000,-5,5) ; // 1 bin = 100 um 
  TString nameY(fSection->GetName()) ;
  nameY.Append("deltaYH1") ;
  fDeltaYH1 = new TH1F(nameY,nameY,100,
		       fSection->GetRawY()-0.6*fSection->GetWireLength(),
		       fSection->GetRawY()+0.6*fSection->GetWireLength()) ;
#endif
};

bool
OTrAlignmentProjection::Add(const OTrTrackDescription* track, 
			    const OTrHit* hit)
{
#ifdef USE_ROOT
  if((hit->GetWire()->GetSection()==fSection)) {
    OTrNodeProjection* nodeProjection = new OTrNodeProjection(hit) ;
    nodeProjection->Project(track) ;
    float noemer = sqrt(1+sqr(track->GetTx()*fSection->GetCosStereoAngle()-
			      track->GetTy()*fSection->GetSinStereoAngle())) ;
    float deltaX = nodeProjection->GetResidualD()*noemer ;
    float deltaY = track->GetTy()*hit->GetWire()->GetRawZ()-
      fSection->GetRawY() ;
    if(fSection->WithinYAcceptance(track->GetTx(),track->GetTy()))
      fDeltaXH1->Fill(deltaX) ;
    if(fabs(deltaX)<hit->GetWire()->GetGede()->GetWirePitch())
      fDeltaYH1->Fill(deltaY) ;
    delete nodeProjection ;
    return true ;
  } 
#endif
  return false ;
}
