#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>

#include "wax/OTrDaq.hh"
//#include "OTrTdcBoard.hh"
//#include "OTrAsdBoard.hh"
#include "wax/OTrParameterizedRTRelation.hh"

//__________________________________________________________________
// OTrDaq
//
// Container for: 
// * OTrTdc's
//
// Author: Wouter Hulsbergen & Ivar Siccama
//__________________________________________________________________

#ifdef __OTR_CLASSDEF__
ClassImp(OTrDaq) ;
void OTrDaq::ShowMembers(TMemberInspector &, char *) {} 
void OTrDaq::Streamer(TBuffer &) {} 
#endif

int OTrDaq::fgDefaultNTdcs = 80 ;

OTrDaq::OTrDaq() : OTrDaqComponent(),  OTrComponentMother(fgDefaultNTdcs), fNProcessedEvents(0) {} ;

OTrDaq::OTrDaq(int nTdcs) :
  OTrDaqComponent(),  OTrComponentMother(nTdcs), fNProcessedEvents(0) {
    cout << "OTrDaq::OTrDaq(" << nTdcs << ")" << endl;
};

OTrDaq::~OTrDaq() {
  WriteTable("caldata/masks/occup_last.dat",kMaskTable) ;
  WriteTable("caldata/tzero/tzero_last.dat",kTZeroTable) ;
  OTrRTRelation* rtr;
  if((rtr=OTrChannel::GetRTRelation05mm())) rtr->Write("caldata/rtrelation/rtr05mm_last.dat") ;
  if((rtr=OTrChannel::GetRTRelation10mm())) rtr->Write("caldata/rtrelation/rtr10mm_last.dat") ;
}

OTrTdcBoard*
OTrDaq::AddTdcBoard(TdcId_t tdcId)
{
  OTrTdcBoard* tdcIt = FindChild(tdcId) ;
  if(!tdcIt) 
    tdcIt =  AddChild(OTrTdcBoard(SetTdcId(GetId(),tdcId))) ;
  //else cout << "WARNING: tdc " << tdcId << " already added." << endl ;
  return tdcIt ;
}

void 
OTrDaq::WriteTable(char * filename, ETableType type) const 
{
  cout << "OTrDaq::WriteTable(\"" << filename << "\")" << endl ;
  ofstream ofs(filename) ;
  ofs << type << " " << GetNChildren() << endl ;
  for(TdcConstIterator tdcIt(*this) ; tdcIt.valid(); tdcIt.next() ) {
    ofs << endl << (*tdcIt).GetTdcId() << endl ;
    for(int asdId=0; asdId<OTrTdcBoard::GetNAsdBoards(); asdId++) {
      for(int chanId=0; chanId<OTrAsdBoard::GetNChannels(); chanId++)
	switch(type) 
	  {
	  case kMaskTable: {
#ifdef USE_ROOT
	    double occupancy = 
	      (*tdcIt).GetAsdBoards()[asdId].GetChannels()[chanId].GetOccupancy() ;
#else
	    double occupancy = 0 ;
#endif
	    ofs << setw(5) << setprecision(3) << (int)(occupancy*100) ;
	    break ;
	  };
	  case kTZeroTable: {
	    ofs << setw(9) << setprecision(3) 
		<< (*tdcIt).GetAsdBoards()[asdId].GetChannels()[chanId].
	      GetTZero().GetValue();
	    break ;
	  };
	  default:
	    ofs << setw(5) << setprecision(3) << "huh" ;
	  };
      ofs << endl ;
    }
  }
}

/*
Sergey Solunin's quality definition 
0 -- channels are supposed to be good;
2 -- low efficiency (50% of neighbours);
3 -- high background in spectra;
4 -- noisy channel;
5 -- oscillations;
6 -- dead channel or one with VERY low efficiency;
7 -- large beginning of spectrum (not so bad);
8 -- two peaks in spectrum (most in sector 2');
9 -- noise 100%;
*/

void OTrDaq::ReadTable(char * filename)
{
  cout << "OTrDaq::ReadTable(\"" << filename << "\")" << endl ;
  ifstream ifs(filename) ;
  if(!ifs) {
    cout << "*** ERROR in OTrDaq::ReadTable : \"" 
	 << filename << "\" does not exist"  << endl ;
    return ;
  }

  int type,nTdc, tdcId ;
  ifs >> type >> nTdc ;
  while(ifs >> tdcId) {
    OTrTdcBoard* tdcIt = GetTdcBoard(tdcId) ;
    double tableEntry ;
    for(int asdId=0; asdId<OTrTdcBoard::GetNAsdBoards(); asdId++) {
      for(int chanId=0; chanId<OTrAsdBoard::GetNChannels(); chanId++)
	{
	  ifs >> tableEntry ;
	  if(tdcIt) {
	    switch (type)
	      {
	      case kMaskTable: {
		tdcIt->GetAsdBoards()[asdId].GetChannels()[chanId].SetMask(tableEntry);
		break ;
	      };
	      case kTZeroTable: {
		OTrFObservable t0(tableEntry,0) ;
		tdcIt->GetAsdBoards()[asdId].GetChannels()[chanId].SetTZero(t0) ;
		break ;
	      };
	      case kQMaskTable: {
		OTrChannel& chan = tdcIt->
		  GetAsdBoards()[asdId].GetChannels()[chanId];
		//int mask = chan.GetMask();
		//if ( mask != 0 && tableEntry != 0 ) { 
		//cout << "new,old = " << tableEntry << " " << mask << endl;
		//}
		if ( tableEntry > 0 ) {
		  chan.SetMask( 100 + tableEntry );
		  //  cout << "mask = " << chan.GetMask() << endl;
		}
		break ;
	      }
	      default: ;
	      }
	  }
	}
    }
  }
}

void OTrDaq::PrintSummary() const
{
  int n = 0;
  for(TdcConstIterator tdcIt(*this) ; tdcIt.valid(); tdcIt.next() ) {
    n++;
    cout << (*tdcIt).GetTdcId() << " : " 
	 << (*tdcIt).GetNProcessedEvents();
    if ( GetNProcessedEvents() != (*tdcIt).GetNProcessedEvents() ) {
      cout << "** not OK **";
    }
    if ( n%10 == 0 ) {
      cout << endl;
      n = 0;
    }
  }
}

void OTrDaq::Reset(bool b)
{
  for(TdcIterator tdcIt(*this) ; tdcIt.valid(); tdcIt.next() )
    (*tdcIt).Reset() ;
} 
