#include <iostream.h>
#include <iomanip.h>
#include <fstream.h>
#include "wax/OTrDetector.hh"
#include "wax/OTrDaq.hh"
//#include "OTrRTRelation.hh"
#include "wax/OTrParameterizedRTRelation.hh"
#include "wax/OTrHit.hh"
#include "wax/OTrLayer.hh"
#include "wax/OTrEvent.hh"

//#define LISTFILE "/afs/desy.de/user/h/hbotr/public/dmon/tables/list_of_tables_otr"
//#define TDCFILE "/afs/desy.de/user/h/hbotr/public/dmon/tables/tdc_fed_otr.dat"
#define LISTFILE "/afs/desy.de/group/hera-b/OTR/db/tables/list_of_tables_otr"
#define TDCFILE  "/afs/desy.de/group/hera-b/OTR/db/tables/tdc_fed_otr.dat"

extern "C" 
{
#include "wax/OTrDecode.h"
  int good_tdc[100]; 
}

//__________________________________________________________________
// OTrDetector
//
// Container for:
// * SuperLayers
// Functionality:
// * Building of channelmap
//
// Author: Wouter Hulsbergen && Ivar Siccama && Sascha Lanyov
//__________________________________________________________________

#ifdef __OTR_CLASSDEF__
ClassImp(OTrDetector) ;
//void OTrDetector::Streamer(TBuffer &) {} ;
//void OTrDetector::ShowMembers(TMemberInspector &, char *) {} ;
#endif

OTrDetector::~OTrDetector() {} ;

OTrDetector::OTrDetector() 
  : OTrDetectorComponent(0), fDaq(0) 
{
  // Default constructor
} ;

inline OTrSuperLayer*
OTrDetector::AddSuperLayer(SuperLayerId_t sulId) {
  OTrSuperLayer* childIt = FindChild(sulId) ;
  if(childIt!=0) return childIt ;
  return AddChild(OTrSuperLayer(SetSuperLayerId(GetId(),sulId))) ;
}

OTrDetector::OTrDetector( const string filename, OTrDaq* daq ) 
  : OTrDetectorComponent(), fDaq(daq)
{ 
  // Load gede geometry table from file. Build up to Section level.
  OTrDetectorComponent::SetDetector(this) ;
  char header[5];
  ifstream infile( filename.c_str() );
  if ( infile.is_open() <= 0 ) {
    cout << "OTrDetector: input file " << filename 
	 << " does not exist." << endl;
  } else {
    while ( infile ) {
      infile >> header;
      //cout << "header: " << header << endl;
      if ( !infile ) { break; }
      OTrGede* gede = new OTrGede( infile );
      if ( !infile ) break; 
      //cout << endl << "Read gede from file: " << *gede ;
      int secId = gede->determineSection() ;
      if ( secId >= 0 ) {
	  int sulId = gede->superLayerId() ;
	  int layId = gede->layerId() ;
	  //	  cout << "sulId = " << sulId << " layId = " << layId 
	  //   << " secId = " << secId << " gede = " << gede 
	  //   << " " << gede->GetSuperLayerName() << endl;
	  AddSuperLayer(sulId)->AddLayer(layId)->AddSection(gede) ;
      }
      //fGedelist.push_back( agede ); (obsolete)
    }
    infile.close();
  }

  // now merge with DAQ table. Build Wires
  //cout << "merging .." << endl;
  Merge() ;
  //cout << "done" << endl;
}

#ifdef USE_ARTE
OTrDetector::OTrDetector( OTrDaq* daq ) 
  : OTrDetectorComponent(), fDaq(daq)

{
  // Constructor reading the geometry directly from ARTE GEDE bank.
  OTrDetectorComponent::SetDetector(this) ;
  cout << "building OTrDetector from the ARTE geometry." << endl;
  for ( ArteTable<GEDE>::iterator itgede = ArteTable<GEDE>::begin() ;
	itgede != ArteTable<GEDE>::end(); itgede++ ) {
    if ( itgede->cmp != 2 ) { continue; }
    OTrGede* gede = new OTrGede( ArtePointer<GEDE>(itgede) );
    int secId = gede->determineSection() ;
    if ( secId>=0 ) {
      int sulId = gede->superLayerId() ;
      int layId = gede->layerId() ;
//       cout << "sulId = " << sulId << " layId = " << layId 
//          << " secId = " << secId << " gede = " << gede->GetGEDE() << endl ;
      AddSuperLayer(sulId)->AddLayer(layId)->AddSection(gede) ;
    }
  }

  // now merge with DAQ table. Build Wires
  Merge() ;
}
#endif

void
OTrDetector::BuildGEDEIndexToSectionMap()
{
  // temporary solution (Ivar)
  fGEDEIndexToSectionMap = vector<const OTrSection*>(1000);
  for(SuperLayerContainer::const_iterator sulIt=GetSuperLayers().begin() ;
      sulIt!=GetSuperLayers().end() ; sulIt++)
    for(OTrSuperLayer::LayerContainer::const_iterator layIt = (*sulIt).GetLayers().begin() ;
	layIt != (*sulIt).GetLayers().end() ; layIt++)
      for(OTrLayer::SectionContainer::const_iterator secIt = (*layIt).GetSections().begin() ;
	  secIt != (*layIt).GetSections().end() ; secIt++) {
	int index = (*secIt).GetGEDEIndex() ;
	// cout << index << " " << (*secIt).GetSectionId() << endl;
	fGEDEIndexToSectionMap[index] = &(*secIt);
      }
}

void
OTrDetector::WriteSectionPositions(char* filename)
{
  // Just some debug info.
  ofstream ofs(filename) ;
  for(SuperLayerContainer::const_iterator sulIt=GetSuperLayers().begin() ;
      sulIt!=GetSuperLayers().end() ; sulIt++)
    for(OTrSuperLayer::LayerContainer::const_iterator layIt = (*sulIt).GetLayers().begin() ;
	layIt != (*sulIt).GetLayers().end() ; layIt++)   
      for(OTrLayer::SectionContainer::const_iterator secIt = (*layIt).GetSections().begin() ; 
	  secIt != (*layIt).GetSections().end() ; secIt++)
     	{
	  ofs << setw(5) << (*secIt).GetSuperLayerId() 
	      << setw(5) << (*secIt).GetLayerId()
	      << setw(5) << (*secIt).GetSectionId() 
	      << setw(9) << setprecision(3) << (*secIt).GetPMin().x() 
	      << setw(9) << setprecision(3) << (*secIt).GetPMin().y() 
	      << setw(9) << setprecision(3) << (*secIt).GetPMin().z() 
	      << setw(9) << setprecision(3) << (*secIt).GetPMax().x() 
	      << setw(9) << setprecision(3) << (*secIt).GetPMax().y() 
	      << setw(9) << setprecision(3) << (*secIt).GetPMax().z() 
	      << endl ;
	};
};

void OTrDetector::Merge() 
{
  // Build complete OTrDetector and OTrDaq, setting all pointers using
  // Dima's OTrDecode toutines.

  int ntot = 0;           // total number of wires
  int nconnected = 0;     // number of unconnected wires

  // initialize the decoding routines from OTrDecode
  unsigned char* decodeTable = OTr_decode_TDC_init(LISTFILE,TDCFILE);

  for( SuperLayerContainer::iterator sulIt=GetSuperLayers().begin();
       sulIt != GetSuperLayers().end(); sulIt++ ) {
    int sulId = sulIt->GetSuperLayerId();
    //cout << "Now connecting superlayer " << sulId << "." << endl; 
    for( OTrSuperLayer::LayerContainer::iterator layIt=sulIt->GetLayers().begin();
 	 layIt!=sulIt->GetLayers().end(); layIt++) {
      //cout << "Now connecting layer " << layIt->GetSuperLayerId() << " " 
      //	   << layIt->GetLayerId() << endl ;
      int layId = layIt->GetLayerId();
      for( OTrLayer::SectionContainer::iterator secIt=layIt->GetSections().begin();
	   secIt!=layIt->GetSections().end(); secIt++ ) {
	//cout << "Now connecting section " << secIt->GetSuperLayerId() << " " 
	//     << layIt->GetLayerId() << " " << secIt->GetSectionId() << endl ;
	secIt->CreateWires();
	int secId = secIt->GetSectionId();
	//cout << "sl = " << sulId << " lay = " << layId << " sec = " << secId 
	//   << " " << secIt->GetNWires() << endl;
	for ( OTrSection::WireContainer::iterator wireIt = secIt->GetWires().begin();
	      wireIt != secIt->GetWires().end(); wireIt++ ) {
	  ntot++;
	  int wire = wireIt->GetWireNumber();
	  int nwire = wire;
	  int subsecId = wire/16 + 1 ;
	  int group_of_16 = subsecId;
	  if ( secId == 104 || secId == 203 ) group_of_16 += 4;
	  else if( secId == 105 || secId == 205 ) group_of_16 += 8;
	  else if( secId == 106 || secId == 206 ) group_of_16 += 9;
	  int tdcId, asdId;
	  int sectorId = secIt->GetSectorNumber();

	  //gives always MC02???	  cout << "Superlayer " << sulIt->GetName() << endl;
	  //if (sulIt->GetName() == "TC02" ) {
	  if (sulId == 33 ) {
	    //cout << "Recalculating wire number for TC02:" << endl;	
	    int mono  = wire%4;
	    int iwire = wire/4;
	    int slay = 1 - mono/2;
	    nwire = 4*iwire + 2*(3-mono) + slay + 1;
	    group_of_16 = (nwire-1)/16 + 1;

	    // 	    cout << " wire=" << wire
	    // 	         << " mono=" << mono
	    // 	         << " iwire=" << iwire
	    // 	         << " slay=" << slay
	    // 	         << " nwire=" << nwire
	    // 		 << " group_of_16=" << group_of_16
	    // 		 << endl;

	    int nwires = secIt->GetGede()->GetNWires();
	    // cout << "nwires = " << nwires << endl;

	    if ( nwire > nwires ) {
	      nwire = nwire-nwires;

	      if (secId == 90) {
		sectorId    = 1;
		group_of_16 = 1;
	      }
	      else if (secId == 100) {
		sectorId    = 6;
		group_of_16 = 1;
	      }
	      else {
		sectorId    = 0;
		group_of_16 = 0;
		};
	      //       cout << "---------------- Adjacent ASD: nwire=" << nwire
	      // 		   << " sectorId=" << sectorId
	      // 		   << " group_of_16=" << group_of_16
	      // 		   << endl;
	    };

	    // Due to wrong cabling in mini-TC
	    if (group_of_16 == 0)   {group_of_16 = 0; }                      // not connected
	    else if (sectorId <= 6) {group_of_16 = 1 + (group_of_16+3)%8; }  // 5 mm
	    else                    {group_of_16 = 1 + (group_of_16+1)%4; }; // 10 mm

	    if      (sectorId == 1) {sectorId = 5; }
	    else if (sectorId == 5) {sectorId = 1; };

	  };

	  if (group_of_16 == 0) {
	    //cout << "This wire is not connected" << endl;
	    continue;
	  }

	  //if(layId==3) layId==1; 
	  //else if(layId==1) layId==3; 

	  int stat = OTrGetConnector(decodeTable, sulId, layId, 
				     sectorId, group_of_16,
				     &tdcId, &asdId );
	  //   if(sectorId==5 && wire==1)
	  // 	     cout << "sulId = " << sulId
	  // 		  << " lay = " << layId
	  // 		  << " stereo = " << secIt->GetSinStereoAngle() 
	  // 		  << " sector = " << sectorId
	  // 		  << " w = " << wire
	  // 		  << " group_of_16 = " << group_of_16
	  // 		  << "           tdc, conn = " << tdcId << " " << asdId << endl;
       
	  if ( stat != 1 ) {	
            cout << "** ERROR in OTrDetector::Merge()" << endl; 
	    cout << "id = " << wireIt->GetId() << " sulId = " << sulId << " lay = " << layId
		 << " sector = " << sectorId
		 << " w = " << wire
		 << " group_of_16 = " << group_of_16
	         << "           tdc, conn = " << tdcId << " " << asdId
		 << endl;
	    //	    exit(1);
	  }
	  OTrTdcBoard* tdcIt = fDaq->AddTdcBoard(tdcId) ;
	  if ( tdcIt == 0 ) {
	    cout << "** ERROR in OTrDetector: tdcId = " << tdcId << endl;
	    continue;
	  }
	  OTrAsdBoard* asdIt = &(tdcIt->GetAsdBoard(asdId-1)) ;
	  if ( asdIt == 0 ) {
	    cout << "** ERROR in OTrDetector: asdId = " << asdId << endl;
	    continue;
	  }
       
	  int channelNr = nwire%16;
	  if (sulId == 33 ) {
	    int channelNr = 15 - channelNr; // wrong orientation of ASD in mini-TC
	  } else // A very add hoc solution .... :)
	    if(secIt->GetXSide()==OTrGede::kXSidePlus)
	      channelNr = channelNr + 1 - 2* (channelNr%2) ;
	  
	  OTrChannel* chan = asdIt->FindChannel( channelNr );
	  if(wireIt->GetChannel()!=0) cout << "connected wire twice ! " << sulId
					   << " " << layId
					   << " " << sectorId
					   << " " << wire
					   << " " << group_of_16
					   << " " << tdcId << endl ;
	  wireIt->SetChannel( chan );
	  if(chan->GetWire()!=0) cout << "connected channel twice ! " <<  sulId
				      << " " << layId
				      << " " << sectorId
				      << " " << wire
				      << " " << group_of_16
				      << " " << tdcId 
				      << "|" 
				      << chan->GetWire()->GetSuperLayerId() << " "
				      << chan->GetWire()->GetLayerId() << " "
				      << chan->GetWire()->GetGede()->GetSectorNumber() << " "
				      << chan->GetWire()->GetWireNumber() << endl ;
	  chan->SetWire( wireIt );
	  nconnected++;
	}
	//	cout << endl;
      }
    }
  }
  cout << "total number of wires: " << ntot 
       << " connected: " << nconnected << endl;
  BuildGEDEIndexToSectionMap();
  //dump();
};

void OTrDetector::dump() const
{
  // checks also if the pointers from wires to sections are set
  // correctly
  int nNotConnectedWires(0) ;
  int nBadConnectedWires(0) ;
  cout << "number of superlayers: " << GetSuperLayers().size() << endl;
  for( SuperLayerContainer::const_iterator sulIt=GetSuperLayers().begin();
       sulIt != GetSuperLayers().end(); sulIt++ ) {
    cout << "superlayer Id = " << sulIt->GetSuperLayerId() << endl;
    for( OTrSuperLayer::LayerContainer::const_iterator layIt=sulIt->GetLayers().begin();
 	 layIt!=sulIt->GetLayers().end(); layIt++) {
      int layId = layIt->GetLayerId();
      cout << "layer Id = " << layId << " ";
      int nsec = 0;
      int nwir = 0;
      for( OTrLayer::SectionContainer::const_iterator secIt=layIt->GetSections().begin();
	   secIt!=layIt->GetSections().end(); secIt++ ) {
	//int secId = secIt->GetSectionId();
	nsec++;
	//cout << " secId = " << secId << " " 
	//   << secIt->GetNWires() << " " << flush;
	for ( OTrSection::WireContainer::const_iterator wireIt = secIt->GetWires().begin();
	      wireIt != secIt->GetWires().end(); wireIt++ ) {
	  nwir++;
	  if ( wireIt->GetSection() != secIt ) {
	    cout << " ** ERROR in OTrDetector: " << wireIt->GetId() << " " 
		 << wireIt->GetSection()->GetSectionId() << " " 
		 << wireIt->GetSection() << " " << secIt << endl;
	  }
	  if ( wireIt->GetChannel()==0) nNotConnectedWires++;
	  else if ( wireIt->GetChannel()->GetWire() != wireIt)
	    nBadConnectedWires++;
	}
      }
      cout << nsec << " sections " << nwir << " wires. " << endl;
    }
  }
  cout << "There are " << nNotConnectedWires << " wires without a channel; "
       << nBadConnectedWires << " bad connected. " << endl ;

  int nNotConnectedChannels(0) ;
  int nBadConnectedChannels(0) ;
  for(OTrDaq::TdcContainer::const_iterator tdcIt = fDaq->GetTdcBoards().begin();
      tdcIt != fDaq->GetTdcBoards().end() ; tdcIt++)
    for(OTrTdcBoard::AsdContainer::const_iterator asdIt = tdcIt->GetAsdBoards().begin();
	asdIt != tdcIt->GetAsdBoards().end(); asdIt++)
      for(OTrAsdBoard::ChannelContainer::const_iterator chanIt = asdIt->GetChannels().begin() ;
	  chanIt != asdIt->GetChannels().end() ; chanIt++)
	if(chanIt->GetWire()==0) nNotConnectedChannels++;
	else if(chanIt->GetWire()->GetChannel()!=chanIt)
	  nBadConnectedChannels++;
  cout << "There are " << nNotConnectedChannels << " channels without a wire; " 
       << nBadConnectedChannels << " bad connected." << endl ;
}

#ifdef USE_ARTE
void
OTrDetector::ResetGEDEs()
{
  // Reset the pointers to the first and last HITBs in each GEDE
  for( vector<OTrSuperLayer>::iterator sulIt=GetSuperLayers().begin();
       sulIt != GetSuperLayers().end(); sulIt++ ) {
    for( vector<OTrLayer>::iterator layIt=sulIt->GetLayers().begin();
 	 layIt!=sulIt->GetLayers().end(); layIt++) {
      for( vector<OTrSection>::iterator secIt=layIt->GetSections().begin();
	   secIt!=layIt->GetSections().end(); secIt++ ) {
	secIt->GetGEDE()->hit1 = ArtePointer<HITB>(0)  ;//ArteTable<HITB>::end() ;
	secIt->GetGEDE()->hitl = ArtePointer<HITB>(-1) ;//ArteTable<HITB>::end() ;
      }
    }
  }
}
#else
void
OTrDetector::ResetGEDEs()
{
  // Dummy routine for use outside arte
}
#endif


/******************** Obsolete stuff etc ... *****************/


// #ifdef USE_ARTE
// OTrEvent*
// OTrDetector::nextmc()
// {
//   // Make a new OTrEvent out of the arte HITB bank. For Monte Carlo
//   // purposes only.

//   //  cout << "nextmc: creating OTrEvent structure" << endl;
//   OTrEvent* event = new OTrEvent() ;

//   for ( ArteTable<HITB>::iterator ihitb = ArteTable<HITB>::begin(); 
// 	ihitb != ArteTable<HITB>::end() ; ihitb++ ) {
//     ArtePointer<HITB> hitb = ArtePointer<HITB>(ihitb);
//     ArtePointer<GEDE> gede = ArtePointer<GEDE>(hitb->gede);
//     if ( gede->cmp != 2 ) { continue; }       // select only OTr hits
//     //int layer = gede->lay;
//     // cout << "layer = " << layer << endl;    
//     //int sector = gede->sec;  // Apparently, this is sometimes wrong
//     // (we have no sector 12 in MC2!)
//     // cout << "sector = " << sector << endl;
 
//     // Get the wirenumber from the DIGB information:
//     for ( ArteRelationIterator<DIGB> idigb(hitb->digb);
//           idigb.valid(); idigb.next() ) {
//       ArtePointer<DIGB> digb = ArtePointer<DIGB>( idigb );
//       int part = (digb->id)/10000;

//       //int digb_wirenr = part/4;            // wire number
//       //int tube = part%4;                   // monolayer

//       // Decoding of PART = 4*WIRE + MONOLAYER + 1
//       int digb_wirenr = (part-1)/4;  // wire (0..31)
//       int monolayer = (part-1)%4;    // monolayer (0..3)

//       //ArtePointer<GESL> gesl = ArtePointer<GESL>(gede->gesl);
//       //int superlayer = gesl.index();
//       /*
// 	if      (!strncmp(gesl->name,"MC02",4)) {superlayer=13;}
// 	else if (!strncmp(gesl->name,"PC03",4)) {superlayer=14;}
// 	else {
// 	cout << "******** Superlayer name not found!" << endl;
// 	continue;
// 	};
// 	*/

//       //cout << "superlayer = " << gesl->name << " " << superlayer; 
//       //cout << " hitb = " << hitb.index() << " gede = " << gede.index()
//       //     << " id: " << digb->id 
//       //     << " part: " << part
//       //     << " monolayer: " << monolayer
//       //     << " digb_wirenr: " << digb_wirenr 
//       //     << " pitch: " << gede->pit << endl;

//       int nsl=gede->nsl; // Number of monolayers
//       int wirenr = digb_wirenr*nsl + monolayer;

//     //   if(hitb->gede==333 || hitb->gede==327)
// // 	cout << hitb->gede << " " << part << " "
// // 	     << digb_wirenr << " " << monolayer << " " << wirenr << " "
// // 	     << hitb->x << " " << hitb->z << endl ;
//       /*
//       int wirenr;
//       if ( tube == 1 ) {
// 	wirenr = digb_wirenr*2 + 1; 
// 	// to be correct, +1 should be at even tube!!!
//       } else {
// 	wirenr = digb_wirenr*2;
//       }
//       */

//       const OTrSection* sec = GetGEDEIndexToSectionMap()[gede.index()];
//       if (!sec) {
//       	cout << "*** Section not found! gede = " << gede.index() << endl;
// 	continue;
//       };

//       const OTrWire* Wire = sec->GetWire(wirenr);
//       if (!Wire) {
//  	cout << "******** Wire not found! wirenr = " << wirenr
//  	     << " gede = " << gede.index()
//              << " monolayer = " << monolayer 
// 	     << ", wire = " << digb_wirenr
// 	     << endl;
// 	continue;
//       };
//       // cout << *Wire << endl;
//       OTrChannel* Channel = Wire->GetChannel();
//       if (!Channel) {
//  	cout << "******** Channel not found for wire "
// 	     << Wire->GetId()
// 	     << endl;
// 	continue;
//       }
//       // cout << "Channel->GetId() = " << Channel->GetId() << endl;

//       // The following uses the `current' value for rtrelation and
//       // tzero. Consequently, it cannot be used to check the
//       // calibration. I propose implementing a routine
//       // OTrChannel::GetMCRTRelation and OTrChannel::GetMCTdcCount.
   
//       ArteRelationIterator<MIMP> imimp(hitb->mimp) ;
//       ArtePointer<MIMP> mimp(imimp) ;
//       float x0 = mimp->x - mimp->z*mimp->tx ;
//       float y0 = mimp->y - mimp->z*mimp->ty ;
//       bool targetConstraint = false ;
//       bool momentumConstraint = false ;//true ;
//       //cout << "momentum: " << mimp->p << endl ;
//       //cout << hitb << " " << hitb->z << " " << hitb->x << " " ;
//       if((!targetConstraint || (fabs(x0)<10 && fabs(y0)<10)) &&
// 	 (!momentumConstraint || (fabs(mimp->p)>10))) { 
// 	float R = fabs(hitb->q) ;
// 	float driftTime = Channel->GetMCRTRelation()->GetT(R).GetValue() ;
// 	OTrChannel::TdcCount_t tdccount = 
// 	  //Channel->GetMCTdcCount(driftTime) ;
// 	  Channel->GetTdcCount(driftTime) ;
// 	OTrHit* hit = Channel->ProcessChannelBuffer( tdccount, event ) ;
// 	if(hit) {
// 	  hit->SetHITB(hitb) ;
// 	  //cout << hit->GetWire()->GetRawZ() << " " << hit->GetWire()->GetRawX() << " " ;
// 	} else cout << "????? Hit not accepted. " ;
// 	// 	     << hitb <<  " " 
// 	// 	     << hitb->gede << " " 
// 	// 	     << hitb->x << " " << hitb->z << endl ;
//       };
//       //cout << endl ;
//       break; // only one DIGB
//     }
//   }
//   //event->DumpHits();
//   return event ;
// }

