#include <iostream>
#include "etb.h"
#include "log.h"
#ifdef USE_ARTE 
//#include "online/etb.h"
#include <arte/arte.hh>
#include <farm/farm.hh>
//typedef EVHD LogHdr ;
#endif

#include "wax/OTrEventBuffer.hh"
#include "wax/OTrEventBuilder.hh"

//__________________________________________________________________
// OTrEventBuilder
//
// Container for: 
// * Eventheader, fedbuffers etc
//
// Functionality:
// * Rerieval of data from file or DOTR bank.
//
// Author: Wouter Hulsbergen && Ivar Siccama
//__________________________________________________________________

#ifdef __OTR_CLASSDEF__
ClassImp(OTrEventBuffer) ;
#endif

const int OTrEventBuffer::fgMaxNFedBuffers = 100 ;
const int OTrEventBuffer::fgMaxEventLength = 128*1024 ;
const int OTrEventBuffer::fgDaqComponentId = 0x400 ;

OTrEventBuffer::OTrEventBuffer() 
  : fDataStream(0), fInputFile(0), fNFedBuffers(0), fFedBuffers(0), fEvtBuffer(0), fEvtHdr(0), fLogHdr(0)
{
  fFedBuffers = new ETB_fed[GetMaxNFedBuffers()];
#ifndef USE_ARTE
  fEvtBuffer = new char[fgMaxEventLength] ;
  fLogHdr = new LogHdr ;
#endif
  fEvtHdr = (EVENT_HDR *) fEvtBuffer ;
  //fEvent = new OTrEvent() ;
}

OTrEventBuffer::OTrEventBuffer(int datastream) 
  : fDataStream(datastream), fInputFile(0), fNFedBuffers(0), fFedBuffers(0), fEvtBuffer(0), 
    fEvtHdr(0), fLogHdr(0)
{
  fFedBuffers = new ETB_fed[GetMaxNFedBuffers()];
  switch(GetDataStream())
    {
    case OTrEventBuilder::kDS_DOTR:
      fEvtBuffer = 0 ;
      fLogHdr = 0 ;
      break;
    default:
      fEvtBuffer = new char[fgMaxEventLength] ;
      fLogHdr = new LogHdr ;
      break;
    }
  fEvtHdr = (EVENT_HDR *) fEvtBuffer ;
}

OTrEventBuffer::~OTrEventBuffer() {
  if(fFedBuffers) delete[] fFedBuffers ;
  if(fEvtBuffer) delete[] fEvtBuffer ;
  if(fInputFile) fclose(fInputFile) ;
  if(fLogHdr) delete fLogHdr ;
}

int OTrEventBuffer::NextFEDBuffers()
{ 
  // In the arte modes, updating of the arte banks is performed
  // independent of WAX and this routine only updates the internal WAX
  // structure according to the data stored in ARTE (DOTR)
  // bank. In standalone mode this routine also takes care that a new
  // event is read (by asking the OTrEventBuffer for a new event
  // header).
  
  int rc = 0;
  switch(GetDataStream()) 
    {
    case OTrEventBuilder::kDS_DOTR:
      rc=DOTRUpdateFedBuffers() ;
      break ;
    case OTrEventBuilder::kDS_FARM:
    case OTrEventBuilder::kDS_STANDALONE:
      // read untill the first event where also the buffers are oke ...
      while((rc=ETBNextEventHdr())==ETB_ok) {
	if(ETBUpdateFedBuffers()==ETB_ok) break ;
	cout << "skipping this event ..." << endl ;
      }
      //rc=ETBNextEventHdr() ;
      //if(rc==ETB_ok) rc=ETBUpdateFedBuffers() ;
      break ;
    default:
      cout << "OTrEventBuffer::UpdateFEDBuffers() : No valid data-type specified ! " << endl ;
      break ;
    }
  return rc ;
}

int
OTrEventBuffer::OpenFile(char* filename, int fileformat)
{
  int rc=0;
  switch(GetDataStream())
    {
    case OTrEventBuilder::kDS_FARM:
    case OTrEventBuilder::kDS_STANDALONE:
      if(fInputFile) {
	//cout << "closing the data file " << endl;
	fclose(fInputFile);
      }
      fInputFile = fopen(filename,"r"); 
      if ( fInputFile ) {
        cout << "Successfully opened file \"" << filename << "\"." << endl;
	if(fileformat==0) fDataStream=OTrEventBuilder::kDS_STANDALONE;
	else fDataStream=OTrEventBuilder::kDS_FARM;
      } else {
	cout << "No such file \"" << filename << "\"." << endl;
	rc=1;
      }
      break ;
    case OTrEventBuilder::kDS_DOTR:
    default: ;
    }
  return rc ;
}

void 
OTrEventBuffer::DumpFedBuffers()
{
  cout << "fNFedBuffers = " << fNFedBuffers << endl;
  for( int i=0; i < fNFedBuffers; i++ ){
    cout << "usevnt_(): i=" << i << " "
	 << "length="         << fFedBuffers[i].hdr.length        << " "
	 << "fedid=0x" << hex << fFedBuffers[i].hdr.fedid  << dec << " "
	 << "recid=0x" << hex << fFedBuffers[i].hdr.recid  << dec << " "
	 << "bxid=0x"  << hex << fFedBuffers[i].hdr.bxid   << dec << endl;
    
    // FED data
    int           l  = fFedBuffers[i].hdr.length - sizeof(  SLB_SUBHDR );
    unsigned char *p = (unsigned char *) fFedBuffers[i].data;
    for( int j=0; j<l; j++ ){
      unsigned char adc = *( p + j );
      if( adc != 0xff ){
	cout << "  j=" << j << " adc=0x" << hex << int( adc ) << dec << endl;
      }
    }
  }
}

#ifdef USE_ARTE
int OTrEventBuffer::ETBNextEventHdr() {
  // Dummy routine for use in ARTE
  return 0;
}

int OTrEventBuffer::ETBUpdateFedBuffers()
{
 // Dummy routine for use in ARTE
  return 0;
}

int OTrEventBuffer::DOTRUpdateFedBuffers()
{
  int rc(ETB_ok) ;
  int recid = 0;
  rc = ARTE_get_recid_DOTR( &recid );
  rc = ARTE_get_feds_DOTR( GetMaxNFedBuffers(), fFedBuffers, &fNFedBuffers );
  if ( rc != ETB_ok ) {
    cout << "** ERROR in OTrEventBuffer::DOTRUpdateFedBuffers, rc = "
	 << rc << endl;
  } else {
    //DumpFedBuffers();
  }
  return rc; 
}

#else // USE_ARTE

int OTrEventBuffer::ETBNextEventHdr()
{
  int rc=1;
  if(fInputFile) {
    switch(GetDataStream())
    {
    case OTrEventBuilder::kDS_STANDALONE:
      rc=ETB_read_event(fInputFile, fEvtHdr, fgMaxEventLength);
      break;
    case OTrEventBuilder::kDS_FARM:
      int D1;
      rc=l4aGetEvt(fInputFile, fEvtHdr, fgMaxEventLength, &D1, fLogHdr );
      break;
    default: ;
    }
    if (rc!=ETB_ok) {
      cout << "*** OTrEventBuffer::NextEventHdr() "
	   << " error reading event from file" << endl;
    }
    if ( feof(fInputFile) ) cout << "At end of datafile" << endl;
  } else {
    cout << "No datafile opened." << endl ;
  }
  return rc;
}

int OTrEventBuffer::ETBUpdateFedBuffers()
{
  // Calls ETB_get_feds(..)
  int rc = ETB_get_feds(fEvtHdr, GetDaqComponentId(), GetMaxNFedBuffers(),
                        fFedBuffers, &fNFedBuffers);
  if (rc!=ETB_ok)
    printf("Error getting FEDs for *event= %x rc= %d \n", (int) fEvtHdr, rc);
  //printf("Getting FEDs for *event= %x rc= %d \n", (int) fEvtHdr, rc);
  return rc ;
}

int OTrEventBuffer::DOTRUpdateFedBuffers()
{
  // Dummy routine for use outside ARTE.
  return 0;
}

#endif // USE_ARTE






