#include "wax/OTrRTCalib.hh"
#include "wax/OTrParameterizedRTRelation.hh"
#include "wax/OTrHit.hh"
#include "wax/OTrWire.hh"
#include "wax/OTrSection.hh"
#include "wax/OTrRTProjection.hh"
#include "wax/OTrChannel.hh"
#include "wax/OTrTrack.hh"
#include "wax/OTrTrackCandidate.hh"
#include "wax/OTrSuperCluster.hh"
#include "wax/OTrReconstructedHit.hh"
#include "wax/OTrTZeroCalib.hh"
#include "wax/OTrEvent.hh"
#ifdef USE_ROOT
#include <TCanvas.h>
#include <TH1.h>
#endif

//__________________________________________________________________
// OTrRTCalib
//
// The RT calibration class
// Container for:
// Functionality:
//
// Author: Wouter Hulsbergen
//__________________________________________________________________

#ifdef __OTR_CLASSDEF__
ClassImp(OTrRTCalib) ;
//const char * OTrRTCalib::DeclFileName() { return 0 ;} ;
//short int OTrRTCalib::Class_Version() { return 0 ;} ;
#endif

int OTrRTCalib::fgUpdateFrequency=20000 ; // tracks

OTrRTCalib::OTrRTCalib() 
  : fCanvas(0), fTZeroCalib(0)
{
  Init() ;
}

OTrRTCalib::OTrRTCalib(OTrTZeroCalib* tzerocalib) 
  : fCanvas(0), fTZeroCalib(tzerocalib)
{
  Init() ;
}

void OTrRTCalib::Init() 
{
  cout << "OTrRTCalib::OTrRTCalib(....) " << endl ;
  fRTProjection = new OTrRTProjection*[2] ;
  fRTProjection[0] = new 
    OTrRTProjection((OTrParameterizedRTRelation*)OTrChannel::GetRTRelation05mm()) ;
  fRTProjection[1] = new 
    OTrRTProjection((OTrParameterizedRTRelation*)OTrChannel::GetRTRelation10mm()) ;
  
#ifdef USE_ROOT
  //ROOTSTUFF 
  fCanvas = 0; //new TCanvas("OTrRTCalibCanvas", "OTrRTCalibCanvas",2); 
  fChi2ProbH1 = new TH1F("CorrectedChi2Prob","",50,0,1) ;
  if(fCanvas) {
    fCanvas->Show(); fCanvas->Clear();
    fCanvas->cd() ; fChi2ProbH1->Draw() ;
    fCanvas->Update();
  }
#endif //USE_ROOT
  cout << "Leaving OTrRTCalib::OTrRTCalib(....) " << endl ;
  Reset() ;
  OTrParameterizedRTRelation::SetCorrectionLevel(1) ;
}

void
OTrRTCalib::Reset()
{
  // Initialisation
  fRTProjection[0]->Reset() ;
  fRTProjection[1]->Reset() ;
  fNTrackCandidates = 0 ;
  fNTracks = 0 ;
  fNHitsTotal = 0 ;
  fNHitsTrack = 0 ;
  fNReconstructedHits = 0;
  fNDofs = 0 ;
  fChiSquare = 0;
#ifdef USE_ROOT
  fChi2ProbH1->Reset();
#endif
}

void
OTrRTCalib::UpdateCanvas()
{
#ifdef USE_ROOT
  if(fCanvas) {
    fCanvas->Modified() ;
    fCanvas->Update();
  }
  fRTProjection[0]->UpdateCanvas() ;
  fRTProjection[1]->UpdateCanvas() ;
#endif
}

void
OTrRTCalib::Add(const OTrEvent* event) 
{
  if(event) {
    //OTrEvent::SuperLayerEventContainer::const_iterator 
    //  sleIt = fEvent->GetSuperLayerEvents().find(2) ;
    // I wanna try with mc-02 only.
    for(OTrEvent::SuperLayerEventContainer::const_iterator 
	  sleIt=event->GetSuperLayerEvents().begin();
	sleIt!=event->GetSuperLayerEvents().end(); sleIt++)
      if(sleIt->first==2) { // do not take mini tc in rt calibration
	for(OTrSuperLayerEvent::ClusterContainer::const_iterator 
	      clIt=(*sleIt).second.GetClusters().begin() ; 
	    clIt!=(*sleIt).second.GetClusters().end() ; clIt++) 
	  Add((*clIt)) ;
      } 
  }
}

void
OTrRTCalib::Add(const OTrTrackCandidate* trackCandidate)
{
  fNTrackCandidates++ ;
  //fNHitsTotal += (*trackCandidate).GetHits().size() ;
  OTrReconstructedTrackInterface* trackp ;
  if((trackp=(*trackCandidate).GetTrack())) {
    fNTracks++ ;
    fNHitsTrack += (*trackp).GetHits().size() ;
    fNDofs    += (*trackp).GetNDof() ;
#ifdef USE_ROOT
    fChi2ProbH1->Fill((*trackp).GetChi2Prob());
#endif
    fChiSquare += (*trackp).GetChi2() ;
    
    for(vector<const OTrSuperCluster*>::const_iterator clIt=trackCandidate->GetClusters().begin() ;
	clIt!=trackCandidate->GetClusters().end(); clIt++)
      for(vector<OTrSectionCluster*>::const_iterator secIt=(*clIt)->GetSectionClusters().begin() ;
	  secIt!=(*clIt)->GetSectionClusters().end() ; secIt++) 
	for(OTrSectionCluster::HitIterator hitIt=(*secIt)->begin(); hitIt!=(*secIt)->end() ; hitIt++) {
	  OTrGede* gede = (OTrGede*)((*hitIt)->GetWire()->GetSection());
	  if(fRTProjection[gede->GetEPitch()]->Add(*trackp,*hitIt))
	    fNReconstructedHits++ ;
	  fNHitsTotal++ ;
	};
  };
  //if(!((++fCycle)%2000)) UpdateCanvas() ;
};

void
OTrRTCalib::Add(const OTrReconstructedTrackInterface* trackp)
{
  fNTracks++ ;
  fNHitsTrack += (*trackp).GetHits().size() ;
  fNDofs    += (*trackp).GetNDof() ;
#ifdef USE_ROOT
  fChi2ProbH1->Fill((*trackp).GetChi2Prob());
#endif
  fChiSquare += (*trackp).GetChi2() ;
  for(OTrReconstructedTrackInterface::HitContainer::const_iterator 
	recHitIt=trackp->GetHits().begin() ;
      recHitIt!=trackp->GetHits().end() ; recHitIt++) {
    const OTrHit* hitIt =  (*recHitIt)->GetMeasuredHit() ;
    OTrSection* gede = hitIt->GetWire()->GetSection() ;
    if(fRTProjection[gede->GetEPitch()]->Add(*trackp,hitIt))
      fNReconstructedHits++ ;
    fNHitsTotal++ ;
  };
  if(!((fNTracks+1)%fgUpdateFrequency)) Fit(true,true,true,true) ;
  // cout << fNTracks << endl ;
};

bool
OTrRTCalib::Fit(bool fitT, bool fitSigma2, bool fit05mm, bool fit10mm)
{
  cout << "Number of selected tracks: " << fNTracks << endl 
       << "chi2tot       " << fChiSquare << endl 
       << "chi2totbar:   " << fChiSquare/fNDofs << endl
       << "nHitsTotal:   " << fNHitsTotal << endl
       << "nHitsTrack:   " << fNHitsTrack << endl
       << "totRecHits:   " << fNReconstructedHits << endl ;
  
  bool converged(true) ;

  if(fitT && fit05mm) converged *= fRTProjection[0]->FitT() ;
  if(fitT && fit10mm) converged *= fRTProjection[1]->FitT() ;
  if(fitSigma2 && fit05mm) converged *= fRTProjection[0]->FitSigma2() ;
  if(fitSigma2 && fit10mm) converged *= fRTProjection[1]->FitSigma2() ;

  fRTProjection[0]->Draw() ;
  fRTProjection[1]->Draw() ;
  UpdateCanvas() ;
 
  //if(fTZeroCalib) fTZeroCalib->CalibFromResiduals() ;
  Reset() ;
  return converged ;
};
