#include "wax/OTrRunSummary.hh"
#include "wax/OTrEvent.hh"

#ifdef USE_ROOT
#include <TCanvas.h>
#include <TH1.h>
#include <TProfile.h>
#endif

void
OTrRunSummary::Add(const OTrEvent* event) 
{
  
  if(event) fEventSummaries.push_back( OTrEventSummary(event)) ;
};

OTrEventSummary::OTrEventSummary() {
  cout << "OTrEventSummary default constructor. Do not use. " << endl ;
};

OTrEventSummary::OTrEventSummary(const OTrEvent* event)
{
  fId = event->GetId() ;
  fNHits   = 0; //event->GetNHits() ;
  fNBadHits = 0 ;
  fNGoodHits = 0 ;
  //fTdcSum = 0 ;

  const OTrSuperLayerEvent* slEvent = event->FindSuperLayerEvent(10) ;
  if ( slEvent ) {
    for(OTrSuperLayerEvent::HitContainer::const_iterator hitIt=slEvent->GetHits().begin() ;
	hitIt!=slEvent->GetHits().end() ; hitIt++)
      if( ((OTrGede*)(*hitIt)->GetWire()->GetSection())->GetEPitch() 
	  == OTrGede::kFiveMM) {
	//fTdcSum += (*hitIt)->GetTdcCounts();
	if( (*hitIt)->GetTdcCounts()>10 && (*hitIt)->GetTdcCounts()<100) 
	  fNBadHits++;
	else if ( (*hitIt)->GetTdcCounts()>170 && (*hitIt)->GetTdcCounts()<200) 
	  fNGoodHits++;
	fNHits++ ;
      }
  }
}


OTrRunSummary::OTrRunSummary() : fEventSummaries() {}

#ifdef USE_ROOT
void
OTrRunSummary::Draw(int nEventIdsPerBin )
{
  // Create histograms
  if(nEventIdsPerBin<=0) nEventIdsPerBin=1;
  int nbins=1; int first =0; int last=1;
  if(fEventSummaries.size()) {
    int firstid = fEventSummaries.front().GetId() ;
    int lastid  = fEventSummaries.back().GetId() ;
    nbins = (lastid-firstid+nEventIdsPerBin-1)/nEventIdsPerBin ;
    first = firstid ;
    last  = firstid + nbins * nEventIdsPerBin ;
  };
  
  cout << nbins << " " << first << " " << last << endl ;
  
  // Create histograms
  static TProfile *prAvTdc(0), *prNHits, *prNBadHits ;
  static TH1F *h1NHits, *h1NBadHits ;
  if(!prAvTdc) {
    prAvTdc    = new TProfile("prAvTdc"    ,"AvTdc"    ,nbins,first,last) ;
    prNHits    = new TProfile("prNHits"    ,"NHits"    ,nbins,first,last) ;
    prNBadHits = new TProfile("prNBadHits" ,"NBadHits" ,nbins,first,last) ;
    h1NHits    = new TH1F("h1NHits"    ,"NHits"    ,nbins,first,last) ;
    h1NBadHits = new TH1F("h1NBadHits" ,"NBadHits" ,nbins,first,last) ;
  } else {
    *prAvTdc    = TProfile("prAvTdc"    ,"AvTdc"    ,nbins,first,last) ;
    *prNHits    = TProfile("prNHits"    ,"NHits"    ,nbins,first,last) ;
    *prNBadHits = TProfile("prNBadHits" ,"NBadHits" ,nbins,first,last) ;
    *h1NHits    = TH1F("h1NHits"    ,"NHits"    ,nbins,first,last) ;
    *h1NBadHits = TH1F("h1NBadHits" ,"NBadHits" ,nbins,first,last) ;
  };
    
  for(Container::const_iterator eventIt = fEventSummaries.begin() ;
      eventIt != fEventSummaries.end() ; eventIt++) {
    float id = eventIt->GetId() ;
    //prAvTdc->Fill( id, eventIt->GetAvTdc() ) ;
    h1NHits->Fill(id,float(eventIt->GetNHits())) ;
    h1NBadHits->Fill(id,float(eventIt->GetNBadHits())) ;
    prNHits->Fill(id,float(eventIt->GetNHits())) ;
    prNBadHits->Fill(id,float(eventIt->GetNBadHits())) ;
  };
};
#endif
