#ifdef USE_ROOT
 #include <TString.h>
 #include <TCanvas.h>
#endif

#include "wax/OTrSuperLayer.hh"
#include "wax/OTrSector.hh"
#include "wax/OTrEvent.hh"

//__________________________________________________________________
// OTrSuperLayer
//
// Container for:
// * Layers
// Functionality:
// * None
//
// Author: Wouter Hulsbergen && Ivar Siccama 
//__________________________________________________________________

#ifdef __OTR_CLASSDEF__
ClassImp(OTrSuperLayer) ;
//void OTrSuperLayer::Streamer(TBuffer &) {} ;
//void OTrSuperLayer::ShowMembers(TMemberInspector &, char *) {} ;
#endif

OTrSuperLayer::~OTrSuperLayer() {
  // Destructor.

  //cout << "OTrSuperLayer::~OTrSuperLayer()"<< endl ;
} ;

OTrSuperLayer::OTrSuperLayer() 
  : OTrDetectorComponent(0), OTrComponentMother(0) 
{
  // Default constructor. Do not use !
  cout << "OTrSuperLayer::OTrSuperLayer() ... please do not use " << endl ;
} ;

OTrSuperLayer::OTrSuperLayer(ComponentId_t id) 
  : OTrDetectorComponent(id), OTrComponentMother(6) 
{
  // Constructor with detector component id.

  //cout << "You created a SuperLayer with id " << GetId() << endl ;
};

OTrLayer*
OTrSuperLayer::AddLayer(LayerId_t layerId)
{
  //cout << "OTrSuperLayer::AddLayer(" << layerId << ")" << endl ; 
  //cout << "OTrSuperLayer::AddLayer(), id = " << GetId() << endl;
  OTrLayer* childIt = FindChild(layerId) ;
  if(childIt!=0) return childIt ;
  return AddChild(OTrLayer(this,SetLayerId(GetId(),layerId))) ;
}

void
OTrSuperLayer::Connect()
{
  for(Iterator layIt(*this); layIt.valid(); layIt.next() )
    (*layIt).Connect(this) ;
  return ;
}

void
OTrSuperLayer::Translate(const ThreeVector& p)
{
  // The first ansatz to alignment: Translate this superlayer with
  // vector p.
  for(Iterator layIt(*this); layIt.valid(); layIt.next() )
    (*layIt).Translate(p) ;
  return ;
}

#ifdef USE_ROOT
TString
OTrSuperLayer::GetName() const
{
  return TString(const_cast<char *>(GetLayers().front().GetSections().front().GetSuperLayerName().c_str())) ;
} 

void
OTrSuperLayer::DumpOccupancySummary( ostream& ofs, bool useMask )
{
  ofs << "Superlayer: " << GetSuperLayerId() << endl ;
  for(LayerContainer::iterator layIt=GetLayers().begin(); 
      layIt!=GetLayers().end(); layIt++) 
    (*layIt).DumpOccupancySummary( ofs, useMask ) ;
}

void 
OTrSuperLayer::Plot(TCanvas* canvas, const int choice, 
		    const OTrEvent* event, const bool use_mask )
{
  canvas->cd() ;
  for(LayerContainer::iterator layIt=GetLayers().begin(); layIt!=GetLayers().end(); layIt++) 
    (*layIt).CreateSectors() ;
  const OTrSuperLayerEvent* sulEvent(0) ;
  if(event) sulEvent = event->FindSuperLayerEvent(GetSuperLayerId());

  typedef vector<TPad*> PadContainer ;
  static PadContainer sectorPads ;

  cout << sectorPads.size() << " " << GetLayers().front().GetSectors().size() << endl ;
  if(sectorPads.size() != GetLayers().front().GetSectors().size()) {
    for(PadContainer::iterator padIt = sectorPads.begin() ;
	padIt != sectorPads.end() ; padIt++) delete *padIt ;
    sectorPads.erase(sectorPads.begin(),sectorPads.end()) ;
    canvas->Clear() ;
    OTrLayer* zeroLayer=0 ;
    for(LayerContainer::iterator layIt=GetLayers().begin(); layIt!=GetLayers().end(); layIt++) 
      if(layIt->GetSections().front().GetGede()->GetEStereoAngle()==OTrGede::kZeroDegrees) {
	zeroLayer=layIt;
	break ;
      }
    if(zeroLayer) {
      float padXmin = 0;
      float padYmin = 0;
      float padXmax = 1;
      float padYmax = 0.95;

      Point3D layPmin = zeroLayer->GetPMin() ; 
      Point3D layPmax = zeroLayer->GetPMax() ;
      float layXmin = layPmin.x() ; float layXmax = layPmax.x() ; 
      float layYmin = layPmin.y() ; float layYmax = layPmax.y() ; 
     //  cout << "layerpos " << layXmin << " " << layYmin << " "
      // 	   << layXmax << " " << layYmax << endl ;

      for(OTrLayer::SectorContainer::iterator sectorIt=zeroLayer->GetSectors().begin() ;
	  sectorIt!=zeroLayer->GetSectors().end(); sectorIt++ ) {
	Point3D sectorPmin = (*sectorIt)->GetPMin() ;
	Point3D sectorPmax = (*sectorIt)->GetPMax() ;
	float xmin = (padXmax-padXmin)*(sectorPmin.x()-layXmin)/(layXmax-layXmin) + padXmin ;
	float xmax = (padXmax-padXmin)*(sectorPmax.x()-layXmin)/(layXmax-layXmin) + padXmin ;
	float ymin = (padYmax-padYmin)*(sectorPmin.y()-layYmin)/(layYmax-layYmin) + padYmin ;
	float ymax = (padYmax-padYmin)*(sectorPmax.y()-layYmin)/(layYmax-layYmin) + padYmin ;
	// cout << "sectorpos: " << sectorPmin.x() << " "
	// 	     << sectorPmin.y() << " " << sectorPmax.x() << " " << sectorPmax.y() << endl ;
	cout << xmin << " " << ymin << " " << xmax << " " << ymax << endl ;
	sectorPads.push_back(new TPad("","",xmin+0.01,ymin+0.01,xmax-0.01,ymax-0.01)) ;
	sectorPads.back()->SetBorderMode(1);
	//sectorPads.back()->SetLeftMargin(0.05);
	//sectorPads.back()->SetBottomMargin(0.05);
	sectorPads.back()->Draw() ;
      }
    } else
      cout << "OTrSuperLayer::Plot(..): no zero degree layer ! " << endl ;
  }
    
  for(vector<OTrLayer>::iterator layIt=GetLayers().begin() ;
      layIt != GetLayers().end() ; layIt++) {
    char* option = "" ;
    if(layIt != GetLayers().begin()) option = "same" ;
    layIt->Plot( sectorPads, choice, option, sulEvent, use_mask );
  }
  canvas->cd() ;
  cout << "end of OTrSuperLayer::Plot" << endl ;
}
#endif
