#ifndef __OTR_CHANNELHISTOGRAMMER_HH__
#define __OTR_CHANNELHISTOGRAMMER_HH__

#ifdef USE_ROOT
#ifndef ROOT_TString
#include "TString.h"
#endif
#ifndef __OTR_CLASSDEF__
#include "Rtypes.h"
#endif
#endif

class OTrChannel ;
class OTrNodeProjection ;
class TH1D ;

class OTrChannelHistogrammer
{
public:
  // Typedefs
  typedef unsigned char TdcCount_t ;
  typedef unsigned short TimeHistogramBin_t ;
  
  // Constructors, desctructor, assignment operator
  OTrChannelHistogrammer() ;
  OTrChannelHistogrammer(const OTrChannelHistogrammer&) ;
  OTrChannelHistogrammer(OTrChannel*) ;
  virtual ~OTrChannelHistogrammer() ;
  OTrChannelHistogrammer& operator=(const OTrChannelHistogrammer& rhs) ;
  
  // Setters and getters
  OTrChannel* GetChannel() const { return fChannel ; } ;
  void SetChannel(OTrChannel* c) { fChannel=c; } ;
  float GetSumTdcCount() const { return fSumTdcCount ; } ;
  float GetSumTdcCount2() const { return fSumTdcCount2 ; } ;
  int GetNHit() const { return fNHit ; } ;
  const TimeHistogramBin_t* GetTimeHistogram() const { return fTimeBins ; } ;
  
  // Static setters and getters
  static int GetNTimeBins() { return fgNTimeBins ; } ;
  static int GetMaximumTimeAxis() { return fgMaximumTimeAxis ; } ;
  static int GetMinimumTimeAxis() { return fgMinimumTimeAxis ; } ;
  static int GetHistogrammingMode() { return fgHistogrammingMode ; } ;
  static void SetTimeHistogramming(bool b=true){ fgTimeHistogramming=b; } ;

  // Others ...
  float GetOccupancy() const ; // return the occupancy
  float GetSigma() const ;     // sigma of TDC values for hits
  float GetAverageTime() const { return fSumTdcCount/fNHit ; }
  float GetAverageTime2() const { return fSumTdcCount2/fNHit ; }
  float GetAverageTResidual() const { 
    if(fSumWeight>0) return fSumTResidual/fSumWeight ; return 0 ; }
  float GetAverageTResidual2() const { 
    if(fSumWeight>0) return fSumTResidual2/fSumWeight ; return 0 ; }
  float GetWidthTResidual() const ;
  int GetNProcessed() const ;  //the number of times a TDC value was processed
  int GetTZeroBin() const ;    // the bin number corresponding to channel's Tzero
  float GetAverageFromTimeH1() const; // the average tdc count in the time histogram
  
  void ProcessChannelBuffer(const TdcCount_t) ;
  void ProcessNodeProjection(const OTrNodeProjection*) ;
  void PerformHistogramming(const TdcCount_t) ;
  float UpdateTZero() ;

  void Reset() ;  // Reset all histogramming data

  // Other statics ...
  static int Bin2Count( const int bin ) ; // the tdc count from a histogram bin number
  
#ifdef USE_ROOT
#ifdef __OTR_CLASSDEF__
  ClassDef(OTrChannelHistogrammer,0) // Histogrammer for OTrChannel
#endif
  virtual TString GetName() const ;
  TH1D* GetTimeH1() const ;
  TH1D* GetTimeH1_B( float tshift = 0 ) const ;
#endif

private:
  OTrChannel* fChannel ; // pointer to mother channel
  int   fNHit ;          // total number of hits
  float fSumTdcCount ;   // sum of tdc counts of all hits
  float fSumTdcCount2 ;  // sum of tdc counts2 of all hits
  TimeHistogramBin_t* fTimeBins ;   // timehisto bins
  float fDChi2DTZero ;   // first derivative of total chi2 to tzero
  float fD2Chi2DTZero2 ; // second derivative of total chi2 to tzero
  float fNNodeP ;
  float fChi2 ;
  float fSumWeight ;
  float fSumTResidual ;
  float fSumTResidual2 ;
  //#ifdef USE_ROOT
  //  TH1F* fTResidualH1 ;
  //#endif

  // For the tdc spectra ...
  static bool fgTimeHistogramming ; // status flag for time histogramming
  static int fgNTimeBins ;          // number of time bins
  static int fgMaximumTimeAxis ;    // maximum histo x-axis
  static int fgMinimumTimeAxis ;    // miniimum histo x-axis
  static int fgHistogrammingMode ;  // histogramming mode
  static int fgTZeroUpdateFrequency ;
} ;

inline int OTrChannelHistogrammer::Bin2Count( const int bin ) 
{ 
  return int( (bin-1)*float(fgMaximumTimeAxis-fgMinimumTimeAxis)/
              float(fgNTimeBins) + fgMinimumTimeAxis );
}

inline void 
OTrChannelHistogrammer::ProcessChannelBuffer(const TdcCount_t count)
{
  fNHit++;
  fSumTdcCount+=count ;
  fSumTdcCount2+=count*count ;
} ;

#endif
