#ifndef __OTR_COMPONENTMOTHER_HH__
#define __OTR_COMPONENTMOTHER_HH__

#ifndef __OTR_COMPONENT_HH__
#include "OTrComponent.hh"
#endif
#ifndef __SGI_STL_VECTOR
#include <vector>
#endif

template <class T>
class OTrComponentMother
{
public:
  typedef vector<T> ChildContainer ;
  typedef OTrComponent::ItemId_t ChildItemId_t ;
protected:
  ///
  OTrComponentMother() {} ; 
  ///
  OTrComponentMother(const OTrComponentMother& rhs) ;
  ///
  OTrComponentMother(int NChildren)  ;
  ///
  inline ChildContainer& GetChildren() { return fChildren ; } ;
  inline const ChildContainer& GetChildren() const { return fChildren ; } ;
  ///
  inline OTrComponentMother& operator=(const OTrComponentMother&) ;
public:
  /// To get a child (via table)
  inline T* GetChild(ChildItemId_t id) ;
  inline const T* GetConstChild(ChildItemId_t id) const ;
  /// To find a child
  T* FindChild(ChildItemId_t id) ;
  inline const T* FindConstChild(ChildItemId_t id) const ;
  /// To add a child
  inline T* AddChild(const T& c) ;
  int GetNChildren() const { return fChildren.size() ; } ;
  class Iterator ;      friend class Iterator ;
  class ConstIterator ; friend class ConstIterator ;
  void Map() ;
private:
  vector<T> fChildren ;
  vector<T*> fChildrenMap ;
  ChildItemId_t fMinId ;
  ChildItemId_t fMaxId ;
};

template <class T>
inline OTrComponentMother<T>::OTrComponentMother(int NChildren) 
  : fChildren(), fChildrenMap() 
{
  fChildren.reserve(NChildren) ;
  fChildrenMap.reserve(NChildren) ;
}

template <class T>
inline void
OTrComponentMother<T>::Map()
{
  //cout << "OTrComponentMother<T>::Map()" << endl ;
  if(fChildren.size()) {
    vector<T>::iterator x=fChildren.begin() ;
    fMinId = x->GetItemId() ;
    fMaxId = fMinId ;
    for(; x!=fChildren.end() ; x++)
      {
	ChildItemId_t id= x->GetItemId() ;
	if(fMinId>id) fMinId=id;
	else if(fMaxId<id) fMaxId=id;
      };
    fChildrenMap.erase(fChildrenMap.begin(),fChildrenMap.end()) ;
    fChildrenMap.resize(fMaxId-fMinId+1,0) ;
    for(x=fChildren.begin();x!=fChildren.end() ; x++)
      fChildrenMap[x->GetItemId()-fMinId] = x;
  }
  //cout << "End of Map" << endl ;
}

template <class T>
inline  OTrComponentMother<T>& 
OTrComponentMother<T>::operator=(const OTrComponentMother& rhs)
{
  //cout << "OTrComponentMother<T>::operator=(const OTrComponentMother& rhs)" << endl;
  fChildren = rhs.fChildren ;
  Map() ;
  return *this;
};

template <class T>
inline 
OTrComponentMother<T>::OTrComponentMother(const OTrComponentMother& rhs)
{ 
  //cout << "OTrComponentMother(const OTrComponentMother& rhs)" << endl ;
  operator=(rhs) ;
};

template <class T>
inline T*
OTrComponentMother<T>::AddChild(const T& c)
{
  //cout << "OTrComponentMother<T>::AddChild(const T& c)" << endl ;
  fChildren.push_back(c) ;
  Map() ;
  return fChildren.end()-1 ;
};

template <class T>
inline T*
OTrComponentMother<T>::GetChild(ChildItemId_t id)
{
  if(id > fMaxId || id < fMinId) return 0;
  return fChildrenMap[id-fMinId] ;
};

template <class T>
inline const T*
OTrComponentMother<T>::GetConstChild(ChildItemId_t id) const
{
  if(id > fMaxId || id < fMinId) return 0;
  return fChildrenMap[id-fMinId] ;
};

#ifndef CINTDICT 

template <class T>
class OTrComponentMother<T>::ConstIterator
{
public:
  ConstIterator(const OTrComponentMother<T>& mother) :
    fChildren(mother.fChildren) { reset() ; } ;
  inline const T* next() { return fCur++ ; } ;
  inline bool valid() const { return fCur != fChildren.end() ; };
  inline const T* reset() { return fCur=fChildren.begin() ; };
  inline const T& operator*() const { return *fCur ; } ;
  inline const T& operator->() const { return *fCur ; } ;
  inline bool operator==(const T* a) const { return fCur==a; }; 
  inline const T* current() const { return fCur ; } ;
private:
  const vector<T>& fChildren;
  vector<T>::const_iterator fCur ; 
} ;

template <class T>
class OTrComponentMother<T>::Iterator
{
public:
  Iterator(OTrComponentMother<T>& mother) :
    fChildren(mother.fChildren) { reset() ; } ;
  inline T* next() { return fCur++ ; } ;
  inline bool valid() const { return fCur != fChildren.end() ; };
  inline T* reset() { return fCur=fChildren.begin() ; };
  inline T& operator*() { return *fCur ; } ;
  inline T& operator->() { return *fCur ; } ;
  inline bool operator==(const T* a) const { return fCur==a; }; 
  inline T* current() { return fCur ; } ;
private:
  vector<T>& fChildren ;
  vector<T>::iterator fCur ; 
} ;

#endif

template <class T>
inline T*
OTrComponentMother<T>::FindChild(ChildItemId_t childId)
{
  for(vector<T>::iterator childIt=fChildren.begin() ;
      childIt!=fChildren.end() ; childIt++)
    if((*childIt).GetItemId()== childId)
      return childIt ;
  return 0;
};

template <class T>
inline const T*
OTrComponentMother<T>::FindConstChild(ChildItemId_t childId) const
{
  for(vector<T>::const_iterator childIt=fChildren.begin() ;
      childIt!=fChildren.end() ; childIt++)
    if((*childIt).GetItemId()==childId)
      return childIt ;
  return 0;
};

#endif
