#ifndef __OTR_DETECTOR_HH__
#define __OTR_DETECTOR_HH__

#ifndef __STRING__
#include <string>
#endif
#ifndef __SGI_STL_VECTOR
#include <vector>
#endif
#ifndef __OTR_SUPERLAYER_HH__
#include "OTrSuperLayer.hh"
#endif

#ifdef __OTR_CLASSDEF__
#ifndef ROOT_Rtypes
#include <Rtypes.h>
#endif
#endif

class OTrDaq ;
class OTrEvent ;

class OTrDetector : public OTrDetectorComponent, 
		    public OTrComponentMother<OTrSuperLayer>
{
public:
  // typedefs, enumerates
  typedef ChildContainer SuperLayerContainer ;

  // constructors, destructos, assignment
  OTrDetector();
  OTrDetector( const string filename, OTrDaq* ) ; // constructor using an ASCII file
#ifdef USE_ARTE
  OTrDetector( OTrDaq* ); // constructor reading directly from ARTE
#endif
  virtual ~OTrDetector() ;

  // setters and getters
  OTrDaq* GetDaq() const { return fDaq ; }
  OTrSuperLayer* GetSuperLayer(SuperLayerId_t id) { return GetChild(id) ; } ;
  const OTrSuperLayer* GetSuperLayer(SuperLayerId_t id) const { 
    return GetConstChild(id) ; } ;
  OTrSuperLayer* FindSuperLayer(SuperLayerId_t id) { return FindChild(id) ; } ;
  const OTrSuperLayer* FindSuperLayer(SuperLayerId_t id) const { 
    return FindConstChild(id) ; } ;
    SuperLayerContainer& GetSuperLayers() { return GetChildren() ; } ;
  const SuperLayerContainer& GetSuperLayers() const { return GetChildren() ; } ;
  const vector<const OTrSection*>& GetGEDEIndexToSectionMap() { 
    return fGEDEIndexToSectionMap ; }
  OTrSuperLayer* AddSuperLayer(SuperLayerId_t id) ;

  // other methods
  void dump() const;
  void WriteSectionPositions(char *filename) ;
  void ResetGEDEs(); // reset the first and last hit of each GEDE
  const OTrSection* GetSectionFromGEDEIndex(int GEDEindex) {
    return fGEDEIndexToSectionMap[GEDEindex] ; }

#ifdef __OTR_CLASSDEF__
  // CINT class definition
  ClassDef(OTrDetector,0) // The outer tracker detector class
#endif

private:
  // private methods
  int build(); // build the detector from the database
  void BuildGEDEIndexToSectionMap() ;
  void OldMerge() ; // obsolete
  void Merge() ;// new merging routine

  // data members
  OTrDaq* fDaq ; // pointer to daq object
  vector<const OTrSection*> fGEDEIndexToSectionMap ;// maps gede indices to pointer to sections
};

#endif
