#ifndef __OTR_EVENTBUILDER_HH__
#define __OTR_EVENTBUILDER_HH__

#ifdef __OTR_CLASSDEF__
 #ifndef ROOT_Rtypes
 #include <Rtypes.h>
 #endif
#else
 #define BIT(n)       (1 << (n))
#endif

class OTrEvent ;
class OTrEventBuffer ;
class OTrDetector ;

class OTrEventBuilder
{
public:
  // typedefs, enumerates
  enum EDataStream { kDS_DOTR      =BIT(1),    // data from DOTR bank
		     kDS_FARM      =BIT(2),    // datafile in Farm (l4) format 
		     kDS_STANDALONE=BIT(3),    // datafile in StandAlone format
		     kDS_MC        =BIT(4),    // monte carlo data (from HITB bank)
		     kDS_DST       =BIT(5)} ;  // dst data (from HITB bank)
  enum EDataFormat { kDF_DAQ       =BIT(6),    // DOTR, FARM or STANDALONE
		     kDF_HITB      =BIT(7) } ; // MC or DST
  // constructors, destructor, assignment
  OTrEventBuilder() ;
  OTrEventBuilder(OTrDetector*) ;
  virtual ~OTrEventBuilder() ;
  
  // setters and getters
  OTrEvent* GetEvent() {return fEvent; }
  OTrDetector* GetDetector() { return fDetector; }
  int GetDataType() const { return fDataType ; }
  OTrEventBuffer* GetEventBuffer() { return fEventBuffer ; }
  int GetNProcessedEvents() { return fNProcessedEvents ; }
  static bool GetPerformReconstruction() { return fgPerformReconstruction ; }
  static void SetPerformReconstruction(bool on=true) { fgPerformReconstruction=on ; }
  // other methods
  OTrEvent* CreateNextEvent() ;
  int GetRunNumber() ;
  int GetEventNumber() ;
#ifdef __OTR_CLASSDEF__
  ClassDef(OTrEventBuilder,0) // Builds the OTrEvent (from HITB or FED buffer)
#endif
private:
  // private methods
  void EraseCurrentEvent() ;
  OTrEvent* DAQCreateEvent() ;
  OTrEvent* HITBCreateEvent() ;
  static int GetOuterTrackerCMP() { return 2 ; }
  int GetDataFormat() const { return GetDataType() & (kDF_DAQ+kDF_HITB) ; }
  int GetDataStream() const { 
    return GetDataType() & (kDS_DOTR+kDS_FARM+kDS_STANDALONE+kDS_MC+kDS_DST) ; }

  // data members
  OTrEventBuffer* fEventBuffer ; // eventbuffer (owner)
  OTrEvent* fEvent ;             // current event (owner)
  OTrDetector* fDetector ;       // detector
  int fDataType ;                // DAQ/HITB + DOTR/FARM/STANDALONE/MC/DST
  int fNProcessedEvents ;        // number of processed events
  static bool fgPerformReconstruction ; // flag for reconstruction
} ;

#endif
