#ifndef __OTR_SPLINERTRELATION_HH__
#define __OTR_SPLINERTRELATION_HH__

#ifndef __OTR_PARAMETERIZEDRTRELATION_HH__
#include "OTrParameterizedRTRelation.hh"
#endif
class EventSample ;
class istream ;
class TH1 ;

class OTrSplineRTRelation : public OTrParameterizedRTRelation
{
public:
  // constructors, destructors, assignment
#ifdef USE_ROOT
  // Calculates initial parameters using integrated time spectrum
  OTrSplineRTRelation(int, int, TH1*, HepDouble, HepDouble, 
		      HepDouble, HepDouble, HepDouble,int tdegree=3, int s2degree=3) ;
#endif
  OTrSplineRTRelation(istream&);        // Initializes parameters from file
  OTrSplineRTRelation(const char* filename) ; // Initializes parameters from file
  virtual ~OTrSplineRTRelation() ;

  // setters and getters
  int GetTDegree() const { return fTDegree; }
  int GetSigma2Degree() const { return fSigma2Degree; }
  const HepVector& GetTKnots() const { return fTKnots ; }
  const HepVector& GetSigma2Knots() const { return fSigma2Knots ; }

  // declared in base class
  virtual void CorrectT0() ;
  virtual void RepairTParameters() ;
  virtual void RepairSigma2Parameters() ;
  virtual vector<OTrFunctionValue> CalcBasisFuncVector(HepDouble,int) const ;
  virtual void SetUniformSigma(HepDouble) ;
  virtual void Write(ostream&) const ;
  virtual void Write(const char* filename) const ;
  virtual void Read(istream&) ; // Read from inputstream
  virtual void Read(const char* filename) ; // Read from inputstream
  virtual void SetNTParameters(int newdegree, int newn) ;
  virtual void SetNSigma2Parameters(int newdegree, int newn) ;

#ifdef __OTR_CLASSDEF__
  ClassDef(OTrSplineRTRelation,0)
#endif
private:
  // private methods
  static HepVector CalcEquidistantKnots(int degree, int npar) ;
  static double EvaluateBSpline(const HepVector&,
                         const int, const int, const double) ;
  static double EvaluateBSplineDerivative(const HepVector&,
                         const int, const int, const double) ;
  static double EvaluateBSplineSecondDerivative(const HepVector&,
			 const int, const int, const double) ;
  static void ChangeNParameters(const int k_old, const HepVector& knots_old,
			       const HepVector& par_old, const int n_new,
			       const int k_new, HepVector& knots_new,
			       HepVector& par_new) ;
  // data members
  int fTDegree ;       // degree of the t(r) spline
  int fSigma2Degree ;  // degree of the V(r) spline
  HepVector fTKnots ;      // position of knots for the t(r) spline
  HepVector fSigma2Knots ; // position of knots for the V(r) spline
 
} ;

#endif







