#ifndef __OTR_SUPERLAYEREVENT_HH__
#define __OTR_SUPERLAYEREVENT_HH__

#ifndef __SGI_STL_VECTOR
#include <vector>
#endif
#ifndef __OTR_SECTIONCLUSTER_HH__
#include "OTrSectionCluster.hh"
#endif
#ifdef USE_ROOT
 class TPad ;
 class TBuffer ;
 #ifndef ROOT_Rtypes
 #include <Rtypes.h>
 #endif
#endif
class OTrHit ;
class OTrSuperCluster ;

class OTrSuperLayerEvent
{
public:
  // typedefs, enumerates
  typedef vector<OTrHit*> HitContainer ;
  typedef vector<OTrSuperCluster*> ClusterContainer ;
  typedef vector<OTrSectionCluster> SectionClusterContainer ;

  // constructors, destructor, assignment
  OTrSuperLayerEvent() ;
  virtual ~OTrSuperLayerEvent() ;

  // setters and getters
  const ClusterContainer& GetClusters() const { return fClusters ; }
  ClusterContainer& GetClusters() { return fClusters ; }
  const HitContainer& GetHits() const { return fHits ; }
  HitContainer& GetHits() { return fHits ; }

  // other inlines
  int GetNHits() const { return fHits.size() ; } ;

  // other methods
  OTrHit* Add( OTrHit* ) ;
  int SearchClusters() ;
  void DumpHits(ostream& os=cout) const ;
  void DumpClusters(ostream& os=cout) const ;
  void PerformHistogramming() ;
  void SortHits(); // sort the hits per superlayer according to GEDE index and x

#ifdef USE_ARTE
  void AppendHITB() ;  // append hits to the HITB bank
  void AppendRSEG() ;  // append clusters to the RSEG bank
#endif

#ifdef USE_ROOT
  void DrawHits() ; // draw all the hits in the event display
  void DrawClusters() ; // draw the clusters in the event display
 #ifdef __OTR_CLASSDEF__
  ClassDef(OTrSuperLayerEvent,0) // The Outer Tracker superlayer event 
 #else
  void Streamer(TBuffer&) ;
 #endif
#endif
  
private:
  // private methods
  void SearchSectionClusters() ;
  void SearchSuperClusters() ;

  // data members
  HitContainer fHits ;                     // Measured hits
  ClusterContainer fClusters ;             // Reconstructed clusters 
  SectionClusterContainer fZeroClusters ;  // Clusters in zero stereo layer
  SectionClusterContainer fPlusClusters ;  // Clusters in plus stereo layer
  SectionClusterContainer fMinusClusters ; // Clusters in minus stereo layer
};

#endif
