#ifndef __OTR_WIRE_HH__
#define __OTR_WIRE_HH__

#ifndef __OTR_COMPONENT_HH__
#include "OTrComponent.hh"
#endif
#ifndef __OTR_GEDE_HH__
//#include "OTrSection.hh"
#include "OTrGede.hh"
#endif

class OTrSection ;
class OTrChannel ;
class OTrSector ;

class OTrWire: public OTrDetectorComponent
{
public:
  // Constructors, destructor, assignment operator
  OTrWire();
  OTrWire( const int wirenr, OTrSection*, ComponentId_t);
  virtual ~OTrWire() ;

  // Getters and Setters
  OTrChannel* GetChannel() const { return fChannel ; }
  OTrSection* GetSection() const { return fSection ; }
  OTrGede* GetGede() const { return (OTrGede*)fSection ; }
  int GetWireNumber() const { return fWireNumber ; }
  void SetChannel( OTrChannel* c ) { fChannel = c; }
  //OTrSector* GetSector() const { return 0 ; }

  // Other public methods
  virtual void Dump(ostream& os=cout) const;
  float GetLength() const { return GetGede()->GetWireLength() ; } ;
  int GetPRWireNumber() const { return GetGede()->GetPRWireNumber(GetWireNumber()) ; }
  int GetMonoLayer() const { return GetWireNumber()%(GetGede()->GetNMonoLayers()) ; }
  float GetRawX() const { return GetGede()->GetRawX(GetWireNumber()) ; }
  float GetRawY() const { return GetGede()->GetRawY(GetWireNumber()) ; }
  float GetRawZ() const { return GetGede()->GetRawZ(GetWireNumber()) ; }
  float GetCentralX() const { return GetGede()->GetCentralX(GetWireNumber()) ; }
  float GetCentralY() const { return GetGede()->GetCentralY(GetWireNumber()) ; }
  ItemId_t GetItemId() const { return GetWireId(); }
  
#ifdef __OTR_CLASSDEF__
  ClassDef(OTrWire,0) // The Outer Tracker wire class (almost obsolete)
#endif
private:
  OTrChannel* fChannel ; // pointer to corresponding TDC channel
  //OTrGede* fGede ; // pointer to mother section
  OTrSection* fSection ; // pointer to mother section
  int fWireNumber; // internal wire number (slight different from arte (+=1) and very
  //different from dima's database and even more different from the
  //Zeuthen stuff :-)
};

#endif
