#ifndef _controller_
#define _controller_

#include <iostream>
#include <fstream>
#include <stdio.h>
#ifdef __OTR_CLASSDEF_CONTROLLER__
  #ifndef ROOT_Rtypes
  #include <Rtypes.h>
  #endif
#endif

class controller ;
extern controller* gOTr ;

class OTrChannel;
class OTrEvent ;
class OTrDetector ;
class OTrEventBuilder ;
class OTrDaq ;
class OTrView;
class OTrLayer;
class OTrTZeroCalib ;
class controller ;
class OTrAlignment ;
 
class OTrRTCalib ;
class OTrRunSummary ;

class controller {
public:
  // constructors, destructor, assignment
  controller();
  virtual ~controller() ;

  // setters and getters
  OTrRunSummary* GetRunSummary() const { return fRunSummary ; } 
  OTrDaq* GetDaq() const { return fDaq ; } ;
  OTrDetector* GetDetector() const { return fDetector ; }
  OTrEventBuilder* GetEventBuilder() const { return fEventBuilder ; }
  OTrTZeroCalib* GetTZeroCalib() const { return fTZeroCalib; } 
  OTrAlignment* GetAlignment() const { return fAlignment ; } 
  OTrView* GetView() const { return fView ; }
  static void SetConfigDirName(const char* name) { strcpy(fgConfigDirName,name) ; }

  // other inlines 
  int next(const int n=1, const int prflag=0) { return Run(n) ; }

  // other methods
  void ProcessOneEvent() ; // Process one event
  int Run(const int n) ;   // Run over n events
  void UpdateRT(bool,bool,bool,bool) ;
  int open(char* filename="/hb/otr2/run98/data/otr_run_0074.dat", int farm=0) ;
  void SetSearchClusters(bool on=true) ; 
  //void temp() ;
  OTrTZeroCalib* CreateTZeroCalib() ;
  OTrRTCalib* CreateRTCalib() ;
  OTrAlignment* CreateAlignment() ;
  void CreateRTRelation(bool) const ;
  void WriteOccupancyMap(char* filename) ;
  void Reset() ;
  void WriteRTRelations(const char* name) ;
  void ReadRTRelations(const char* name) ;
  static void temp(const char* name) ;

#ifdef USE_ARTE
  int EventRowSummary( ostream& os = cout ) const; // print a short ROW summary
#endif

#ifdef USE_ROOT
  void Display(); // display the event
  int plot( const int sl = -1, const int choice = 2, const bool use_mask = 1 ); // plot histograms for each section of a certain superlayer
  int plot_combined( const int sl = 13 ); // plot the combined occupancy plots (5mm cells OR-ed)
  void PlotChannelTdcSpectra(int tdcId) const ;
  void PlotAllTdcSpectra() const ;
  void DrawRTRelation() const ;
  void PerformRawogramming() ; // Histogramming for the raw event
  void PerformRecogramming() ; // Histogramming for the clusters and tracks
  void DrawTxTy() const ;
  void SetCombinedLayer( const int slId, const int layId );
  /// print a short summary, used to identify missing FED problems
  void PrintSummary() const; // print a short summary, used to identify missing FED problems
#endif //USE_ROOT
#ifdef __OTR_CLASSDEF_CONTROLLER__
  ClassDef(controller,0)
#endif   
private:
  OTrDaq* fDaq ;                  // pointer to the daq
  OTrDetector* fDetector ;        // pointer to the detector
  OTrEventBuilder* fEventBuilder ;// the event builder
  OTrEvent* fEvent;               // the current event (not owner !)
  OTrRunSummary* fRunSummary ;    // run summary
  OTrTZeroCalib* fTZeroCalib;     // tzero calibration
  OTrRTCalib* fRTCalib ;          // rt calibration
  OTrAlignment* fAlignment ;      // alignment
  OTrView* fView;                 // view for the ROOT detector display
  OTrLayer* fCombinedLayer;       // layer for which combined occupancy plots are to be filled
  static char fgConfigDirName[256] ; // Name of WAX configuration directory
} ;

#endif
